@@ -2,37 +2,32 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 // FloatingWindow.cpp - For GNOME 3.26 and above floating window will be used.
 //
 ////////////////////////////////////////////////////////////////////////////
 
+#include "pgAdmin4.h"
 #include "FloatingWindow.h"
 #include "ui_FloatingWindow.h"
 
+#include <QMenu>
+#include <QMenuBar>
+
+
 FloatingWindow::FloatingWindow(QWidget *parent) :
-    QMainWindow(parent),
-    ui(new Ui::FloatingWindow)
+    QMainWindow(parent)
 {
-    m_newAction = Q_NULLPTR;
-    m_configAction = Q_NULLPTR;
-    m_logAction = Q_NULLPTR;
-    m_quitAction = Q_NULLPTR;
-    m_menuActions = Q_NULLPTR;
-    m_floatingWindowMenu = Q_NULLPTR;
-
-    ui->setupUi(this);
 }
 
-FloatingWindow::~FloatingWindow()
-{
-    delete ui;
-}
 
 bool FloatingWindow::Init()
 {
+    ui = new Ui::FloatingWindow;
+    ui->setupUi(this);
+
     // Creating Menu
     createMenu();
 
@@ -50,12 +45,13 @@ bool FloatingWindow::Init()
     return true;
 }
 
+
 // Create the menu
 void FloatingWindow::createMenu()
 {
     createActions();
 
-    m_floatingWindowMenu = menuBar()->addMenu(QString(tr("&%1")).arg(PGA_APP_NAME));
+    m_floatingWindowMenu = menuBar()->addMenu(tr("&pgAdmin 4"));
     m_floatingWindowMenu->addAction(m_newAction);
     m_floatingWindowMenu->addAction(m_copyUrlAction);
     m_floatingWindowMenu->addSeparator();
@@ -65,19 +61,24 @@ void FloatingWindow::createMenu()
     m_floatingWindowMenu->addAction(m_quitAction);
 }
 
+
 // Create the menu actions
 void FloatingWindow::createActions()
 {
-    m_newAction = new QAction(QString(tr("&New %1 window...")).arg(PGA_APP_NAME), this);
+    m_newAction = new QAction(tr("&New pgAdmin 4 window..."), this);
+    m_newAction->setEnabled(false);
     connect(m_newAction, SIGNAL(triggered()), m_menuActions, SLOT(onNew()));
 
     m_copyUrlAction = new QAction(tr("&Copy server URL"), this);
+    m_copyUrlAction->setEnabled(false);
     connect(m_copyUrlAction, SIGNAL(triggered()), m_menuActions, SLOT(onCopyUrl()));
 
     m_configAction = new QAction(tr("C&onfigure..."), this);
+    m_configAction->setEnabled(false);
     connect(m_configAction, SIGNAL(triggered()), m_menuActions, SLOT(onConfig()));
 
     m_logAction = new QAction(tr("&View log..."), this);
+    m_logAction->setEnabled(false);
     connect(m_logAction, SIGNAL(triggered()), m_menuActions, SLOT(onLog()));
 
     m_quitAction = new QAction(tr("&Shut down server"), this);
@@ -85,12 +86,23 @@ void FloatingWindow::createActions()
     connect(m_quitAction, SIGNAL(triggered()), m_menuActions, SLOT(onQuit()));
 }
 
-void FloatingWindow::enableShutdownMenu()
+
+void FloatingWindow::enablePostStartOptions()
 {
+    if (m_newAction != Q_NULLPTR)
+        m_newAction->setEnabled(true);
+
+    if (m_copyUrlAction != Q_NULLPTR)
+        m_copyUrlAction->setEnabled(true);
+
+    if (m_configAction != Q_NULLPTR)
+        m_configAction->setEnabled(true);
+
+    if (m_logAction != Q_NULLPTR)
+        m_logAction->setEnabled(true);
+
     if (m_quitAction != Q_NULLPTR)
-    {
         m_quitAction->setEnabled(true);
-    }
 }
 
 void FloatingWindow::setMenuActions(MenuActions * menuActions)
@@ -98,6 +110,34 @@ void FloatingWindow::setMenuActions(MenuActions * menuActions)
     m_menuActions = menuActions;
 }
 
+// Enable the View Log option
+void FloatingWindow::enableViewLogOption()
+{
+    if (m_logAction != Q_NULLPTR)
+        m_logAction->setEnabled(true);
+}
+
+// Disable the View Log option
+void FloatingWindow::disableViewLogOption()
+{
+    if (m_logAction != Q_NULLPTR)
+        m_logAction->setEnabled(false);
+}
+
+// Enable the configure option
+void FloatingWindow::enableConfigOption()
+{
+    if (m_configAction != Q_NULLPTR)
+        m_configAction->setEnabled(true);
+}
+
+// Disable the configure option
+void FloatingWindow::disableConfigOption()
+{
+    if (m_configAction != Q_NULLPTR)
+        m_configAction->setEnabled(false);
+}
+
 void FloatingWindow::closeEvent(QCloseEvent * event)
 {
     // Emit the signal to shut down the python server.
