@@ -2,18 +2,16 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 // FloatingWindow.h - For GNOME 3.26 and above floating window will be used.
 //
 ////////////////////////////////////////////////////////////////////////////
 
-
 #ifndef FLOATINGWINDOW_H
 #define FLOATINGWINDOW_H
 
-#include "pgAdmin4.h"
 #include "MenuActions.h"
 
 #include <QMainWindow>
@@ -28,10 +26,13 @@ class FloatingWindow : public QMainWindow
 
 public:
     explicit FloatingWindow(QWidget *parent = Q_NULLPTR);
-    ~FloatingWindow();
 
     bool Init();
-    void enableShutdownMenu();
+    void enablePostStartOptions();
+    void enableViewLogOption();
+    void disableViewLogOption();
+    void enableConfigOption();
+    void disableConfigOption();
     void setMenuActions(MenuActions * menuActions);
 
 private:
@@ -41,14 +42,14 @@ private:
     void createActions();
     void closeEvent(QCloseEvent * event);
 
-    QAction *m_newAction;
-    QAction *m_copyUrlAction;
-    QAction *m_configAction;
-    QAction *m_logAction;
-    QAction *m_quitAction;
+    QAction *m_newAction = Q_NULLPTR;
+    QAction *m_copyUrlAction = Q_NULLPTR;
+    QAction *m_configAction = Q_NULLPTR;
+    QAction *m_logAction = Q_NULLPTR;
+    QAction *m_quitAction = Q_NULLPTR;
 
-    QMenu *m_floatingWindowMenu;
-    MenuActions *m_menuActions;
+    QMenu *m_floatingWindowMenu = Q_NULLPTR;
+    MenuActions *m_menuActions = Q_NULLPTR;
 
 signals:
     void shutdownSignal(QUrl);
