@@ -2,7 +2,7 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 // LogWindow.h - Log viewer window
@@ -13,6 +13,7 @@
 #define LOGWINDOW_H
 
 #include <QDialog>
+#include <QPlainTextEdit>
 
 namespace Ui {
 class LogWindow;
@@ -23,10 +24,8 @@ class LogWindow : public QDialog
     Q_OBJECT
 
 public:
-    explicit LogWindow(QWidget *parent = Q_NULLPTR, QString logFile = "");
-    ~LogWindow();
-
-    void ReadLog();
+    explicit LogWindow(QWidget *parent = Q_NULLPTR);
+    void LoadLog();
 
 private slots:
     void reload();
@@ -34,7 +33,8 @@ private slots:
 private:
     Ui::LogWindow *ui;
 
-    QString m_logFile;
+    void initLogWindow();
+    int readLog(QString logFile, QPlainTextEdit *logWidget);
 };
 
 #endif // LOGWINDOW_H
