@@ -2,7 +2,7 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 // Logger.cpp - Logger Utility
@@ -11,13 +11,18 @@
 
 #include "pgAdmin4.h"
 #include "Logger.h"
+
+#include <QDateTime>
+#include <QTextStream>
+#include <QStandardPaths>
+
 Logger* Logger::m_pThis = Q_NULLPTR;
-QString Logger::m_sFileName = "";
 QFile* Logger::m_Logfile = Q_NULLPTR;
 
 Logger::Logger()
 {
 }
+
 Logger::~Logger()
 {
 }
@@ -27,9 +32,8 @@ Logger* Logger::GetLogger()
     if (m_pThis == Q_NULLPTR)
     {
         m_pThis = new Logger();
-        m_sFileName = QDir::homePath() + (QString("/.%1.startup.log").arg(PGA_APP_NAME)).remove(" ");
         m_Logfile = new QFile;
-        m_Logfile->setFileName(m_sFileName);
+        m_Logfile->setFileName(g_startupLogFile);
         m_Logfile->open(QIODevice::WriteOnly | QIODevice::Text);
         m_Logfile->setPermissions(QFile::ReadOwner|QFile::WriteOwner);
     }
@@ -37,9 +41,9 @@ Logger* Logger::GetLogger()
     return m_pThis;
 }
 
-void Logger::Log(const QString& sMessage)
+void Logger::Log(const QString& sMessage) const
 {
-    QString text = QDateTime::currentDateTime().toString("dd.MM.yyyy hh:mm:ss ") + sMessage + "\n";
+    QString text = QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss: ") + sMessage + "\n";
     if (m_Logfile != Q_NULLPTR)
     {
         QTextStream out(m_Logfile);
