@@ -2,7 +2,7 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 // Logger.h - Logger Utility
@@ -20,15 +20,13 @@ class Logger : public QObject
 public:
     static Logger* GetLogger();
     static void ReleaseLogger();
-    void Log(const QString& sMessage);
+    void Log(const QString& sMessage) const;
 
 private:
     Logger();
     virtual ~Logger();
 
-private:
     static Logger* m_pThis;
-    static QString m_sFileName;
     static QFile *m_Logfile;
 };
 
