@@ -4,7 +4,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -12,7 +12,6 @@
 # This utility will allow us to change the copyright year information from
 # all the files present in FILE_EXTENSIONS variable at once
 
-from __future__ import print_function
 import os
 import sys
 import re
@@ -33,7 +32,7 @@ def is_code_file(filename, extensions=ALLOWED_FILE_EXTENSIONS):
 
 
 # Main function which will iterate and replace the copyright year
-def findReplace(directory, find, replace):
+def find_replace(directory, find, replace):
     total = 0
     COPYRIGHT_PATTERN = re.compile(
         r'(Copyright \(C\) \d{{4}} - ){0},'.format(find)
@@ -72,7 +71,7 @@ def findReplace(directory, find, replace):
                     print("Done")
                 else:
                     print("N/A")
-            except Exception as e:
+            except Exception:
                 failed.append(current_file)
                 print("FAILED")
 
@@ -96,7 +95,7 @@ if __name__ == '__main__':
     # Search and Replace the Copyright information from Parent folder
     parent_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 
-    failed, files_affected = findReplace(parent_dir, sys.argv[1], sys.argv[2])
+    failed, files_affected = find_replace(parent_dir, sys.argv[1], sys.argv[2])
 
     if len(failed) > 0:
         print("Failed to process the following files:\n", "\n\t".join(failed))
