@@ -4,12 +4,12 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
-# This utility will generate a iist of all dependencies, their upstream
+# This utility will generate a list of all dependencies, their upstream
 # repos and licence information.
 
 import os
@@ -52,10 +52,7 @@ def get_python_deps():
                                 "/../requirements.txt")
 
     with open(req_file, 'r') as req_file_p:
-        if sys.version_info[0] >= 3:
-            required = req_file_p.read().splitlines()
-        else:
-            required = req_file_p.read().decode("utf-8").splitlines()
+        required = req_file_p.read().splitlines()
 
     # Get the package info from the requirements file
     requirements = pkg_resources.parse_requirements(required)
@@ -68,7 +65,7 @@ def get_python_deps():
 
         try:
             distribution = pkg_resources.get_distribution(pkg)
-        except IndexError as e:
+        except IndexError:
             # The package probably isn't required on this version of Python,
             # thus we have no info about it.
             have_unknowns = True
@@ -91,6 +88,9 @@ def get_python_deps():
 
             # Next one....
             continue
+        except pkg_resources.VersionConflict:
+            # Next one....
+            continue
 
         try:
             metadata = distribution.get_metadata_lines('METADATA')
@@ -168,7 +168,6 @@ def get_js_deps():
         if module[2] != "":
             licence = module[2]
 
-        url = "Unknown"
         if module[3] != "":
             url = module[3]
 
@@ -183,17 +182,19 @@ def dump_header():
     print(textwrap.fill(
         "pgAdmin 4 is built on C++, Python and Javascript, and is "
         "dependent on various third party libraries. These are "
-        "automatically compiled from the system, requirements.txt."
-        "and packages.json and list below.",
+        "automatically compiled from the system, requirements.txt "
+        "and packages.json and listed below.",
         width=79) + "\n")
 
 
 def dump_cplusplus():
     print_title("C++ Dependencies")
     print_table_header()
-    print_row("QT", "4.6.2+", "LGPL v2.1/3", "http://www.qt.io/")
-    print_row("Python", "2.7/3.4+", "PSF", "https://www.python.org/")
-    print_summary(2)
+    print_row("QT", "5+", "LGPL v2.1/3", "https://www.qt.io/")
+    print_row("QDarkStyleSheet", "2.8.1", "MIT (code), CC BY 4.0 (images)",
+              "https://github.com/ColinDuquesnoy/QDarkStyleSheet")
+    print_row("Python", "3.4+", "PSF", "https://www.python.org/")
+    print_summary(3)
 
 
 def dump_python():
