@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -222,10 +222,6 @@ def upgrade():
             sql = "INSERT INTO keys (name, value) VALUES ('SECURITY_PASSWORD_SALT', 'SuperSecret3')"
         db.engine.execute(sql)
 
-        # Finally, update the schema version
-
-    # version.value = config.SETTINGS_SCHEMA_VERSION
-
     db.engine.execute(
         'UPDATE version set value="%s" WHERE name = "ConfigDB"' % config.SETTINGS_SCHEMA_VERSION
     )
@@ -233,9 +229,5 @@ def upgrade():
 
 
 def downgrade():
-    # ### commands auto generated by Alembic - please adjust! ###
-    print(u"""
-    Cannot downgrade from this version
-    Exiting...""")
-    sys.exit(1)
-    # ### end Alembic commands ###
+    # pgAdmin only upgrades, downgrade not implemented.
+    pass
