@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -12,41 +12,49 @@ such as setup of logging, dynamic loading of modules etc."""
 import logging
 import os
 import sys
+import re
+import ipaddress
+from types import MethodType
 from collections import defaultdict
 from importlib import import_module
 
 from flask import Flask, abort, request, current_app, session, url_for
 from werkzeug.exceptions import HTTPException
 from flask_babelex import Babel, gettext
+from flask_babelex import gettext as _
 from flask_login import user_logged_in, user_logged_out
 from flask_mail import Mail
 from flask_paranoid import Paranoid
 from flask_security import Security, SQLAlchemyUserDatastore, current_user
-from flask_security.utils import login_user
+from flask_security.utils import login_user, logout_user
 from werkzeug.datastructures import ImmutableDict
 from werkzeug.local import LocalProxy
 from werkzeug.utils import find_modules
 
-from pgadmin.model import db, Role, Server, ServerGroup, \
+from pgadmin.model import db, Role, Server, SharedServer, ServerGroup, \
     User, Keys, Version, SCHEMA_VERSION as CURRENT_SCHEMA_VERSION
-from pgadmin.utils import PgAdminModule, driver
+from pgadmin.utils import PgAdminModule, driver, KeyManager
 from pgadmin.utils.preferences import Preferences
 from pgadmin.utils.session import create_session_interface, pga_unauthorised
 from pgadmin.utils.versioned_template_loader import VersionedTemplateLoader
 from datetime import timedelta
 from pgadmin.setup import get_version, set_version
-from pgadmin.utils.ajax import internal_server_error
+from pgadmin.utils.ajax import internal_server_error, make_json_response
+from pgadmin.utils.csrf import pgCSRFProtect
+from pgadmin import authenticate
+from pgadmin.utils.security_headers import SecurityHeaders
+from pgadmin.utils.constants import KERBEROS
 
+# Explicitly set the mime-types so that a corrupted windows registry will not
+# affect pgAdmin 4 to be load properly. This will avoid the issues that may
+# occur due to security fix of X_CONTENT_TYPE_OPTIONS = "nosniff".
+import mimetypes
+mimetypes.add_type('application/javascript', '.js')
+mimetypes.add_type('text/css', '.css')
 
-# If script is running under python3, it will not have the xrange function
-# defined
 winreg = None
-if sys.version_info[0] >= 3:
-    xrange = range
-    if os.name == 'nt':
-        import winreg
-elif os.name == 'nt':
-    import _winreg as winreg
+if os.name == 'nt':
+    import winreg
 
 
 class PgAdmin(Flask):
@@ -56,6 +64,8 @@ class PgAdmin(Flask):
             extensions=['jinja2.ext.autoescape', 'jinja2.ext.with_'],
             loader=VersionedTemplateLoader(self)
         )
+        self.logout_hooks = []
+
         super(PgAdmin, self).__init__(*args, **kwargs)
 
     def find_submodules(self, basemodule):
@@ -161,6 +171,11 @@ class PgAdmin(Flask):
                           for key, value in menu_items.items())
         return menu_items
 
+    def register_logout_hook(self, module):
+        if hasattr(module, 'on_logout') and \
+                isinstance(getattr(module, 'on_logout'), MethodType):
+            self.logout_hooks.append(module)
+
 
 def _find_blueprint():
     if request.blueprint:
@@ -176,6 +191,11 @@ def create_app(app_name=None):
     if not app_name:
         app_name = config.APP_NAME
 
+    # Check if app is created for CLI operations or Web
+    cli_mode = False
+    if app_name.endswith('-cli'):
+        cli_mode = True
+
     # Only enable password related functionality in server mode.
     if config.SERVER_MODE is True:
         # Some times we need to access these config params where application
@@ -205,6 +225,10 @@ def create_app(app_name=None):
     # Setup logging and log the application startup
     ##########################################################################
 
+    # We won't care about errors in the logging system, we are more
+    # interested in application errors.
+    logging.raiseExceptions = False
+
     # Add SQL level logging, and set the base logging level
     logging.addLevelName(25, 'SQL')
     app.logger.setLevel(logging.DEBUG)
@@ -215,7 +239,7 @@ def create_app(app_name=None):
     # stream handler thus ensuring all the logging goes through the pgAdmin
     # logger.
     logger = logging.getLogger('werkzeug')
-    logger.setLevel(logging.INFO)
+    logger.setLevel(config.CONSOLE_LOG_LEVEL)
 
     # Set SQLITE_PATH to TEST_SQLITE_PATH while running test cases
     if (
@@ -223,17 +247,20 @@ def create_app(app_name=None):
         os.environ['PGADMIN_TESTING_MODE'] == '1'
     ):
         config.SQLITE_PATH = config.TEST_SQLITE_PATH
+        config.MASTER_PASSWORD_REQUIRED = False
+        config.UPGRADE_CHECK_ENABLED = False
 
-    # Ensure the various working directories exist
-    from pgadmin.setup import create_app_data_directory, db_upgrade
-    create_app_data_directory(config)
+    if not cli_mode:
+        # Ensure the various working directories exist
+        from pgadmin.setup import create_app_data_directory
+        create_app_data_directory(config)
 
-    # File logging
-    fh = logging.FileHandler(config.LOG_FILE, encoding='utf-8')
-    fh.setLevel(config.FILE_LOG_LEVEL)
-    fh.setFormatter(logging.Formatter(config.FILE_LOG_FORMAT))
-    app.logger.addHandler(fh)
-    logger.addHandler(fh)
+        # File logging
+        fh = logging.FileHandler(config.LOG_FILE, encoding='utf-8')
+        fh.setLevel(config.FILE_LOG_LEVEL)
+        fh.setFormatter(logging.Formatter(config.FILE_LOG_FORMAT))
+        app.logger.addHandler(fh)
+        logger.addHandler(fh)
 
     # Console logging
     ch = logging.StreamHandler()
@@ -270,7 +297,7 @@ def create_app(app_name=None):
                 if user is not None:
                     user_id = user.id
             user_language = Preferences.raw_value(
-                'miscellaneous', 'user_language', None, user_id
+                'misc', 'user_language', 'user_language', user_id
             )
             if user_language is not None:
                 language = user_language
@@ -294,11 +321,16 @@ def create_app(app_name=None):
     # Setup authentication
     ##########################################################################
 
-    app.config['SQLALCHEMY_DATABASE_URI'] = u'sqlite:///{0}?timeout={1}' \
-        .format(config.SQLITE_PATH.replace(u'\\', u'/'),
+    app.config['SQLALCHEMY_DATABASE_URI'] = 'sqlite:///{0}?timeout={1}' \
+        .format(config.SQLITE_PATH.replace('\\', '/'),
                 getattr(config, 'SQLITE_TIMEOUT', 500)
                 )
 
+    # Override USER_DOES_NOT_EXIST and INVALID_PASSWORD messages from flask.
+    app.config['SECURITY_MSG_USER_DOES_NOT_EXIST'] = \
+        app.config['SECURITY_MSG_INVALID_PASSWORD'] = \
+        (gettext("Incorrect username or password."), "error")
+
     # Create database connection object and mailer
     db.init_app(app)
 
@@ -308,11 +340,22 @@ def create_app(app_name=None):
     with app.app_context():
         # Run migration for the first time i.e. create database
         from config import SQLITE_PATH
+        from pgadmin.setup import db_upgrade
 
         # If version not available, user must have aborted. Tables are not
         # created and so its an empty db
         if not os.path.exists(SQLITE_PATH) or get_version() == -1:
-            db_upgrade(app)
+            # If running in cli mode then don't try to upgrade, just raise
+            # the exception
+            if not cli_mode:
+                db_upgrade(app)
+            else:
+                if not os.path.exists(SQLITE_PATH):
+                    raise FileNotFoundError(
+                        'SQLite database file "' + SQLITE_PATH +
+                        '" does not exists.')
+                raise RuntimeError('Specified SQLite database file '
+                                   'is not valid.')
         else:
             schema_version = get_version()
 
@@ -326,10 +369,15 @@ def create_app(app_name=None):
                 set_version(CURRENT_SCHEMA_VERSION)
                 db.session.commit()
 
+        if os.name != 'nt':
+            os.chmod(config.SQLITE_PATH, 0o600)
+
     Mail(app)
 
-    import pgadmin.utils.paths as paths
-    paths.init_app(app)
+    # Don't bother paths when running in cli mode
+    if not cli_mode:
+        import pgadmin.utils.paths as paths
+        paths.init_app(app)
 
     # Setup Flask-Security
     user_datastore = SQLAlchemyUserDatastore(db, User, Role)
@@ -352,7 +400,10 @@ def create_app(app_name=None):
         'CSRF_SESSION_KEY': config.CSRF_SESSION_KEY,
         'SECRET_KEY': config.SECRET_KEY,
         'SECURITY_PASSWORD_SALT': config.SECURITY_PASSWORD_SALT,
-        'SESSION_COOKIE_DOMAIN': config.SESSION_COOKIE_DOMAIN
+        'SESSION_COOKIE_DOMAIN': config.SESSION_COOKIE_DOMAIN,
+        # CSRF Token expiration till session expires
+        'WTF_CSRF_TIME_LIMIT': getattr(config, 'CSRF_TIME_LIMIT', None),
+        'WTF_CSRF_METHODS': ['GET', 'POST', 'PUT', 'DELETE'],
     }))
 
     security.init_app(app, user_datastore)
@@ -364,12 +415,13 @@ def create_app(app_name=None):
     app.permanent_session_lifetime = timedelta(
         days=config.SESSION_EXPIRATION_TIME)
 
-    app.session_interface = create_session_interface(
-        app, config.SESSION_SKIP_PATHS
-    )
+    if not cli_mode:
+        app.session_interface = create_session_interface(
+            app, config.SESSION_SKIP_PATHS
+        )
 
     # Make the Session more secure against XSS & CSRF when running in web mode
-    if config.SERVER_MODE:
+    if config.SERVER_MODE and config.ENHANCED_COOKIE_PROTECTION:
         paranoid = Paranoid(app)
         paranoid.redirect_view = 'browser.index'
 
@@ -377,6 +429,7 @@ def create_app(app_name=None):
     # Load all available server drivers
     ##########################################################################
     driver.init_app(app)
+    authenticate.init_app(app)
 
     ##########################################################################
     # Register language to the preferences after login
@@ -390,7 +443,7 @@ def create_app(app_name=None):
             language = data['language']
 
             # Set the user language preference
-            misc_preference = Preferences.module('miscellaneous')
+            misc_preference = Preferences.module('misc')
             user_languages = misc_preference.preference(
                 'user_language'
             )
@@ -450,7 +503,7 @@ def create_app(app_name=None):
 
             try:
                 proc_arch64 = os.environ['PROCESSOR_ARCHITEW6432'].lower()
-            except Exception as e:
+            except Exception:
                 proc_arch64 = None
 
             if proc_arch == 'x86' and not proc_arch64:
@@ -464,10 +517,10 @@ def create_app(app_name=None):
                     try:
                         root_key = winreg.OpenKey(
                             winreg.HKEY_LOCAL_MACHINE,
-                            "SOFTWARE\\" + server_type + "\Services", 0,
+                            "SOFTWARE\\" + server_type + "\\Services", 0,
                             winreg.KEY_READ | arch_key
                         )
-                        for i in xrange(0, winreg.QueryInfoKey(root_key)[0]):
+                        for i in range(0, winreg.QueryInfoKey(root_key)[0]):
                             inst_id = winreg.EnumKey(root_key, i)
                             inst_key = winreg.OpenKey(root_key, inst_id)
 
@@ -480,16 +533,14 @@ def create_app(app_name=None):
                             svr_port = winreg.QueryValueEx(inst_key, 'Port')[0]
                             svr_discovery_id = inst_id
                             svr_comment = gettext(
-                                "Auto-detected %s installation with the data "
-                                "directory at %s" % (
+                                "Auto-detected {0} installation with the data "
+                                "directory at {1}").format(
                                     winreg.QueryValueEx(
                                         inst_key, 'Display Name'
                                     )[0],
                                     winreg.QueryValueEx(
                                         inst_key, 'Data Directory'
-                                    )[0]
-                                )
-                            )
+                                    )[0])
 
                             add_server(
                                 user_id, servergroup_id, svr_name,
@@ -498,14 +549,11 @@ def create_app(app_name=None):
                             )
 
                             inst_key.Close()
-                    except Exception as e:
+                    except Exception:
                         pass
         else:
             # We use the postgres-winreg.ini file on non-Windows
-            try:
-                from configparser import ConfigParser
-            except ImportError:
-                from ConfigParser import ConfigParser  # Python 2
+            from configparser import ConfigParser
 
             registry = ConfigParser()
 
@@ -534,20 +582,14 @@ def create_app(app_name=None):
                     svr_discovery_id = section
                     description = registry.get(section, 'Description')
                     data_directory = registry.get(section, 'DataDirectory')
-                    if hasattr(str, 'decode'):
-                        description = description.decode('utf-8')
-                        data_directory = data_directory.decode('utf-8')
-                    svr_comment = gettext(u"Auto-detected %s installation "
-                                          u"with the data directory at %s" % (
-                                              description,
-                                              data_directory
-                                          )
-                                          )
+                    svr_comment = gettext("Auto-detected {0} installation "
+                                          "with the data directory at {1}"
+                                          ).format(description, data_directory)
                     add_server(user_id, servergroup_id, svr_name,
                                svr_superuser, svr_port, svr_discovery_id,
                                svr_comment)
 
-        except Exception as e:
+        except Exception:
             pass
 
     @user_logged_in.connect_via(app)
@@ -555,10 +597,28 @@ def create_app(app_name=None):
     def force_session_write(app, user):
         session.force_write = True
 
+    @user_logged_in.connect_via(app)
+    def store_crypt_key(app, user):
+        # in desktop mode, master password is used to encrypt/decrypt
+        # and is stored in the keyManager memory
+        if config.SERVER_MODE and 'password' in request.form:
+            current_app.keyManager.set(request.form['password'])
+
     @user_logged_out.connect_via(app)
-    def clear_current_user_connections(app, user):
+    def current_user_cleanup(app, user):
         from config import PG_DEFAULT_DRIVER
         from pgadmin.utils.driver import get_driver
+        from flask import current_app
+
+        # remove key
+        current_app.keyManager.reset()
+
+        for mdl in current_app.logout_hooks:
+            try:
+                mdl.on_logout(user)
+            except Exception as e:
+                current_app.logger.exception(e)
+
         _driver = get_driver(PG_DEFAULT_DRIVER)
         _driver.gc_own()
 
@@ -568,6 +628,42 @@ def create_app(app_name=None):
     for module in app.find_submodules('pgadmin'):
         app.logger.info('Registering blueprint module: %s' % module)
         app.register_blueprint(module)
+        app.register_logout_hook(module)
+
+    @app.before_request
+    def limit_host_addr():
+        """
+        This function validate the hosts from ALLOWED_HOSTS before allowing
+        HTTP request to avoid Host Header Injection attack
+        :return: None/JSON response with 403 HTTP status code
+        """
+        client_host = str(request.host).split(':')[0]
+        valid = True
+        allowed_hosts = config.ALLOWED_HOSTS
+
+        if len(allowed_hosts) != 0:
+            regex = re.compile(
+                r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}(?:/\d{1,2}|)')
+            # Create separate list for ip addresses and host names
+            ip_set = list(filter(lambda ip: regex.match(ip), allowed_hosts))
+            host_set = list(filter(lambda ip: not regex.match(ip),
+                                   allowed_hosts))
+            is_ip = regex.match(client_host)
+            if is_ip:
+                ip_address = []
+                for ip in ip_set:
+                    ip_address.extend(list(ipaddress.ip_network(ip)))
+                valid = ip_address.__contains__(
+                    ipaddress.ip_address(client_host)
+                )
+            else:
+                valid = host_set.__contains__(client_host)
+
+        if not valid:
+            return make_json_response(
+                status=403, success=0,
+                errormsg=_("403 FORBIDDEN")
+            )
 
     ##########################################################################
     # Handle the desktop login
@@ -579,14 +675,14 @@ def create_app(app_name=None):
 
         # Check the auth key is valid, if it's set, and we're not in server
         # mode, and it's not a help file request.
-        if not config.SERVER_MODE and app.PGADMIN_KEY != '':
-            if (
-                ('key' not in request.args or
-                 request.args['key'] != app.PGADMIN_KEY) and
-                request.cookies.get('PGADMIN_KEY') != app.PGADMIN_KEY and
-                request.endpoint != 'help.static'
-            ):
-                abort(401)
+
+        if not config.SERVER_MODE and app.PGADMIN_INT_KEY != '' and ((
+            'key' not in request.args or
+            request.args['key'] != app.PGADMIN_INT_KEY) and
+            request.cookies.get('PGADMIN_INT_KEY') != app.PGADMIN_INT_KEY and
+            request.endpoint != 'help.static'
+        ):
+            abort(401)
 
         if not config.SERVER_MODE and not current_user.is_authenticated:
             user = user_datastore.get_user(config.DESKTOP_USER)
@@ -601,6 +697,22 @@ def create_app(app_name=None):
                 )
                 abort(401)
             login_user(user)
+        elif config.SERVER_MODE and\
+                app.PGADMIN_EXTERNAL_AUTH_SOURCE ==\
+                KERBEROS and \
+                not current_user.is_authenticated and \
+                request.endpoint in ('redirects.index', 'security.login'):
+            return authenticate.login()
+
+        # if the server is restarted the in memory key will be lost
+        # but the user session may still be active. Logout the user
+        # to get the key again when login
+        if config.SERVER_MODE and current_user.is_authenticated and \
+                app.PGADMIN_EXTERNAL_AUTH_SOURCE != \
+                KERBEROS and \
+                current_app.keyManager.get() is None and \
+                request.endpoint not in ('security.login', 'security.logout'):
+            logout_user()
 
     @app.after_request
     def after_request(response):
@@ -609,10 +721,14 @@ def create_app(app_name=None):
             if config.COOKIE_DEFAULT_DOMAIN and \
                     config.COOKIE_DEFAULT_DOMAIN != 'localhost':
                 domain['domain'] = config.COOKIE_DEFAULT_DOMAIN
-            response.set_cookie('PGADMIN_KEY', value=request.args['key'],
+            response.set_cookie('PGADMIN_INT_KEY', value=request.args['key'],
                                 path=config.COOKIE_DEFAULT_PATH,
+                                secure=config.SESSION_COOKIE_SECURE,
+                                httponly=config.SESSION_COOKIE_HTTPONLY,
+                                samesite=config.SESSION_COOKIE_SAMESITE,
                                 **domain)
 
+        SecurityHeaders.set_response_headers(response)
         return response
 
     ##########################################################################
@@ -650,19 +766,24 @@ def create_app(app_name=None):
             values.pop(config.APP_VERSION_PARAM)
 
     ##########################################################################
-    # Minify output
+    # Minify output. Not required in desktop mode
     ##########################################################################
-    # HTMLMIN doesn't work with Python 2.6.
-    if not config.DEBUG and sys.version_info >= (2, 7):
-        from flask_htmlmin import HTMLMIN
-        HTMLMIN(app)
+    if not config.DEBUG and config.SERVER_MODE:
+        from flask_compress import Compress
+        Compress(app)
+
+    from pgadmin.misc.themes import themes
+    themes(app)
 
     @app.context_processor
     def inject_blueprint():
-        """Inject a reference to the current blueprint, if any."""
+        """
+        Inject a reference to the current blueprint, if any.
+        """
+
         return {
             'current_app': current_app,
-            'current_blueprint': current_blueprint
+            'current_blueprint': current_blueprint,
         }
 
     @app.errorhandler(Exception)
@@ -678,8 +799,16 @@ def create_app(app_name=None):
         current_app.logger.error(e, exc_info=True)
         return e
 
+    # Intialize the key manager
+    app.keyManager = KeyManager()
+
+    ##########################################################################
+    # Protection against CSRF attacks
+    ##########################################################################
+    with app.app_context():
+        pgCSRFProtect.init_app(app)
+
     ##########################################################################
     # All done!
     ##########################################################################
-
     return app
