@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -15,10 +15,11 @@ from smtplib import SMTPConnectError, SMTPResponseException, \
     SMTPServerDisconnected, SMTPDataError, SMTPHeloError, SMTPException, \
     SMTPAuthenticationError, SMTPSenderRefused, SMTPRecipientsRefused
 from socket import error as SOCKETErrorException
+from urllib.request import urlopen
 
 import six
 from flask import current_app, render_template, url_for, make_response, \
-    flash, Response, request, after_this_request, redirect
+    flash, Response, request, after_this_request, redirect, session
 from flask_babelex import gettext
 from flask_gravatar import Gravatar
 from flask_login import current_user, login_required
@@ -28,8 +29,8 @@ from flask_security.recoverable import reset_password_token_status, \
     generate_reset_password_token, update_password
 from flask_security.signals import reset_password_instructions_sent
 from flask_security.utils import config_value, do_flash, get_url, \
-    get_message, slash_url_suffix, login_user, send_mail
-from flask_security.views import _security, _commit, _render_json, _ctx
+    get_message, slash_url_suffix, login_user, send_mail, logout_user
+from flask_security.views import _security, _commit, _ctx
 from werkzeug.datastructures import MultiDict
 
 import config
@@ -37,16 +38,39 @@ from pgadmin import current_blueprint
 from pgadmin.settings import get_setting
 from pgadmin.utils import PgAdminModule
 from pgadmin.utils.ajax import make_json_response
+from pgadmin.utils.csrf import pgCSRFProtect
 from pgadmin.utils.preferences import Preferences
+from pgadmin.utils.menu import MenuItem
 from pgadmin.browser.register_browser_preferences import \
     register_browser_preferences
+from pgadmin.utils.master_password import validate_master_password, \
+    set_masterpass_check_text, cleanup_master_password, get_crypt_key, \
+    set_crypt_key, process_masterpass_disabled
+from pgadmin.model import User
+from pgadmin.utils.constants import MIMETYPE_APP_JS, PGADMIN_NODE,\
+    INTERNAL, KERBEROS
 
 try:
-    import urllib.request as urlreq
+    from flask_security.views import default_render_json
 except ImportError as e:
-    import urllib2 as urlreq
+    # Support Flask-Security-Too == 3.2
+    import sys
+    if sys.version_info < (3, 8):
+        from flask_security.views import _render_json as default_render_json
 
 MODULE_NAME = 'browser'
+BROWSER_STATIC = 'browser.static'
+JQUERY_ACIPLUGIN = 'jquery.aciplugin'
+BROWSER_INDEX = 'browser.index'
+PGADMIN_BROWSER = 'pgAdmin.Browser'
+PASS_ERROR_MSG = gettext('Your password has not been changed.')
+SMTP_SOCKET_ERROR = gettext(
+    'SMTP Socket error: {error}\n {pass_error}').format(
+        error={}, pass_error=PASS_ERROR_MSG)
+SMTP_ERROR = gettext('SMTP error: {error}\n {pass_error}').format(
+    error={}, pass_error=PASS_ERROR_MSG)
+PASS_ERROR = gettext('Error: {error}\n {pass_error}').format(
+    error={}, pass_error=PASS_ERROR_MSG)
 
 
 class BrowserModule(PgAdminModule):
@@ -67,7 +91,7 @@ class BrowserModule(PgAdminModule):
             ('static', 'vendor/codemirror/addon/dialog/dialog.css'),
             ('static', context_menu_file),
             ('static', wcdocker_file),
-            ('browser.static', 'vendor/aciTree/css/aciTree.css')
+            (BROWSER_STATIC, 'vendor/aciTree/css/aciTree.css')
         ]:
             stylesheets.append(url_for(endpoint, filename=filename))
         return stylesheets
@@ -109,9 +133,9 @@ class BrowserModule(PgAdminModule):
             'preloaded': True
         })
         scripts.append({
-            'name': 'jquery.aciplugin',
+            'name': JQUERY_ACIPLUGIN,
             'path': url_for(
-                'browser.static',
+                BROWSER_STATIC,
                 filename='vendor/aciTree/jquery.aciPlugin.min'
             ),
             'deps': ['jquery'],
@@ -121,21 +145,21 @@ class BrowserModule(PgAdminModule):
         scripts.append({
             'name': 'jquery.acitree',
             'path': url_for(
-                'browser.static',
+                BROWSER_STATIC,
                 filename='vendor/aciTree/jquery.aciTree' if
                 current_app.debug else 'vendor/aciTree/jquery.aciTree.min'
             ),
-            'deps': ['jquery', 'jquery.aciplugin'],
+            'deps': ['jquery', JQUERY_ACIPLUGIN],
             'exports': 'aciPluginClass.plugins.aciTree',
             'preloaded': True
         })
         scripts.append({
             'name': 'jquery.acisortable',
             'path': url_for(
-                'browser.static',
+                BROWSER_STATIC,
                 filename='vendor/aciTree/jquery.aciSortable.min'
             ),
-            'deps': ['jquery', 'jquery.aciplugin'],
+            'deps': ['jquery', JQUERY_ACIPLUGIN],
             'exports': 'aciPluginClass.plugins.aciSortable',
             'when': None,
             'preloaded': True
@@ -143,10 +167,10 @@ class BrowserModule(PgAdminModule):
         scripts.append({
             'name': 'jquery.acifragment',
             'path': url_for(
-                'browser.static',
+                BROWSER_STATIC,
                 filename='vendor/aciTree/jquery.aciFragment.min'
             ),
-            'deps': ['jquery', 'jquery.aciplugin'],
+            'deps': ['jquery', JQUERY_ACIPLUGIN],
             'exports': 'aciPluginClass.plugins.aciFragment',
             'when': None,
             'preloaded': True
@@ -165,18 +189,18 @@ class BrowserModule(PgAdminModule):
 
         scripts.append({
             'name': 'pgadmin.browser.datamodel',
-            'path': url_for('browser.static', filename='js/datamodel'),
+            'path': url_for(BROWSER_STATIC, filename='js/datamodel'),
             'preloaded': True
         })
 
         for name, script in [
-            ['pgadmin.browser', 'js/browser'],
+            [PGADMIN_BROWSER, 'js/browser'],
             ['pgadmin.browser.endpoints', 'js/endpoints'],
             ['pgadmin.browser.error', 'js/error']
         ]:
             scripts.append({
                 'name': name,
-                'path': url_for('browser.index') + script,
+                'path': url_for(BROWSER_INDEX) + script,
                 'preloaded': True
             })
 
@@ -187,7 +211,7 @@ class BrowserModule(PgAdminModule):
         ]:
             scripts.append({
                 'name': name,
-                'path': url_for('browser.index') + script,
+                'path': url_for(BROWSER_INDEX) + script,
                 'preloaded': True,
                 'deps': ['pgadmin.browser.datamodel']
             })
@@ -198,12 +222,12 @@ class BrowserModule(PgAdminModule):
             ['pgadmin.browser.frame', 'js/frame']
         ]:
             scripts.append({
-                'name': name, 'path': url_for('browser.static', filename=end),
+                'name': name, 'path': url_for(BROWSER_STATIC, filename=end),
                 'preloaded': True})
 
         scripts.append({
             'name': 'pgadmin.browser.node.ui',
-            'path': url_for('browser.static', filename='js/node.ui'),
+            'path': url_for(BROWSER_STATIC, filename='js/node.ui'),
             'when': 'server_group'
         })
 
@@ -211,6 +235,40 @@ class BrowserModule(PgAdminModule):
             scripts.extend(module.get_own_javascripts())
         return scripts
 
+    def get_own_menuitems(self):
+        return {
+            'file_items': [
+                MenuItem(
+                    name='mnu_locklayout',
+                    module=PGADMIN_BROWSER,
+                    label=gettext('Lock Layout'),
+                    priority=999,
+                    menu_items=[MenuItem(
+                        name='mnu_lock_none',
+                        module=PGADMIN_BROWSER,
+                        callback='mnu_lock_none',
+                        priority=0,
+                        label=gettext('None'),
+                        checked=True
+                    ), MenuItem(
+                        name='mnu_lock_docking',
+                        module=PGADMIN_BROWSER,
+                        callback='mnu_lock_docking',
+                        priority=1,
+                        label=gettext('Prevent Docking'),
+                        checked=False
+                    ), MenuItem(
+                        name='mnu_lock_full',
+                        module=PGADMIN_BROWSER,
+                        callback='mnu_lock_full',
+                        priority=2,
+                        label=gettext('Full Lock'),
+                        checked=False
+                    )]
+                )
+            ]
+        }
+
     def register_preferences(self):
         register_browser_preferences(self)
 
@@ -219,7 +277,12 @@ class BrowserModule(PgAdminModule):
         Returns:
             list: a list of url endpoints exposed to the client.
         """
-        return ['browser.index', 'browser.nodes']
+        return [BROWSER_INDEX, 'browser.nodes',
+                'browser.check_master_password',
+                'browser.set_master_password',
+                'browser.reset_master_password',
+                'browser.lock_layout'
+                ]
 
 
 blueprint = BrowserModule(MODULE_NAME, __name__)
@@ -310,15 +373,15 @@ class BrowserPluginModule(PgAdminModule):
 
         if self.module_use_template_javascript:
             scripts.extend([{
-                'name': 'pgadmin.node.%s' % self.node_type,
-                'path': url_for('browser.index') +
-                        '%s/module' % self.node_type,
+                'name': PGADMIN_NODE % self.node_type,
+                'path': url_for(BROWSER_INDEX
+                                ) + '%s/module' % self.node_type,
                 'when': self.script_load,
                 'is_template': True
             }])
         else:
             scripts.extend([{
-                'name': 'pgadmin.node.%s' % self.node_type,
+                'name': PGADMIN_NODE % self.node_type,
                 'path': url_for(
                     '%s.static' % self.name,
                     filename=('js/%s' % self.node_type)
@@ -364,7 +427,7 @@ class BrowserPluginModule(PgAdminModule):
             "_type": node_type,
             "_id": node_id,
             "_pid": parent_id,
-            "module": 'pgadmin.node.%s' % node_type
+            "module": PGADMIN_NODE % node_type
         }
         for key in kwargs:
             obj.setdefault(key, kwargs[key])
@@ -477,7 +540,99 @@ class BrowserPluginModule(PgAdminModule):
         )
 
 
+def _get_logout_url():
+    if config.SERVER_MODE and\
+            session['_auth_source_manager_obj']['current_source'] == \
+            KERBEROS:
+        return '{0}?next={1}'.format(url_for(
+            'authenticate.kerberos_logout'), url_for(BROWSER_INDEX))
+
+    return '{0}?next={1}'.format(
+        url_for('security.logout'), url_for(BROWSER_INDEX))
+
+
+def _get_supported_browser():
+    """
+    This function return supported browser.
+    :return: browser name, browser known, browser version
+    """
+    browser = request.user_agent.browser
+    version = request.user_agent.version and int(
+        request.user_agent.version.split('.')[0])
+
+    browser_name = None
+    browser_known = True
+    if browser == 'chrome' and version < 72:
+        browser_name = 'Chrome'
+    elif browser == 'firefox' and version < 65:
+        browser_name = 'Firefox'
+    # comparing EdgeHTML engine version
+    elif browser == 'edge' and version < 18:
+        browser_name = 'Edge'
+        # browser version returned by edge browser is actual EdgeHTML
+        # engine version. Below code gets actual browser version using
+        # EdgeHTML version
+        engine_to_actual_browser_version = {
+            16: 41,
+            17: 42,
+            18: 44
+        }
+        version = engine_to_actual_browser_version.get(version, '< 44')
+    elif browser == 'safari' and version < 12:
+        browser_name = 'Safari'
+    elif browser == 'msie':
+        browser_name = 'Internet Explorer'
+    elif browser != 'chrome' and browser != 'firefox' and \
+            browser != 'edge' and browser != 'safari':
+        browser_name = browser
+        browser_known = False
+
+    return browser_name, browser_known, version
+
+
+def check_browser_upgrade():
+    """
+    This function is used to check the browser version.
+    :return:
+    """
+    data = None
+    url = '%s?version=%s' % (config.UPGRADE_CHECK_URL, config.APP_VERSION)
+    current_app.logger.debug('Checking version data at: %s' % url)
+
+    try:
+        # Do not wait for more than 5 seconds.
+        # It stuck on rendering the browser.html, while working in the
+        # broken network.
+        if os.path.exists(config.CA_FILE):
+            response = urlopen(url, data, 5, cafile=config.CA_FILE)
+        else:
+            response = urlopen(url, data, 5)
+        current_app.logger.debug(
+            'Version check HTTP response code: %d' % response.getcode()
+        )
+
+        if response.getcode() == 200:
+            data = json.loads(response.read().decode('utf-8'))
+            current_app.logger.debug('Response data: %s' % data)
+    except Exception:
+        current_app.logger.exception('Exception when checking for update')
+
+    if data is not None and \
+        data[config.UPGRADE_CHECK_KEY]['version_int'] > \
+            config.APP_VERSION_INT:
+        msg = render_template(
+            MODULE_NAME + "/upgrade.html",
+            current_version=config.APP_VERSION,
+            upgrade_version=data[config.UPGRADE_CHECK_KEY]['version'],
+            product_name=config.APP_NAME,
+            download_url=data[config.UPGRADE_CHECK_KEY]['download_url']
+        )
+
+        flash(msg, 'warning')
+
+
 @blueprint.route("/")
+@pgCSRFProtect.exempt
 @login_required
 def index():
     """Render and process the main browser window."""
@@ -493,53 +648,55 @@ def index():
             base_url=None
         )
 
+    # Check the browser is a support version
+    # NOTE: If the checks here are updated, make sure the supported versions
+    # at https://www.pgadmin.org/faq/#11 are updated to match!
+    if config.CHECK_SUPPORTED_BROWSER:
+        browser_name, browser_known, version = _get_supported_browser()
+
+        if browser_name is not None:
+            msg = render_template(
+                MODULE_NAME + "/browser.html",
+                version=version,
+                browser=browser_name,
+                known=browser_known
+            )
+
+            flash(msg, 'warning')
+
     # Get the current version info from the website, and flash a message if
     # the user is out of date, and the check is enabled.
     if config.UPGRADE_CHECK_ENABLED:
-        data = None
-        url = '%s?version=%s' % (config.UPGRADE_CHECK_URL, config.APP_VERSION)
-        current_app.logger.debug('Checking version data at: %s' % url)
+        check_browser_upgrade()
 
-        try:
-            # Do not wait for more than 5 seconds.
-            # It stuck on rendering the browser.html, while working in the
-            # broken network.
-            if os.path.exists(config.CA_FILE):
-                response = urlreq.urlopen(url, data, 5, cafile=config.CA_FILE)
-            else:
-                response = urlreq.urlopen(url, data, 5)
-            current_app.logger.debug(
-                'Version check HTTP response code: %d' % response.getcode()
-            )
+    auth_only_internal = False
+    auth_source = []
 
-            if response.getcode() == 200:
-                data = json.loads(response.read().decode('utf-8'))
-                current_app.logger.debug('Response data: %s' % data)
-        except Exception:
-            current_app.logger.exception('Exception when checking for update')
+    session['allow_save_password'] = True
 
-        if data is not None:
-            if data['pgadmin4']['version_int'] > config.APP_VERSION_INT:
-                msg = render_template(
-                    MODULE_NAME + "/upgrade.html",
-                    current_version=config.APP_VERSION,
-                    upgrade_version=data['pgadmin4']['version'],
-                    product_name=config.APP_NAME,
-                    download_url=data['pgadmin4']['download_url']
-                )
+    if config.SERVER_MODE:
+        if len(config.AUTHENTICATION_SOURCES) == 1\
+                and INTERNAL in config.AUTHENTICATION_SOURCES:
+            auth_only_internal = True
+        auth_source = session['_auth_source_manager_obj'][
+            'source_friendly_name']
 
-                flash(msg, 'warning')
+        if session['_auth_source_manager_obj']['current_source'] == KERBEROS:
+            session['allow_save_password'] = False
 
     response = Response(render_template(
         MODULE_NAME + "/index.html",
-        username=current_user.email,
+        username=current_user.username,
+        auth_source=auth_source,
         is_admin=current_user.has_role("Administrator"),
-        _=gettext
+        logout_url=_get_logout_url(),
+        _=gettext,
+        auth_only_internal=auth_only_internal
     ))
 
     # Set the language cookie after login, so next time the user will have that
     # same option at the login time.
-    misc_preference = Preferences.module('miscellaneous')
+    misc_preference = Preferences.module('misc')
     user_languages = misc_preference.preference(
         'user_language'
     )
@@ -554,12 +711,16 @@ def index():
 
     response.set_cookie("PGADMIN_LANGUAGE", value=language,
                         path=config.COOKIE_DEFAULT_PATH,
+                        secure=config.SESSION_COOKIE_SECURE,
+                        httponly=config.SESSION_COOKIE_HTTPONLY,
+                        samesite=config.SESSION_COOKIE_SAMESITE,
                         **domain)
 
     return response
 
 
 @blueprint.route("/js/utils.js")
+@pgCSRFProtect.exempt
 @login_required
 def utils():
     layout = get_setting('Browser/Layout', default='')
@@ -600,7 +761,7 @@ def utils():
         from pgadmin.utils.driver import get_driver
         driver = get_driver(PG_DEFAULT_DRIVER)
         pg_libpq_version = driver.libpq_version()
-    except Exception as e:
+    except Exception:
         pg_libpq_version = 0
 
     for submodule in current_blueprint.submodules:
@@ -619,64 +780,42 @@ def utils():
             editor_insert_pair_brackets=insert_pair_brackets,
             editor_indent_with_tabs=editor_indent_with_tabs,
             app_name=config.APP_NAME,
+            app_version_int=config.APP_VERSION_INT,
             pg_libpq_version=pg_libpq_version,
-            support_ssh_tunnel=config.SUPPORT_SSH_TUNNEL
+            support_ssh_tunnel=config.SUPPORT_SSH_TUNNEL,
+            logout_url=_get_logout_url()
         ),
-        200, {'Content-Type': 'application/x-javascript'})
+        200, {'Content-Type': MIMETYPE_APP_JS})
 
 
 @blueprint.route("/js/endpoints.js")
+@pgCSRFProtect.exempt
 def exposed_urls():
     return make_response(
         render_template('browser/js/endpoints.js'),
-        200, {'Content-Type': 'application/x-javascript'}
+        200, {'Content-Type': MIMETYPE_APP_JS}
     )
 
 
 @blueprint.route("/js/error.js")
+@pgCSRFProtect.exempt
 @login_required
 def error_js():
     return make_response(
         render_template('browser/js/error.js', _=gettext),
-        200, {'Content-Type': 'application/x-javascript'})
-
-
-@blueprint.route("/js/node.js")
-@login_required
-def node_js():
-    prefs = Preferences.module('paths')
-
-    pg_help_path_pref = prefs.preference('pg_help_path')
-    pg_help_path = pg_help_path_pref.get()
-
-    edbas_help_path_pref = prefs.preference('edbas_help_path')
-    edbas_help_path = edbas_help_path_pref.get()
-
-    return make_response(
-        render_template('browser/js/node.js',
-                        pg_help_path=pg_help_path,
-                        edbas_help_path=edbas_help_path,
-                        _=gettext
-                        ),
-        200, {'Content-Type': 'application/x-javascript'})
+        200, {'Content-Type': MIMETYPE_APP_JS})
 
 
 @blueprint.route("/js/messages.js")
+@pgCSRFProtect.exempt
 def messages_js():
     return make_response(
         render_template('browser/js/messages.js', _=gettext),
-        200, {'Content-Type': 'application/x-javascript'})
-
-
-@blueprint.route("/js/collection.js")
-@login_required
-def collection_js():
-    return make_response(
-        render_template('browser/js/collection.js', _=gettext),
-        200, {'Content-Type': 'application/x-javascript'})
+        200, {'Content-Type': MIMETYPE_APP_JS})
 
 
 @blueprint.route("/browser.css")
+@pgCSRFProtect.exempt
 @login_required
 def browser_css():
     """Render and return CSS snippets from the nodes and modules."""
@@ -702,6 +841,155 @@ def get_nodes():
     return make_json_response(data=nodes)
 
 
+def form_master_password_response(existing=True, present=False, errmsg=None):
+    content_new = (
+        gettext("Set Master Password"),
+        "<br/>".join([
+            gettext("Please set a master password for pgAdmin."),
+            gettext("This will be used to secure and later unlock saved "
+                    "passwords and other credentials.")])
+    )
+    content_existing = (
+        gettext("Unlock Saved Passwords"),
+        "<br/>".join([
+            gettext("Please enter your master password."),
+            gettext("This is required to unlock saved passwords and "
+                    "reconnect to the database server(s).")])
+    )
+
+    return make_json_response(data={
+        'present': present,
+        'title': content_existing[0] if existing else content_new[0],
+        'content': render_template(
+            'browser/master_password.html',
+            content_text=content_existing[1] if existing else content_new[1],
+            errmsg=errmsg
+        ),
+        'reset': existing
+    })
+
+
+@blueprint.route("/master_password", endpoint="check_master_password",
+                 methods=["GET"])
+def check_master_password():
+    """
+    Checks if the master password is available in the memory
+    This password will be used to encrypt/decrypt saved server passwords
+    """
+    return make_json_response(data=get_crypt_key()[0])
+
+
+@blueprint.route("/master_password", endpoint="reset_master_password",
+                 methods=["DELETE"])
+def reset_master_password():
+    """
+    Removes the master password and remove all saved passwords
+    This password will be used to encrypt/decrypt saved server passwords
+    """
+    cleanup_master_password()
+    return make_json_response(data=get_crypt_key()[0])
+
+
+@blueprint.route("/master_password", endpoint="set_master_password",
+                 methods=["POST"])
+def set_master_password():
+    """
+    Set the master password and store in the memory
+    This password will be used to encrypt/decrypt saved server passwords
+    """
+
+    data = None
+
+    if hasattr(request.data, 'decode'):
+        data = request.data.decode('utf-8')
+
+    if data != '':
+        data = json.loads(data)
+
+    # Master password is not applicable for server mode
+    if not config.SERVER_MODE and config.MASTER_PASSWORD_REQUIRED:
+
+        # if master pass is set previously
+        if current_user.masterpass_check is not None:
+            if data.get('button_click') and \
+                    not validate_master_password(data.get('password')):
+                return form_master_password_response(
+                    existing=True,
+                    present=False,
+                    errmsg=gettext("Incorrect master password")
+                )
+
+        if data != '' and data.get('password', '') != '':
+
+            # store the master pass in the memory
+            set_crypt_key(data.get('password'))
+
+            if current_user.masterpass_check is None:
+                # master check is not set, which means the server password
+                # data is old and is encrypted with old key
+                # Re-encrypt with new key
+
+                from pgadmin.browser.server_groups.servers.utils \
+                    import reencrpyt_server_passwords
+                reencrpyt_server_passwords(
+                    current_user.id, current_user.password,
+                    data.get('password'))
+
+            # set the encrypted sample text with the new
+            # master pass
+            set_masterpass_check_text(data.get('password'))
+
+        elif not get_crypt_key()[0] and \
+                current_user.masterpass_check is not None:
+            return form_master_password_response(
+                existing=True,
+                present=False,
+            )
+        elif not get_crypt_key()[0]:
+            error_message = None
+            if data.get('button_click') and data.get('password') == '':
+                # If user attempted to enter a blank password, then throw error
+                error_message = gettext("Master password cannot be empty")
+            return form_master_password_response(
+                existing=False,
+                present=False,
+                errmsg=error_message
+            )
+
+    # if master password is disabled now, but was used once then
+    # remove all the saved passwords
+    process_masterpass_disabled()
+
+    if config.SERVER_MODE and current_user.masterpass_check is None:
+
+        crypt_key = get_crypt_key()[1]
+        from pgadmin.browser.server_groups.servers.utils \
+            import reencrpyt_server_passwords
+        reencrpyt_server_passwords(
+            current_user.id, current_user.password, crypt_key)
+
+        set_masterpass_check_text(crypt_key)
+
+    return form_master_password_response(
+        present=True,
+    )
+
+
+@blueprint.route("/lock_layout", endpoint="lock_layout", methods=["PUT"])
+def lock_layout():
+    data = None
+
+    if hasattr(request.data, 'decode'):
+        data = request.data.decode('utf-8')
+
+    if data != '':
+        data = json.loads(data)
+
+    blueprint.lock_layout.set(data['value'])
+
+    return make_json_response()
+
+
 # Only register route if SECURITY_CHANGEABLE is set to True
 # We can't access app context here so cannot
 # use app.config['SECURITY_CHANGEABLE']
@@ -710,6 +998,7 @@ def get_nodes():
 if hasattr(config, 'SECURITY_CHANGEABLE') and config.SECURITY_CHANGEABLE:
     @blueprint.route("/change_password", endpoint="change_password",
                      methods=['GET', 'POST'])
+    @pgCSRFProtect.exempt
     @login_required
     def change_password():
         """View function which handles a change password request."""
@@ -728,9 +1017,7 @@ if hasattr(config, 'SECURITY_CHANGEABLE') and config.SECURITY_CHANGEABLE:
             except SOCKETErrorException as e:
                 # Handle socket errors which are not covered by SMTPExceptions.
                 logging.exception(str(e), exc_info=True)
-                flash(gettext(u'SMTP Socket error: {}\n'
-                              u'Your password has not been changed.'
-                              ).format(e),
+                flash(gettext(SMTP_SOCKET_ERROR).format(e),
                       'danger')
                 has_error = True
             except (SMTPConnectError, SMTPResponseException,
@@ -739,19 +1026,14 @@ if hasattr(config, 'SECURITY_CHANGEABLE') and config.SECURITY_CHANGEABLE:
                     SMTPRecipientsRefused) as e:
                 # Handle smtp specific exceptions.
                 logging.exception(str(e), exc_info=True)
-                flash(gettext(u'SMTP error: {}\n'
-                              u'Your password has not been changed.'
-                              ).format(e),
+                flash(gettext(SMTP_ERROR).format(e),
                       'danger')
                 has_error = True
             except Exception as e:
                 # Handle other exceptions.
                 logging.exception(str(e), exc_info=True)
                 flash(
-                    gettext(
-                        u'Error: {}\n'
-                        u'Your password has not been changed.'
-                    ).format(e),
+                    gettext(PASS_ERROR).format(e),
                     'danger'
                 )
                 has_error = True
@@ -759,12 +1041,21 @@ if hasattr(config, 'SECURITY_CHANGEABLE') and config.SECURITY_CHANGEABLE:
             if request.json is None and not has_error:
                 after_this_request(_commit)
                 do_flash(*get_message('PASSWORD_CHANGE'))
+
+                old_key = get_crypt_key()[1]
+                set_crypt_key(form.new_password.data, False)
+
+                from pgadmin.browser.server_groups.servers.utils \
+                    import reencrpyt_server_passwords
+                reencrpyt_server_passwords(
+                    current_user.id, old_key, form.new_password.data)
+
                 return redirect(get_url(_security.post_change_view) or
                                 get_url(_security.post_login_view))
 
         if request.json and not has_error:
             form.user = current_user
-            return _render_json(form)
+            return default_render_json(form)
 
         return _security.render_template(
             config_value('CHANGE_PASSWORD_TEMPLATE'),
@@ -793,6 +1084,7 @@ if hasattr(config, 'SECURITY_RECOVERABLE') and config.SECURITY_RECOVERABLE:
 
     @blueprint.route("/reset_password", endpoint="forgot_password",
                      methods=['GET', 'POST'])
+    @pgCSRFProtect.exempt
     @anonymous_user_required
     def forgot_password():
         """View function that handles a forgotten password request."""
@@ -805,43 +1097,54 @@ if hasattr(config, 'SECURITY_RECOVERABLE') and config.SECURITY_RECOVERABLE:
             form = form_class()
 
         if form.validate_on_submit():
-            try:
-                send_reset_password_instructions(form.user)
-            except SOCKETErrorException as e:
-                # Handle socket errors which are not covered by SMTPExceptions.
-                logging.exception(str(e), exc_info=True)
-                flash(gettext(u'SMTP Socket error: {}\n'
-                              u'Your password has not been changed.'
-                              ).format(e),
-                      'danger')
-                has_error = True
-            except (SMTPConnectError, SMTPResponseException,
-                    SMTPServerDisconnected, SMTPDataError, SMTPHeloError,
-                    SMTPException, SMTPAuthenticationError, SMTPSenderRefused,
-                    SMTPRecipientsRefused) as e:
+            # Check the Authentication source of the User
+            user = User.query.filter_by(
+                email=form.data['email'],
+                auth_source=INTERNAL
+            ).first()
 
-                # Handle smtp specific exceptions.
-                logging.exception(str(e), exc_info=True)
-                flash(gettext(u'SMTP error: {}\n'
-                              u'Your password has not been changed.'
-                              ).format(e),
-                      'danger')
-                has_error = True
-            except Exception as e:
-                # Handle other exceptions.
-                logging.exception(str(e), exc_info=True)
-                flash(gettext(u'Error: {}\n'
-                              u'Your password has not been changed.'
-                              ).format(e),
+            if user is None:
+                # If the user is not an internal user, raise the exception
+                flash(gettext('Your account is authenticated using an '
+                              'external {} source. '
+                              'Please contact the administrators of this '
+                              'service if you need to reset your password.'
+                              ).format(form.user.auth_source),
                       'danger')
                 has_error = True
+            if not has_error:
+                try:
+                    send_reset_password_instructions(form.user)
+                except SOCKETErrorException as e:
+                    # Handle socket errors which are not
+                    # covered by SMTPExceptions.
+                    logging.exception(str(e), exc_info=True)
+                    flash(gettext(SMTP_SOCKET_ERROR).format(e),
+                          'danger')
+                    has_error = True
+                except (SMTPConnectError, SMTPResponseException,
+                        SMTPServerDisconnected, SMTPDataError, SMTPHeloError,
+                        SMTPException, SMTPAuthenticationError,
+                        SMTPSenderRefused, SMTPRecipientsRefused) as e:
+
+                    # Handle smtp specific exceptions.
+                    logging.exception(str(e), exc_info=True)
+                    flash(gettext(SMTP_ERROR).format(e),
+                          'danger')
+                    has_error = True
+                except Exception as e:
+                    # Handle other exceptions.
+                    logging.exception(str(e), exc_info=True)
+                    flash(gettext(PASS_ERROR).format(e),
+                          'danger')
+                    has_error = True
 
             if request.json is None and not has_error:
                 do_flash(*get_message('PASSWORD_RESET_REQUEST',
                                       email=form.user.email))
 
         if request.json and not has_error:
-            return _render_json(form, include_user=False)
+            return default_render_json(form, include_user=False)
 
         return _security.render_template(
             config_value('FORGOT_PASSWORD_TEMPLATE'),
@@ -859,10 +1162,10 @@ if hasattr(config, 'SECURITY_RECOVERABLE') and config.SECURITY_RECOVERABLE:
         methods=['GET', 'POST'],
         endpoint='reset_password'
     )
+    @pgCSRFProtect.exempt
     @anonymous_user_required
     def reset_password(token):
         """View function that handles a reset password request."""
-
         expired, invalid, user = reset_password_token_status(token)
 
         if invalid:
@@ -881,9 +1184,7 @@ if hasattr(config, 'SECURITY_RECOVERABLE') and config.SECURITY_RECOVERABLE:
             except SOCKETErrorException as e:
                 # Handle socket errors which are not covered by SMTPExceptions.
                 logging.exception(str(e), exc_info=True)
-                flash(gettext(u'SMTP Socket error: {}\n'
-                              u'Your password has not been changed.'
-                              ).format(e),
+                flash(gettext(SMTP_SOCKET_ERROR).format(e),
                       'danger')
                 has_error = True
             except (SMTPConnectError, SMTPResponseException,
@@ -893,17 +1194,13 @@ if hasattr(config, 'SECURITY_RECOVERABLE') and config.SECURITY_RECOVERABLE:
 
                 # Handle smtp specific exceptions.
                 logging.exception(str(e), exc_info=True)
-                flash(gettext(u'SMTP error: {}\n'
-                              u'Your password has not been changed.'
-                              ).format(e),
+                flash(gettext(SMTP_ERROR).format(e),
                       'danger')
                 has_error = True
             except Exception as e:
                 # Handle other exceptions.
                 logging.exception(str(e), exc_info=True)
-                flash(gettext(u'Error: {}\n'
-                              u'Your password has not been changed.'
-                              ).format(e),
+                flash(gettext(PASS_ERROR).format(e),
                       'danger')
                 has_error = True
 
