@@ -1,9 +1,18 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define('pgadmin.node.cast', [
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
-  'underscore.string', 'sources/pgadmin', 'pgadmin.browser',
+  'sources/pgadmin', 'pgadmin.browser',
   'pgadmin.alertifyjs', 'pgadmin.backform', 'pgadmin.browser.collection',
-], function(gettext, url_for, $, _, S, pgAdmin, pgBrowser, alertify, Backform) {
-    // Extend the collection class for cast
+], function(gettext, url_for, $, _, pgAdmin, pgBrowser, alertify, Backform) {
+  // Extend the collection class for cast
   if (!pgBrowser.Nodes['coll-cast']) {
     pgAdmin.Browser.Nodes['coll-cast'] =
       pgAdmin.Browser.Collection.extend({
@@ -14,7 +23,7 @@ define('pgadmin.node.cast', [
       });
   }
 
-    // Extend the node class for cast
+  // Extend the node class for cast
   if (!pgBrowser.Nodes['cast']) {
     pgAdmin.Browser.Nodes['cast'] = pgAdmin.Browser.Node.extend({
       parent_type: 'database',
@@ -29,13 +38,13 @@ define('pgadmin.node.cast', [
       hasDepends: true,
       Init: function() {
 
-          // Avoid multiple registration of menus
+        // Avoid multiple registration of menus
         if (this.initialized)
           return;
 
         this.initialized = true;
 
-          // Add context menus for cast
+        // Add context menus for cast
         pgBrowser.add_menus([{
           name: 'create_cast_on_database', node: 'database', module: this,
           applies: ['object', 'context'], callback: 'show_obj_properties',
@@ -56,7 +65,7 @@ define('pgadmin.node.cast', [
 
       },
 
-        // Define the backform model for cast node
+      // Define the backform model for cast node
       model: pgAdmin.Browser.Node.Model.extend({
         idAttribute: 'oid',
         defaults: {
@@ -70,16 +79,16 @@ define('pgadmin.node.cast', [
           description: undefined,      // Comment on the cast
         },
 
-          // Define the schema for cast
+        // Define the schema for cast
         schema: [{
           id: 'name', label: gettext('Name'), cell: 'string',
-          editable: false, type: 'text', disabled: true, cellHeaderClasses: 'width_percent_50',
+          editable: false, type: 'text', readonly: true, cellHeaderClasses: 'width_percent_50',
         },{
           id: 'oid', label: gettext('OID'), cell: 'string',
-          editable: false, type: 'text', disabled: true, mode: ['properties'],
+          editable: false, type: 'text', mode: ['properties'],
         },{
           id: 'srctyp', label: gettext('Source type'), url: 'get_type',
-          type: 'text', group: gettext('Definition'), disabled: function(m) {
+          type: 'text', group: gettext('Definition'), readonly: function(m) {
             return !m.isNew();
           }, mode: ['create'],
 
@@ -90,7 +99,7 @@ define('pgadmin.node.cast', [
             return rows;
           },
 
-            /*
+          /*
              * Control is extended to create cast name from source type and destination type
              * once their values are changed
              */
@@ -98,10 +107,10 @@ define('pgadmin.node.cast', [
 
             onChange: function() {
               Backform.NodeAjaxOptionsControl.prototype.onChange.apply(
-                    this, arguments
-                    );
+                this, arguments
+              );
 
-                 /*
+              /*
                   * On source type change, check if both source type and
                   * target type are set, if yes then fetch values from both
                   * controls and generate cast name
@@ -112,21 +121,21 @@ define('pgadmin.node.cast', [
                     trgtype != undefined && trgtype != '')
                 this.model.set('name', srctype+'->'+trgtype);
               else
-                   this.model.unset('name');
+                this.model.unset('name');
             },
           }),
         },
 
-          /*
+        /*
            * Text control for viewing source type in properties and
            * edit mode only
            */
         {
           id: 'srctyp', label: gettext('Source type'), type: 'text',
-          group: gettext('Definition'), disabled: true, mode:['properties','edit'],
+          group: gettext('Definition'), readonly: true, mode:['properties','edit'],
         },{
           id: 'trgtyp', label: gettext('Target type'), url: 'get_type',
-          type: 'text', group: gettext('Definition'), disabled: function(m) {
+          type: 'text', group: gettext('Definition'), readonly: function(m) {
             return !m.isNew();
           }, mode: ['create'],
           transform: function(rows) {
@@ -136,7 +145,7 @@ define('pgadmin.node.cast', [
             return rows;
           },
 
-            /*
+          /*
              * Control is extended to create cast name from source type and destination type
              * once their values are changed
              */
@@ -144,10 +153,10 @@ define('pgadmin.node.cast', [
 
             onChange: function() {
               Backform.NodeAjaxOptionsControl.prototype.onChange.apply(
-                 this, arguments
-                 );
+                this, arguments
+              );
 
-                 /*
+              /*
                   * on target type change, check if both source type and
                   * target type are set, if yes then fetch values from both
                   * controls and generate cast name
@@ -158,27 +167,27 @@ define('pgadmin.node.cast', [
                   trgtype != undefined && trgtype != '')
                 this.model.set('name', srcType+'->'+trgtype);
               else
-                 this.model.unset('name');
+                this.model.unset('name');
             },
           }),
         },
-          /*
+        /*
            * Text control for viewing target type in properties and
            * edit mode only
            */
         {
           id: 'trgtyp', label: gettext('Target type'), type: 'text',
-          group: gettext('Definition'), disabled: true, mode:['properties','edit'],
+          group: gettext('Definition'), readonly: true, mode:['properties','edit'],
         },
 
-          /*
+        /*
            * Proname field is dependent on source type and target type.
            * On source and target type changed event,
            * associated functions will be fetch using ajax call
            */
         {
           id: 'proname', label: gettext('Function'), deps:['srctyp', 'trgtyp'],
-          type: 'text', disabled: function(m) { return !m.isNew(); },
+          type: 'text', readonly: function(m) { return !m.isNew(); },
           group: gettext('Definition'), mode: ['create'],
           control: 'node-ajax-options',
           options: function(control) {
@@ -188,13 +197,13 @@ define('pgadmin.node.cast', [
 
             if(srcTyp != undefined && srcTyp != '' &&
                  trgtyp != undefined && trgtyp != '')
-              {
+            {
               var node = control.field.get('schema_node'),
                 _url = node.generate_url.apply(
-                 node, [
-                   null, 'get_functions', control.field.get('node_data'), false,
-                   control.field.get('node_info'),
-                 ]);
+                  node, [
+                    null, 'get_functions', control.field.get('node_data'), false,
+                    control.field.get('node_info'),
+                  ]);
               $.ajax({
                 type: 'POST',
                 timeout: 30000,
@@ -203,15 +212,15 @@ define('pgadmin.node.cast', [
                 async: false,
                 data: {'srctyp' : srcTyp, 'trgtyp' : trgtyp},
               })
-               // On success return function list from server
-              .done(function(result) {
-                res = result.data;
-                return res;
-              })
-               // On failure show error appropriate error message to user
-              .fail(function(xhr, status, error) {
-                alertify.pgRespErrorNotify(xhr, error);
-              });
+              // On success return function list from server
+                .done(function(result) {
+                  res = result.data;
+                  return res;
+                })
+              // On failure show error appropriate error message to user
+                .fail(function(xhr, status, error) {
+                  alertify.pgRespErrorNotify(xhr, error);
+                });
             }
             return res;
           },
@@ -222,10 +231,10 @@ define('pgadmin.node.cast', [
          */
         {
           id: 'proname', label: gettext('Function'), type: 'text',
-          group: gettext('Definition'), disabled: true, mode:['properties','edit'],
+          group: gettext('Definition'), readonly: true, mode:['properties','edit'],
         },{
           id: 'castcontext', label: gettext('Context'),
-          options:{'onText':'IMPLICIT','offText':'EXPLICIT'},
+          options:{'onText':'IMPLICIT','offText':'EXPLICIT', width: '90'},
           editable: false, type: 'string', group: gettext('Definition'),
           mode:['create'],
           control: Backform.SwitchControl.extend({
@@ -239,7 +248,7 @@ define('pgadmin.node.cast', [
          * edit mode
          */
         {
-          id: 'castcontext', label: gettext('Context'), disabled: true,
+          id: 'castcontext', label: gettext('Context'), readonly: true,
           options:[{
             label: 'IMPLICIT', value: 'IMPLICIT',
           },{
@@ -250,12 +259,7 @@ define('pgadmin.node.cast', [
           mode:['properties', 'edit'],
         },{
           id: 'syscast', label: gettext('System cast?'),
-          cell: 'switch', type: 'switch', mode: ['properties'], disabled: true,
-          options: {
-            'onText': gettext('Yes'), 'offText': gettext('No'),
-            'onColor': 'success', 'offColor': 'primary',
-            'size': 'small',
-          },
+          cell: 'switch', type: 'switch', mode: ['properties'],
         },{
           id: 'description', label: gettext('Comment'),
           type: 'multiline', cellHeaderClasses: 'width_percent_50',
