@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -20,28 +20,11 @@ from pgadmin.browser.utils import PGChildNodeView
 from pgadmin.utils.ajax import make_json_response, \
     make_response as ajax_response, internal_server_error, gone
 from pgadmin.utils.driver import get_driver
+from pgadmin.browser.server_groups.servers.databases.extensions.utils \
+    import get_extension_details
 from config import PG_DEFAULT_DRIVER
-
-# As unicode type is not available in python3
-# If we check a variable is "isinstance(variable, str)
-# it breaks in python 3 as variable type is not string its unicode.
-# We assign basestring as str type if it is python3, unicode
-# if it is python2.
-
-try:
-    unicode = unicode
-except NameError:
-    # 'unicode' is undefined, must be Python 3
-    str = str
-    unicode = str
-    bytes = bytes
-    basestring = (str, bytes)
-else:
-    # 'unicode' exists, must be Python 2
-    str = str
-    unicode = unicode
-    bytes = str
-    basestring = basestring
+from pgadmin.tools.schema_diff.node_registry import SchemaDiffRegistry
+from pgadmin.tools.schema_diff.compare import SchemaDiffObjectCompare
 
 
 class ExtensionModule(CollectionNodeModule):
@@ -52,8 +35,8 @@ class ExtensionModule(CollectionNodeModule):
         class and define methods to get child nodes, to load its own
         javascript file.
     """
-    NODE_TYPE = "extension"
-    COLLECTION_LABEL = gettext("Extensions")
+    _NODE_TYPE = "extension"
+    _COLLECTION_LABEL = gettext("Extensions")
 
     def __init__(self, *args, **kwargs):
         """
@@ -81,7 +64,7 @@ class ExtensionModule(CollectionNodeModule):
         Load the module script for extension, when any of the database nodes
         are initialized.
         """
-        return databases.DatabaseModule.NODE_TYPE
+        return databases.DatabaseModule.node_type
 
     @property
     def module_use_template_javascript(self):
@@ -96,7 +79,7 @@ class ExtensionModule(CollectionNodeModule):
 blueprint = ExtensionModule(__name__)
 
 
-class ExtensionView(PGChildNodeView):
+class ExtensionView(PGChildNodeView, SchemaDiffObjectCompare):
     """
     This is a class for extension nodes which inherits the
     properties and methods from NodeView class and define
@@ -109,6 +92,7 @@ class ExtensionView(PGChildNodeView):
     * ids - id with type and name of extension module being used.
     * operations - function routes mappings defined.
     """
+    EXT_TEMPLATE_PATH = 'extensions/sql'
     node_type = blueprint.node_type
 
     parent_ids = [
@@ -152,7 +136,21 @@ class ExtensionView(PGChildNodeView):
                 PG_DEFAULT_DRIVER
             ).connection_manager(kwargs['sid'])
             self.conn = self.manager.connection(did=kwargs['did'])
-            self.template_path = 'extensions/sql'
+            self.template_path = self.EXT_TEMPLATE_PATH
+
+            self.datlastsysoid = \
+                self.manager.db_info[kwargs['did']]['datlastsysoid'] \
+                if self.manager.db_info is not None and \
+                kwargs['did'] in self.manager.db_info else 0
+
+            self.datistemplate = False
+            if (
+                self.manager.db_info is not None and
+                kwargs['did'] in self.manager.db_info and
+                'datistemplate' in self.manager.db_info[kwargs['did']]
+            ):
+                self.datistemplate = self.manager.db_info[
+                    kwargs['did']]['datistemplate']
 
             return f(*args, **kwargs)
 
@@ -163,7 +161,8 @@ class ExtensionView(PGChildNodeView):
         """
         Fetches all extensions properties and render into properties tab
         """
-        SQL = render_template("/".join([self.template_path, 'properties.sql']))
+        SQL = render_template("/".join([self.template_path,
+                                        self._PROPERTIES_SQL]))
         status, res = self.conn.execute_dict(SQL)
 
         if not status:
@@ -179,7 +178,8 @@ class ExtensionView(PGChildNodeView):
         Lists all extensions under the Extensions Collection node
         """
         res = []
-        SQL = render_template("/".join([self.template_path, 'properties.sql']))
+        SQL = render_template("/".join([self.template_path,
+                                        self._PROPERTIES_SQL]))
         status, rset = self.conn.execute_2darray(SQL)
         if not status:
             return internal_server_error(errormsg=rset)
@@ -187,7 +187,7 @@ class ExtensionView(PGChildNodeView):
         for row in rset['rows']:
             res.append(
                 self.blueprint.generate_browser_node(
-                    row['eid'],
+                    row['oid'],
                     did,
                     row['name'],
                     'icon-extension'
@@ -203,7 +203,8 @@ class ExtensionView(PGChildNodeView):
         """
         This function will fetch the properties of extension
         """
-        SQL = render_template("/".join([self.template_path, 'properties.sql']),
+        SQL = render_template("/".join([self.template_path,
+                                        self._PROPERTIES_SQL]),
                               eid=eid)
         status, rset = self.conn.execute_2darray(SQL)
         if not status:
@@ -212,7 +213,7 @@ class ExtensionView(PGChildNodeView):
         for row in rset['rows']:
             return make_json_response(
                 data=self.blueprint.generate_browser_node(
-                    row['eid'],
+                    row['oid'],
                     did,
                     row['name'],
                     'icon-extension'
@@ -227,21 +228,37 @@ class ExtensionView(PGChildNodeView):
         """
         Fetch the properties of a single extension and render in properties tab
         """
+        status, res = self._fetch_properties(did, eid)
+        if not status:
+            return res
+
+        return ajax_response(
+            response=res,
+            status=200
+        )
+
+    def _fetch_properties(self, did, eid):
+        """
+        This function fetch the properties of the extension.
+        :param did:
+        :param eid:
+        :return:
+        """
         SQL = render_template("/".join(
-            [self.template_path, 'properties.sql']), eid=eid)
+            [self.template_path, self._PROPERTIES_SQL]), eid=eid)
         status, res = self.conn.execute_dict(SQL)
         if not status:
-            return internal_server_error(errormsg=res)
+            return False, internal_server_error(errormsg=res)
 
         if len(res['rows']) == 0:
-            return gone(
+            return False, gone(
                 gettext("Could not find the extension information.")
             )
 
-        return ajax_response(
-            response=res['rows'][0],
-            status=200
-        )
+        res['rows'][0]['is_sys_obj'] = (
+            res['rows'][0]['oid'] <= self.datlastsysoid or self.datistemplate)
+
+        return True, res['rows'][0]
 
     @check_precondition
     def create(self, gid, sid, did):
@@ -262,13 +279,13 @@ class ExtensionView(PGChildNodeView):
                     status=410,
                     success=0,
                     errormsg=gettext(
-                        "Could not find the required parameter (%s)." % arg
-                    )
+                        "Could not find the required parameter ({})."
+                    ).format(arg)
                 )
 
         status, res = self.conn.execute_dict(
             render_template(
-                "/".join([self.template_path, 'create.sql']),
+                "/".join([self.template_path, self._CREATE_SQL]),
                 data=data
             )
         )
@@ -276,26 +293,21 @@ class ExtensionView(PGChildNodeView):
         if not status:
             return internal_server_error(errormsg=res)
 
-        status, rset = self.conn.execute_dict(
-            render_template(
-                "/".join([self.template_path, 'properties.sql']),
-                ename=data['name']
+        status, res = get_extension_details(
+            self.conn, data['name'],
+            "/".join([self.template_path, self._PROPERTIES_SQL]))
+        if not status:
+            return internal_server_error(errormsg=res)
+
+        return jsonify(
+            node=self.blueprint.generate_browser_node(
+                res['oid'],
+                did,
+                res['name'],
+                'icon-extension'
             )
         )
 
-        if not status:
-            return internal_server_error(errormsg=rset)
-
-        for row in rset['rows']:
-            return jsonify(
-                node=self.blueprint.generate_browser_node(
-                    row['eid'],
-                    did,
-                    row['name'],
-                    'icon-extension'
-                )
-            )
-
     @check_precondition
     def update(self, gid, sid, did, eid):
         """
@@ -308,7 +320,7 @@ class ExtensionView(PGChildNodeView):
         try:
             SQL, name = self.getSQL(gid, sid, data, did, eid)
             # Most probably this is due to error
-            if not isinstance(SQL, (str, unicode)):
+            if not isinstance(SQL, str):
                 return SQL
             SQL = SQL.strip('\n').strip(' ')
             status, res = self.conn.execute_dict(SQL)
@@ -327,7 +339,7 @@ class ExtensionView(PGChildNodeView):
             return internal_server_error(errormsg=str(e))
 
     @check_precondition
-    def delete(self, gid, sid, did, eid=None):
+    def delete(self, gid, sid, did, eid=None, only_sql=False):
         """
         This function will drop/drop cascade a extension object
         """
@@ -339,13 +351,13 @@ class ExtensionView(PGChildNodeView):
         else:
             data = {'ids': [eid]}
 
-        cascade = True if self.cmd == 'delete' else False
+        cascade = self._check_cascade_operation()
 
         try:
             for eid in data['ids']:
                 # check if extension with eid exists
                 SQL = render_template("/".join(
-                    [self.template_path, 'delete.sql']), eid=eid)
+                    [self.template_path, self._DELETE_SQL]), eid=eid)
                 status, name = self.conn.execute_scalar(SQL)
                 if not status:
                     return internal_server_error(errormsg=name)
@@ -364,8 +376,13 @@ class ExtensionView(PGChildNodeView):
 
                 # drop extension
                 SQL = render_template("/".join(
-                    [self.template_path, 'delete.sql']
+                    [self.template_path, self._DELETE_SQL]
                 ), name=name, cascade=cascade)
+
+                # Used for schema diff tool
+                if only_sql:
+                    return SQL
+
                 status, res = self.conn.execute_scalar(SQL)
                 if not status:
                     return internal_server_error(errormsg=res)
@@ -386,7 +403,7 @@ class ExtensionView(PGChildNodeView):
         try:
             SQL, name = self.getSQL(gid, sid, data, did, eid)
             # Most probably this is due to error
-            if not isinstance(SQL, (str, unicode)):
+            if not isinstance(SQL, str):
                 return SQL
             SQL = SQL.strip('\n').strip(' ')
             if SQL == '':
@@ -409,7 +426,7 @@ class ExtensionView(PGChildNodeView):
 
         if eid is not None:
             SQL = render_template("/".join(
-                [self.template_path, 'properties.sql']
+                [self.template_path, self._PROPERTIES_SQL]
             ), eid=eid)
             status, res = self.conn.execute_dict(SQL)
             if not status:
@@ -425,12 +442,12 @@ class ExtensionView(PGChildNodeView):
                 if arg not in data:
                     data[arg] = old_data[arg]
             SQL = render_template("/".join(
-                [self.template_path, 'update.sql']
+                [self.template_path, self._UPDATE_SQL]
             ), data=data, o_data=old_data)
             return SQL, data['name'] if 'name' in data else old_data['name']
         else:
             SQL = render_template("/".join(
-                [self.template_path, 'create.sql']
+                [self.template_path, self._CREATE_SQL]
             ), data=data)
             return SQL, data['name']
 
@@ -463,31 +480,34 @@ class ExtensionView(PGChildNodeView):
         )
 
     @check_precondition
-    def sql(self, gid, sid, did, eid):
+    def sql(self, gid, sid, did, eid, json_resp=True):
         """
         This function will generate sql for the sql panel
         """
         SQL = render_template("/".join(
-            [self.template_path, 'properties.sql']
+            [self.template_path, self._PROPERTIES_SQL]
         ), eid=eid)
         status, res = self.conn.execute_dict(SQL)
         if not status:
             return internal_server_error(errormsg=res)
         if len(res['rows']) == 0:
             return gone(
-                _("Could not find the extension on the server.")
+                gettext("Could not find the extension on the server.")
             )
 
         result = res['rows'][0]
 
         SQL = render_template("/".join(
-            [self.template_path, 'create.sql']
+            [self.template_path, self._CREATE_SQL]
         ),
             data=result,
             conn=self.conn,
             display_comments=True
         )
 
+        if not json_resp:
+            return SQL
+
         return ajax_response(response=SQL)
 
     @check_precondition
@@ -526,6 +546,57 @@ class ExtensionView(PGChildNodeView):
             status=200
         )
 
+    @check_precondition
+    def fetch_objects_to_compare(self, sid, did):
+        """
+        This function will fetch the list of all the extensions for
+        specified database id.
 
+        :param sid: Server Id
+        :param did: Database Id
+        :return:
+        """
+        res = dict()
+
+        sql = render_template("/".join([self.template_path,
+                                        'properties.sql']))
+        status, rset = self.conn.execute_2darray(sql)
+        if not status:
+            return internal_server_error(errormsg=rset)
+
+        for row in rset['rows']:
+            status, data = self._fetch_properties(did, row['oid'])
+            if status:
+                res[row['name']] = data
+
+        return res
+
+    def get_sql_from_diff(self, **kwargs):
+        """
+        This function is used to get the DDL/DML statements.
+        :param kwargs
+        :return:
+        """
+        gid = kwargs.get('gid')
+        sid = kwargs.get('sid')
+        did = kwargs.get('did')
+        oid = kwargs.get('oid')
+        data = kwargs.get('data', None)
+        drop_sql = kwargs.get('drop_sql', False)
+
+        if data:
+            sql, name = self.getSQL(gid=gid, sid=sid, did=did, data=data,
+                                    eid=oid)
+        else:
+            if drop_sql:
+                sql = self.delete(gid=gid, sid=sid, did=did,
+                                  eid=oid, only_sql=True)
+            else:
+                sql = self.sql(gid=gid, sid=sid, did=did, eid=oid,
+                               json_resp=False)
+        return sql
+
+
+SchemaDiffRegistry(blueprint.node_type, ExtensionView, 'Database')
 # Register and add ExtensionView as blueprint
 ExtensionView.register_node_view(blueprint)
