@@ -2,16 +2,16 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
 
 define('pgadmin.node.extension', [
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
-  'underscore.string', 'sources/pgadmin', 'pgadmin.browser',
+  'sources/pgadmin', 'pgadmin.browser',
   'pgadmin.backform', 'pgadmin.browser.collection',
-], function(gettext, url_for, $, _, S, pgAdmin, pgBrowser, Backform) {
+], function(gettext, url_for, $, _, pgAdmin, pgBrowser, Backform) {
 
   /*
    * Create and Add an Extension Collection into nodes
@@ -24,7 +24,7 @@ define('pgadmin.node.extension', [
     pgAdmin.Browser.Nodes['coll-extension'] =
       pgAdmin.Browser.Collection.extend({
         node: 'extension',
-        label: gettext('Extension'),
+        label: gettext('Extensions'),
         type: 'coll-extension',
         columns: ['name', 'owner', 'comment'],
       });
@@ -93,12 +93,12 @@ define('pgadmin.node.extension', [
        * of the model in schema.
        */
       model: pgAdmin.Browser.Node.Model.extend({
-        idAttribute: 'eid',
+        idAttribute: 'oid',
         schema: [
           {
             id: 'name', label: gettext('Name'), first_empty: true,
             type: 'text', mode: ['properties', 'create', 'edit'],
-            visible: true, url:'avails', disabled: function(m) {
+            visible: true, url:'avails', readonly: function(m) {
               return !m.isNew();
             },
             transform: function(data, cell) {
@@ -181,8 +181,8 @@ define('pgadmin.node.extension', [
             }),
           },
           {
-            id: 'eid', label: gettext('OID'), cell: 'string',
-            type: 'text', disabled: true, mode: ['properties'],
+            id: 'oid', label: gettext('OID'), cell: 'string',
+            type: 'text', mode: ['properties'],
           },
           {
             id: 'owner', label: gettext('Owner'), control: 'node-list-by-name',
@@ -205,11 +205,6 @@ define('pgadmin.node.extension', [
           {
             id: 'relocatable', label: gettext('Relocatable?'), cell: 'switch',
             group: gettext('Definition'), type: 'switch', mode: ['properties'],
-            options: {
-              'onText': gettext('Yes'), 'offText': gettext('No'),
-              'onColor': 'success', 'offColor': 'primary',
-              'size': 'small',
-            },
           },
           {
             id: 'version', label: gettext('Version'), cell: 'string',
@@ -233,10 +228,12 @@ define('pgadmin.node.extension', [
               });
               return res;
             },
-          },
-          {
+          },{
+            id: 'is_sys_obj', label: gettext('System extension?'),
+            cell:'boolean', type: 'switch', mode: ['properties'],
+          },{
             id: 'comment', label: gettext('Comment'), cell: 'string',
-            type: 'multiline', disabled: true,
+            type: 'multiline', readonly: true,
           },
         ],
         validate: function() {
@@ -251,7 +248,7 @@ define('pgadmin.node.extension', [
           if (_.isUndefined(name) || _.isNull(name) ||
             String(name).replace(/^\s+|\s+$/g, '') == '') {
             err['name'] = gettext('Name cannot be empty.');
-            errmsg = errmsg || err['name'];
+            errmsg = err['name'];
             this.errorModel.set('name', errmsg);
             return errmsg;
           }
