@@ -2,22 +2,17 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
 import os
-import sys
 
 from pgadmin.browser.server_groups.servers.databases.external_tables import \
     ExternalTablesView
 from pgadmin.utils.route import BaseTestGenerator
-
-if sys.version_info < (3, 3):
-    from mock import MagicMock, patch
-else:
-    from unittest.mock import MagicMock, patch
+from unittest.mock import MagicMock, patch
 
 
 class TestExternalTablesView(BaseTestGenerator):
@@ -345,8 +340,8 @@ class TestExternalTablesView(BaseTestGenerator):
         self.manager.connection.assert_called_with(
             **self.expected_manager_connection_to_be_called_with
         )
-        self.assertEquals(self.manager, external_tables_view.manager)
-        self.assertEquals(self.connection, external_tables_view.connection)
+        self.assertEqual(self.manager, external_tables_view.manager)
+        self.assertEqual(self.connection, external_tables_view.connection)
 
     @patch('pgadmin.browser.server_groups.servers.databases.external_tables'
            '.render_template')
