@@ -1,8 +1,17 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define('pgadmin.node.foreign_server', [
-  'sources/gettext', 'jquery', 'underscore', 'underscore.string', 'sources/pgadmin',
+  'sources/gettext', 'jquery', 'underscore', 'sources/pgadmin',
   'pgadmin.browser', 'pgadmin.backform', 'pgadmin.browser.collection',
   'pgadmin.browser.server.privilege',
-], function(gettext, $, _, S, pgAdmin, pgBrowser, Backform) {
+], function(gettext, $, _, pgAdmin, pgBrowser, Backform) {
 
   // Extend the browser's node model class to create a Options model
   var OptionsModel = pgAdmin.Browser.Node.Model.extend({
@@ -12,22 +21,22 @@ define('pgadmin.node.foreign_server', [
       fsrvvalue: undefined,
     },
 
-        // Defining schema for the Options model
+    // Defining schema for the Options model
     schema: [
-          {id: 'fsrvoption', label: gettext('Options'), type:'text', cellHeaderClasses:'width_percent_50', group: null, editable: true},
-          {id: 'fsrvvalue', label: gettext('Value'), type: 'text', cellHeaderClasses:'width_percent_50', group:null, editable: true},
+      {id: 'fsrvoption', label: gettext('Options'), type:'text', cellHeaderClasses:'width_percent_50', group: null, editable: true},
+      {id: 'fsrvvalue', label: gettext('Value'), type: 'text', cellHeaderClasses:'width_percent_50', group:null, editable: true},
     ],
 
-        /* validate function is used to validate the input given by
+    /* validate function is used to validate the input given by
          * the user. In case of error, message will be displayed on
          * the browser for the respective control.
          */
     validate: function() {
-          // Validation for the option name
+      // Validation for the option name
       if (_.isUndefined(this.get('fsrvoption')) ||
             _.isNull(this.get('fsrvoption')) ||
             String(this.get('fsrvoption')).replace(/^\s+|\s+$/g, '') == '') {
-        var msg = 'Please enter an option name';
+        var msg = gettext('Please enter an option name.');
         this.errorModel.set('fsrvoption', msg);
         return msg;
       } else {
@@ -92,7 +101,7 @@ define('pgadmin.node.foreign_server', [
 
       // Defining model for foreign server node
       model: pgAdmin.Browser.Node.Model.extend({
-        idAttribute: 'fsrvid',
+        idAttribute: 'oid',
         defaults: {
           name: undefined,
           fsrvtype: undefined,
@@ -100,6 +109,7 @@ define('pgadmin.node.foreign_server', [
           fsrvvalue: undefined,
           fsrvoptions: [],
           fsrvowner: undefined,
+          is_sys_obj: undefined,
           description: undefined,
           fsrvacl: [],
         },
@@ -125,8 +135,8 @@ define('pgadmin.node.foreign_server', [
             );
           },
         },{
-          id: 'fsrvid', label: gettext('OID'), cell: 'string',
-          type: 'text', disabled: true, mode: ['properties'],
+          id: 'oid', label: gettext('OID'), cell: 'string',
+          type: 'text', mode: ['properties'],
         },{
           id: 'fsrvowner', label: gettext('Owner'), type: 'text',
           control: Backform.NodeListByNameControl, node: 'role',
@@ -139,6 +149,9 @@ define('pgadmin.node.foreign_server', [
         },{
           id: 'fsrvversion', label: gettext('Version'), cell: 'string',
           group: gettext('Definition'), type: 'text',
+        },{
+          id: 'is_sys_obj', label: gettext('System foreign server?'),
+          cell:'boolean', type: 'switch', mode: ['properties'],
         },{
           id: 'description', label: gettext('Comment'), cell: 'string',
           type: 'multiline',
@@ -153,7 +166,7 @@ define('pgadmin.node.foreign_server', [
           mode: ['edit', 'create'], canAdd: true, canDelete: true, uniqueCol : ['grantee'],
         },{
           id: 'acl', label: gettext('Privileges'), type: 'text',
-          group: gettext('Security'), mode: ['properties'], disabled: true,
+          group: gettext('Security'), mode: ['properties'],
         },
         ],
 
