@@ -1,17 +1,26 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define('pgadmin.node.foreign_data_wrapper', [
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
-  'underscore.string', 'sources/pgadmin', 'pgadmin.browser', 'pgadmin.backform',
+  'sources/pgadmin', 'pgadmin.browser', 'pgadmin.backform',
   'pgadmin.browser.collection', 'pgadmin.browser.server.privilege',
-], function(gettext, url_for, $, _, S, pgAdmin, pgBrowser, Backform) {
+], function(gettext, url_for, $, _, pgAdmin, pgBrowser, Backform) {
 
-    // Extend the browser's node model class to create a Options model
+  // Extend the browser's node model class to create a Options model
   var OptionsModel = pgBrowser.Node.Model.extend({
     idAttribute: 'fdwoption',
     defaults: {
       fdwoption: undefined,
       fdwvalue: undefined,
     },
-        // Defining schema for the Options model
+    // Defining schema for the Options model
     schema: [{
       id: 'fdwoption', label: gettext('Option'), type:'text',
       cellHeaderClasses:'width_percent_50', editable: true,
@@ -19,16 +28,16 @@ define('pgadmin.node.foreign_data_wrapper', [
       id: 'fdwvalue', label: gettext('Value'), type: 'text',
       cellHeaderClasses:'width_percent_50', group:null, editable: true,
     }],
-        /* validate function is used to validate the input given by
+    /* validate function is used to validate the input given by
          * the user. In case of error, message will be displayed on
          * the browser for the respective control.
          */
     validate: function() {
-          // Validation for the option name
+      // Validation for the option name
       if (_.isUndefined(this.get('fdwoption')) ||
             _.isNull(this.get('fdwoption')) ||
             String(this.get('fdwoption')).replace(/^\s+|\s+$/g, '') == '') {
-        var msg = 'Please enter an option name';
+        var msg = gettext('Please enter an option name.');
         this.errorModel.set('fdwoption', msg);
         return msg;
       } else {
@@ -95,10 +104,11 @@ define('pgadmin.node.foreign_data_wrapper', [
 
       // Defining model for foreign data wrapper node
       model: pgBrowser.Node.Model.extend({
-        idAttribute: 'fdwoid',
+        idAttribute: 'oid',
         defaults: {
           name: undefined,
           fdwowner: undefined,
+          is_sys_obj: undefined,
           comment: undefined,
           fdwvalue: undefined,
           fdwhan: undefined,
@@ -121,15 +131,15 @@ define('pgadmin.node.foreign_data_wrapper', [
         // Defining schema for the foreign data wrapper node
         schema: [{
           id: 'name', label: gettext('Name'), cell: 'string',
-          type: 'text', disabled: function() {
-            // name field will be disabled only if edit mode and server version is below 9.2
+          type: 'text', readonly: function() {
+            // name field will be disabled only if edit mode
             return (
-              this.mode == 'edit' && this.node_info.server.version < 90200
+              this.mode == 'edit'
             );
           },
         },{
-          id: 'fdwoid', label: gettext('OID'), cell: 'string',
-          type: 'text', disabled: true, mode: ['properties'],
+          id: 'oid', label: gettext('OID'), cell: 'string',
+          type: 'text', mode: ['properties'],
         },{
           id: 'fdwowner', label: gettext('Owner'), type: 'text',
           control: Backform.NodeListByNameControl, node: 'role',
@@ -137,6 +147,9 @@ define('pgadmin.node.foreign_data_wrapper', [
         },{
           id: 'fdwhan', label: gettext('Handler'), type: 'text', control: 'node-ajax-options',
           group: gettext('Definition'), mode: ['edit', 'create', 'properties'], url:'get_handlers',
+        },{
+          id: 'is_sys_obj', label: gettext('System foreign data wrapper?'),
+          cell:'boolean', type: 'switch', mode: ['properties'],
         },{
           id: 'description', label: gettext('Comment'), cell: 'string',
           type: 'multiline',
@@ -159,9 +172,9 @@ define('pgadmin.node.foreign_data_wrapper', [
           }), control: 'unique-col-collection',
         },{
           id: 'acl', label: gettext('Privileges'), type: 'text',
-          group: gettext('Security'), mode: ['properties'], disabled: true,
+          group: gettext('Security'), mode: ['properties'],
         }],
-         /* validate function is used to validate the input given by
+        /* validate function is used to validate the input given by
           * the user. In case of error, message will be displayed on
           * the browser for the respective control.
           */
