@@ -2,27 +2,24 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
-from __future__ import print_function
-
 import uuid
-
 from pgadmin.browser.server_groups.servers.databases.tests import \
     utils as database_utils
 from pgadmin.utils.route import BaseTestGenerator
 from regression import parent_node_dict
 from regression.python_test_utils import test_utils as utils
 from . import utils as language_utils
+from unittest.mock import patch
 
 
 class LanguagesDeleteTestCase(BaseTestGenerator):
-    scenarios = [
-        ('Language delete test case', dict(url='/browser/language/obj/'))
-    ]
+    scenarios = utils.generate_scenarios(
+        'language_delete', language_utils.test_cases)
 
     def setUp(self):
         self.server_data = parent_node_dict["database"][-1]
@@ -42,12 +39,37 @@ class LanguagesDeleteTestCase(BaseTestGenerator):
                                                           self.lang_name)
 
     def runTest(self):
-        """This function will delete language under test database."""
+        """This function contains the test cases for delete language"""
 
-        response = self.tester.delete("{0}{1}/{2}/{3}/{4}".format(
+        if self.is_positive_test:
+            response = self.delete_language()
+            actual_response_code = response.status_code
+            expected_status_code = self.expected_data['status_code']
+        elif hasattr(self, 'error_in_getting_language'):
+            with patch(self.mock_data["function_name"],
+                       return_value=[eval(self.mock_data["return_value"])]):
+                response = self.delete_language()
+                actual_response_code = response.status_code
+                expected_status_code = self.expected_data['status_code']
+
+        elif self.error_in_deleting_language:
+            # with patch('pgadmin.utils.driver.psycopg2.connection.Connection'
+            #            '.execute_scalar',
+            #            side_effect=[(True, True), (
+            #                False, self.expected_data["message"])]):
+            with patch(self.mock_data["function_name"],
+                       side_effect=[eval(self.mock_data["return_value"])]):
+                response = self.delete_language()
+                actual_response_code = response.status_code
+                expected_status_code = self.expected_data['status_code']
+
+        self.assertEqual(actual_response_code, expected_status_code)
+
+    def delete_language(self):
+        """This function will delete language under test database."""
+        return self.tester.delete("{0}{1}/{2}/{3}/{4}".format(
             self.url, utils.SERVER_GROUP, self.server_id, self.db_id,
             self.language_id), follow_redirects=True)
-        self.assertEquals(response.status_code, 200)
 
     def tearDown(self):
         """This function disconnect the test database."""
