@@ -2,12 +2,11 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
-from __future__ import print_function
 
 import uuid
 import json
@@ -21,9 +20,8 @@ from . import utils as language_utils
 
 
 class LanguagesDeleteMultipleTestCase(BaseTestGenerator):
-    scenarios = [
-        ('Language delete test case', dict(url='/browser/language/obj/'))
-    ]
+    scenarios = utils.generate_scenarios('delete_multiple',
+                                         language_utils.test_cases)
 
     def setUp(self):
         self.server_data = parent_node_dict["database"][-1]
@@ -59,7 +57,7 @@ class LanguagesDeleteMultipleTestCase(BaseTestGenerator):
             data=json.dumps(data),
             content_type='html/json'
         )
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
 
     def tearDown(self):
         """This function disconnect the test database."""
