@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -11,7 +11,5 @@ from pgadmin.utils.route import BaseTestGenerator
 
 
 class CollationTestGenerator(BaseTestGenerator):
-
-    def generate_tests(self):
+    def runTest(self):
         return
-
