@@ -1,20 +1,31 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define('pgadmin.node.fts_configuration', [
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore', 'backbone',
   'sources/pgadmin', 'pgadmin.browser', 'pgadmin.backform', 'pgadmin.backgrid',
-  'pgadmin.node.schema.dir/child', 'pgadmin.browser.collection',
+  'pgadmin.node.schema.dir/child', 'pgadmin.node.schema.dir/schema_child_tree_node',
+  'pgadmin.browser.collection',
 ], function(
   gettext, url_for, $, _, Backbone, pgAdmin, pgBrowser, Backform, Backgrid,
-  schemaChild
+  schemaChild, schemaChildTreeNode
 ) {
 
   // Model for tokens control
   var TokenModel = pgAdmin.Browser.Node.Model.extend({
+    idAttribute: 'token',
     defaults: {
       token: undefined,
       dictname: undefined,
     },
     keys: ['token'],
-        // Define the schema for the token/dictionary list
+    // Define the schema for the token/dictionary list
     schema: [{
       id: 'token', label: gettext('Token'), type:'text', group: null,
       cellHeaderClasses:'width_percent_50',
@@ -23,10 +34,11 @@ define('pgadmin.node.fts_configuration', [
       id: 'dictname', label: gettext('Dictionaries'), type: 'text', group:null,
       cellHeaderClasses:'width_percent_50', editable: true,
       cell:Backgrid.Extension.MultiSelectAjaxCell, url: 'dictionaries',
+      cache_level:'fts_configuration', cache_node:'fts_configuration',
     }],
-        // Validation for token and dictionary list
+    // Validation for token and dictionary list
     validate: function() {
-            // Clear any existing errors.
+      // Clear any existing errors.
       var msg;
       this.errorModel.clear();
       var token = this.get('token');
@@ -51,14 +63,14 @@ define('pgadmin.node.fts_configuration', [
     },
   });
 
-// Customized control for token control
+  // Customized control for token control
   var TokenControl =  Backform.TokenControl =
     Backform.UniqueColCollectionControl.extend({
 
       initialize: function() {
         Backform.UniqueColCollectionControl.prototype.initialize.apply(
-        this, arguments
-      );
+          this, arguments
+        );
 
         var self = this,
           headerSchema = [{
@@ -72,14 +84,14 @@ define('pgadmin.node.fts_configuration', [
                 Backform.NodeAjaxOptionsControl.prototype.initialize.apply(
                   this,
                   arguments
-                 );
-                var self = this,
-                  url = self.field.get('url') || self.defaults.url,
-                  m = self.model.top || self.model;
+                );
+                var _self = this,
+                  url = _self.field.get('url') || _self.defaults.url,
+                  m = _self.model.top || _self.model;
 
-                /* Fetch the tokens/dict list from 'self' node.
-                 * Here 'self' refers to unique collection control where
-                 * 'self' refers to nodeAjaxOptions control for dictionary
+                /* Fetch the tokens/dict list from '_self' node.
+                 * Here '_self' refers to unique collection control where
+                 * '_self' refers to nodeAjaxOptions control for dictionary
                  */
                 if (url) {
                   var node = this.field.get('schema_node'),
@@ -97,6 +109,9 @@ define('pgadmin.node.fts_configuration', [
                                     pgAdmin.Browser.Nodes[cache_node])
                                || node;
 
+                  // Clear the cache to get the latest dictionaries and parsers.
+                  cache_node.clear_cache(this);
+
                   /*
                    * We needs to check, if we have already cached data
                    * for this url. If yes - use it, and do not bother about
@@ -107,26 +122,26 @@ define('pgadmin.node.fts_configuration', [
                   // Fetch token/dictionary list
                   if (this.field.get('version_compatible') &&
                     (_.isUndefined(data) || _.isNull(data))) {
-                    m.trigger('pgadmin:view:fetching', m, self.field);
+                    m.trigger('pgadmin:view:fetching', m, _self.field);
                     $.ajax({
                       async: false,
                       url: full_url,
                     })
-                    .done(function(res) {
-                    /*
+                      .done(function(res) {
+                        /*
                      * We will cache this data for short period of time for
                      * avoiding same calls.
                      */
-                      data = cache_node.cache(url,
-                               node_info,
-                               cache_level,
-                               res.data
-                             );
-                    })
-                    .fail(function() {
-                      m.trigger('pgadmin:view:fetch:error', m, self.field);
-                    });
-                    m.trigger('pgadmin:view:fetched', m, self.field);
+                        data = cache_node.cache(url,
+                          node_info,
+                          cache_level,
+                          res.data
+                        );
+                      })
+                      .fail(function() {
+                        m.trigger('pgadmin:view:fetch:error', m, _self.field);
+                      });
+                    m.trigger('pgadmin:view:fetched', m, _self.field);
                   }
 
                   // It is feasible that the data may not have been fetched.
@@ -136,11 +151,11 @@ define('pgadmin.node.fts_configuration', [
                    * Transform the data
                    */
                   var transform = (this.field.get('transform')
-                                || self.defaults.transform);
+                                || _self.defaults.transform);
                   if (transform && _.isFunction(transform)) {
-                    self.field.set('options', transform.bind(self, data));
+                    _self.field.set('options', transform.bind(_self, data));
                   } else {
-                    self.field.set('options', data);
+                    _self.field.set('options', data);
                   }
                 }
               },
@@ -159,16 +174,16 @@ define('pgadmin.node.fts_configuration', [
           // Grid columns backgrid
           gridCols = ['token', 'dictname'];
 
-      // Creating model for header control which is used to add new tokens
+        // Creating model for header control which is used to add new tokens
         self.headerData = new (Backbone.Model.extend({
           defaults: headerDefaults,
           schema: headerSchema,
         }))({});
 
-      // Creating view from header schema in tokens control
+        // Creating view from header schema in tokens control
         var headerGroups = Backform.generateViewSchema(
-          self.field.get('node_info'), self.headerData, 'create',
-          self.field.get('schema_node'), self.field.get('node_data')
+            self.field.get('node_info'), self.headerData, 'create',
+            self.field.get('schema_node'), self.field.get('node_data')
           ),
           fields = [];
 
@@ -177,13 +192,13 @@ define('pgadmin.node.fts_configuration', [
         });
         self.headerFields = new Backform.Fields(fields);
 
-      // creating grid using grid columns
+        // creating grid using grid columns
         self.gridSchema = Backform.generateGridColumnsFromModel(
           self.field.get('node_info'), self.field.get('model'),
           'edit', gridCols, self.field.get('schema_node')
-      );
+        );
 
-      // Providing behaviour control functions to header and grid control
+        // Providing behaviour control functions to header and grid control
         self.controls = [];
         self.listenTo(self.headerData, 'change', self.headerDataChanged);
         self.listenTo(self.headerData, 'select2', self.headerDataChanged);
@@ -191,7 +206,7 @@ define('pgadmin.node.fts_configuration', [
         self.listenTo(self.collection, 'remove', self.onAddorRemoveTokens);
       },
 
-    // Template for creating header view
+      // Template for creating header view
       generateHeader: function(data) {
         var header = [
           '<div class="subnode-header-form">',
@@ -202,7 +217,7 @@ define('pgadmin.node.fts_configuration', [
           '   </div>',
           '   <div class="col-6" header="token"></div>',
           '   <div class="col-2">',
-          '     <button class="btn btn-sm-sq btn-default add fa fa-plus" <%=canAdd ? "" : "disabled=\'disabled\'"%> ></button>',
+          '     <button class="btn btn-sm-sq btn-primary-icon add fa fa-plus" <%=canAdd ? "" : "disabled=\'disabled\'"%> ><span class="sr-only">' + gettext('Add Token') + '</span></button>',
           '   </div>',
           '  </div>',
           ' </div>',
@@ -224,19 +239,19 @@ define('pgadmin.node.fts_configuration', [
           });
 
           $header.find('div[header="' + field.get('name') + '"]').append(
-          control.render().$el
-        );
+            control.render().$el
+          );
 
           control.$el.find('.control-label').remove();
           controls.push(control);
         });
 
-      // We should not show add button in properties mode
+        // We should not show add button in properties mode
         if (data.mode == 'properties') {
           $header.find('button.add').remove();
         }
 
-      // Disable add button in token control in create mode
+        // Disable add button in token control in create mode
         if(data.mode == 'create') {
           $header.find('button.add').attr('disabled', true);
         }
@@ -245,13 +260,13 @@ define('pgadmin.node.fts_configuration', [
         return $header;
       },
 
-    // Providing event handler for add button in header
+      // Providing event handler for add button in header
       events: _.extend(
-              {}, Backform.UniqueColCollectionControl.prototype.events,
-              {'click button.add': 'addTokens'}
-            ),
+        {}, Backform.UniqueColCollectionControl.prototype.events,
+        {'click button.add': 'addTokens'}
+      ),
 
-    // Show token/dictionary grid
+      // Show token/dictionary grid
       showGridControl: function(data) {
 
         var self = this,
@@ -259,14 +274,14 @@ define('pgadmin.node.fts_configuration', [
           $gridBody = $('<div></div>', {
             class:'pgadmin-control-group backgrid form-group col-12 object subnode',
           }).append(
-               titleTmpl({label: data.label})
+            titleTmpl({label: data.label})
           );
 
         $gridBody.append(self.generateHeader(data));
 
         var gridColumns = _.clone(this.gridSchema.columns);
 
-      // Insert Delete Cell into Grid
+        // Insert Delete Cell into Grid
         if (data.disabled == false && data.canDelete) {
           gridColumns.unshift({
             name: 'pg-backform-delete', label: '',
@@ -277,9 +292,9 @@ define('pgadmin.node.fts_configuration', [
 
         if (self.grid) {
           self.grid.remove();
-          self.grid.null;
+          self.grid = null;
         }
-      // Initialize a new Grid instance
+        // Initialize a new Grid instance
         var grid = self.grid = new Backgrid.Grid({
           columns: gridColumns,
           collection: self.collection,
@@ -289,21 +304,21 @@ define('pgadmin.node.fts_configuration', [
 
         $gridBody.append(self.$grid);
 
-      // Find selected dictionaries in grid and show it all together
+        // Find selected dictionaries in grid and show it all together
         setTimeout(function() {
           self.headerData.set({
             'token': self.$header.find(
-            'div[header="token"] select'
+              'div[header="token"] select'
             ).val(),
           }, {silent:true}
           );
         }, 10);
 
-      // Render node grid
+        // Render node grid
         return $gridBody;
       },
 
-    // When user change the header control to add a new token
+      // When user change the header control to add a new token
       headerDataChanged: function() {
         var self = this,
           data = this.headerData.toJSON(),
@@ -316,57 +331,54 @@ define('pgadmin.node.fts_configuration', [
         self.$header.find('button.add').prop('disabled', inSelected);
       },
 
-    // Get called when user click on add button header
+      // Get called when user click on add button header
       addTokens: function(ev) {
         ev.preventDefault();
         var self = this,
           token = self.headerData.get('token');
 
-        if (!token || token == '') {
-          return false;
-        }
+        if (token && token != '') {
+          var coll = self.model.get(self.field.get('name')),
+            m = new (self.field.get('model'))(
+              self.headerData.toJSON(), {
+                silent: true, top: self.model.top,
+                collection: coll, handler: coll,
+              }),
+            checkVars = ['token'],
+            idx = -1;
 
-        var coll = self.model.get(self.field.get('name')),
-          m = new (self.field.get('model'))(
-                self.headerData.toJSON(), {
-                  silent: true, top: self.model.top,
-                  collection: coll, handler: coll,
-                }),
-          checkVars = ['token'],
-          idx = -1;
-
-      // Find if token exists in grid
-        self.collection.each(function(m) {
-          _.each(checkVars, function(v) {
-            var val = m.get(v);
-            if(val == token) {
-              idx = coll.indexOf(m);
-            }
+          // Find if token exists in grid
+          self.collection.each(function(local_model) {
+            _.each(checkVars, function(v) {
+              var val = local_model.get(v);
+              if(val == token) {
+                idx = coll.indexOf(local_model);
+              }
+            });
           });
-        });
 
 
 
-      // remove 'm' if duplicate value found.
-        if (idx == -1) {
-          coll.add(m);
-          idx = coll.indexOf(m);
+          // remove 'm' if duplicate value found.
+          if (idx == -1) {
+            coll.add(m);
+            idx = coll.indexOf(m);
+          }
+          self.$grid.find('.new').removeClass('new');
+          var newRow = self.grid.body.rows[idx].$el;
+          newRow.addClass('new');
+          //$(newRow).pgMakeVisible('table-bordered');
+          $(newRow).pgMakeVisible('backform-tab');
         }
-        self.$grid.find('.new').removeClass('new');
-        var newRow = self.grid.body.rows[idx].$el;
-        newRow.addClass('new');
-      //$(newRow).pgMakeVisible('table-bordered');
-        $(newRow).pgMakeVisible('backform-tab');
-
 
         return false;
       },
 
-    // When user delete token/dictionary entry from grid
+      // When user delete token/dictionary entry from grid
       onAddorRemoveTokens: function() {
         var self = this;
 
-      /*
+        /*
        * Wait for collection to be updated before checking for the button to
        * be enabled, or not.
        */
@@ -376,7 +388,7 @@ define('pgadmin.node.fts_configuration', [
         }, 10);
       },
 
-    // When control is about to destroy
+      // When control is about to destroy
       remove: function() {
       /*
        * Stop listening the events registered by this control.
@@ -385,13 +397,13 @@ define('pgadmin.node.fts_configuration', [
         this.listenTo(this.headerData, 'select2', this.headerDataChanged);
         this.listenTo(this.collection, 'remove', this.onAddorRemoveTokens);
 
-      // Remove header controls.
+        // Remove header controls.
         _.each(this.controls, function(control) {
           control.remove();
         });
         TokenControl.__super__.remove.apply(this, arguments);
 
-      // Remove the header model
+        // Remove the header model
         delete (this.headerData);
 
       },
@@ -406,6 +418,8 @@ define('pgadmin.node.fts_configuration', [
         label: gettext('FTS Configurations'),
         type: 'coll-fts_configuration',
         columns: ['name', 'description'],
+        canDrop: schemaChildTreeNode.isTreeItemOfChildOfSchema,
+        canDropCascade: schemaChildTreeNode.isTreeItemOfChildOfSchema,
       });
   }
 
@@ -456,6 +470,7 @@ define('pgadmin.node.fts_configuration', [
         defaults: {
           name: undefined,        // FTS Configuration name
           owner: undefined,       // FTS Configuration owner
+          is_sys_obj: undefined,  // Is system object
           description: undefined, // Comment on FTS Configuration
           schema: undefined,      // Schema name FTS Configuration belongs to
           prsname: undefined,    // FTS parser list for FTS Configuration node
@@ -480,7 +495,7 @@ define('pgadmin.node.fts_configuration', [
           type: 'text', cellHeaderClasses: 'width_percent_50',
         },{
           id: 'oid', label: gettext('OID'), cell: 'string',
-          editable: false, type: 'text', disabled: true, mode:['properties'],
+          editable: false, type: 'text', mode:['properties'],
         },{
           id: 'owner', label: gettext('Owner'), cell: 'string',
           type: 'text', mode: ['properties', 'edit','create'], node: 'role',
@@ -490,6 +505,9 @@ define('pgadmin.node.fts_configuration', [
           type: 'text', mode: ['create','edit'], node: 'schema',
           control: 'node-list-by-id', cache_node: 'database',
           cache_level: 'database',
+        },{
+          id: 'is_sys_obj', label: gettext('System FTS configuration?'),
+          cell:'boolean', type: 'switch', mode: ['properties'],
         },{
           id: 'description', label: gettext('Comment'), cell: 'string',
           type: 'multiline', cellHeaderClasses: 'width_percent_50',
@@ -501,24 +519,24 @@ define('pgadmin.node.fts_configuration', [
           //disable parser when user select copy_config manually and vica-versa
           disabled: function(m) {
             var copy_config = m.get('copy_config');
-            return m.isNew() &&
-                    (_.isNull(copy_config) ||
+            return (_.isNull(copy_config) ||
                     _.isUndefined(copy_config) ||
                     copy_config === '') ? false : true;
           },
+          readonly: function(m) {return !m.isNew();},
         },{
-          id: 'copy_config', label: gettext('Copy Config'),type: 'text',
+          id: 'copy_config', label: gettext('Copy config'),type: 'text',
           mode: ['create'], group: gettext('Definition'),
           control: 'node-ajax-options', url: 'copyConfig', deps: ['prsname'],
 
           //disable copy_config when user select parser manually and vica-versa
           disabled: function(m) {
             var parser = m.get('prsname');
-            return m.isNew() &&
-                    (_.isNull(parser) ||
+            return (_.isNull(parser) ||
                     _.isUndefined(parser) ||
                     parser === '') ? false : true;
           },
+          readonly: function(m) {return !m.isNew();},
         },{
           id: 'tokens', label: gettext('Tokens'), type: 'collection',
           group: gettext('Tokens'), control: TokenControl,
@@ -539,7 +557,7 @@ define('pgadmin.node.fts_configuration', [
           var copy_config_or_parser = !(parser === '' ||
                                         _.isUndefined(parser) ||
                                         _.isNull(parser)) ?
-                                        this.get('prsname') : this.get('copy_config');
+            this.get('prsname') : this.get('copy_config');
           var schema = this.get('schema');
 
           // Clear the existing error model
