@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -20,14 +20,11 @@ from config import PG_DEFAULT_DRIVER
 from pgadmin.browser.server_groups.servers.databases.schemas.utils \
     import SchemaChildModule
 from pgadmin.browser.utils import PGChildNodeView
-from pgadmin.utils import IS_PY2
 from pgadmin.utils.ajax import make_json_response, internal_server_error, \
     make_response as ajax_response, gone
 from pgadmin.utils.driver import get_driver
-
-# If we are in Python3
-if not IS_PY2:
-    unicode = str
+from pgadmin.tools.schema_diff.node_registry import SchemaDiffRegistry
+from pgadmin.tools.schema_diff.compare import SchemaDiffObjectCompare
 
 
 class FtsDictionaryModule(SchemaChildModule):
@@ -52,8 +49,8 @@ class FtsDictionaryModule(SchemaChildModule):
       - Load the module script for FTS Dictionary, when any of the schema
       node is initialized.
     """
-    NODE_TYPE = 'fts_dictionary'
-    COLLECTION_LABEL = _('FTS Dictionaries')
+    _NODE_TYPE = 'fts_dictionary'
+    _COLLECTION_LABEL = _('FTS Dictionaries')
 
     def __init__(self, *args, **kwargs):
         self.min_ver = None
@@ -85,13 +82,13 @@ class FtsDictionaryModule(SchemaChildModule):
         Load the module script for fts template, when any of the schema
         node is initialized.
         """
-        return databases.DatabaseModule.NODE_TYPE
+        return databases.DatabaseModule.node_type
 
 
 blueprint = FtsDictionaryModule(__name__)
 
 
-class FtsDictionaryView(PGChildNodeView):
+class FtsDictionaryView(PGChildNodeView, SchemaDiffObjectCompare):
     """
     class FtsDictionaryView(PGChildNodeView)
 
@@ -105,10 +102,6 @@ class FtsDictionaryView(PGChildNodeView):
     * __init__(**kwargs)
       - Method is used to initialize the FtsDictionaryView and it's base view.
 
-    * module_js()
-      - This property defines (if javascript) exists for this node.
-        Override this property for your own logic
-
     * check_precondition()
       - This function will behave as a decorator which will checks
         database connection before running view, it will also attaches
@@ -163,6 +156,9 @@ class FtsDictionaryView(PGChildNodeView):
     * dependencies(self, gid, sid, did, scid, dcid):
       - This function get the dependencies and return ajax response for node.
 
+    * compare(**kwargs):
+      - This function will compare the fts dictionaries nodes from two
+        different schemas.
     """
 
     node_type = blueprint.node_type
@@ -192,29 +188,18 @@ class FtsDictionaryView(PGChildNodeView):
         'stats': [{'get': 'statistics'}],
         'dependency': [{'get': 'dependencies'}],
         'dependent': [{'get': 'dependents'}],
-        'module.js': [{}, {}, {'get': 'module_js'}],
         'fetch_templates': [{'get': 'fetch_templates'},
-                            {'get': 'fetch_templates'}],
+                            {'get': 'fetch_templates'}]
     })
 
+    keys_to_ignore = ['oid', 'oid-2', 'schema']
+
     def _init_(self, **kwargs):
         self.conn = None
         self.template_path = None
         self.manager = None
         super(FtsDictionaryView, self).__init__(**kwargs)
 
-    def module_js(self):
-        """
-        Load JS file (fts_dictionary.js) for this module.
-        """
-        return make_response(
-            render_template(
-                "fts_dictionary/js/fts_dictionary.js",
-                _=_
-            ),
-            200, {'Content-Type': 'application/x-javascript'}
-        )
-
     def check_precondition(f):
         """
         This function will behave as a decorator which will checks
@@ -231,8 +216,22 @@ class FtsDictionaryView(PGChildNodeView):
             self.conn = self.manager.connection(did=kwargs['did'])
             driver = get_driver(PG_DEFAULT_DRIVER)
             self.qtIdent = driver.qtIdent
+            self.datlastsysoid = \
+                self.manager.db_info[kwargs['did']]['datlastsysoid'] \
+                if self.manager.db_info is not None and \
+                kwargs['did'] in self.manager.db_info else 0
+
+            self.datistemplate = False
+            if (
+                self.manager.db_info is not None and
+                kwargs['did'] in self.manager.db_info and
+                'datistemplate' in self.manager.db_info[kwargs['did']]
+            ):
+                self.datistemplate = self.manager.db_info[
+                    kwargs['did']]['datistemplate']
+
             # Set the template path for the SQL scripts
-            self.template_path = 'fts_dictionary/sql/#{0}#'.format(
+            self.template_path = 'fts_dictionaries/sql/#{0}#'.format(
                 self.manager.version)
 
             return f(*args, **kwargs)
@@ -271,7 +270,7 @@ class FtsDictionaryView(PGChildNodeView):
         """
 
         sql = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             scid=scid
         )
         status, res = self.conn.execute_dict(sql)
@@ -302,7 +301,7 @@ class FtsDictionaryView(PGChildNodeView):
 
         res = []
         sql = render_template(
-            "/".join([self.template_path, 'nodes.sql']),
+            "/".join([self.template_path, self._NODES_SQL]),
             scid=scid
         )
         status, rset = self.conn.execute_2darray(sql)
@@ -337,7 +336,7 @@ class FtsDictionaryView(PGChildNodeView):
         """
 
         sql = render_template(
-            "/".join([self.template_path, 'nodes.sql']),
+            "/".join([self.template_path, self._NODES_SQL]),
             dcid=dcid
         )
         status, rset = self.conn.execute_2darray(sql)
@@ -370,39 +369,55 @@ class FtsDictionaryView(PGChildNodeView):
             scid: Schema Id
             dcid: fts dictionary id
         """
+        status, res = self._fetch_properties(scid, dcid)
+        if not status:
+            return res
 
+        return ajax_response(
+            response=res,
+            status=200
+        )
+
+    def _fetch_properties(self, scid, dcid):
+        """
+        This function is used to fetch the properties of specified object.
+
+        :param scid:
+        :param dcid:
+        :return:
+        """
         sql = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             scid=scid,
             dcid=dcid
         )
         status, res = self.conn.execute_dict(sql)
 
         if not status:
-            return internal_server_error(errormsg=res)
+            return False, internal_server_error(errormsg=res)
 
         if len(res['rows']) == 0:
-            return gone(_(
+            return False, gone(_(
                 "Could not find the FTS Dictionary node in the database node."
             ))
 
+        res['rows'][0]['is_sys_obj'] = (
+            res['rows'][0]['oid'] <= self.datlastsysoid or self.datistemplate)
+
         # Handle templates and its schema name properly
-        if res['rows'][0]['template_schema'] is not None:
-            if res['rows'][0]['template_schema'] != "pg_catalog":
-                res['rows'][0]['template'] = self.qtIdent(
-                    self.conn, res['rows'][0]['template_schema'],
-                    res['rows'][0]['template']
-                )
+        if res['rows'][0]['template_schema'] is not None and \
+                res['rows'][0]['template_schema'] != "pg_catalog":
+            res['rows'][0]['template'] = self.qtIdent(
+                self.conn, res['rows'][0]['template_schema'],
+                res['rows'][0]['template']
+            )
 
         if res['rows'][0]['options'] is not None:
             res['rows'][0]['options'] = self.tokenize_options(
                 res['rows'][0]['options']
             )
 
-        return ajax_response(
-            response=res['rows'][0],
-            status=200
-        )
+        return True, res['rows'][0]
 
     @check_precondition
     def create(self, gid, sid, did, scid):
@@ -430,11 +445,12 @@ class FtsDictionaryView(PGChildNodeView):
                     status=410,
                     success=0,
                     errormsg=_(
-                        "Could not find the required parameter (%s)." % arg)
+                        "Could not find the required parameter ({})."
+                    ).format(arg)
                 )
         # Fetch schema name from schema oid
         sql = render_template(
-            "/".join([self.template_path, 'schema.sql']),
+            "/".join([self.template_path, self._SCHEMA_SQL]),
             data=data,
             conn=self.conn,
         )
@@ -448,7 +464,7 @@ class FtsDictionaryView(PGChildNodeView):
         new_data = data.copy()
         new_data['schema'] = schema
         sql = render_template(
-            "/".join([self.template_path, 'create.sql']),
+            "/".join([self.template_path, self._CREATE_SQL]),
             data=new_data,
             conn=self.conn,
         )
@@ -459,7 +475,7 @@ class FtsDictionaryView(PGChildNodeView):
         # We need dcid to add object in tree at browser,
         # Below sql will give the same
         sql = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             name=data['name'],
             scid=data['schema']
         )
@@ -493,7 +509,7 @@ class FtsDictionaryView(PGChildNodeView):
         # Fetch sql query to update fts dictionary
         sql, name = self.get_sql(gid, sid, did, scid, data, dcid)
         # Most probably this is due to error
-        if not isinstance(sql, (str, unicode)):
+        if not isinstance(sql, str):
             return sql
 
         sql = sql.strip('\n').strip(' ')
@@ -503,7 +519,7 @@ class FtsDictionaryView(PGChildNodeView):
 
         if dcid is not None:
             sql = render_template(
-                "/".join([self.template_path, 'properties.sql']),
+                "/".join([self.template_path, self._PROPERTIES_SQL]),
                 dcid=dcid
             )
 
@@ -526,7 +542,7 @@ class FtsDictionaryView(PGChildNodeView):
         )
 
     @check_precondition
-    def delete(self, gid, sid, did, scid, dcid=None):
+    def delete(self, gid, sid, did, scid, dcid=None, only_sql=False):
         """
         This function will drop the FTS Dictionary object
         :param gid: group id
@@ -534,6 +550,7 @@ class FtsDictionaryView(PGChildNodeView):
         :param did: database id
         :param scid: schema id
         :param dcid: FTS Dictionary id
+        :param only_sql: Return only sql if True
         """
         if dcid is None:
             data = request.form if request.form else json.loads(
@@ -543,23 +560,19 @@ class FtsDictionaryView(PGChildNodeView):
             data = {'ids': [dcid]}
 
         # Below will decide if it's simple drop or drop with cascade call
-        if self.cmd == 'delete':
-            # This is a cascade operation
-            cascade = True
-        else:
-            cascade = False
+
+        cascade = self._check_cascade_operation()
 
         try:
             for dcid in data['ids']:
                 # Get name for FTS Dictionary from dcid
                 sql = render_template("/".join([self.template_path,
-                                                'delete.sql']),
+                                                self._DELETE_SQL]),
                                       dcid=dcid)
                 status, res = self.conn.execute_dict(sql)
                 if not status:
                     return internal_server_error(errormsg=res)
-
-                if not res['rows']:
+                elif not res['rows']:
                     return make_json_response(
                         success=0,
                         errormsg=_(
@@ -574,12 +587,16 @@ class FtsDictionaryView(PGChildNodeView):
                 # Drop FTS Dictionary
                 result = res['rows'][0]
                 sql = render_template("/".join([self.template_path,
-                                                'delete.sql']),
+                                                self._DELETE_SQL]),
                                       name=result['name'],
                                       schema=result['schema'],
                                       cascade=cascade
                                       )
 
+                # Used for schema diff tool
+                if only_sql:
+                    return sql
+
                 status, res = self.conn.execute_scalar(sql)
                 if not status:
                     return internal_server_error(errormsg=res)
@@ -603,18 +620,22 @@ class FtsDictionaryView(PGChildNodeView):
         :param scid: schema id
         :param dcid: FTS Dictionary id
         """
-        # data = request.args
         data = {}
         for k, v in request.args.items():
             try:
-                data[k] = json.loads(v, encoding='utf-8')
+                # comments should be taken as is because if user enters a
+                # json comment it is parsed by loads which should not happen
+                if k in ('description',):
+                    data[k] = v
+                else:
+                    data[k] = json.loads(v, encoding='utf-8')
             except ValueError:
                 data[k] = v
 
         # Fetch sql query for modified data
         SQL, name = self.get_sql(gid, sid, did, scid, data, dcid)
         # Most probably this is due to error
-        if not isinstance(SQL, (str, unicode)):
+        if not isinstance(SQL, str):
             return SQL
 
         if SQL == '':
@@ -625,6 +646,49 @@ class FtsDictionaryView(PGChildNodeView):
             status=200
         )
 
+    def _get_sql_for_create(self, data, schema):
+        """
+        This function is used to get the create sql.
+        :param data:
+        :param schema:
+        :return:
+        """
+        # Replace schema oid with schema name
+        new_data = data.copy()
+        new_data['schema'] = schema
+
+        if (
+            'template' in new_data and
+            'name' in new_data and
+            'schema' in new_data
+        ):
+            sql = render_template("/".join([self.template_path,
+                                            self._CREATE_SQL]),
+                                  data=new_data,
+                                  conn=self.conn
+                                  )
+        else:
+            sql = "-- definition incomplete"
+        return sql
+
+    def _check_template_name_and_schema_name(self, data, old_data):
+        """
+        This function is used to check the template and schema name.
+        :param data:
+        :param old_data:
+        :return:
+        """
+        if 'schema' not in data:
+            data['schema'] = old_data['schema']
+
+        # Handle templates and its schema name properly
+        if old_data['template_schema'] is not None and \
+                old_data['template_schema'] != "pg_catalog":
+            old_data['template'] = self.qtIdent(
+                self.conn, old_data['template_schema'],
+                old_data['template']
+            )
+
     def get_sql(self, gid, sid, did, scid, data, dcid=None):
         """
         This function will return SQL for model data
@@ -638,7 +702,7 @@ class FtsDictionaryView(PGChildNodeView):
         # Fetch sql for update
         if dcid is not None:
             sql = render_template(
-                "/".join([self.template_path, 'properties.sql']),
+                "/".join([self.template_path, self._PROPERTIES_SQL]),
                 dcid=dcid,
                 scid=scid
             )
@@ -646,24 +710,16 @@ class FtsDictionaryView(PGChildNodeView):
             status, res = self.conn.execute_dict(sql)
             if not status:
                 return internal_server_error(errormsg=res)
-
-            if len(res['rows']) == 0:
+            elif len(res['rows']) == 0:
                 return gone(_("Could not find the FTS Dictionary node."))
 
             old_data = res['rows'][0]
-
-            # Handle templates and its schema name properly
-            if old_data['template_schema'] is not None:
-                if old_data['template_schema'] != "pg_catalog":
-                    old_data['template'] = self.qtIdent(
-                        self.conn, old_data['template_schema'],
-                        old_data['template']
-                    )
+            self._check_template_name_and_schema_name(data, old_data)
 
             # If user has changed the schema then fetch new schema directly
             # using its oid otherwise fetch old schema name using its oid
             sql = render_template(
-                "/".join([self.template_path, 'schema.sql']),
+                "/".join([self.template_path, self._SCHEMA_SQL]),
                 data=data)
 
             status, new_schema = self.conn.execute_scalar(sql)
@@ -677,7 +733,7 @@ class FtsDictionaryView(PGChildNodeView):
 
             # Fetch old schema name using old schema oid
             sql = render_template(
-                "/".join([self.template_path, 'schema.sql']),
+                "/".join([self.template_path, self._SCHEMA_SQL]),
                 data=old_data)
 
             status, old_schema = self.conn.execute_scalar(sql)
@@ -688,7 +744,7 @@ class FtsDictionaryView(PGChildNodeView):
             old_data['schema'] = old_schema
 
             sql = render_template(
-                "/".join([self.template_path, 'update.sql']),
+                "/".join([self.template_path, self._UPDATE_SQL]),
                 data=new_data, o_data=old_data
             )
             # Fetch sql query for modified data
@@ -698,29 +754,14 @@ class FtsDictionaryView(PGChildNodeView):
             return sql.strip('\n'), old_data['name']
         else:
             # Fetch schema name from schema oid
-            sql = render_template("/".join([self.template_path, 'schema.sql']),
-                                  data=data)
+            sql = render_template("/".join([self.template_path,
+                                            self._SCHEMA_SQL]), data=data)
 
             status, schema = self.conn.execute_scalar(sql)
             if not status:
                 return internal_server_error(errormsg=schema)
 
-            # Replace schema oid with schema name
-            new_data = data.copy()
-            new_data['schema'] = schema
-
-            if (
-                'template' in new_data and
-                'name' in new_data and
-                'schema' in new_data
-            ):
-                sql = render_template("/".join([self.template_path,
-                                                'create.sql']),
-                                      data=new_data,
-                                      conn=self.conn
-                                      )
-            else:
-                sql = u"-- definition incomplete"
+            sql = self._get_sql_for_create(data, schema)
             return sql.strip('\n'), data['name']
 
     @check_precondition
@@ -740,7 +781,6 @@ class FtsDictionaryView(PGChildNodeView):
         if not status:
             return internal_server_error(errormsg=rset)
 
-        datlastsysoid = self.manager.db_info[did]['datlastsysoid']
         # Empty set is added before actual list as initially it will be visible
         # at template control while creating a new FTS Dictionary
         res = [{'label': '', 'value': ''}]
@@ -758,7 +798,7 @@ class FtsDictionaryView(PGChildNodeView):
         )
 
     @check_precondition
-    def sql(self, gid, sid, did, scid, dcid):
+    def sql(self, gid, sid, did, scid, dcid, **kwargs):
         """
         This function will reverse generate sql for sql panel
         :param gid: group id
@@ -766,10 +806,13 @@ class FtsDictionaryView(PGChildNodeView):
         :param did: database id
         :param scid: schema id
         :param dcid: FTS Dictionary id
+        :param json_resp: True then return json response
         """
+        json_resp = kwargs.get('json_resp', True)
+        target_schema = kwargs.get('target_schema', None)
 
         sql = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             scid=scid,
             dcid=dcid
         )
@@ -784,12 +827,12 @@ class FtsDictionaryView(PGChildNodeView):
             ))
 
         # Handle templates and its schema name properly
-        if res['rows'][0]['template_schema'] is not None:
-            if res['rows'][0]['template_schema'] != "pg_catalog":
-                res['rows'][0]['template'] = self.qtIdent(
-                    self.conn, res['rows'][0]['template_schema'],
-                    res['rows'][0]['template']
-                )
+        if res['rows'][0]['template_schema'] is not None and \
+                res['rows'][0]['template_schema'] != "pg_catalog":
+            res['rows'][0]['template'] = self.qtIdent(
+                self.conn, res['rows'][0]['template_schema'],
+                res['rows'][0]['template']
+            )
 
         if res['rows'][0]['options'] is not None:
             res['rows'][0]['options'] = self.tokenize_options(
@@ -801,7 +844,7 @@ class FtsDictionaryView(PGChildNodeView):
 
         # Fetch schema name from schema oid
         sql = render_template("/".join(
-            [self.template_path, 'schema.sql']), data=res['rows'][0])
+            [self.template_path, self._SCHEMA_SQL]), data=res['rows'][0])
 
         status, schema = self.conn.execute_scalar(sql)
 
@@ -810,20 +853,25 @@ class FtsDictionaryView(PGChildNodeView):
 
         # Replace schema oid with schema name
         res['rows'][0]['schema'] = schema
+        if target_schema:
+            res['rows'][0]['schema'] = target_schema
 
-        sql = render_template("/".join([self.template_path, 'create.sql']),
+        sql = render_template("/".join([self.template_path,
+                                        self._CREATE_SQL]),
                               data=res['rows'][0],
                               conn=self.conn, is_displaying=True)
 
-        sql_header = u"""-- Text Search Dictionary: {0}
-
--- DROP TEXT SEARCH DICTIONARY {0};
-
+        sql_header = """-- Text Search Dictionary: {0}.{1}\n\n""".format(
+            res['rows'][0]['schema'], res['rows'][0]['name'])
+        sql_header += """-- DROP TEXT SEARCH DICTIONARY {0};\n
 """.format(self.qtIdent(self.conn, res['rows'][0]['schema'],
                         res['rows'][0]['name']))
 
         sql = sql_header + sql
 
+        if not json_resp:
+            return sql
+
         return ajax_response(response=sql.strip('\n'))
 
     @check_precondition
@@ -864,5 +912,61 @@ class FtsDictionaryView(PGChildNodeView):
             status=200
         )
 
+    @check_precondition
+    def fetch_objects_to_compare(self, sid, did, scid):
+        """
+        This function will fetch the list of all the fts dictionaries for
+        specified schema id.
 
+        :param sid: Server Id
+        :param did: Database Id
+        :param scid: Schema Id
+        :return:
+        """
+        res = dict()
+        SQL = render_template("/".join([self.template_path,
+                                        self._NODES_SQL]), scid=scid)
+        status, rset = self.conn.execute_2darray(SQL)
+        if not status:
+            return internal_server_error(errormsg=res)
+
+        for row in rset['rows']:
+            status, data = self._fetch_properties(scid, row['oid'])
+            if status:
+                res[row['name']] = data
+
+        return res
+
+    def get_sql_from_diff(self, **kwargs):
+        """
+        This function is used to get the DDL/DML statements.
+        :param kwargs
+        :return:
+        """
+        gid = kwargs.get('gid')
+        sid = kwargs.get('sid')
+        did = kwargs.get('did')
+        scid = kwargs.get('scid')
+        oid = kwargs.get('oid')
+        data = kwargs.get('data', None)
+        drop_sql = kwargs.get('drop_sql', False)
+        target_schema = kwargs.get('target_schema', None)
+
+        if data:
+            sql, name = self.get_sql(gid=gid, sid=sid, did=did, scid=scid,
+                                     data=data, dcid=oid)
+        else:
+            if drop_sql:
+                sql = self.delete(gid=gid, sid=sid, did=did,
+                                  scid=scid, dcid=oid, only_sql=True)
+            elif target_schema:
+                sql = self.sql(gid=gid, sid=sid, did=did, scid=scid, dcid=oid,
+                               target_schema=target_schema, json_resp=False)
+            else:
+                sql = self.sql(gid=gid, sid=sid, did=did, scid=scid, dcid=oid,
+                               json_resp=False)
+        return sql
+
+
+SchemaDiffRegistry(blueprint.node_type, FtsDictionaryView)
 FtsDictionaryView.register_node_view(blueprint)
