@@ -2,20 +2,23 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
-from __future__ import print_function
 
 import os
 import sys
 import traceback
+import json
 
 from regression.python_test_utils.test_utils import get_db_connection
 
 file_name = os.path.basename(__file__)
+CURRENT_PATH = os.path.dirname(os.path.realpath(__file__))
+with open(CURRENT_PATH + "/fts_templates_test_data.json") as data_file:
+    test_cases = json.load(data_file)
 
 
 def create_fts_template(server, db_name, schema_name, fts_temp_name):
@@ -30,17 +33,20 @@ def create_fts_template(server, db_name, schema_name, fts_temp_name):
                                        server['sslmode'])
         pg_cursor = connection.cursor()
 
-        query = "DROP TEXT SEARCH TEMPLATE IF EXISTS " + schema_name + "." + fts_temp_name
+        query = "DROP TEXT SEARCH TEMPLATE IF EXISTS " + schema_name + "." + \
+                fts_temp_name
         pg_cursor.execute(query)
 
-        query = "CREATE TEXT SEARCH TEMPLATE " + schema_name + "." + fts_temp_name + \
+        query = "CREATE TEXT SEARCH TEMPLATE " + schema_name + "." + \
+                fts_temp_name + \
                 "(INIT=dispell_init, LEXIZE=dispell_lexize)"
         pg_cursor.execute(query)
         connection.commit()
 
         # Get 'oid' from newly created template
         pg_cursor.execute("select oid from pg_catalog.pg_ts_template where "
-                          "tmplname = '%s' order by oid ASC limit 1" % fts_temp_name)
+                          "tmplname = '%s' order by oid ASC limit 1" %
+                          fts_temp_name)
 
         oid = pg_cursor.fetchone()
         fts_temp_id = ''
