@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -63,6 +63,9 @@ class TriggerFuncAddTestCase(BaseTestGenerator):
                 {
                     "name": "enable_sort",
                     "value": True
+                }, {
+                    "name": "search_path",
+                    "value": "public, pg_temp"
                 }
             ]
         }
@@ -83,6 +86,6 @@ class TriggerFuncAddTestCase(BaseTestGenerator):
                 '/', data=json.dumps(data), content_type='html/json'
             )
 
-            self.assertEquals(response.status_code, 200)
+            self.assertEqual(response.status_code, 200)
         # Disconnect the database
         database_utils.disconnect_database(self, self.server_id, self.db_id)
