@@ -2,31 +2,31 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
 """Implements Sequence Node"""
 
-import simplejson as json
 from functools import wraps
-import pgadmin.browser.server_groups.servers.databases as database
-from flask import render_template, make_response, request, jsonify
+
+import simplejson as json
+from flask import render_template, request, jsonify
 from flask_babelex import gettext as _
+
+import pgadmin.browser.server_groups.servers.databases as database
+from config import PG_DEFAULT_DRIVER
 from pgadmin.browser.server_groups.servers.databases.schemas.utils \
     import SchemaChildModule
 from pgadmin.browser.server_groups.servers.utils import parse_priv_from_db, \
     parse_priv_to_db
 from pgadmin.browser.utils import PGChildNodeView
+from pgadmin.tools.schema_diff.compare import SchemaDiffObjectCompare
+from pgadmin.tools.schema_diff.node_registry import SchemaDiffRegistry
 from pgadmin.utils.ajax import make_json_response, internal_server_error, \
     make_response as ajax_response, gone
 from pgadmin.utils.driver import get_driver
-from config import PG_DEFAULT_DRIVER
-from pgadmin.utils import IS_PY2
-# If we are in Python3
-if not IS_PY2:
-    unicode = str
 
 
 class SequenceModule(SchemaChildModule):
@@ -52,8 +52,8 @@ class SequenceModule(SchemaChildModule):
 
     """
 
-    NODE_TYPE = 'sequence'
-    COLLECTION_LABEL = _("Sequences")
+    _NODE_TYPE = 'sequence'
+    _COLLECTION_LABEL = _("Sequences")
 
     def __init__(self, *args, **kwargs):
         super(SequenceModule, self).__init__(*args, **kwargs)
@@ -73,7 +73,7 @@ class SequenceModule(SchemaChildModule):
         Load the module script for database, when any of the database node is
         initialized.
         """
-        return database.DatabaseModule.NODE_TYPE
+        return database.DatabaseModule.node_type
 
     @property
     def node_inode(self):
@@ -88,8 +88,10 @@ class SequenceModule(SchemaChildModule):
 blueprint = SequenceModule(__name__)
 
 
-class SequenceView(PGChildNodeView):
+class SequenceView(PGChildNodeView, SchemaDiffObjectCompare):
     node_type = blueprint.node_type
+    node_label = "Sequence"
+    node_icon = "icon-%s" % node_type
 
     parent_ids = [
         {'type': 'int', 'id': 'gid'},
@@ -113,21 +115,10 @@ class SequenceView(PGChildNodeView):
         'msql': [{'get': 'msql'}, {'get': 'msql'}],
         'stats': [{'get': 'statistics'}, {'get': 'statistics'}],
         'dependency': [{'get': 'dependencies'}],
-        'dependent': [{'get': 'dependents'}],
-        'module.js': [{}, {}, {'get': 'module_js'}]
+        'dependent': [{'get': 'dependents'}]
     })
 
-    def module_js(self):
-        """
-        This property defines whether javascript exists for this node.
-        """
-        return make_response(
-            render_template(
-                "sequence/js/sequence.js",
-                _=_
-            ),
-            200, {'Content-Type': 'application/x-javascript'}
-        )
+    keys_to_ignore = ['oid', 'oid-2', 'schema', 'current_value']
 
     def check_precondition(action=None):
         """
@@ -149,8 +140,20 @@ class SequenceView(PGChildNodeView):
                     self.conn = self.manager.connection(did=kwargs['did'])
                 else:
                     self.conn = self.manager.connection()
+                self.datlastsysoid = \
+                    self.manager.db_info[kwargs['did']]['datlastsysoid'] \
+                    if self.manager.db_info is not None and \
+                    kwargs['did'] in self.manager.db_info else 0
+                self.datistemplate = False
+                if (
+                    self.manager.db_info is not None and
+                    kwargs['did'] in self.manager.db_info and
+                    'datistemplate' in self.manager.db_info[kwargs['did']]
+                ):
+                    self.datistemplate = self.manager.db_info[
+                        kwargs['did']]['datistemplate']
 
-                self.template_path = 'sequence/sql/#{0}#'.format(
+                self.template_path = 'sequences/sql/#{0}#'.format(
                     self.manager.version
                 )
                 self.acl = ['r', 'w', 'U']
@@ -176,15 +179,17 @@ class SequenceView(PGChildNodeView):
 
         """
         SQL = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             scid=scid
         )
         status, res = self.conn.execute_dict(SQL)
 
         if not status:
             return internal_server_error(errormsg=res)
+
+        sequence_nodes = self._get_sequence_nodes(res['rows'])
         return ajax_response(
-            response=res['rows'],
+            response=sequence_nodes,
             status=200
         )
 
@@ -205,7 +210,7 @@ class SequenceView(PGChildNodeView):
         """
         res = []
         SQL = render_template(
-            "/".join([self.template_path, 'nodes.sql']),
+            "/".join([self.template_path, self._NODES_SQL]),
             scid=scid,
             seid=seid
         )
@@ -215,25 +220,26 @@ class SequenceView(PGChildNodeView):
 
         if seid is not None:
             if len(rset['rows']) == 0:
-                return gone(errormsg=_("Could not find the sequence."))
+                return gone(errormsg=self.not_found_error_msg())
             row = rset['rows'][0]
             return make_json_response(
                 data=self.blueprint.generate_browser_node(
                     row['oid'],
                     scid,
                     row['name'],
-                    icon="icon-%s" % self.node_type
+                    icon=self.node_icon
                 ),
                 status=200
             )
 
-        for row in rset['rows']:
+        sequence_nodes = self._get_sequence_nodes(rset['rows'])
+        for row in sequence_nodes:
             res.append(
                 self.blueprint.generate_browser_node(
                     row['oid'],
                     scid,
                     row['name'],
-                    icon="icon-%s" % self.node_type
+                    icon=self.node_icon
                 ))
 
         return make_json_response(
@@ -241,6 +247,31 @@ class SequenceView(PGChildNodeView):
             status=200
         )
 
+    def _get_sequence_nodes(self, nodes):
+        """
+        This function is used to iterate through all the sequences node and
+        hiding sequences created as part of an IDENTITY column.
+        :param nodes:
+        :return:
+        """
+        # If show_system_objects is true then no need to hide any sequences.
+        if self.blueprint.show_system_objects:
+            return nodes
+
+        seq_nodes = []
+        for row in nodes:
+            system_seq = self._get_dependency(row['oid'],
+                                              show_system_objects=True)
+            seq = [dep for dep in system_seq
+                   if dep['type'] == 'column' and dep['field'] == 'internal']
+            if len(seq) > 0:
+                continue
+
+            # Append the node into the newly created list
+            seq_nodes.append(row)
+
+        return seq_nodes
+
     @check_precondition(action='properties')
     def properties(self, gid, sid, did, scid, seid):
         """
@@ -256,26 +287,45 @@ class SequenceView(PGChildNodeView):
         Returns:
 
         """
-        SQL = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+        status, res = self._fetch_properties(scid, seid)
+        if not status:
+            return res
+
+        return ajax_response(
+            response=res,
+            status=200
+        )
+
+    def _fetch_properties(self, scid, seid):
+        """
+        This function is used to fetch the properties of the specified object.
+        :param scid:
+        :param seid:
+        :return:
+        """
+
+        sql = render_template(
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             scid=scid, seid=seid
         )
-        status, res = self.conn.execute_dict(SQL)
+        status, res = self.conn.execute_dict(sql)
 
         if not status:
-            return internal_server_error(errormsg=res)
+            return False, internal_server_error(errormsg=res)
+        elif len(res['rows']) == 0:
+            return False, gone(self.not_found_error_msg())
 
-        if len(res['rows']) == 0:
-            return gone(_("Could not find the sequence in the database."))
+        res['rows'][0]['is_sys_obj'] = (
+            res['rows'][0]['oid'] <= self.datlastsysoid or self.datistemplate)
 
         for row in res['rows']:
-            SQL = render_template(
+            sql = render_template(
                 "/".join([self.template_path, 'get_def.sql']),
                 data=row
             )
-            status, rset1 = self.conn.execute_dict(SQL)
+            status, rset1 = self.conn.execute_dict(sql)
             if not status:
-                return internal_server_error(errormsg=rset1)
+                return False, internal_server_error(errormsg=rset1)
 
             row['current_value'] = rset1['rows'][0]['last_value']
             row['minimum'] = rset1['rows'][0]['min_value']
@@ -285,24 +335,15 @@ class SequenceView(PGChildNodeView):
             row['cache'] = rset1['rows'][0]['cache_value']
             row['cycled'] = rset1['rows'][0]['is_cycled']
 
-            sec_lbls = []
-            if 'securities' in row and row['securities'] is not None:
-                for sec in row['securities']:
-                    import re
-                    sec = re.search(r'([^=]+)=(.*$)', sec)
-                    sec_lbls.append({
-                        'provider': sec.group(1),
-                        'label': sec.group(2)
-                    })
-            row['securities'] = sec_lbls
+            self._add_securities_to_row(row)
 
-        SQL = render_template(
-            "/".join([self.template_path, 'acl.sql']),
+        sql = render_template(
+            "/".join([self.template_path, self._ACL_SQL]),
             scid=scid, seid=seid
         )
-        status, dataclres = self.conn.execute_dict(SQL)
+        status, dataclres = self.conn.execute_dict(sql)
         if not status:
-            return internal_server_error(errormsg=res)
+            return False, internal_server_error(errormsg=res)
 
         for row in dataclres['rows']:
             priv = parse_priv_from_db(row)
@@ -311,10 +352,19 @@ class SequenceView(PGChildNodeView):
             else:
                 res['rows'][0][row['deftype']] = [priv]
 
-        return ajax_response(
-            response=res['rows'][0],
-            status=200
-        )
+        return True, res['rows'][0]
+
+    def _add_securities_to_row(self, row):
+        sec_lbls = []
+        if 'securities' in row and row['securities'] is not None:
+            for sec in row['securities']:
+                import re
+                sec = re.search(r'([^=]+)=(.*$)', sec)
+                sec_lbls.append({
+                    'provider': sec.group(1),
+                    'label': sec.group(2)
+                })
+        row['securities'] = sec_lbls
 
     @check_precondition(action="create")
     def create(self, gid, sid, did, scid):
@@ -331,9 +381,9 @@ class SequenceView(PGChildNodeView):
 
         """
         required_args = [
-            u'name',
-            u'schema',
-            u'seqowner',
+            'name',
+            'schema',
+            'seqowner',
         ]
 
         data = request.form if request.form else json.loads(
@@ -346,42 +396,47 @@ class SequenceView(PGChildNodeView):
                     status=400,
                     success=0,
                     errormsg=_(
-                        "Could not find the required parameter (%s)." % arg
-                    )
+                        "Could not find the required parameter ({})."
+                    ).format(arg)
                 )
-        # The SQL below will execute CREATE DDL only
-        SQL = render_template(
-            "/".join([self.template_path, 'create.sql']),
-            data=data, conn=self.conn
-        )
-        status, msg = self.conn.execute_scalar(SQL)
+
+        try:
+            # The SQL below will execute CREATE DDL only
+            sql = render_template(
+                "/".join([self.template_path, self._CREATE_SQL]),
+                data=data, conn=self.conn
+            )
+        except Exception as e:
+            return internal_server_error(errormsg=e)
+
+        status, msg = self.conn.execute_scalar(sql)
         if not status:
             return internal_server_error(errormsg=msg)
 
         if 'relacl' in data:
-            data['relacl'] = parse_priv_to_db(data['relacl'], 'DATABASE')
+            data['relacl'] = parse_priv_to_db(data['relacl'], self.acl)
 
         # The SQL below will execute rest DMLs because we cannot execute
         # CREATE with any other
-        SQL = render_template(
-            "/".join([self.template_path, 'grant.sql']),
+        sql = render_template(
+            "/".join([self.template_path, self._GRANT_SQL]),
             data=data, conn=self.conn
         )
-        SQL = SQL.strip('\n').strip(' ')
-        if SQL and SQL != "":
-            status, msg = self.conn.execute_scalar(SQL)
+        sql = sql.strip('\n').strip(' ')
+        if sql and sql != "":
+            status, msg = self.conn.execute_scalar(sql)
             if not status:
                 return internal_server_error(errormsg=msg)
 
         # We need oid of newly created sequence.
-        SQL = render_template(
-            "/".join([self.template_path, 'get_oid.sql']),
+        sql = render_template(
+            "/".join([self.template_path, self._OID_SQL]),
             name=data['name'],
             schema=data['schema']
         )
-        SQL = SQL.strip('\n').strip(' ')
+        sql = sql.strip('\n').strip(' ')
 
-        status, rset = self.conn.execute_2darray(SQL)
+        status, rset = self.conn.execute_2darray(sql)
         if not status:
             return internal_server_error(errormsg=rset)
 
@@ -391,12 +446,12 @@ class SequenceView(PGChildNodeView):
                 row['oid'],
                 row['relnamespace'],
                 data['name'],
-                icon="icon-%s" % self.node_type
+                icon=self.node_icon
             )
         )
 
     @check_precondition(action='delete')
-    def delete(self, gid, sid, did, scid, seid=None):
+    def delete(self, gid, sid, did, scid, seid=None, only_sql=False):
         """
         This function will drop the object
 
@@ -406,6 +461,7 @@ class SequenceView(PGChildNodeView):
           did: Database ID
           scid: Schema ID
           seid: Sequence ID
+          only_sql: Return SQL only if True
 
         Returns:
 
@@ -418,38 +474,37 @@ class SequenceView(PGChildNodeView):
             data = {'ids': [seid]}
 
         # Below will decide if it's simple drop or drop with cascade call
-        if self.cmd == 'delete':
-            # This is a cascade operation
-            cascade = True
-        else:
-            cascade = False
+
+        cascade = self._check_cascade_operation()
 
         try:
             for seid in data['ids']:
-                SQL = render_template(
-                    "/".join([self.template_path, 'properties.sql']),
+                sql = render_template(
+                    "/".join([self.template_path, self._PROPERTIES_SQL]),
                     scid=scid, seid=seid
                 )
-                status, res = self.conn.execute_dict(SQL)
+                status, res = self.conn.execute_dict(sql)
                 if not status:
                     return internal_server_error(errormsg=res)
 
-                if not res['rows']:
+                elif not res['rows']:
                     return make_json_response(
                         success=0,
                         errormsg=_(
                             'Error: Object not found.'
                         ),
-                        info=_(
-                            'The specified sequence could not be found.\n'
-                        )
+                        info=self.not_found_error_msg()
                     )
 
-                SQL = render_template(
-                    "/".join([self.template_path, 'delete.sql']),
+                sql = render_template(
+                    "/".join([self.template_path, self._DELETE_SQL]),
                     data=res['rows'][0], cascade=cascade
                 )
-                status, res = self.conn.execute_scalar(SQL)
+
+                if only_sql:
+                    return sql
+
+                status, res = self.conn.execute_scalar(sql)
                 if not status:
                     return internal_server_error(errormsg=res)
 
@@ -479,22 +534,22 @@ class SequenceView(PGChildNodeView):
         data = request.form if request.form else json.loads(
             request.data, encoding='utf-8'
         )
-        SQL, name = self.getSQL(gid, sid, did, data, scid, seid)
+        sql, name = self.get_SQL(gid, sid, did, data, scid, seid)
         # Most probably this is due to error
-        if not isinstance(SQL, (str, unicode)):
-            return SQL
+        if not isinstance(sql, str):
+            return sql
 
-        SQL = SQL.strip('\n').strip(' ')
+        sql = sql.strip('\n').strip(' ')
 
-        status, res = self.conn.execute_scalar(SQL)
+        status, res = self.conn.execute_scalar(sql)
         if not status:
             return internal_server_error(errormsg=res)
 
-        SQL = render_template(
-            "/".join([self.template_path, 'nodes.sql']),
+        sql = render_template(
+            "/".join([self.template_path, self._NODES_SQL]),
             seid=seid
         )
-        status, rset = self.conn.execute_2darray(SQL)
+        status, rset = self.conn.execute_2darray(sql)
         if not status:
             return internal_server_error(errormsg=rset)
         row = rset['rows'][0]
@@ -504,7 +559,7 @@ class SequenceView(PGChildNodeView):
                 seid,
                 row['schema'],
                 row['name'],
-                icon="icon-%s" % self.node_type
+                icon=self.node_icon
             )
         )
 
@@ -524,7 +579,12 @@ class SequenceView(PGChildNodeView):
         data = {}
         for k, v in request.args.items():
             try:
-                data[k] = json.loads(v, encoding='utf-8')
+                # comments should be taken as is because if user enters a
+                # json comment it is parsed by loads which should not happen
+                if k in ('comment',):
+                    data[k] = v
+                else:
+                    data[k] = json.loads(v, encoding='utf-8')
             except ValueError:
                 data[k] = v
 
@@ -540,24 +600,24 @@ class SequenceView(PGChildNodeView):
                         status=400,
                         success=0,
                         errormsg=_(
-                            "Could not find the required parameter (%s)." % arg
-                        )
+                            "Could not find the required parameter ({})."
+                        ).format(arg)
                     )
-        SQL, name = self.getSQL(gid, sid, did, data, scid, seid)
+        sql, name = self.get_SQL(gid, sid, did, data, scid, seid)
         # Most probably this is due to error
-        if not isinstance(SQL, (str, unicode)):
-            return SQL
+        if not isinstance(sql, str):
+            return sql
 
-        SQL = SQL.strip('\n').strip(' ')
-        if SQL == '':
-            SQL = "--modified SQL"
+        sql = sql.strip('\n').strip(' ')
+        if sql == '':
+            sql = "--modified SQL"
 
         return make_json_response(
-            data=SQL,
+            data=sql,
             status=200
         )
 
-    def getSQL(self, gid, sid, did, data, scid, seid=None):
+    def get_SQL(self, gid, sid, did, data, scid, seid=None):
         """
         This function will generate sql from model data.
 
@@ -570,67 +630,70 @@ class SequenceView(PGChildNodeView):
         """
 
         required_args = [
-            u'name'
+            'name'
         ]
 
         if seid is not None:
-            SQL = render_template(
-                "/".join([self.template_path, 'properties.sql']),
+            sql = render_template(
+                "/".join([self.template_path, self._PROPERTIES_SQL]),
                 scid=scid, seid=seid
             )
-            status, res = self.conn.execute_dict(SQL)
+            status, res = self.conn.execute_dict(sql)
             if not status:
                 return internal_server_error(errormsg=res)
-            if len(res['rows']) == 0:
-                return gone(_("Could not find the sequence in the database."))
+            elif len(res['rows']) == 0:
+                return gone(self.not_found_error_msg())
 
             # Making copy of output for further processing
             old_data = dict(res['rows'][0])
             old_data = self._formatter(old_data, scid, seid)
 
-            # To format privileges data coming from client
-            for key in ['relacl']:
-                if key in data and data[key] is not None:
-                    if 'added' in data[key]:
-                        data[key]['added'] = parse_priv_to_db(
-                            data[key]['added'], self.acl
-                        )
-                    if 'changed' in data[key]:
-                        data[key]['changed'] = parse_priv_to_db(
-                            data[key]['changed'], self.acl
-                        )
-                    if 'deleted' in data[key]:
-                        data[key]['deleted'] = parse_priv_to_db(
-                            data[key]['deleted'], self.acl
-                        )
+            self._format_privilege_data(data)
 
             # If name is not present with in update data then copy it
             # from old data
             for arg in required_args:
                 if arg not in data:
                     data[arg] = old_data[arg]
-            SQL = render_template(
-                "/".join([self.template_path, 'update.sql']),
+            sql = render_template(
+                "/".join([self.template_path, self._UPDATE_SQL]),
                 data=data, o_data=old_data, conn=self.conn
             )
-            return SQL, data['name'] if 'name' in data else old_data['name']
+            return sql, data['name'] if 'name' in data else old_data['name']
         else:
             # To format privileges coming from client
             if 'relacl' in data:
                 data['relacl'] = parse_priv_to_db(data['relacl'], self.acl)
 
-            SQL = render_template(
-                "/".join([self.template_path, 'create.sql']),
+            sql = render_template(
+                "/".join([self.template_path, self._CREATE_SQL]),
                 data=data, conn=self.conn
             )
-            SQL += render_template(
-                "/".join([self.template_path, 'grant.sql']),
+            sql += render_template(
+                "/".join([self.template_path, self._GRANT_SQL]),
                 data=data, conn=self.conn
             )
-            return SQL, data['name']
+            return sql, data['name']
+
+    def _format_privilege_data(self, data):
+        # To format privileges data coming from client
+        for key in ['relacl']:
+            if key in data and data[key] is not None:
+                if 'added' in data[key]:
+                    data[key]['added'] = parse_priv_to_db(
+                        data[key]['added'], self.acl
+                    )
+                if 'changed' in data[key]:
+                    data[key]['changed'] = parse_priv_to_db(
+                        data[key]['changed'], self.acl
+                    )
+                if 'deleted' in data[key]:
+                    data[key]['deleted'] = parse_priv_to_db(
+                        data[key]['deleted'], self.acl
+                    )
 
     @check_precondition(action="sql")
-    def sql(self, gid, sid, did, scid, seid):
+    def sql(self, gid, sid, did, scid, seid, **kwargs):
         """
         This function will generate sql for sql panel
 
@@ -640,24 +703,27 @@ class SequenceView(PGChildNodeView):
             did: Database ID
             scid: Schema ID
             seid: Sequence ID
+            json_resp: json response or plain text response
         """
+        json_resp = kwargs.get('json_resp', True)
+        target_schema = kwargs.get('target_schema', None)
 
-        SQL = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+        sql = render_template(
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             scid=scid, seid=seid
         )
-        status, res = self.conn.execute_dict(SQL)
+        status, res = self.conn.execute_dict(sql)
         if not status:
             return internal_server_error(errormsg=res)
         if len(res['rows']) == 0:
-            return gone(_("Could not find the sequence in the database."))
+            return gone(self.not_found_error_msg())
 
         for row in res['rows']:
-            SQL = render_template(
+            sql = render_template(
                 "/".join([self.template_path, 'get_def.sql']),
                 data=row
             )
-            status, rset1 = self.conn.execute_dict(SQL)
+            status, rset1 = self.conn.execute_dict(sql)
             if not status:
                 return internal_server_error(errormsg=rset1)
 
@@ -665,26 +731,35 @@ class SequenceView(PGChildNodeView):
             row['minimum'] = rset1['rows'][0]['min_value']
             row['maximum'] = rset1['rows'][0]['max_value']
             row['increment'] = rset1['rows'][0]['increment_by']
+            row['start'] = rset1['rows'][0]['start_value']
             row['cache'] = rset1['rows'][0]['cache_value']
             row['cycled'] = rset1['rows'][0]['is_cycled']
 
         result = res['rows'][0]
+        if target_schema:
+            result['schema'] = target_schema
+
         result = self._formatter(result, scid, seid)
-        SQL, name = self.getSQL(gid, sid, did, result, scid)
+        sql, name = self.get_SQL(gid, sid, did, result, scid)
         # Most probably this is due to error
-        if not isinstance(SQL, (str, unicode)):
-            return SQL
-        SQL = SQL.strip('\n').strip(' ')
+        if not isinstance(sql, str):
+            return sql
+        sql = sql.strip('\n').strip(' ')
 
-        sql_header = u"""-- SEQUENCE: {0}
+        # Return sql for schema diff
+        if not json_resp:
+            return sql
 
--- DROP SEQUENCE {0};
+        sql_header = """-- SEQUENCE: {0}.{1}\n\n""".format(
+            result['schema'], result['name'])
+
+        sql_header += """-- DROP SEQUENCE {0};
 
 """.format(self.qtIdent(self.conn, result['schema'], result['name']))
 
-        SQL = sql_header + SQL
+        sql = sql_header + sql
 
-        return ajax_response(response=SQL)
+        return ajax_response(response=sql)
 
     def _formatter(self, data, scid, seid):
         """
@@ -707,9 +782,9 @@ class SequenceView(PGChildNodeView):
             data['securities'] = seclabels
 
         # We need to parse & convert ACL coming from database to json format
-        SQL = render_template("/".join([self.template_path, 'acl.sql']),
+        sql = render_template("/".join([self.template_path, self._ACL_SQL]),
                               scid=scid, seid=seid)
-        status, acl = self.conn.execute_dict(SQL)
+        status, acl = self.conn.execute_dict(sql)
         if not status:
             return internal_server_error(errormsg=acl)
 
@@ -755,7 +830,15 @@ class SequenceView(PGChildNodeView):
             scid: Schema ID
             seid: Sequence ID
         """
-        dependencies_result = self.get_dependencies(self.conn, seid)
+
+        return ajax_response(
+            response=self._get_dependency(seid),
+            status=200
+        )
+
+    def _get_dependency(self, seid, show_system_objects=None):
+        dependencies_result = self.get_dependencies(self.conn, seid, None,
+                                                    show_system_objects)
 
         # Get missing dependencies.
         # A Corner case, reported by Guillaume Lelarge, could be found at:
@@ -785,11 +868,7 @@ class SequenceView(PGChildNodeView):
             dependencies_result.append({'type': 'column',
                                         'name': ref_name,
                                         'field': dep_type})
-
-        return ajax_response(
-            response=dependencies_result,
-            status=200
-        )
+        return dependencies_result
 
     @check_precondition(action="stats")
     def statistics(self, gid, sid, did, scid, seid=None):
@@ -813,7 +892,7 @@ class SequenceView(PGChildNodeView):
             # Get schema name
             status, schema_name = self.conn.execute_scalar(
                 render_template(
-                    'schema/pg/#{0}#/sql/get_name.sql'.format(
+                    'schemas/pg/#{0}#/sql/get_name.sql'.format(
                         self.manager.version
                     ),
                     scid=scid
@@ -838,5 +917,62 @@ class SequenceView(PGChildNodeView):
             status=200
         )
 
+    @check_precondition(action="fetch_objects_to_compare")
+    def fetch_objects_to_compare(self, sid, did, scid):
+        """
+        This function will fetch the list of all the sequences for
+        specified schema id.
 
+        :param sid: Server Id
+        :param did: Database Id
+        :param scid: Schema Id
+        :return:
+        """
+        res = dict()
+        sql = render_template("/".join([self.template_path,
+                                        self._NODES_SQL]), scid=scid)
+        status, rset = self.conn.execute_2darray(sql)
+        if not status:
+            return internal_server_error(errormsg=res)
+
+        for row in rset['rows']:
+            status, data = self._fetch_properties(scid, row['oid'])
+            if status:
+                res[row['name']] = data
+
+        return res
+
+    def get_sql_from_diff(self, **kwargs):
+        """
+        This function is used to get the DDL/DML statements.
+        :param kwargs
+        :return:
+        """
+        gid = kwargs.get('gid')
+        sid = kwargs.get('sid')
+        did = kwargs.get('did')
+        scid = kwargs.get('scid')
+        oid = kwargs.get('oid')
+        data = kwargs.get('data', None)
+        drop_sql = kwargs.get('drop_sql', False)
+        target_schema = kwargs.get('target_schema', None)
+
+        if data:
+            if target_schema:
+                data['schema'] = target_schema
+            sql, name = self.get_SQL(gid, sid, did, data, scid, oid)
+        else:
+            if drop_sql:
+                sql = self.delete(gid=gid, sid=sid, did=did,
+                                  scid=scid, seid=oid, only_sql=True)
+            elif target_schema:
+                sql = self.sql(gid=gid, sid=sid, did=did, scid=scid, seid=oid,
+                               target_schema=target_schema, json_resp=False)
+            else:
+                sql = self.sql(gid=gid, sid=sid, did=did, scid=scid, seid=oid,
+                               json_resp=False)
+        return sql
+
+
+SchemaDiffRegistry(blueprint.node_type, SequenceView)
 SequenceView.register_node_view(blueprint)
