@@ -2,19 +2,19 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
 
 define('pgadmin.node.sequence', [
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
-  'underscore.string', 'sources/pgadmin', 'pgadmin.browser', 'pgadmin.backform',
+  'sources/pgadmin', 'pgadmin.browser', 'pgadmin.backform',
   'pgadmin.node.schema.dir/child', 'pgadmin.node.schema.dir/schema_child_tree_node',
   'pgadmin.browser.collection',
 ], function(
-  gettext, url_for, $, _, S, pgAdmin, pgBrowser, Backform, schemaChild,
-   schemaChildTreeNode
+  gettext, url_for, $, _, pgAdmin, pgBrowser, Backform, schemaChild,
+  schemaChildTreeNode
 ) {
 
   // Extend the browser's collection class for sequence collection
@@ -80,6 +80,7 @@ define('pgadmin.node.sequence', [
           oid: undefined,
           seqowner: undefined,
           schema: undefined,
+          is_sys_obj: undefined,
           comment: undefined,
           increment: undefined,
           start: undefined,
@@ -128,6 +129,9 @@ define('pgadmin.node.sequence', [
             }
             return true;
           }, cache_node: 'database', cache_level: 'database',
+        },{
+          id: 'is_sys_obj', label: gettext('System sequence?'),
+          cell:'boolean', type: 'switch', mode: ['properties'],
         },{
           id: 'comment', label: gettext('Comment'), type: 'multiline',
           mode: ['properties', 'create', 'edit'],
@@ -137,13 +141,9 @@ define('pgadmin.node.sequence', [
         },{
           id: 'increment', label: gettext('Increment'), type: 'int',
           mode: ['properties', 'create', 'edit'], group: gettext('Definition'),
-          min: 1,
         },{
           id: 'start', label: gettext('Start'), type: 'int',
-          mode: ['properties', 'create'], group: gettext('Definition'),
-          disabled: function(m) {
-            return !m.isNew();
-          },
+          mode: ['properties', 'create', 'edit'], group: gettext('Definition'),
         },{
           id: 'minimum', label: gettext('Minimum'), type: 'int',
           mode: ['properties', 'create', 'edit'], group: gettext('Definition'),
@@ -157,13 +157,9 @@ define('pgadmin.node.sequence', [
         },{
           id: 'cycled', label: gettext('Cycled'), type: 'switch',
           mode: ['properties', 'create', 'edit'], group: gettext('Definition'),
-          options: {
-            'onText': gettext('Yes'), 'offText': gettext('No'),
-            'onColor': 'success', 'offColor': 'primary',
-          },
         }, pgBrowser.SecurityGroupSchema,{
           id: 'acl', label: gettext('Privileges'), type: 'text',
-          group: gettext('Security'), mode: ['properties'], disabled: true,
+          group: gettext('Security'), mode: ['properties'],
         },{
           id: 'relacl', label: gettext('Privileges'), group: 'security',
           model: pgBrowser.Node.PrivilegeRoleModel.extend({
@@ -172,7 +168,7 @@ define('pgadmin.node.sequence', [
           editable: false, type: 'collection', canAdd: true, canDelete: true,
           control: 'unique-col-collection',
         },{
-          id: 'securities', label: gettext('Security Labels'), canAdd: true,
+          id: 'securities', label: gettext('Security labels'), canAdd: true,
           model: pgBrowser.SecLabelModel, editable: false,
           type: 'collection', canEdit: false, group: 'security',
           mode: ['edit', 'create'], canDelete: true,
@@ -266,6 +262,9 @@ define('pgadmin.node.sequence', [
             start_lt = gettext('Start value cannot be less than minimum value.'),
             start_gt = gettext('Start value cannot be greater than maximum value.');
 
+          if (_.isEmpty(minimum) || _.isEmpty(maximum))
+            return null;
+
           if ((minimum == 0 && maximum == 0) ||
               (parseInt(minimum, 10) >= parseInt(maximum, 10))) {
             this.errorModel.set('minimum', min_lt);
