@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -25,7 +25,7 @@ class SequenceAddTestCase(BaseTestGenerator):
     scenarios = [
         # Fetching default URL for sequence node.
         (
-            'Fetch sequence Node URL (valid optional data)',
+            'Create sequence with positive values',
             dict(
                 url='/browser/sequence/obj/',
                 # Valid optional data
@@ -42,19 +42,19 @@ class SequenceAddTestCase(BaseTestGenerator):
             )
         ),
         (
-            'Fetch sequence Node URL (invalid optional data)',
+            'Create sequence with negative values',
             dict(
                 url='/browser/sequence/obj/',
-                # Optional fields should be int but we are passing empty str
+                # Valid optional data
                 data={
-                    "cache": "",
-                    "cycled": False,
-                    "increment": "",
-                    "maximum": "",
-                    "minimum": "",
+                    "cache": "1",
+                    "cycled": True,
+                    "increment": "-5",
+                    "maximum": "-10",
+                    "minimum": "-40",
                     "name": "test_sequence_add_%s" % (str(uuid.uuid4())[1:8]),
                     "securities": [],
-                    "start": ""
+                    "start": "-30"
                 }
             )
         )
@@ -119,7 +119,7 @@ class SequenceAddTestCase(BaseTestGenerator):
             '/' + str(schema_id) + '/',
             data=json.dumps(self.data),
             content_type='html/json')
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
 
     def tearDown(self):
         # Disconnect the database
