@@ -2,12 +2,13 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
 import uuid
+from unittest.mock import patch
 
 from pgadmin.browser.server_groups.servers.databases.schemas.tables.tests \
     import utils as tables_utils
@@ -23,11 +24,17 @@ from . import utils as columns_utils
 
 class ColumnGetTestCase(BaseTestGenerator):
     """This class will get column under table node."""
-    scenarios = [
-        ('Fetch columns under table node', dict(url='/browser/column/obj/'))
-    ]
+    url = '/browser/column/obj/'
+
+    # Generates scenarios
+    scenarios = utils.generate_scenarios("column_get",
+                                         columns_utils.test_cases)
 
     def setUp(self):
+        # Load test data
+        self.data = self.test_data
+
+        # Create db connection
         self.db_name = parent_node_dict["database"][-1]["db_name"]
         schema_info = parent_node_dict["schema"][-1]
         self.server_id = schema_info["server_id"]
@@ -36,6 +43,8 @@ class ColumnGetTestCase(BaseTestGenerator):
                                                  self.server_id, self.db_id)
         if not db_con['data']["connected"]:
             raise Exception("Could not connect to database to add a table.")
+
+        # Create schema
         self.schema_id = schema_info["schema_id"]
         self.schema_name = schema_info["schema_name"]
         schema_response = schema_utils.verify_schemas(self.server,
@@ -43,27 +52,56 @@ class ColumnGetTestCase(BaseTestGenerator):
                                                       self.schema_name)
         if not schema_response:
             raise Exception("Could not find the schema to add a table.")
+
+        # Create table
         self.table_name = "table_column_%s" % (str(uuid.uuid4())[1:8])
         self.table_id = tables_utils.create_table(self.server, self.db_name,
                                                   self.schema_name,
                                                   self.table_name)
+
+        # Create column
         self.column_name = "test_column_delete_%s" % (str(uuid.uuid4())[1:8])
         self.column_id = columns_utils.create_column(self.server,
                                                      self.db_name,
                                                      self.schema_name,
                                                      self.table_name,
                                                      self.column_name)
+        if self.is_list:
+            # Create column
+            self.column_name_1 = "test_column_delete_%s" % \
+                                 (str(uuid.uuid4())[1:8])
+            self.column_id_1 = columns_utils.create_column(self.server,
+                                                           self.db_name,
+                                                           self.schema_name,
+                                                           self.table_name,
+                                                           self.column_name_1)
 
     def runTest(self):
         """This function will fetch the column under table node."""
-        response = self.tester.get(self.url + str(utils.SERVER_GROUP) +
-                                   '/' + str(self.server_id) + '/' +
-                                   str(self.db_id) + '/' +
-                                   str(self.schema_id) + '/' +
-                                   str(self.table_id) + '/' +
-                                   str(self.column_id),
-                                   follow_redirects=True)
-        self.assertEquals(response.status_code, 200)
+        if self.is_positive_test:
+            if self.is_list:
+                response = columns_utils.api_get(self, "")
+            else:
+                response = columns_utils.api_get(self)
+
+            # Assert response
+            utils.assert_status_code(self, response)
+        else:
+            if self.mocking_required:
+                with patch(self.mock_data["function_name"],
+                           side_effect=eval(self.mock_data["return_value"])):
+                    if self.is_list:
+                        response = columns_utils.api_get(self, "")
+                    else:
+                        response = columns_utils.api_get(self)
+            else:
+                if 'column_id' in self.data:
+                    self.column_id = self.data['column_id']
+                response = columns_utils.api_get(self)
+
+            # Assert response
+            utils.assert_status_code(self, response)
+            utils.assert_error_message(self, response)
 
     def tearDown(self):
         # Disconnect the database
