@@ -2,19 +2,19 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
 
 define('pgadmin.node.compound_trigger', [
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
-  'underscore.string', 'sources/pgadmin', 'pgadmin.browser',
+  'sources/pgadmin', 'pgadmin.browser',
   'pgadmin.backform', 'pgadmin.alertifyjs',
   'pgadmin.node.schema.dir/schema_child_tree_node',
   'pgadmin.browser.collection',
 ], function(
-  gettext, url_for, $, _, S, pgAdmin, pgBrowser, Backform, alertify,
+  gettext, url_for, $, _, pgAdmin, pgBrowser, Backform, alertify,
   SchemaChildTreeNode
 ) {
 
@@ -42,6 +42,7 @@ define('pgadmin.node.compound_trigger', [
       sqlAlterHelp: 'sql-altertcompoundtrigger.html',
       sqlCreateHelp: 'sql-createcompoundtrigger.html',
       dialogHelp: url_for('help.static', {'filename': 'compound_trigger_dialog.html'}),
+      url_jump_after_node: 'schema',
       Init: function() {
         /* Avoid mulitple registration of menus */
         if (this.initialized)
@@ -108,7 +109,7 @@ define('pgadmin.node.compound_trigger', [
           $.ajax({
             url: obj.generate_url(i, 'enable' , d, true),
             type:'PUT',
-            data: {'enable' : true},
+            data: {'is_enable_trigger' : 'O'},
             dataType: 'json',
           })
             .done(function(res) {
@@ -146,7 +147,7 @@ define('pgadmin.node.compound_trigger', [
           $.ajax({
             url: obj.generate_url(i, 'enable' , d, true),
             type:'PUT',
-            data: {'enable' : false},
+            data: {'is_enable_trigger' : 'D'},
             dataType: 'json',
           })
             .done(function(res) {
@@ -182,16 +183,23 @@ define('pgadmin.node.compound_trigger', [
           type: 'text', disabled: 'inSchema',
         },{
           id: 'oid', label: gettext('OID'), cell: 'string',
-          type: 'int', disabled: true, mode: ['properties'],
+          type: 'int', mode: ['properties'],
         },{
           id: 'is_enable_trigger', label: gettext('Trigger enabled?'),
-          type: 'switch', mode: ['edit', 'properties'],
+          mode: ['edit', 'properties'],
           disabled: function() {
             if(this.node_info && ('catalog' in this.node_info || 'view' in this.node_info)) {
               return true;
             }
             return false;
           },
+          options: [
+            {label: gettext('Enable'), value: 'O'},
+            {label: gettext('Enable Replica'), value: 'R'},
+            {label: gettext('Enable Always'), value: 'A'},
+            {label: gettext('Disable'), value: 'D'},
+          ],
+          control: 'select2', select2: { allowClear: false, width: '100%' },
         },{
           type: 'nested', control: 'fieldset', mode: ['create','edit', 'properties'],
           label: gettext('FOR Events'), group: gettext('Events'), contentClass: 'row',
@@ -206,8 +214,9 @@ define('pgadmin.node.compound_trigger', [
               var evn_insert = m.get('evnt_insert');
               if (!_.isUndefined(evn_insert) && m.node_info['server']['server_type'] == 'ppas')
                 return false;
-              return m.inSchemaWithModelCheck.apply(this, [m]);
+              return m.inSchema.apply(this, [m]);
             },
+            readonly: 'inEditMode',
           },{
             id: 'evnt_update', label: gettext('UPDATE'),
             type: 'switch', mode: ['create','edit', 'properties'],
@@ -219,8 +228,9 @@ define('pgadmin.node.compound_trigger', [
               var evn_update = m.get('evnt_update');
               if (!_.isUndefined(evn_update) && m.node_info['server']['server_type'] == 'ppas')
                 return false;
-              return m.inSchemaWithModelCheck.apply(this, [m]);
+              return m.inSchema.apply(this, [m]);
             },
+            readonly: 'inEditMode',
           },{
             id: 'evnt_delete', label: gettext('DELETE'),
             type: 'switch', mode: ['create','edit', 'properties'],
@@ -232,8 +242,9 @@ define('pgadmin.node.compound_trigger', [
               var evn_delete = m.get('evnt_delete');
               if (!_.isUndefined(evn_delete) && m.node_info['server']['server_type'] == 'ppas')
                 return false;
-              return m.inSchemaWithModelCheck.apply(this, [m]);
+              return m.inSchema.apply(this, [m]);
             },
+            readonly: 'inEditMode',
           },{
             id: 'evnt_truncate', label: gettext('TRUNCATE'),
             type: 'switch', mode: ['create','edit', 'properties'],
@@ -249,12 +260,13 @@ define('pgadmin.node.compound_trigger', [
 
               if (!_.isUndefined(evn_truncate) && m.node_info['server']['server_type'] == 'ppas')
                 return false;
-              return m.inSchemaWithModelCheck.apply(this, [m]);
+              return m.inSchema.apply(this, [m]);
             },
           }],
+          readonly: 'inEditMode',
         },{
           id: 'whenclause', label: gettext('When'),
-          type: 'text', disabled: 'inSchemaWithModelCheck',
+          type: 'text', disabled: 'inSchema', readonly: 'inEditMode',
           mode: ['create', 'edit', 'properties'],
           control: 'sql-field', visible: true, group: gettext('Events'),
         },{
@@ -266,10 +278,6 @@ define('pgadmin.node.compound_trigger', [
             if(this.node_info &&  'catalog' in this.node_info) {
               return true;
             }
-            //Disable in edit mode
-            if (!m.isNew()) {
-              return true;
-            }
             // Enable column only if update event is set true
             var isUpdate = m.get('evnt_update');
             if(!_.isUndefined(isUpdate) && isUpdate) {
@@ -277,6 +285,7 @@ define('pgadmin.node.compound_trigger', [
             }
             return true;
           },
+          readonly: 'inEditMode',
         },{
           id: 'prosrc', label: gettext('Code'), group: gettext('Code'),
           type: 'text', mode: ['create', 'edit'],
@@ -293,7 +302,8 @@ define('pgadmin.node.compound_trigger', [
           },
         },{
           id: 'is_sys_trigger', label: gettext('System trigger?'), cell: 'string',
-          type: 'switch', disabled: 'inSchemaWithModelCheck', mode: ['properties'],
+          type: 'switch', disabled: 'inSchema', mode: ['properties'],
+          readonly: 'inEditMode',
         },{
           id: 'description', label: gettext('Comment'), cell: 'string',
           type: 'multiline', mode: ['properties', 'create', 'edit'],
@@ -375,17 +385,8 @@ define('pgadmin.node.compound_trigger', [
           }
           return false;
         },
-        // We will check if we are under schema node & in 'create' mode
-        inSchemaWithModelCheck: function(m) {
-          if(this.node_info &&  'schema' in this.node_info) {
-            // We will disable control if it's in 'edit' mode
-            if (m.isNew()) {
-              return false;
-            } else {
-              return true;
-            }
-          }
-          return true;
+        inEditMode: function(m) {
+          return !m.isNew();
         },
         // Checks weather to enable/disable control
         inSchemaWithColumnCheck: function(m) {
@@ -396,11 +397,7 @@ define('pgadmin.node.compound_trigger', [
               return false;
             } else {
               // if we are in edit mode
-              if (!_.isUndefined(m.get('attnum')) && m.get('attnum') >= 1 ) {
-                return false;
-              } else {
-                return true;
-              }
+              return !(!_.isUndefined(m.get('attnum')) && m.get('attnum') >= 1 );
             }
           }
           return true;
