@@ -2,7 +2,7 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
@@ -19,6 +19,7 @@ define('pgadmin.node.exclusion_constraint', [
   var ExclusionConstraintColumnModel = pgBrowser.Node.Model.extend({
     defaults: {
       column: undefined,
+      is_exp: false,
       oper_class: undefined,
       order: false,
       nulls_order: false,
@@ -32,8 +33,20 @@ define('pgadmin.node.exclusion_constraint', [
       return d;
     },
     schema: [{
-      id: 'column', label: gettext('Column'), type:'text', editable: false,
+      id: 'column', label: gettext('Col/Exp'), type:'text', editable: false,
       cell:'string',
+    },{
+      id: 'is_exp', label: '', type:'boolean', editable: false,
+      cell: Backgrid.StringCell.extend({
+        formatter: {
+          fromRaw: function (rawValue) {
+            return rawValue ? 'E' : 'C';
+          },
+          toRaw: function (val) {
+            return val;
+          },
+        },
+      }), visible: false,
     },{
       id: 'oper_class', label: gettext('Operator class'), type:'text',
       node: 'table', url: 'get_oper_class', first_empty: true,
@@ -175,7 +188,7 @@ define('pgadmin.node.exclusion_constraint', [
 
           self.column.set('options', []);
 
-          if (url && !_.isUndefined(col_type) && !_.isNull(col_type) && col_type != '') {
+          if (url) {
             var node = this.column.get('schema_node'),
               eventHandler = m.top || m,
               node_info = this.column.get('node_info'),
@@ -210,9 +223,11 @@ define('pgadmin.node.exclusion_constraint', [
     validate: function() {
       this.errorModel.clear();
       var operator = this.get('operator'),
-        column_name = this.get('column');
+        column_name = this.get('column'),
+        is_exp = this.get('is_exp');
       if (_.isUndefined(operator) || _.isNull(operator)) {
         var msg = gettext('Please specify operator for column: ') + column_name;
+        if(is_exp) msg = gettext('Please specify operator for expression: ') + column_name;
         this.errorModel.set('operator', msg);
         return msg;
       }
@@ -231,8 +246,15 @@ define('pgadmin.node.exclusion_constraint', [
         var self = this,
           node = 'exclusion_constraint',
           headerSchema = [{
-            id: 'column', label:'', type:'text',
-            node: 'column', control: Backform.NodeListByNameControl.extend({
+            id: 'is_exp', label: gettext('Is expression ?'), type: 'switch',
+            control: 'switch', controlLabelClassName: 'control-label pg-el-sm-4 pg-el-12',
+            controlsClassName: 'pgadmin-controls pg-el-sm-6 pg-el-12',
+          },{
+            id: 'column', label: gettext('Column'), type:'text',
+            controlLabelClassName: 'control-label pg-el-sm-4 pg-el-12',
+            controlsClassName: 'pgadmin-controls pg-el-sm-6 pg-el-12',
+            node: 'column', deps: ['is_exp'],
+            control: Backform.NodeListByNameControl.extend({
               initialize: function() {
                 // Here we will decide if we need to call URL
                 // Or fetch the data from parent columns collection
@@ -310,7 +332,8 @@ define('pgadmin.node.exclusion_constraint', [
                 }
               },
               template: _.template([
-                '<div class="<%=Backform.controlsClassName%> <%=extraClasses.join(\' \')%>">',
+                '<span class="<%=controlLabelClassName%>"><%=label%></span>',
+                '<div class="<%=controlsClassName%> <%=extraClasses.join(\' \')%>">',
                 '  <select class="pgadmin-node-select form-control" name="<%=name%>" style="width:100%;" value="<%-value%>" <%=disabled ? "disabled" : ""%> <%=required ? "required" : ""%> >',
                 '    <% for (var i=0; i < options.length; i++) { %>',
                 '    <% var option = options[i]; %>',
@@ -323,7 +346,7 @@ define('pgadmin.node.exclusion_constraint', [
               // This will only get called in case of NodeListByNameControl.
 
               var that = this,
-                node = that.field.get('schema_node'),
+                schema_node = that.field.get('schema_node'),
                 res = [],
                 col_types = [],
                 filter = that.field.get('filter') || function() { return true; };
@@ -332,14 +355,14 @@ define('pgadmin.node.exclusion_constraint', [
 
               _.each(rows, function(r) {
                 if (filter(r)) {
-                  var l = (_.isFunction(node['node_label']) ?
-                      (node['node_label']).apply(node, [r, that.model, that]) :
+                  var l = (_.isFunction(schema_node['node_label']) ?
+                      (schema_node['node_label']).apply(schema_node, [r, that.model, that]) :
                       r.label),
-                    image = (_.isFunction(node['node_image']) ?
-                      (node['node_image']).apply(
-                        node, [r, that.model, that]
+                    image = (_.isFunction(schema_node['node_image']) ?
+                      (schema_node['node_image']).apply(
+                        schema_node, [r, that.model, that]
                       ) :
-                      (node['node_image'] || ('icon-' + node.type)));
+                      (schema_node['node_image'] || ('icon-' + schema_node.type)));
                   res.push({
                     'value': r.label,
                     'image': image,
@@ -358,15 +381,26 @@ define('pgadmin.node.exclusion_constraint', [
             },
             select2: {
               allowClear: false, width: 'style',
-              placeholder: 'Select column',
+              placeholder: gettext('Select column'),
             }, first_empty: !self.model.isNew(),
-            disabled: function() {
+            readonly: function() {
               return !_.isUndefined(self.model.get('oid'));
             },
+            disabled: function(m) {
+              return m.get('is_exp');
+            },
+          },{
+            id: 'exp', label: gettext('Expression'), type: 'text',
+            editable: true, deps: ['is_exp'],
+            controlLabelClassName: 'control-label pg-el-sm-4 pg-el-12',
+            controlsClassName: 'pgadmin-controls pg-el-sm-6 pg-el-12',
+            disabled: function(m) {
+              return !m.get('is_exp');
+            },
           }],
-          headerDefaults = {column: null},
+          headerDefaults = {is_exp: false, column: null, exp: null},
 
-          gridCols = ['column', 'oper_class', 'order', 'nulls_order', 'operator'];
+          gridCols = ['is_exp', 'column', 'oper_class', 'order', 'nulls_order', 'operator'];
 
         self.headerData = new (Backbone.Model.extend({
           defaults: headerDefaults,
@@ -396,22 +430,16 @@ define('pgadmin.node.exclusion_constraint', [
       },
 
       generateHeader: function(data) {
+        var isNew = _.isUndefined(this.model.get('oid'));
         var header = [
-          '<div class="subnode-header-form">',
+          '<div class="subnode-header-form '+ (isNew ? '' : 'd-none') +'">',
           ' <div>',
-          '  <div class="row">',
-          '   <div class="col-4">',
-          '    <label class="control-label"><%-column_label%></label>',
-          '   </div>',
-          '   <div class="col-4" header="column"></div>',
-          '  </div>',
+          '  <div header="is_exp"></div>',
+          '  <div header="column"></div>',
+          '  <div header="exp"></div>',
           ' </div>',
           '</div>'].join('\n');
 
-        _.extend(data, {
-          column_label: gettext('Column'),
-        });
-
         var self = this,
           headerTmpl = _.template(header),
           $header = $(headerTmpl(data)),
@@ -450,7 +478,7 @@ define('pgadmin.node.exclusion_constraint', [
           titleTmpl = _.template([
             '<div class="subnode-header">',
             '  <label class="control-label pg-el-sm-10"><%-label%></label>',
-            '  <button class="btn btn-sm-sq btn-secondary add fa fa-plus" <%=canAdd ? "" : "disabled=\'disabled\'"%> title="' + _('Add new row') + '"></button>',
+            '  <button class="btn btn-sm-sq btn-primary-icon add fa fa-plus" <%=canAdd ? "" : "disabled=\'disabled\'"%> title="' + gettext('Add new row') + '"></button>',
             '</div>'].join('\n')),
           $gridBody =
           $('<div class=\'pgadmin-control-group backgrid form-group col-12 object subnode\'></div>').append(
@@ -473,7 +501,7 @@ define('pgadmin.node.exclusion_constraint', [
 
         if (self.grid) {
           self.grid.remove();
-          self.grid.null;
+          self.grid = null;
         }
         // Initialize a new Grid instance
         var grid = self.grid = new Backgrid.Grid({
@@ -514,20 +542,24 @@ define('pgadmin.node.exclusion_constraint', [
         }
 
         if (self.control_data.canAdd) {
-          self.collection.each(function(m) {
-            if (!inSelected) {
-              _.each(checkVars, function(v) {
-                if (!inSelected) {
-                  val = m.get(v);
-                  inSelected = ((
-                    (_.isUndefined(val) || _.isNull(val)) &&
-                      (_.isUndefined(data[v]) || _.isNull(data[v]))
-                  ) ||
-                      (val == data[v]));
-                }
-              });
-            }
-          });
+          if(data['is_exp']) {
+            inSelected = false;
+          } else {
+            self.collection.each(function(m) {
+              if (!inSelected) {
+                _.each(checkVars, function(v) {
+                  if (!inSelected) {
+                    val = m.get(v);
+                    inSelected = ((
+                      (_.isUndefined(val) || _.isNull(val)) &&
+                        (_.isUndefined(data[v]) || _.isNull(data[v]))
+                    ) ||
+                        (val == data[v]));
+                  }
+                });
+              }
+            });
+          }
         }
         else {
           inSelected = true;
@@ -538,44 +570,47 @@ define('pgadmin.node.exclusion_constraint', [
 
       addColumns: function(ev) {
         ev.preventDefault();
-        var self = this,
-          column = self.headerData.get('column');
+        var self = this;
+        let newHeaderData = {
+          is_exp: self.headerData.get('is_exp'),
+          column: self.headerData.get('is_exp') ? self.headerData.get('exp') : self.headerData.get('column'),
+        };
 
-        if (!column || column == '') {
-          return false;
-        }
+        if (newHeaderData.column && newHeaderData.column != '') {
+          var coll = self.model.get(self.field.get('name')),
+            m = new (self.field.get('model'))(
+              newHeaderData, {
+                silent: true, top: self.model.top,
+                collection: coll, handler: coll,
+              }),
+            col_types =self.field.get('col_types') || [];
 
-        var coll = self.model.get(self.field.get('name')),
-          m = new (self.field.get('model'))(
-            self.headerData.toJSON(), {
-              silent: true, top: self.model.top,
-              collection: coll, handler: coll,
-            }),
-          col_types =self.field.get('col_types') || [];
-
-        for(var i=0; i < col_types.length; i++) {
-          var col_type = col_types[i];
-          if (col_type['name'] ==  m.get('column')) {
-            m.set({'col_type':col_type['type']});
-            break;
+          if(!m.get('is_exp')) {
+            for(var i=0; i < col_types.length; i++) {
+              var col_type = col_types[i];
+              if (col_type['name'] ==  m.get('column')) {
+                m.set({'col_type':col_type['type']});
+                break;
+              }
+            }
           }
-        }
 
-        coll.add(m);
+          coll.add(m);
 
-        var idx = coll.indexOf(m);
+          var idx = coll.indexOf(m);
 
-        // idx may not be always > -1 because our UniqueColCollection may
-        // remove 'm' if duplicate value found.
-        if (idx > -1) {
-          self.$grid.find('.new').removeClass('new');
+          // idx may not be always > -1 because our UniqueColCollection may
+          // remove 'm' if duplicate value found.
+          if (idx > -1) {
+            self.$grid.find('.new').removeClass('new');
 
-          var newRow = self.grid.body.rows[idx].$el;
+            var newRow = self.grid.body.rows[idx].$el;
 
-          newRow.addClass('new');
-          $(newRow).pgMakeVisible('backform-tab');
-        } else {
-          //delete m;
+            newRow.addClass('new');
+            $(newRow).pgMakeVisible('backform-tab');
+          } else {
+            //delete m;
+          }
         }
 
         return false;
@@ -629,6 +664,7 @@ define('pgadmin.node.exclusion_constraint', [
       hasDepends: true,
       hasStatistics: true,
       statsPrettifyFields: [gettext('Index size')],
+      url_jump_after_node: 'schema',
       Init: function() {
         /* Avoid multiple registration of menus */
         if (this.initialized)
@@ -654,6 +690,7 @@ define('pgadmin.node.exclusion_constraint', [
         defaults: {
           name: undefined,
           oid: undefined,
+          is_sys_obj: undefined,
           comment: undefined,
           spcname: undefined,
           amname: 'gist',
@@ -671,6 +708,9 @@ define('pgadmin.node.exclusion_constraint', [
         },{
           id: 'oid', label: gettext('OID'), cell: 'string',
           type: 'text' , mode: ['properties'],
+        },{
+          id: 'is_sys_obj', label: gettext('System exclusion constraint?'),
+          cell:'boolean', type: 'switch', mode: ['properties'],
         },{
           id: 'comment', label: gettext('Comment'), cell: 'string',
           type: 'multiline', mode: ['properties', 'create', 'edit'],
@@ -693,8 +733,7 @@ define('pgadmin.node.exclusion_constraint', [
           select2:{allowClear:false},
           filter: function(m) {
             // Don't show pg_global tablespace in selection.
-            if (m.label == 'pg_global') return false;
-            else return true;
+            return (m.label !== 'pg_global');
           },
         },{
           id: 'amname', label: gettext('Access method'),
@@ -736,7 +775,7 @@ define('pgadmin.node.exclusion_constraint', [
             },
           }),
           select2:{allowClear:true},
-          disabled: function(m) {
+          readonly: function(m) {
             return ((_.has(m, 'handler') &&
               !_.isUndefined(m.handler) &&
                 !_.isUndefined(m.get('oid'))) || (_.isFunction(m.isNew) && !m.isNew()));
@@ -747,7 +786,7 @@ define('pgadmin.node.exclusion_constraint', [
         },{
           id: 'condeferrable', label: gettext('Deferrable?'),
           type: 'switch', group: gettext('Definition'), deps: ['index'],
-          disabled: function(m) {
+          readonly: function(m) {
             return ((_.has(m, 'handler') &&
               !_.isUndefined(m.handler) &&
                 !_.isUndefined(m.get('oid'))) || (_.isFunction(m.isNew) && !m.isNew()));
@@ -757,12 +796,6 @@ define('pgadmin.node.exclusion_constraint', [
           type: 'switch', group: gettext('Definition'),
           deps: ['condeferrable'],
           disabled: function(m) {
-            if((_.has(m, 'handler') &&
-              !_.isUndefined(m.handler) &&
-                !_.isUndefined(m.get('oid'))) || (_.isFunction(m.isNew) && !m.isNew())) {
-              return true;
-            }
-
             // Disable if condeferred is false or unselected.
             if(m.get('condeferrable') == true) {
               return false;
@@ -774,16 +807,23 @@ define('pgadmin.node.exclusion_constraint', [
               return true;
             }
           },
+          readonly: function(m) {
+            if((_.has(m, 'handler') &&
+              !_.isUndefined(m.handler) &&
+                !_.isUndefined(m.get('oid'))) || (_.isFunction(m.isNew) && !m.isNew())) {
+              return true;
+            }
+          },
         },{
           id: 'indconstraint', label: gettext('Constraint'), cell: 'string',
           type: 'multiline', mode: ['create', 'edit', 'properties'], editable: false,
-          group: gettext('Definition'), disabled: function(m) {
+          group: gettext('Definition'), readonly: function(m) {
             return ((_.has(m, 'handler') &&
               !_.isUndefined(m.handler) &&
                 !_.isUndefined(m.get('oid'))) || (_.isFunction(m.isNew) && !m.isNew()));
           },
         },{
-          id: 'columns', label: gettext('Columns'),
+          id: 'columns', label: gettext('Columns/Expressions'),
           type: 'collection', group: gettext('Columns'),
           deps:['amname'], canDelete: true, editable: false,
           canAdd: function(m) {
@@ -794,7 +834,7 @@ define('pgadmin.node.exclusion_constraint', [
           },
           control: ExclusionConstraintColumnControl,
           model: ExclusionConstraintColumnModel,
-          disabled: function(m) {
+          readonly: function(m) {
             return ((_.has(m, 'handler') &&
               !_.isUndefined(m.handler) &&
                 !_.isUndefined(m.get('oid'))) || (_.isFunction(m.isNew) && !m.isNew()));
@@ -978,18 +1018,6 @@ define('pgadmin.node.exclusion_constraint', [
           }),
           deps: ['index'], node: 'column',
           disabled: function(m) {
-            // If we are in table edit mode then
-            if (_.has(m, 'top') && !_.isUndefined(m.top)
-              && !m.top.isNew()) {
-              // If OID is undefined then user is trying to add
-              // new constraint which should be allowed for Unique
-              return !_.isUndefined(m.get('oid'));
-            }
-
-            // We can't update columns of existing index constraint.
-            if (!m.isNew()) {
-              return true;
-            }
             // Disable if index is selected.
             var index = m.get('index');
             if(_.isUndefined(index) || index == '') {
@@ -1000,6 +1028,18 @@ define('pgadmin.node.exclusion_constraint', [
               return true;
             }
           },
+          readonly: function(m) {
+            // If we are in table edit mode then
+            if (_.has(m, 'top') && !_.isUndefined(m.top)
+              && !m.top.isNew()) {
+              // If OID is undefined then user is trying to add
+              // new constraint which should be allowed for Unique
+              return !_.isUndefined(m.get('oid'));
+            }
+
+            // We can't update columns of existing index constraint.
+            return !m.isNew();
+          },
         }],
         validate: function() {
           this.errorModel.clear();
