@@ -2,14 +2,15 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
 import json
 import uuid
-
+from unittest.mock import patch
+from pgadmin.utils import server_utils as server_utils
 from pgadmin.browser.server_groups.servers.databases.schemas.tables.tests \
     import utils as tables_utils
 from pgadmin.browser.server_groups.servers.databases.schemas.tests import \
@@ -24,20 +25,36 @@ from . import utils as exclusion_utils
 
 class ExclusionConstraintAddTestCase(BaseTestGenerator):
     """This class will add new exclusion constraint to existing table"""
-    scenarios = [
-        ('Add Exclusion Constraint URL',
-         dict(url='/browser/exclusion_constraint/obj/'))
-    ]
+    url = '/browser/exclusion_constraint/obj/'
+
+    # Generates scenarios from cast_test_data.json file
+    scenarios = utils.generate_scenarios("exclusion_constraint_create",
+                                         exclusion_utils.test_cases)
 
     def setUp(self):
+        # Load test data
+        self.data = self.test_data
+
+        # Create db connection
         self.db_name = parent_node_dict["database"][-1]["db_name"]
         schema_info = parent_node_dict["schema"][-1]
         self.server_id = schema_info["server_id"]
         self.db_id = schema_info["db_id"]
+
+        # Check DB version
+        server_con = server_utils.connect_server(self, self.server_id)
+        if not server_con["info"] == "Server connected.":
+            raise Exception("Could not connect to server to add "
+                            "a table.")
+        self.db_version = server_con["data"]["version"]
+
+        # Create db connection
         db_con = database_utils.connect_database(self, utils.SERVER_GROUP,
                                                  self.server_id, self.db_id)
         if not db_con['data']["connected"]:
             raise Exception("Could not connect to database to add a table.")
+
+        # Create schema
         self.schema_id = schema_info["schema_id"]
         self.schema_name = schema_info["schema_name"]
         schema_response = schema_utils.verify_schemas(self.server,
@@ -45,6 +62,8 @@ class ExclusionConstraintAddTestCase(BaseTestGenerator):
                                                       self.schema_name)
         if not schema_response:
             raise Exception("Could not find the schema to add a table.")
+
+        # Create table
         self.table_name = "table_for_exclusion_%s" % (str(uuid.uuid4())[1:8])
         self.table_id = tables_utils.create_table(self.server, self.db_name,
                                                   self.schema_name,
@@ -52,28 +71,39 @@ class ExclusionConstraintAddTestCase(BaseTestGenerator):
 
     def runTest(self):
         """This function will add exclusion constraint to existing table."""
-        self.index_name = "test_index_add_%s" % (str(uuid.uuid4())[1:8])
-        data = {"name": self.index_name,
-                "spcname": "pg_default",
-                "amname": "btree",
-                "columns": [
-                    {"column": "id", "sort_order": False, "nulls": False,
-                     "operator": "="}],
-                "include": ["name"]
-                }
-        response = self.tester.post(
-            self.url + str(utils.SERVER_GROUP) + '/' +
-            str(self.server_id) + '/' + str(self.db_id) +
-            '/' + str(self.schema_id) + '/' + str(self.table_id) + '/',
-            data=json.dumps(data),
-            content_type='html/json')
-        self.assertEquals(response.status_code, 200)
+        if "name" in self.data:
+            constraint_name = self.data["name"] + (str(uuid.uuid4())[1:8])
+            self.data["name"] = constraint_name
+        elif self.db_version < 110000:
+            constraint_name = self.table_name + '_' + \
+                self.data["columns"][0]['column'] + '_excl'
+        else:
+            constraint_name = self.table_name + '_' + \
+                self.data["columns"][0]['column'] + '_name_excl'
 
-        index_response = exclusion_utils.verify_exclusion_constraint(
-            self.server, self.db_name, self.index_name)
+        if self.is_positive_test:
+            response = exclusion_utils.api_create(self)
 
-        if not index_response:
-            raise Exception("Could not find the constraint added.")
+            # Assert response
+            utils.assert_status_code(self, response)
+
+            # Verify in backend
+            cross_check_res = exclusion_utils.verify_exclusion_constraint(
+                self.server, self.db_name, constraint_name)
+            self.assertIsNotNone(cross_check_res,
+                                 "Could not find the newly created exclusion "
+                                 "constraint.")
+        else:
+            if self.mocking_required:
+                with patch(self.mock_data["function_name"],
+                           side_effect=eval(self.mock_data["return_value"])):
+                    response = exclusion_utils.api_create(self)
+            else:
+                response = exclusion_utils.api_create(self)
+
+            # Assert response
+            utils.assert_status_code(self, response)
+            utils.assert_error_message(self, response)
 
     def tearDown(self):
         # Disconnect the database
