@@ -2,7 +2,7 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
@@ -25,10 +25,11 @@ define('pgadmin.node.primary_key', [
       hasSQL: true,
       hasDepends: true,
       hasStatistics: true,
-      statsPrettifyFields: ['Index size'],
+      statsPrettifyFields: [gettext('Index size')],
       parent_type: ['table','partition'],
       canDrop: true,
       canDropCascade: true,
+      url_jump_after_node: 'schema',
       Init: function() {
         /* Avoid multiple registration of menus */
         if (this.initialized)
@@ -103,6 +104,7 @@ define('pgadmin.node.primary_key', [
         defaults: {
           name: undefined,
           oid: undefined,
+          is_sys_obj: undefined,
           comment: undefined,
           spcname: undefined,
           index: undefined,
@@ -122,6 +124,9 @@ define('pgadmin.node.primary_key', [
           id: 'oid', label: gettext('OID'), cell: 'string',
           type: 'text' , mode: ['properties'], editable: false,
           cellHeaderClasses:'width_percent_20',
+        },{
+          id: 'is_sys_obj', label: gettext('System primary key?'),
+          cell:'boolean', type: 'switch', mode: ['properties'],
         },{
           id: 'comment', label: gettext('Comment'), cell: 'string',
           type: 'multiline', mode: ['properties', 'create', 'edit'],
@@ -314,7 +319,7 @@ define('pgadmin.node.primary_key', [
 
               this.stopListening(this.model, 'change:' + name, this.render);
 
-                /*
+              /*
                  * Iterate through all the values, and find out how many are already
                  * present in the collection.
                  */
@@ -329,7 +334,7 @@ define('pgadmin.node.primary_key', [
                 }
               });
 
-                /*
+              /*
                  * Adding new values
                  */
 
@@ -344,7 +349,7 @@ define('pgadmin.node.primary_key', [
                 collection.add(m);
               });
 
-                /*
+              /*
                  * Removing unwanted!
                  */
               _.each(removed, function(v) {
@@ -367,6 +372,14 @@ define('pgadmin.node.primary_key', [
                 Backform.MultiSelectAjaxControl.prototype.remove.apply(this, arguments);
               }
             },
+            render: function() {
+              var index = this.model.get('index');
+              if(!_.isUndefined(index) && index != '') {
+                var col = this.model.get('columns');
+                col.reset([], {silent: true});
+              }
+              return Backform.Select2Control.prototype.render.apply(this, arguments);
+            },
           }),
           deps: ['index'], node: 'column',
           model: pgBrowser.Node.Model.extend({
@@ -387,12 +400,12 @@ define('pgadmin.node.primary_key', [
             return res;
           },
           select2:{allowClear:false},
-          disabled: function(m) {
+          readonly: function(m) {
             // If we are in table edit mode then
             if (_.has(m, 'top') && !_.isUndefined(m.top)
               && !m.top.isNew()) {
-                // If OID is undefined then user is trying to add
-                // new constraint which should be allowed for Unique
+              // If OID is undefined then user is trying to add
+              // new constraint which should be allowed for Unique
               return !_.isUndefined(m.get('oid'));
             }
 
@@ -400,15 +413,11 @@ define('pgadmin.node.primary_key', [
             if (!m.isNew()) {
               return true;
             }
+          },
+          disabled: function(m) {
             // Disable if index is selected.
             var index = m.get('index');
-            if(_.isUndefined(index) || index == '') {
-              return false;
-            } else {
-              var col = m.get('columns');
-              col.reset();
-              return true;
-            }
+            return !(_.isUndefined(index) || index == '');
           },
         },{
           id: 'include', label: gettext('Include columns'),
@@ -484,12 +493,12 @@ define('pgadmin.node.primary_key', [
             },
           }),
           deps: ['index'], node: 'column',
-          disabled: function(m) {
+          readonly: function(m) {
             // If we are in table edit mode then
             if (_.has(m, 'top') && !_.isUndefined(m.top)
               && !m.top.isNew()) {
-                // If OID is undefined then user is trying to add
-                // new constraint which should be allowed for Unique
+              // If OID is undefined then user is trying to add
+              // new constraint which should be allowed for Unique
               return !_.isUndefined(m.get('oid'));
             }
 
@@ -497,13 +506,16 @@ define('pgadmin.node.primary_key', [
             if (!m.isNew()) {
               return true;
             }
+          },
+          disabled: function(m) {
             // Disable if index is selected.
             var index = m.get('index');
             if(_.isUndefined(index) || index == '') {
               return false;
             } else {
-              var col = m.get('columns');
-              col.reset();
+              setTimeout(function(){
+                m.set('include', []);
+              },10);
               return true;
             }
           },
@@ -515,8 +527,7 @@ define('pgadmin.node.primary_key', [
           select2:{allowClear:false},
           filter: function(m) {
             // Don't show pg_global tablespace in selection.
-            if (m.label == 'pg_global') return false;
-            else return true;
+            return (m.label != 'pg_global');
           },
           disabled: function(m) {
             // Disable if index is selected.
@@ -533,18 +544,15 @@ define('pgadmin.node.primary_key', [
           },
         },{
           id: 'index', label: gettext('Index'),
+          mode: ['create'],
           type: 'text', group: gettext('Definition'),
           control: Backform.NodeListByNameControl.extend({
             initialize:function() {
-              if (_.isUndefined(this.model.top)) {
-                Backform.NodeListByNameControl.prototype.initialize.apply(this,arguments);
-              } else {
-                Backform.Control.prototype.initialize.apply(this,arguments);
-              }
+              Backform.NodeListByNameControl.prototype.initialize.apply(this, arguments);
             },
           }),
           select2:{allowClear:true}, node: 'index',
-          disabled: function(m) {
+          readonly: function(m) {
             // If we are in table edit mode then disable it
             if (_.has(m, 'top') && !_.isUndefined(m.top)
               && !m.top.isNew()) {
@@ -576,12 +584,12 @@ define('pgadmin.node.primary_key', [
         },{
           id: 'condeferrable', label: gettext('Deferrable?'),
           type: 'switch', group: gettext('Definition'), deps: ['index'],
-          disabled: function(m) {
+          readonly: function(m) {
             // If we are in table edit mode then
             if (_.has(m, 'top') && !_.isUndefined(m.top)
               && !m.top.isNew()) {
-                // If OID is undefined then user is trying to add
-                // new constraint which should allowed for Unique
+              // If OID is undefined then user is trying to add
+              // new constraint which should allowed for Unique
               return !_.isUndefined(m.get('oid'));
             }
 
@@ -589,6 +597,8 @@ define('pgadmin.node.primary_key', [
             if (!m.isNew()) {
               return true;
             }
+          },
+          disabled: function(m) {
             // Disable if index is selected.
             var index = m.get('index');
             if(_.isUndefined(index) || index == '') {
@@ -605,12 +615,12 @@ define('pgadmin.node.primary_key', [
           id: 'condeferred', label: gettext('Deferred?'),
           type: 'switch', group: gettext('Definition'),
           deps: ['condeferrable'],
-          disabled: function(m) {
+          readonly: function(m) {
             // If we are in table edit mode then
             if (_.has(m, 'top') && !_.isUndefined(m.top)
               && !m.top.isNew()) {
-                // If OID is undefined then user is trying to add
-                // new constraint which should allowed for Unique
+              // If OID is undefined then user is trying to add
+              // new constraint which should allowed for Unique
               return !_.isUndefined(m.get('oid'));
             }
 
@@ -618,6 +628,8 @@ define('pgadmin.node.primary_key', [
             if (!m.isNew()) {
               return true;
             }
+          },
+          disabled: function(m) {
             // Disable if condeferred is false or unselected.
             if(m.get('condeferrable') == true) {
               return false;
@@ -643,7 +655,7 @@ define('pgadmin.node.primary_key', [
 
           if ((_.isUndefined(index) || String(index).replace(/^\s+|\s+$/g, '') == '') &&
             (_.isUndefined(columns) || _.isNull(columns) || columns.length < 1)) {
-            var msg = gettext('Please specify columns for %(node)s', {node: gettext('Primary key')});
+            var msg = gettext('Please specify columns for %s.', gettext('Primary key'));
             this.errorModel.set('columns', msg);
             return msg;
           }
