@@ -1,3 +1,12 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define('pgadmin.node.index', [
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
   'backbone', 'sources/pgadmin', 'pgadmin.browser', 'pgadmin.alertifyjs',
@@ -15,12 +24,11 @@ define('pgadmin.node.index', [
         node: 'index',
         label: gettext('Indexes'),
         type: 'coll-index',
-        sqlAlterHelp: 'sql-alterindex.html',
-        sqlCreateHelp: 'sql-createindex.html',
-        dialogHelp: url_for('help.static', {'filename': 'index_dialog.html'}),
         columns: ['name', 'description'],
         hasStatistics: true,
-        statsPrettifyFields: ['Size', 'Index size'],
+        statsPrettifyFields: [gettext('Size'), gettext('Index size')],
+        canDrop: SchemaChildTreeNode.isTreeItemOfChildOfSchema,
+        canDropCascade: SchemaChildTreeNode.isTreeItemOfChildOfSchema,
       });
   }
 
@@ -65,7 +73,7 @@ define('pgadmin.node.index', [
     schema: [
       {
         id: 'colname', label: gettext('Column'), cell: 'node-list-by-name',
-        type: 'text', disabled: 'inSchemaWithModelCheck', editable: function(m) {
+        type: 'text', disabled: 'inSchema', readonly: 'isEditMode', editable: function(m) {
           // Header cell then skip
           if (m instanceof Backbone.Collection) {
             return false;
@@ -76,7 +84,7 @@ define('pgadmin.node.index', [
       },{
         id: 'collspcname', label: gettext('Collation'),
         cell: NodeAjaxOptionsDepsCell,
-        type: 'text', disabled: 'inSchemaWithModelCheck', editable: function(m) {
+        type: 'text', disabled: 'inSchema', readonly: 'isEditMode', editable: function(m) {
           // Header cell then skip
           if (m instanceof Backbone.Collection) {
             return false;
@@ -84,20 +92,20 @@ define('pgadmin.node.index', [
           return !(m.inSchemaWithModelCheck.apply(this, arguments));
         },
         control: 'node-ajax-options', url: 'get_collations', node: 'index',
+        url_jump_after_node: 'schema',
       },{
         id: 'op_class', label: gettext('Operator class'),
         cell: NodeAjaxOptionsDepsCell, tags: true,
         type: 'text', disabled: 'checkAccessMethod',
         editable: function(m) {
           // Header cell then skip
-          if (m instanceof Backbone.Collection) {
-            return false;
-          } else if (m.inSchemaWithModelCheck.apply(this, arguments)) {
+          if (m instanceof Backbone.Collection || m.inSchemaWithModelCheck.apply(this, arguments)) {
             return false;
           }
           return !(m.checkAccessMethod.apply(this, arguments));
         },
         control: 'node-ajax-options', url: 'get_op_class', node: 'index',
+        url_jump_after_node: 'schema',
         deps: ['amname'], transform: function(data, control) {
           /* We need to extract data from collection according
            * to access method selected by user if not selected
@@ -136,8 +144,6 @@ define('pgadmin.node.index', [
         deps: ['amname'],
         options: {
           'onText': 'DESC', 'offText': 'ASC',
-          'onColor': 'success', 'offColor': 'primary',
-          'size': 'small',
         },
       },{
         id: 'nulls', label: gettext('NULLs'),
@@ -159,8 +165,6 @@ define('pgadmin.node.index', [
         deps: ['amname', 'sort_order'],
         options: {
           'onText': 'FIRST', 'offText': 'LAST',
-          'onColor': 'success', 'offColor': 'primary',
-          'size': 'small',
         },
       },
     ],
@@ -181,15 +185,14 @@ define('pgadmin.node.index', [
       }
       return false;
     },
+    isEditMode: function(m) {
+      return !m.top.isNew();
+    },
     // We will check if we are under schema node & in 'create' mode
     inSchemaWithModelCheck: function(m) {
       if(m.top.node_info &&  'schema' in m.top.node_info) {
         // We will disable control if it's in 'edit' mode
-        if (m.top.isNew()) {
-          return false;
-        } else {
-          return true;
-        }
+        return !m.top.isNew();
       }
       return true;
     },
@@ -201,8 +204,8 @@ define('pgadmin.node.index', [
         _.isNull(parent_model.get('amname')) ||
           String(parent_model.get('amname')).replace(/^\s+|\s+$/g, '') == '' ||
           parent_model.get('amname') === 'btree') {
-            // We need to set nulls to true if sort_order is set to desc
-            // nulls first is default for desc
+        // We need to set nulls to true if sort_order is set to desc
+        // nulls first is default for desc
         if(m.get('sort_order') == true && m.previous('sort_order') ==  false) {
           setTimeout(function() { m.set('nulls', true); }, 10);
         }
@@ -220,13 +223,15 @@ define('pgadmin.node.index', [
       collection_type: ['coll-table', 'coll-view'],
       sqlAlterHelp: 'sql-alterindex.html',
       sqlCreateHelp: 'sql-createindex.html',
+      dialogHelp: url_for('help.static', {'filename': 'index_dialog.html'}),
       type: 'index',
       label: gettext('Index'),
       hasSQL:  true,
       hasDepends: true,
       hasStatistics: true,
-      statsPrettifyFields: ['Size', 'Index size'],
-      width: '45%',
+      width: pgBrowser.stdW.md + 'px',
+      statsPrettifyFields: [gettext('Size'), gettext('Index size')],
+      url_jump_after_node: 'schema',
       Init: function() {
         /* Avoid mulitple registration of menus */
         if (this.initialized)
@@ -285,7 +290,7 @@ define('pgadmin.node.index', [
           type: 'text', disabled: 'inSchema',
         },{
           id: 'oid', label: gettext('OID'), cell: 'string',
-          type: 'int', disabled: true, mode: ['edit', 'properties'],
+          type: 'int', readonly: true, mode: ['properties'],
         },{
           id: 'spcname', label: gettext('Tablespace'), cell: 'string',
           control: 'node-list-by-name', node: 'tablespace',
@@ -302,7 +307,8 @@ define('pgadmin.node.index', [
         },{
           id: 'amname', label: gettext('Access Method'), cell: 'string',
           type: 'text', mode: ['properties', 'create', 'edit'],
-          disabled: 'inSchemaWithModelCheck', url: 'get_access_methods',
+          disabled: 'inSchema', readonly: 'isEditMode', url: 'get_access_methods',
+          url_jump_after_node: 'schema',
           group: gettext('Definition'), select2: {'allowClear': true},
           control: Backform.NodeAjaxOptionsControl.extend({
             // When access method changes we need to clear columns collection
@@ -340,7 +346,7 @@ define('pgadmin.node.index', [
           type: 'array', group: gettext('Definition'),
           editable: false,
           canDelete: true, canAdd: true, mode: ['properties'],
-          disabled: 'inSchemaWithModelCheck',
+          disabled: 'inSchema', readonly: 'isEditMode',
           visible: function(m) {
             if(!_.isUndefined(m.node_info) && !_.isUndefined(m.node_info.server)
               && !_.isUndefined(m.node_info.server.version) &&
@@ -379,7 +385,7 @@ define('pgadmin.node.index', [
           min: 10, max:100, group: gettext('Definition'),
         },{
           id: 'indisunique', label: gettext('Unique?'), cell: 'string',
-          type: 'switch', disabled: 'inSchemaWithModelCheck',
+          type: 'switch', disabled: 'inSchema', readonly: 'isEditMode',
           group: gettext('Definition'),
         },{
           id: 'indisclustered', label: gettext('Clustered?'), cell: 'string',
@@ -387,42 +393,34 @@ define('pgadmin.node.index', [
           group: gettext('Definition'),
         },{
           id: 'indisvalid', label: gettext('Valid?'), cell: 'string',
-          type: 'switch', disabled: true, mode: ['properties'],
+          type: 'switch', mode: ['properties'],
           group: gettext('Definition'),
         },{
           id: 'indisprimary', label: gettext('Primary?'), cell: 'string',
-          type: 'switch', disabled: true, mode: ['properties'],
+          type: 'switch', mode: ['properties'],
           group: gettext('Definition'),
         },{
           id: 'is_sys_idx', label: gettext('System index?'), cell: 'string',
-          type: 'switch', disabled: true, mode: ['properties'],
+          type: 'switch', mode: ['properties'],
         },{
           id: 'isconcurrent', label: gettext('Concurrent build?'), cell: 'string',
-          type: 'switch', disabled: 'inSchemaWithModelCheck',
+          type: 'switch', disabled: 'inSchema', readonly: 'isEditMode',
           mode: ['create', 'edit'], group: gettext('Definition'),
         },{
           id: 'indconstraint', label: gettext('Constraint'), cell: 'string',
-          type: 'text', disabled: 'inSchemaWithModelCheck', mode: ['create', 'edit'],
+          type: 'text', disabled: 'inSchema', readonly: 'isEditMode', mode: ['create', 'edit'],
           control: 'sql-field', visible: true, group: gettext('Definition'),
         },{
           id: 'columns', label: gettext('Columns'), type: 'collection', deps: ['amname'],
           group: gettext('Definition'), model: ColumnModel, mode: ['edit', 'create'],
           canAdd: function(m) {
             // We will disable it if it's in 'edit' mode
-            if (m.isNew()) {
-              return true;
-            } else {
-              return false;
-            }
+            return m.isNew();
           },
           canEdit: false,
           canDelete: function(m) {
             // We will disable it if it's in 'edit' mode
-            if (m.isNew()) {
-              return true;
-            } else {
-              return false;
-            }
+            return m.isNew();
           },
           control: 'unique-col-collection', uniqueCol : ['colname'],
           columns: ['colname', 'op_class', 'sort_order', 'nulls', 'collspcname'],
@@ -431,7 +429,7 @@ define('pgadmin.node.index', [
           type: 'array', group: gettext('Definition'),
           editable: false,
           canDelete: true, canAdd: true, mode: ['edit', 'create'],
-          disabled: 'inSchemaWithModelCheck',
+          disabled: 'inSchema', readonly: 'isEditMode',
           visible: function(m) {
             if(!_.isUndefined(m.node_info) && !_.isUndefined(m.node_info.server)
               && !_.isUndefined(m.node_info.server.version) &&
@@ -487,12 +485,6 @@ define('pgadmin.node.index', [
             this.errorModel.set('name', msg);
             return msg;
           }
-          if (_.isUndefined(this.get('spcname'))
-            || String(this.get('spcname')).replace(/^\s+|\s+$/g, '') == '') {
-            msg = gettext('Tablespace cannot be empty.');
-            this.errorModel.set('spcname', msg);
-            return msg;
-          }
           if (_.isUndefined(this.get('amname'))
             || String(this.get('amname')).replace(/^\s+|\s+$/g, '') == '') {
             msg = gettext('Access method cannot be empty.');
@@ -521,15 +513,14 @@ define('pgadmin.node.index', [
           }
           return false;
         },
+        isEditMode: function(m) {
+          return !m.isNew();
+        },
         // We will check if we are under schema node & in 'create' mode
         inSchemaWithModelCheck: function(m) {
           if(this.node_info &&  'schema' in this.node_info) {
             // We will disable control if it's in 'edit' mode
-            if (m.isNew()) {
-              return false;
-            } else {
-              return true;
-            }
+            return !m.isNew();
           }
           return true;
         },
@@ -542,11 +533,7 @@ define('pgadmin.node.index', [
               return false;
             } else {
               // if we are in edit mode
-              if (!_.isUndefined(m.get('attnum')) && m.get('attnum') >= 1 ) {
-                return false;
-              } else {
-                return true;
-              }
+              return (_.isUndefined(m.get('attnum')) || m.get('attnum') < 1 );
             }
           }
           return true;
