@@ -2,7 +2,7 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
@@ -248,10 +248,13 @@ define('pgadmin.node.table_partition_utils', [
       values_in: undefined,
       values_modulus: undefined,
       values_remainder: undefined,
+      is_sub_partitioned: false,
+      sub_partition_type: 'range',
     },
     keys:['partition_name'],
     schema: [{
       id: 'oid', label: gettext('OID'), type: 'text',
+      mode: ['properties'],
     },{
       id: 'is_attach', label:gettext('Operation'), cell: 'switch', type: 'switch',
       options: {'onText': gettext('Attach'), 'offText': gettext('Create'), 'width': 65},
@@ -261,6 +264,11 @@ define('pgadmin.node.table_partition_utils', [
           return true;
         return false;
       },
+      disabled: function(m) {
+        if (m instanceof Backbone.Model && m.isNew() && !m.top.isNew())
+          return false;
+        return true;
+      },
     },{
       id: 'partition_name', label: gettext('Name'), type: 'text', cell:'string',
       cellHeaderClasses: 'width_percent_15',
@@ -269,6 +277,11 @@ define('pgadmin.node.table_partition_utils', [
           return true;
         return false;
       }, cellFunction: getPartitionCell,
+      disabled: function(m) {
+        if (m instanceof Backbone.Model && m.isNew())
+          return false;
+        return true;
+      },
     },{
       id: 'is_default', label: gettext('Default'), type: 'switch', cell:'switch',
       cellHeaderClasses: 'width_percent_5',  min_version: 110000,
@@ -283,6 +296,16 @@ define('pgadmin.node.table_partition_utils', [
           return true;
         return false;
       },
+      disabled: function(m) {
+        if(m.handler && m.handler.top &&
+          m.handler.top.attributes &&
+          (m.handler.top.attributes.partition_type === 'range' ||
+          m.handler.top.attributes.partition_type === 'list') &&
+          m instanceof Backbone.Model && m.isNew() &&
+          m.handler.top.node_info.server.version >= 110000)
+          return false;
+        return true;
+      },
     },{
       id: 'values_from', label: gettext('From'), type:'text',
       cell:Backgrid.Extension.StringDepCell, deps: ['is_default'],
@@ -295,6 +318,14 @@ define('pgadmin.node.table_partition_utils', [
           return true;
         return false;
       },
+      disabled: function(m) {
+        if(m.handler && m.handler.top &&
+          m.handler.top.attributes &&
+          m.handler.top.attributes.partition_type === 'range' &&
+          m instanceof Backbone.Model && m.isNew() && m.get('is_default') !== true)
+          return false;
+        return true;
+      },
     },{
       id: 'values_to', label: gettext('To'), type:'text',
       cell:Backgrid.Extension.StringDepCell, deps: ['is_default'],
@@ -307,6 +338,14 @@ define('pgadmin.node.table_partition_utils', [
           return true;
         return false;
       },
+      disabled: function(m) {
+        if(m.handler && m.handler.top &&
+          m.handler.top.attributes &&
+          m.handler.top.attributes.partition_type === 'range' &&
+          m instanceof Backbone.Model && m.isNew() && m.get('is_default') !== true)
+          return false;
+        return true;
+      },
     },{
       id: 'values_in', label: gettext('In'), type:'text',
       cell:Backgrid.Extension.StringDepCell, deps: ['is_default'],
@@ -319,6 +358,14 @@ define('pgadmin.node.table_partition_utils', [
           return true;
         return false;
       },
+      disabled: function(m) {
+        if(m.handler && m.handler.top &&
+          m.handler.top.attributes &&
+          m.handler.top.attributes.partition_type === 'list' &&
+          m instanceof Backbone.Model && m.isNew() && m.get('is_default') !== true)
+          return false;
+        return true;
+      },
     },{
       id: 'values_modulus', label: gettext('Modulus'), type:'int',
       cell:Backgrid.Extension.StringDepCell,
@@ -331,6 +378,14 @@ define('pgadmin.node.table_partition_utils', [
           return true;
         return false;
       },
+      disabled: function(m) {
+        if(m.handler && m.handler.top &&
+          m.handler.top.attributes &&
+          m.handler.top.attributes.partition_type === 'hash' &&
+          m instanceof Backbone.Model && m.isNew())
+          return false;
+        return true;
+      },
     },{
       id: 'values_remainder', label: gettext('Remainder'), type:'int',
       cell:Backgrid.Extension.StringDepCell,
@@ -343,6 +398,131 @@ define('pgadmin.node.table_partition_utils', [
           return true;
         return false;
       },
+      disabled: function(m) {
+        if(m.handler && m.handler.top &&
+          m.handler.top.attributes &&
+          m.handler.top.attributes.partition_type === 'hash' &&
+          m instanceof Backbone.Model && m.isNew())
+          return false;
+        return true;
+      },
+    },{
+      id: 'is_sub_partitioned', label:gettext('Partitioned table?'), cell: 'switch',
+      group: 'Partition', type: 'switch', mode: ['properties', 'create', 'edit'],
+      deps: ['is_attach'],
+      disabled: function(m) {
+        if(!m.isNew())
+          return true;
+
+        if (m.get('is_attach')) {
+          setTimeout( function() {
+            m.set('is_sub_partitioned', false);
+          }, 10);
+          return true;
+        }
+        return false;
+      },
+    },{
+      id: 'sub_partition_type', label:gettext('Partition Type'),
+      editable: false, type: 'select2', select2: {allowClear: false},
+      group: 'Partition', deps: ['is_sub_partitioned'],
+      options: function() {
+        var options = [{
+          label: gettext('Range'), value: 'range',
+        },{
+          label: gettext('List'), value: 'list',
+        }];
+
+        if(!_.isUndefined(this.node_info) && !_.isUndefined(this.node_info.server)
+          && !_.isUndefined(this.node_info.server.version) &&
+            this.node_info.server.version >= 110000) {
+          options.push({
+            label: gettext('Hash'), value: 'hash',
+          });
+        }
+        return options;
+      },
+      visible: function(m) {
+        if (m.isNew())
+          return true;
+        return false;
+      },
+      disabled: function(m) {
+        if (!m.isNew() || !m.get('is_sub_partitioned'))
+          return true;
+        return false;
+      },
+    },{
+      id: 'sub_partition_keys', label:gettext('Partition Keys'),
+      model: Backform.PartitionKeyModel,
+      subnode: Backform.PartitionKeyModel,
+      editable: true, type: 'collection',
+      group: 'Partition', mode: ['properties', 'create', 'edit'],
+      deps: ['is_sub_partitioned', 'sub_partition_type'],
+      canEdit: false, canDelete: true,
+      control: 'sub-node-collection',
+      canAdd: function(m) {
+        if (m.isNew() && m.get('is_sub_partitioned'))
+          return true;
+        return false;
+      },
+      canAddRow: function(m) {
+        var columns = m.top.get('columns'),
+          typename = m.top.get('typname'),
+          columns_exist= false;
+
+        var max_row_count = 1000;
+        if (m.get('sub_partition_type') && m.get('sub_partition_type') == 'list')
+          max_row_count = 1;
+
+        /* If columns are not specified by the user then it may be
+           * possible that he/she selected 'OF TYPE', so we should check
+           * for that as well.
+           */
+        if (columns.length <= 0 && !_.isUndefined(typename)
+          && !_.isNull(typename) && m.of_types_tables.length > 0){
+          _.each(m.of_types_tables, function(data) {
+            if (data.label == typename && data.oftype_columns.length > 0){
+              columns_exist = true;
+            }
+          });
+        } else if (columns.length > 0) {
+          columns_exist = _.some(columns.pluck('name'));
+        }
+
+        return (m.get('sub_partition_keys') &&
+          m.get('sub_partition_keys').length < max_row_count && columns_exist
+        );
+
+      },
+      disabled: function(m) {
+        if (m.get('sub_partition_keys') && m.get('sub_partition_keys').models.length > 0) {
+          setTimeout(function () {
+            var coll = m.get('sub_partition_keys');
+            coll.remove(coll.filter(function() { return true; }));
+
+          }, 10);
+        }
+      },
+      visible: function(m) {
+        if (m.isNew())
+          return true;
+        return false;
+      },
+    },{
+      id: 'sub_partition_scheme', label: gettext('Partition Scheme'),
+      type: 'note', group: 'Partition', mode: ['edit'],
+      visible: function(m) {
+        if (!m.isNew() && !_.isUndefined(m.get('sub_partition_scheme')) &&
+          m.get('sub_partition_scheme') != '')
+          return true;
+        return false;
+      },
+      disabled: function(m) {
+        if (!m.isNew()) {
+          this.text = m.get('sub_partition_scheme');
+        }
+      },
     }],
     validate: function() {
       var partition_name = this.get('partition_name'),
@@ -352,11 +532,14 @@ define('pgadmin.node.table_partition_utils', [
         values_in = this.get('values_in'),
         values_modulus = this.get('values_modulus'),
         values_remainder = this.get('values_remainder'),
+        is_sub_partitioned = this.get('is_sub_partitioned'),
+        sub_partition_keys = this.get('sub_partition_keys'),
         msg;
 
       // Have to clear existing validation before initiating current state
       // validation only
       this.errorModel.clear();
+      this.top.errorModel.clear();
 
       if (_.isUndefined(partition_name) || _.isNull(partition_name) ||
        String(partition_name).replace(/^\s+|\s+$/g, '') === '') {
@@ -365,6 +548,13 @@ define('pgadmin.node.table_partition_utils', [
         return msg;
       }
 
+      if (is_sub_partitioned && this.isNew() &&
+        !_.isNull(sub_partition_keys) && sub_partition_keys.length <= 0) {
+        msg = gettext('Please specify at least one key for partitioned table.');
+        this.top.errorModel.set('sub_partition_keys', msg);
+        return msg;
+      }
+
       if (this.top.get('partition_type') === 'range') {
         if (is_default !== true && (_.isUndefined(values_from) ||
          _.isNull(values_from) || String(values_from).replace(/^\s+|\s+$/g, '') === '')) {
