@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -35,7 +35,10 @@ class TestColumnAclSql(SQLTemplateTestBase):
         self.table_id, self.column_id = cursor.fetchone()
 
     def generate_sql(self, version):
-        template_file = self.get_template_file(version, "acl.sql")
+        file_path = os.path.join(os.path.dirname(__file__), "..", "templates",
+                                 "columns", "sql")
+        template_file = self.get_template_file(version, file_path,
+                                               "acl.sql")
         template = file_as_template(template_file)
         public_schema_id = 2200
         sql = template.render(scid=public_schema_id,
@@ -47,10 +50,3 @@ class TestColumnAclSql(SQLTemplateTestBase):
 
     def assertions(self, fetch_result, descriptions):
         self.assertEqual(0, len(fetch_result))
-
-    @staticmethod
-    def get_template_file(version, filename):
-        return os.path.join(
-            os.path.dirname(__file__), "..", "templates", "column", "sql",
-            version, filename
-        )
