@@ -1,33 +1,23 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define('pgadmin.node.trigger', [
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
-  'underscore.string', 'sources/pgadmin', 'pgadmin.browser',
+  'sources/pgadmin', 'pgadmin.browser',
   'pgadmin.backform', 'pgadmin.alertifyjs',
   'pgadmin.node.schema.dir/schema_child_tree_node',
   'pgadmin.browser.collection',
 ], function(
-  gettext, url_for, $, _, S, pgAdmin, pgBrowser, Backform, alertify,
+  gettext, url_for, $, _, pgAdmin, pgBrowser, Backform, alertify,
   SchemaChildTreeNode
 ) {
 
-  Backform.CustomSwitchControl = Backform.SwitchControl.extend({
-    template: _.template([
-      '<label class="<%=Backform.controlLabelClassName%> custom_switch_label_class"><%=label%></label>',
-      '<div class="<%=Backform.controlsClassName%> custom_switch_control_class">',
-      '  <div class="checkbox">',
-      '    <label>',
-      '      <input type="checkbox" class="<%=extraClasses.join(\' \')%>"',
-      '        name="<%=name%>" <%=value ? "checked=\'checked\'" : ""%>',
-      '        <%=disabled ? "disabled" : ""%> <%=required ? "required" : ""%> />',
-      '    </label>',
-      '  </div>',
-      '</div>',
-      '<% if (helpMessage && helpMessage.length) { %>',
-      '  <span class="<%=Backform.helpMessageClassName%>"><%=helpMessage%></span>',
-      '<% } %>',
-    ].join('\n')),
-    className: 'pgadmin-control-group form-group col-6',
-  });
-
   if (!pgBrowser.Nodes['coll-trigger']) {
     pgAdmin.Browser.Nodes['coll-trigger'] =
       pgAdmin.Browser.Collection.extend({
@@ -35,6 +25,8 @@ define('pgadmin.node.trigger', [
         label: gettext('Triggers'),
         type: 'coll-trigger',
         columns: ['name', 'description'],
+        canDrop: SchemaChildTreeNode.isTreeItemOfChildOfSchema,
+        canDropCascade: SchemaChildTreeNode.isTreeItemOfChildOfSchema,
       });
   }
 
@@ -46,10 +38,11 @@ define('pgadmin.node.trigger', [
       label: gettext('Trigger'),
       hasSQL:  true,
       hasDepends: true,
-      width: '650px',
+      width: pgBrowser.stdW.sm + 'px',
       sqlAlterHelp: 'sql-altertrigger.html',
       sqlCreateHelp: 'sql-createtrigger.html',
       dialogHelp: url_for('help.static', {'filename': 'trigger_dialog.html'}),
+      url_jump_after_node: 'schema',
       Init: function() {
         /* Avoid mulitple registration of menus */
         if (this.initialized)
@@ -116,28 +109,28 @@ define('pgadmin.node.trigger', [
           $.ajax({
             url: obj.generate_url(i, 'enable' , d, true),
             type:'PUT',
-            data: {'enable' : true},
+            data: {'is_enable_trigger' : 'O'},
             dataType: 'json',
           })
-          .done(function(res) {
-            if (res.success == 1) {
-              alertify.success(res.info);
-              t.removeIcon(i);
-              data.icon = 'icon-trigger';
-              t.addIcon(i, {icon: data.icon});
+            .done(function(res) {
+              if (res.success == 1) {
+                alertify.success(res.info);
+                t.removeIcon(i);
+                data.icon = 'icon-trigger';
+                t.addIcon(i, {icon: data.icon});
+                t.unload(i);
+                t.setInode(false);
+                t.deselect(i);
+                // Fetch updated data from server
+                setTimeout(function() {
+                  t.select(i);
+                }, 10);
+              }
+            })
+            .fail(function(xhr, status, error) {
+              alertify.pgRespErrorNotify(xhr, error);
               t.unload(i);
-              t.setInode(false);
-              t.deselect(i);
-              // Fetch updated data from server
-              setTimeout(function() {
-                t.select(i);
-              }, 10);
-            }
-          })
-          .fail(function(xhr, status, error) {
-            alertify.pgRespErrorNotify(xhr, error);
-            t.unload(i);
-          });
+            });
         },
         /* Disable trigger */
         disable_trigger: function(args) {
@@ -154,28 +147,28 @@ define('pgadmin.node.trigger', [
           $.ajax({
             url: obj.generate_url(i, 'enable' , d, true),
             type:'PUT',
-            data: {'enable' : false},
+            data: {'is_enable_trigger' : 'D'},
             dataType: 'json',
           })
-          .done(function(res) {
-            if (res.success == 1) {
-              alertify.success(res.info);
-              t.removeIcon(i);
-              data.icon = 'icon-trigger-bad';
-              t.addIcon(i, {icon: data.icon});
+            .done(function(res) {
+              if (res.success == 1) {
+                alertify.success(res.info);
+                t.removeIcon(i);
+                data.icon = 'icon-trigger-bad';
+                t.addIcon(i, {icon: data.icon});
+                t.unload(i);
+                t.setInode(false);
+                t.deselect(i);
+                // Fetch updated data from server
+                setTimeout(function() {
+                  t.select(i);
+                }, 10);
+              }
+            })
+            .fail(function(xhr, status, error) {
+              alertify.pgRespErrorNotify(xhr, error, gettext('Disable trigger failed'));
               t.unload(i);
-              t.setInode(false);
-              t.deselect(i);
-              // Fetch updated data from server
-              setTimeout(function() {
-                t.select(i);
-              }, 10);
-            }
-          })
-          .fail(function(xhr, status, error) {
-            alertify.pgRespErrorNotify(xhr, error, gettext('Disable trigger failed'));
-            t.unload(i);
-          });
+            });
         },
       },
       canDrop: SchemaChildTreeNode.isTreeItemOfChildOfSchema,
@@ -192,11 +185,23 @@ define('pgadmin.node.trigger', [
           type: 'text', disabled: 'inSchema',
         },{
           id: 'oid', label: gettext('OID'), cell: 'string',
-          type: 'int', disabled: true, mode: ['properties'],
+          type: 'int', mode: ['properties'],
         },{
           id: 'is_enable_trigger', label: gettext('Trigger enabled?'),
-          type: 'switch', disabled: 'inSchema', mode: ['edit', 'properties'],
-          group: gettext('Definition'),
+          mode: ['edit', 'properties'], group: gettext('Definition'),
+          disabled: function() {
+            if(this.node_info && ('catalog' in this.node_info || 'view' in this.node_info)) {
+              return true;
+            }
+            return false;
+          },
+          options: [
+            {label: gettext('Enable'), value: 'O'},
+            {label: gettext('Enable Replica'), value: 'R'},
+            {label: gettext('Enable Always'), value: 'A'},
+            {label: gettext('Disable'), value: 'D'},
+          ],
+          control: 'select2', select2: { allowClear: false, width: '100%' },
         },{
           id: 'is_row_trigger', label: gettext('Row trigger?'),
           type: 'switch', group: gettext('Definition'),
@@ -204,10 +209,13 @@ define('pgadmin.node.trigger', [
           deps: ['is_constraint_trigger'],
           disabled: function(m) {
             // Disabled if table is a partitioned table.
+            if (!m.isNew())
+              return true;
+
             if (_.has(m, 'node_info') && _.has(m.node_info, 'table') &&
               _.has(m.node_info.table, 'is_partitioned') &&
                m.node_info.table.is_partitioned && m.node_info.server.version < 110000
-               )
+            )
             {
               setTimeout(function(){
                 m.set('is_row_trigger', false);
@@ -222,7 +230,7 @@ define('pgadmin.node.trigger', [
             if(!m.inSchemaWithModelCheck.apply(this, [m])) {
               if(!_.isUndefined(is_constraint_trigger) &&
                 is_constraint_trigger === true) {
-                  // change it's model value
+                // change it's model value
                 setTimeout(function() { m.set('is_row_trigger', true); }, 10);
                 return true;
               } else {
@@ -243,12 +251,17 @@ define('pgadmin.node.trigger', [
           type: 'switch',
           mode: ['create','edit', 'properties'],
           group: gettext('Definition'),
+          deps: ['tfunction'],
           disabled: function(m) {
             // Disabled if table is a partitioned table.
+            var tfunction = m.get('tfunction');
             if ((_.has(m, 'node_info') && _.has(m.node_info, 'table') &&
               _.has(m.node_info.table, 'is_partitioned') &&
                 m.node_info.table.is_partitioned) ||
-                _.indexOf(Object.keys(m.node_info), 'view') != -1) {
+                _.indexOf(Object.keys(m.node_info), 'view') != -1 ||
+                (m.node_info.server.server_type === 'ppas' &&
+                !_.isUndefined(tfunction) &&
+                 tfunction === 'Inline EDB-SPL')) {
               setTimeout(function(){
                 m.set('is_constraint_trigger', false);
               },10);
@@ -271,7 +284,7 @@ define('pgadmin.node.trigger', [
                 is_constraint_trigger === true) {
                 return false;
               } else {
-                  // If value is already set then reset it to false
+                // If value is already set then reset it to false
                 if(m.get('tgdeferrable')) {
                   setTimeout(function() { m.set('tgdeferrable', false); }, 10);
                 }
@@ -295,11 +308,11 @@ define('pgadmin.node.trigger', [
                 tgdeferrable) {
                 return false;
               } else {
-                  // If value is already set then reset it to false
+                // If value is already set then reset it to false
                 if(m.get('tginitdeferred')) {
                   setTimeout(function() { m.set('tginitdeferred', false); }, 10);
                 }
-                  // If constraint trigger is set then do not disable
+                // If constraint trigger is set then do not disable
                 return m.get('is_constraint_trigger') ? false : true;
               }
             } else {
@@ -308,10 +321,10 @@ define('pgadmin.node.trigger', [
             }
           },
         },{
-          id: 'tfunction', label: gettext('Trigger Function'),
+          id: 'tfunction', label: gettext('Trigger function'),
           type: 'text', disabled: 'inSchemaWithModelCheck',
           mode: ['create','edit', 'properties'], group: gettext('Definition'),
-          control: 'node-ajax-options', url: 'get_triggerfunctions',
+          control: 'node-ajax-options', url: 'get_triggerfunctions', url_jump_after_node: 'schema',
           cache_node: 'trigger_function',
         },{
           id: 'tgargs', label: gettext('Arguments'), cell: 'string',
@@ -326,7 +339,7 @@ define('pgadmin.node.trigger', [
               if(server_type === 'ppas' &&
                 !_.isUndefined(tfunction) &&
                   tfunction === 'Inline EDB-SPL') {
-                    // Disable and clear its value
+                // Disable and clear its value
                 m.set('tgargs', undefined);
                 return true;
               } else {
@@ -342,8 +355,8 @@ define('pgadmin.node.trigger', [
           mode: ['create','edit', 'properties'], group: gettext('Events'),
           options: function(control) {
             var table_options = [
-              {label: 'BEFORE', value: 'BEFORE'},
-              {label: 'AFTER', value: 'AFTER'}],
+                {label: 'BEFORE', value: 'BEFORE'},
+                {label: 'AFTER', value: 'AFTER'}],
               view_options = [
                 {label: 'BEFORE', value: 'BEFORE'},
                 {label: 'AFTER', value: 'AFTER'},
@@ -355,9 +368,10 @@ define('pgadmin.node.trigger', [
               return view_options;
             }
           },
-          // If create mode then by default open composite type
           control: 'select2', select2: { allowClear: false, width: '100%' },
           disabled: function(m) {
+            if (!m.isNew())
+              return true;
             // If contraint trigger is set to True then only enable it
             var is_constraint_trigger = m.get('is_constraint_trigger');
             if(!m.inSchemaWithModelCheck.apply(this, [m])) {
@@ -380,15 +394,17 @@ define('pgadmin.node.trigger', [
           },
         },{
           type: 'nested', control: 'fieldset', mode: ['create','edit', 'properties'],
-          label: gettext('Events'), group: gettext('Events'),
+          label: gettext('Events'), group: gettext('Events'), contentClass: 'row',
           schema:[{
             id: 'evnt_insert', label: gettext('INSERT'),
             type: 'switch', mode: ['create','edit', 'properties'],
             group: gettext('Events'),
-            control: Backform.CustomSwitchControl,
+            extraToggleClasses: 'pg-el-sm-6',
+            controlLabelClassName: 'control-label pg-el-sm-5 pg-el-12',
+            controlsClassName: 'pgadmin-controls pg-el-sm-7 pg-el-12',
             disabled: function(m) {
               var evn_insert = m.get('evnt_insert');
-              if (!_.isUndefined(evn_insert) && m.node_info['server']['server_type'] == 'ppas')
+              if (!_.isUndefined(evn_insert) && m.node_info['server']['server_type'] == 'ppas' && m.isNew())
                 return false;
               return m.inSchemaWithModelCheck.apply(this, [m]);
             },
@@ -396,10 +412,12 @@ define('pgadmin.node.trigger', [
             id: 'evnt_update', label: gettext('UPDATE'),
             type: 'switch', mode: ['create','edit', 'properties'],
             group: gettext('Events'),
-            control: Backform.CustomSwitchControl,
+            extraToggleClasses: 'pg-el-sm-6',
+            controlLabelClassName: 'control-label pg-el-sm-5 pg-el-12',
+            controlsClassName: 'pgadmin-controls pg-el-sm-7 pg-el-12',
             disabled: function(m) {
               var evn_update = m.get('evnt_update');
-              if (!_.isUndefined(evn_update) && m.node_info['server']['server_type'] == 'ppas')
+              if (!_.isUndefined(evn_update) && m.node_info['server']['server_type'] == 'ppas' && m.isNew())
                 return false;
               return m.inSchemaWithModelCheck.apply(this, [m]);
             },
@@ -407,37 +425,38 @@ define('pgadmin.node.trigger', [
             id: 'evnt_delete', label: gettext('DELETE'),
             type: 'switch', mode: ['create','edit', 'properties'],
             group: gettext('Events'),
-            control: Backform.CustomSwitchControl,
+            extraToggleClasses: 'pg-el-sm-6',
+            controlLabelClassName: 'control-label pg-el-sm-5 pg-el-12',
+            controlsClassName: 'pgadmin-controls pg-el-sm-7 pg-el-12',
             disabled: function(m) {
               var evn_delete = m.get('evnt_delete');
-              if (!_.isUndefined(evn_delete) && m.node_info['server']['server_type'] == 'ppas')
+              if (!_.isUndefined(evn_delete) && m.node_info['server']['server_type'] == 'ppas' && m.isNew())
                 return false;
               return m.inSchemaWithModelCheck.apply(this, [m]);
             },
           },{
             id: 'evnt_truncate', label: gettext('TRUNCATE'),
-            type: 'switch', group: gettext('Events'),
-            control: Backform.CustomSwitchControl,
+            type: 'switch', group: gettext('Events'),deps: ['is_row_trigger', 'is_constraint_trigger'],
+            extraToggleClasses: 'pg-el-sm-6',
+            controlLabelClassName: 'control-label pg-el-sm-5 pg-el-12',
+            controlsClassName: 'pgadmin-controls pg-el-sm-7 pg-el-12',
             disabled: function(m) {
               var is_constraint_trigger = m.get('is_constraint_trigger'),
                 is_row_trigger = m.get('is_row_trigger'),
                 server_type = m.node_info['server']['server_type'];
-              if(!m.inSchemaWithModelCheck.apply(this, [m])) {
-                // We will enabale truncate only for EDB PPAS
-                // and both triggers row & constarint are set to false
-                if(server_type === 'ppas' &&
-                  !_.isUndefined(is_constraint_trigger) &&
-                    !_.isUndefined(is_row_trigger) &&
-                    is_constraint_trigger === false &&
-                    is_row_trigger === false) {
-                  return false;
-                } else {
-                  return true;
-                }
-              } else {
-                // Disable it
+              if (is_row_trigger == true){
+                setTimeout(function(){
+                  m.set('evnt_truncate', false);
+                },10);
                 return true;
               }
+
+              if (server_type === 'ppas' &&
+                  !_.isUndefined(is_constraint_trigger) &&
+                    !_.isUndefined(is_row_trigger) &&
+                    is_constraint_trigger === false && m.isNew())
+                return false;
+              return m.inSchemaWithModelCheck.apply(this, [m]);
             },
           }],
         },{
@@ -480,19 +499,18 @@ define('pgadmin.node.trigger', [
         },{
           id: 'prosrc', label: gettext('Code'), group: gettext('Code'),
           type: 'text', mode: ['create', 'edit'], deps: ['tfunction'],
-          control: 'sql-field', visible: true,
+          tabPanelCodeClass: 'sql-code-control',
+          control: Backform.SqlCodeControl,
+          visible: true,
           disabled: function(m) {
             // We will enable it only when EDB PPAS and trigger function is
             // set to Inline EDB-SPL
             var tfunction = m.get('tfunction'),
               server_type = m.node_info['server']['server_type'];
 
-            if(server_type === 'ppas' &&
-              !_.isUndefined(tfunction) &&
-                tfunction === 'Inline EDB-SPL')
-              return false;
-            else
-              return true;
+            return (server_type !== 'ppas' ||
+            _.isUndefined(tfunction) ||
+              tfunction !== 'Inline EDB-SPL');
           },
         },{
           id: 'is_sys_trigger', label: gettext('System trigger?'), cell: 'string',
@@ -556,11 +574,7 @@ define('pgadmin.node.trigger', [
         inSchemaWithModelCheck: function(m) {
           if(this.node_info &&  'schema' in this.node_info) {
             // We will disable control if it's in 'edit' mode
-            if (m.isNew()) {
-              return false;
-            } else {
-              return true;
-            }
+            return !m.isNew();
           }
           return true;
         },
@@ -573,18 +587,16 @@ define('pgadmin.node.trigger', [
               return false;
             } else {
               // if we are in edit mode
-              if (!_.isUndefined(m.get('attnum')) && m.get('attnum') >= 1 ) {
-                return false;
-              } else {
-                return true;
-              }
+              return (_.isUndefined(m.get('attnum')) || m.get('attnum') < 1 );
             }
           }
           return true;
         },
         // Disable/Enable Transition tables
         disableTransition: function(m) {
-          var flag = true,
+          if (!m.isNew())
+            return true;
+          var flag = false,
             evnt = null,
             name = this.name,
             evnt_count = 0;
@@ -611,8 +623,7 @@ define('pgadmin.node.trigger', [
           // Disable New transition table if both UPDATE and INSERT events are disabled
           if(!m.get('is_constraint_trigger') && m.get('fires') == 'AFTER' &&
             (m.get('evnt_update') || m.get(evnt)) && evnt_count == 1) {
-            if (m.get('evnt_update') && (_.size(m.get('columns')) >= 1 && m.get('columns')[0] != '')) flag = true;
-            else flag = false;
+            flag = (m.get('evnt_update') && (_.size(m.get('columns')) >= 1 && m.get('columns')[0] != ''));
           }
 
           flag && setTimeout(function() {
@@ -627,11 +638,21 @@ define('pgadmin.node.trigger', [
       canCreate: SchemaChildTreeNode.isTreeItemOfChildOfSchema,
       // Check to whether trigger is disable ?
       canCreate_with_trigger_enable: function(itemData, item, data) {
+        var treeData = this.getTreeNodeHierarchy(item);
+        if ('view' in treeData) {
+          return false;
+        }
+
         return itemData.icon === 'icon-trigger-bad' &&
           this.canCreate.apply(this, [itemData, item, data]);
       },
       // Check to whether trigger is enable ?
       canCreate_with_trigger_disable: function(itemData, item, data) {
+        var treeData = this.getTreeNodeHierarchy(item);
+        if ('view' in treeData) {
+          return false;
+        }
+
         return itemData.icon === 'icon-trigger' &&
           this.canCreate.apply(this, [itemData, item, data]);
       },
