@@ -2,7 +2,7 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
@@ -36,16 +36,8 @@ define('pgadmin.node.type', [
       Backgrid.Extension.DependentCell.prototype.initialize.apply(this, arguments);
     },
     dependentChanged: function () {
-      this.$el.empty();
-      var model = this.model;
-      var column = this.column;
-      var editable = this.column.get('editable');
-
-      var is_editable = _.isFunction(editable) ? !!editable.apply(column, [model]) : !!editable;
-      if (is_editable){ this.$el.addClass('editable'); }
-      else { this.$el.removeClass('editable'); }
-
-      this.delegateEvents();
+      this.model.set(this.column.get('name'), null);
+      this.render();
       return this;
     },
     remove: Backgrid.Extension.DependentCell.prototype.remove,
@@ -111,7 +103,7 @@ define('pgadmin.node.type', [
       // Note: There are ambiguities in the PG catalogs and docs between
       // precision and scale. In the UI, we try to follow the docs as
       // closely as possible, therefore we use Length/Precision and Scale
-      id: 'tlength', label: gettext('Length/precision'), deps: ['type'], type: 'text',
+      id: 'tlength', label: gettext('Length/Precision'), deps: ['type'], type: 'text',
       disabled: false, cell: IntegerDepCell,
       editable: function(m) {
         // We will store type from selected from combobox
@@ -214,13 +206,13 @@ define('pgadmin.node.type', [
         this.errorModel.set('type', errmsg);
         return errmsg;
       }
-      // Validation for Length/precision field (see comments above if confused about the naming!)
+      // Validation for Length/Precision field (see comments above if confused about the naming!)
       else if (this.get('is_tlength')
         && !_.isUndefined(this.get('tlength'))) {
         if (this.get('tlength') < this.get('min_val'))
-          errmsg = gettext('Length/precision should not be less than %(value)s', {value: this.get('min_val')});
+          errmsg = gettext('Length/Precision should not be less than %s.', this.get('min_val'));
         if (this.get('tlength') > this.get('max_val') )
-          errmsg = gettext('Length/precision should not be greater than %(value)s', {value: this.get('max_val')});
+          errmsg = gettext('Length/Precision should not be greater than %s.', this.get('max_val'));
           // If we have any error set then throw it to user
         if(errmsg) {
           this.errorModel.set('tlength', errmsg);
@@ -231,9 +223,9 @@ define('pgadmin.node.type', [
       else if (this.get('is_precision')
         && !_.isUndefined(this.get('precision'))) {
         if (this.get('precision') < this.get('min_val'))
-          errmsg = gettext('Scale should not be less than %(value)s', {value: this.get('min_val')});
+          errmsg = gettext('Scale should not be less than %s.', this.get('min_val'));
         if (this.get('precision') > this.get('max_val'))
-          errmsg = gettext('Scale should not be greater than %(value)s', {value: this.get('max_val')});
+          errmsg = gettext('Scale should not be greater than %s.', this.get('max_val'));
           // If we have any error set then throw it to user
         if(errmsg) {
           this.errorModel.set('precision', errmsg);
@@ -269,6 +261,7 @@ define('pgadmin.node.type', [
       collection_type: 'coll-type',
       hasSQL: true,
       hasDepends: true,
+      width: pgBrowser.stdW.md + 'px',
       Init: function() {
         /* Avoid multiple registration of menus */
         if (this.initialized)
@@ -326,7 +319,7 @@ define('pgadmin.node.type', [
           disabled: 'schemaCheck',
         },{
           id: 'oid', label: gettext('OID'), cell: 'string',
-          type: 'text' , mode: ['properties'], disabled: true,
+          type: 'text' , mode: ['properties'],
         },{
           id: 'typeowner', label: gettext('Owner'), cell: 'string',
           control: 'node-list-by-name',
@@ -346,16 +339,16 @@ define('pgadmin.node.type', [
           control: 'node-list-by-name', select2: {allowClear: false},
         },{
           id: 'typtype', label: gettext('Type'),
-          mode: ['create','edit'], disabled: 'inSchemaWithModelCheck',
+          mode: ['create','edit'], disabled: 'inSchema', readonly: 'inEditMode',
           group: gettext('Definition'),
           select2: { allowClear: false },
           options: function() {
             return [
-              {label: 'Composite', value: 'c'},
-              {label: 'Enumeration', value: 'e'},
-              {label: 'External', value: 'b'},
-              {label: 'Range', value: 'r'},
-              {label: 'Shell', value: 'p'},
+              {label: gettext('Composite'), value: 'c'},
+              {label: gettext('Enumeration'), value: 'e'},
+              {label: gettext('External'), value: 'b'},
+              {label: gettext('Range'), value: 'r'},
+              {label: gettext('Shell'), value: 'p'},
             ];
           },
           // If create mode then by default open composite type
@@ -385,11 +378,7 @@ define('pgadmin.node.type', [
           group: gettext('Definition'), mode: ['edit', 'create'],
           canAdd: true, canEdit: false, canDelete: function(m) {
             // We will disable it if it's in 'edit' mode
-            if (m.isNew()) {
-              return true;
-            } else {
-              return false;
-            }
+            return m.isNew();
           },
           disabled: 'inSchema', deps: ['typtype'],
           control: 'unique-col-collection', uniqueCol : ['label'],
@@ -408,7 +397,8 @@ define('pgadmin.node.type', [
             control: 'node-ajax-options',
             select2: { allowClear: true, placeholder: '', width: '100%' },
             url: 'get_stypes', type: 'text', mode: ['properties', 'create', 'edit'],
-            group: gettext('Range Type'), disabled: 'inSchemaWithModelCheck',
+            group: gettext('Range Type'), disabled: 'inSchema',
+            readonly: 'inEditMode',
             transform: function(d, self){
               self.model.subtypes =  d;
               return d;
@@ -416,7 +406,7 @@ define('pgadmin.node.type', [
           },{
             id: 'opcname', label: gettext('Subtype operator class'), cell: 'string',
             mode: ['properties', 'create', 'edit'], group: gettext('Range Type'),
-            disabled: 'inSchemaWithModelCheck', deps: ['typname'],
+            disabled: 'inSchema', readonly: 'inEditMode', deps: ['typname'],
             control: 'select', options: function() {
               var l_typname = this.model.get('typname'),
                 self = this,
@@ -483,7 +473,7 @@ define('pgadmin.node.type', [
             id: 'rngcanonical', label: gettext('Canonical function'), cell: 'string',
             type: 'text', mode: ['properties', 'create', 'edit'],
             group: gettext('Range Type'),
-            disabled: 'inSchemaWithModelCheck', deps: ['name', 'typname'],
+            disabled: 'inSchema', readonly: 'inEditMode', deps: ['name', 'typname'],
             control: 'select', options: function() {
               var name = this.model.get('name'),
                 self = this,
@@ -517,7 +507,7 @@ define('pgadmin.node.type', [
             id: 'rngsubdiff', label: gettext('Subtype diff function'), cell: 'string',
             type: 'text', mode: ['properties', 'create', 'edit'],
             group: gettext('Range Type'),
-            disabled: 'inSchemaWithModelCheck', deps: ['opcname'],
+            disabled: 'inSchema', readonly: 'inEditMode', deps: ['opcname'],
             control: 'select', options: function() {
               var l_typname = this.model.get('typname'),
                 l_opcname = this.model.get('opcname'),
@@ -563,7 +553,7 @@ define('pgadmin.node.type', [
             id: 'typinput', label: gettext('Input function'),
             cell: 'string',type: 'text',
             mode: ['properties', 'create', 'edit'], group: gettext('Required'),
-            disabled: 'inSchemaWithModelCheck',
+            disabled: 'inSchema', readonly: 'inEditMode',
             control: 'node-ajax-options', url: 'get_external_functions',
             transform: 'external_func_combo',
             select2: { allowClear: true, placeholder: '', width: '100%' },
@@ -572,8 +562,8 @@ define('pgadmin.node.type', [
             cell: 'string',
             type: 'text', mode: ['properties', 'create', 'edit'],
             group: gettext('Required'),
-            disabled: 'inSchemaWithModelCheck'
-            ,control: 'node-ajax-options', url: 'get_external_functions',
+            disabled: 'inSchema', readonly: 'inEditMode',
+            control: 'node-ajax-options', url: 'get_external_functions',
             transform: 'external_func_combo',
             select2: { allowClear: true, placeholder: '', width: '100%' },
           },{
@@ -582,23 +572,23 @@ define('pgadmin.node.type', [
             id: 'typreceive', label: gettext('Receive function'),
             cell: 'string', type: 'text', group: gettext('Optional-1'),
             mode: ['properties', 'create', 'edit'],
-            disabled: 'inSchemaWithModelCheck'
-            ,control: 'node-ajax-options', url: 'get_external_functions',
+            disabled: 'inSchema', readonly: 'inEditMode',
+            control: 'node-ajax-options', url: 'get_external_functions',
             transform: 'external_func_combo',
             select2: { allowClear: true, placeholder: '', width: '100%' },
           },{
             id: 'typsend', label: gettext('Send function'),
             cell: 'string', group: gettext('Optional-1'),
             type: 'text', mode: ['properties', 'create', 'edit'],
-            disabled: 'inSchemaWithModelCheck'
-            ,control: 'node-ajax-options', url: 'get_external_functions',
+            disabled: 'inSchema', readonly: 'inEditMode',
+            control: 'node-ajax-options', url: 'get_external_functions',
             transform: 'external_func_combo',
             select2: { allowClear: true, placeholder: '', width: '100%' },
           },{
             id: 'typmodin', label: gettext('Typmod in function'),
             cell: 'string', type: 'text',
             mode: ['properties', 'create', 'edit'], group: gettext('Optional-1'),
-            disabled: 'inSchemaWithModelCheck',
+            disabled: 'inSchema', readonly: 'inEditMode',
             control: 'node-ajax-options', url: 'get_external_functions',
             select2: { allowClear: true, placeholder: '', width: '100%' },
             transform: function(d) {
@@ -615,7 +605,7 @@ define('pgadmin.node.type', [
             id: 'typmodout', label: gettext('Typmod out function'),
             cell: 'string', group: gettext('Optional-1'),
             type: 'text', mode: ['properties', 'create', 'edit'],
-            disabled: 'inSchemaWithModelCheck',
+            disabled: 'inSchema', readonly: 'inEditMode',
             control: 'node-ajax-options', url: 'get_external_functions',
             select2: { allowClear: true, placeholder: '', width: '100%' },
             transform: function(d) {
@@ -632,30 +622,30 @@ define('pgadmin.node.type', [
             id: 'typlen', label: gettext('Internal length'),
             cell: 'integer', group: gettext('Optional-1'),
             type: 'int', mode: ['properties', 'create', 'edit'],
-            disabled: 'inSchemaWithModelCheck',
+            disabled: 'inSchema', readonly: 'inEditMode',
           },{
             id: 'variable', label: gettext('Variable?'), cell: 'switch',
             group: gettext('Optional-1'), type: 'switch',
             mode: ['create','edit'],
-            disabled: 'inSchemaWithModelCheck',
+            disabled: 'inSchema', readonly: 'inEditMode',
           },{
             id: 'typdefault', label: gettext('Default?'),
             cell: 'string', group: gettext('Optional-1'),
             type: 'text', mode: ['properties', 'create','edit'],
-            disabled: 'inSchemaWithModelCheck',
+            disabled: 'inSchema', readonly: 'inEditMode',
           },{
             id: 'typanalyze', label: gettext('Analyze function'),
             cell: 'string', group: gettext('Optional-1'),
             type: 'text', mode: ['properties', 'create','edit'],
-            disabled: 'inSchemaWithModelCheck'
-            ,control: 'node-ajax-options', url: 'get_external_functions',
+            disabled: 'inSchema', readonly: 'inEditMode',
+            control: 'node-ajax-options', url: 'get_external_functions',
             transform: 'external_func_combo',
             select2: { allowClear: true, placeholder: '', width: '100%' },
           },{
             id: 'typcategory', label: gettext('Category type'),
             cell: 'string', group: gettext('Optional-1'),
             type: 'text', mode: ['properties', 'create','edit'],
-            disabled: 'inSchemaWithModelCheck', control: 'select2',
+            disabled: 'inSchema', readonly: 'inEditMode', control: 'select2',
             select2: { allowClear: true, placeholder: '', width: '100%' },
             options: [
               {label :'', value : ''},
@@ -677,7 +667,7 @@ define('pgadmin.node.type', [
           },{
             id: 'typispreferred', label: gettext('Preferred?'), cell: 'switch',
             type: 'switch', mode: ['properties', 'create','edit'],
-            disabled: 'inSchemaWithModelCheck',
+            disabled: 'inSchema', readonly: 'inEditMode',
             group: gettext('Optional-1'),
           },{
             id: 'spacer_ctrl_optional_2', group: gettext('Optional-2'), mode: ['edit', 'create'], type: 'spacer',
@@ -685,16 +675,17 @@ define('pgadmin.node.type', [
             id: 'element', label: gettext('Element type'), cell: 'string',
             control: 'node-ajax-options', group: gettext('Optional-2'),
             type: 'text', mode: ['properties', 'create', 'edit'],
-            disabled: 'inSchemaWithModelCheck', url: 'get_types',
+            disabled: 'inSchema', readonly: 'inEditMode', url: 'get_types',
           },{
             id: 'typdelim', label: gettext('Delimiter'), cell: 'string',
             type: 'text', mode: ['properties', 'create', 'edit'],
-            group: gettext('Optional-2'), disabled: 'inSchemaWithModelCheck',
+            group: gettext('Optional-2'), disabled: 'inSchema',
+            readonly: 'inEditMode',
           },{
             id: 'typalign', label: gettext('Alignment type'),
             cell: 'string', group: gettext('Optional-2'),
             type: 'text', mode: ['properties', 'create', 'edit'],
-            disabled: 'inSchemaWithModelCheck', control: 'select2',
+            disabled: 'inSchema', readonly: 'inEditMode', control: 'select2',
             select2: { allowClear: true, placeholder: '', width: '100%' },
             options: [
               {label :'', value : ''},
@@ -707,7 +698,7 @@ define('pgadmin.node.type', [
             id: 'typstorage', label: gettext('Storage type'),
             type: 'text', mode: ['properties', 'create', 'edit'],
             group: gettext('Optional-2'), cell: 'string',
-            disabled: 'inSchemaWithModelCheck', control: 'select2',
+            disabled: 'inSchema', readonly: 'inEditMode', control: 'select2',
             select2: { allowClear: true, placeholder: '', width: '100%' },
             options: [
               {label :'', value : ''},
@@ -720,12 +711,12 @@ define('pgadmin.node.type', [
             id: 'typbyval', label: gettext('Passed by value?'),
             cell: 'switch',
             type: 'switch', mode: ['properties', 'create', 'edit'],
-            disabled: 'inSchemaWithModelCheck', group: gettext('Optional-2'),
+            disabled: 'inSchema', readonly: 'inEditMode', group: gettext('Optional-2'),
           },{
             id: 'is_collatable', label: gettext('Collatable?'),
             cell: 'switch',  min_version: 90100, group: gettext('Optional-2'),
             type: 'switch', mode: ['properties', 'create', 'edit'],
-            disabled: 'inSchemaWithModelCheck',
+            disabled: 'inSchema', readonly: 'inEditMode',
             // End of extension tab
           }],
         },{
@@ -778,7 +769,7 @@ define('pgadmin.node.type', [
               if(acl.length > 0)
                 acl.reset();
             }
-            return !(m.get('typtype') === 'p');
+            return (m.get('typtype') !== 'p');
           },
         },{
           id: 'seclabels', label: gettext('Security labels'),
@@ -794,7 +785,7 @@ define('pgadmin.node.type', [
               if(secLabs.length > 0)
                 secLabs.reset();
             }
-            return !(m.get('typtype') === 'p');
+            return (m.get('typtype') !== 'p');
           },
         }],
         validate: function() {
@@ -878,6 +869,9 @@ define('pgadmin.node.type', [
           }
           return false;
         },
+        inEditMode: function(m) {
+          return !m.isNew();
+        },
         schemaCheck: function(m) {
           if(this.node_info && 'schema' in this.node_info)
           {
@@ -889,34 +883,6 @@ define('pgadmin.node.type', [
           }
           return true;
         },
-        // We will check if we are under schema node & in 'create' mode
-        inSchemaWithModelCheck: function(m) {
-          if(this.node_info &&  'schema' in this.node_info)
-          {
-            // We will disbale control if it's in 'edit' mode
-            if (m.isNew()) {
-              return false;
-            } else {
-              return true;
-            }
-
-          }
-          return true;
-        },
-        // We want to enable only in edit mode
-        inSchemaWithEditMode: function(m) {
-          if(this.node_info &&  'schema' in this.node_info)
-          {
-            // We will disbale control if it's in 'edit' mode
-            if (m.isNew()) {
-              return true;
-            } else {
-              return false;
-            }
-
-          }
-          return true;
-        },
         // Function will help us to fill combobox
         external_func_combo: function(d) {
           var result = [];
