@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -17,13 +17,13 @@ from pgadmin.utils.route import BaseTestGenerator
 from regression import parent_node_dict
 from regression.python_test_utils import test_utils as utils
 from . import utils as types_utils
+from unittest.mock import patch
 
 
 class TypesDeleteTestCase(BaseTestGenerator):
     """ This class will delete type under schema node. """
-    scenarios = [
-        ('Delete type under schema node', dict(url='/browser/type/obj/'))
-    ]
+    scenarios = utils.generate_scenarios('types_delete',
+                                         types_utils.test_cases)
 
     def setUp(self):
         self.db_name = parent_node_dict["database"][-1]["db_name"]
@@ -46,20 +46,48 @@ class TypesDeleteTestCase(BaseTestGenerator):
                                                self.schema_name, self.type_name
                                                )
 
-    def runTest(self):
-        """ This function will delete type under schema node. """
-        type_response = types_utils.verify_type(self.server, self.db_name,
-                                                self.type_name)
-        if not type_response:
-            raise  Exception("Could not find the type to delete.")
-        response = self.tester.delete(
+    def delete_type(self):
+        """
+        This function deletes type
+        :return: type delete
+        """
+        return self.tester.delete(
             "{0}{1}/{2}/{3}/{4}/{5}".format(self.url, utils.SERVER_GROUP,
                                             self.server_id, self.db_id,
                                             self.schema_id, self.type_id
                                             ),
             follow_redirects=True
         )
-        self.assertEquals(response.status_code, 200)
+
+    def runTest(self):
+        """ This function will delete type under schema node. """
+        type_response = types_utils.verify_type(self.server, self.db_name,
+                                                self.type_name)
+        if not type_response:
+            raise Exception("Could not find the type to delete.")
+
+        if self.is_positive_test:
+            response = self.delete_type()
+
+        else:
+            if hasattr(self, "internal_server_error"):
+                return_value_object = eval(self.mock_data["return_value"])
+                with patch(self.mock_data["function_name"],
+                           side_effect=[return_value_object]):
+                    response = self.delete_type()
+
+            if hasattr(self, "error_in_db"):
+                return_value_object = eval(self.mock_data["return_value"])
+                with patch(self.mock_data["function_name"],
+                           side_effect=[return_value_object]):
+                    response = self.delete_type()
+
+            if hasattr(self, "wrong_type_id"):
+                self.type_id = 99999
+                response = self.delete_type()
+
+        self.assertEqual(response.status_code,
+                         self.expected_data["status_code"])
 
     def tearDown(self):
         # Disconnect the database
