@@ -1,10 +1,19 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define('pgadmin.node.view', [
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
   'sources/pgadmin', 'pgadmin.browser', 'pgadmin.backform',
-  'pgadmin.node.schema.dir/child', 'pgadmin.browser.server.privilege',
-  'pgadmin.node.rule',
+  'pgadmin.node.schema.dir/child', 'pgadmin.node.schema.dir/schema_child_tree_node',
+  'pgadmin.browser.server.privilege', 'pgadmin.node.rule',
 ], function(
-  gettext, url_for, $, _, pgAdmin, pgBrowser, Backform, schemaChild
+  gettext, url_for, $, _, pgAdmin, pgBrowser, Backform, schemaChild, schemaChildTreeNode
 ) {
 
 
@@ -22,6 +31,8 @@ define('pgadmin.node.view', [
         label: gettext('Views'),
         type: 'coll-view',
         columns: ['name', 'owner'],
+        canDrop: schemaChildTreeNode.isTreeItemOfChildOfSchema,
+        canDropCascade: schemaChildTreeNode.isTreeItemOfChildOfSchema,
       });
   }
 
@@ -51,7 +62,7 @@ define('pgadmin.node.view', [
 
         this.initialized = true;
 
-          /**
+        /**
           Add "create view" menu option into context and object menu
           for the following nodes:
           coll-view, view and schema.
@@ -102,7 +113,7 @@ define('pgadmin.node.view', [
           type: 'text', disabled: 'notInSchema',
         },{
           id: 'oid', label: gettext('OID'), cell: 'string',
-          type: 'text', disabled: true, mode: ['properties'],
+          type: 'text', mode: ['properties'],
         },{
           id: 'owner', label: gettext('Owner'), cell: 'string', control: 'node-list-by-name',
           node: 'role', disabled: 'notInSchema', select2: { allowClear: false },
@@ -113,7 +124,7 @@ define('pgadmin.node.view', [
           select2: { allowClear: false }, cache_node: 'database',
         },{
           id: 'system_view', label: gettext('System view?'), cell: 'string',
-          type: 'switch', disabled: true, mode: ['properties'],
+          type: 'switch', mode: ['properties'],
         },{
           id: 'acl', label: gettext('Privileges'),
           mode: ['properties'], type: 'text', group: gettext('Security'),
@@ -121,7 +132,7 @@ define('pgadmin.node.view', [
           id: 'comment', label: gettext('Comment'), cell: 'string',
           type: 'multiline', disabled: 'notInSchema',
         },{
-          id: 'security_barrier', label: gettext('Security barrier'),
+          id: 'security_barrier', label: gettext('Security barrier?'),
           type: 'switch', min_version: '90200', group: gettext('Definition'),
           disabled: 'notInSchema',
         },{
@@ -140,10 +151,53 @@ define('pgadmin.node.view', [
             label: gettext('Cascaded'), value: 'cascaded',
           }],
         },{
-          id: 'definition', label: gettext('Definition'), cell: 'string',
-          type: 'text', mode: ['create', 'edit'], group: gettext('Definition'),
-          control: Backform.SqlFieldControl,
+          id: 'definition', label: gettext('Code'), cell: 'string',
+          type: 'text', mode: ['create', 'edit'], group: gettext('Code'),
+          tabPanelCodeClass: 'sql-code-control',
           disabled: 'notInSchema',
+          control: Backform.SqlCodeControl.extend({
+            onChange: function() {
+              Backform.SqlCodeControl.prototype.onChange.apply(this, arguments);
+
+              if (!this.model || !(
+                this.model.changed &&
+                this.model.node_info.server.server_type == 'pg' &&
+                // No need to check this when creating a view
+                this.model.get('oid') !== undefined
+              ) || !(
+                this.model.origSessAttrs &&
+                this.model.changed.definition != this.model.origSessAttrs.definition
+              )) {
+                this.model.warn_text = undefined;
+                return;
+              }
+
+              let old_def = this.model.origSessAttrs.definition &&
+                this.model.origSessAttrs.definition.replace(
+                  /\s/gi, ''
+                ).split('FROM'),
+                new_def = [];
+
+              if (this.model.changed.definition !== undefined) {
+                new_def = this.model.changed.definition.replace(
+                  /\s/gi, ''
+                ).split('FROM');
+              }
+
+              if ((old_def.length != new_def.length) || (
+                old_def.length > 1 && (
+                  old_def[0] != new_def[0]
+                )
+              )) {
+                this.model.warn_text = gettext(
+                  'Changing the columns in a view requires dropping and re-creating the view. This may fail if other objects are dependent upon this view, or may cause procedural functions to fail if they are not modified to take account of the changes.'
+                ) + '<br><br><b>' + gettext('Do you wish to continue?') +
+                '</b>';
+              } else {
+                this.model.warn_text = undefined;
+              }
+            },
+          }),
         }, pgBrowser.SecurityGroupSchema, {
           // Add Privilege Control
           id: 'datacl', label: gettext('Privileges'), type: 'collection',
@@ -169,7 +223,7 @@ define('pgadmin.node.view', [
           if (_.isUndefined(field_name) || _.isNull(field_name) ||
             String(field_name).replace(/^\s+|\s+$/g, '') == '') {
             err['name'] = gettext('Please specify name.');
-            errmsg = errmsg || err['name'];
+            errmsg = err['name'];
             this.errorModel.set('name', errmsg);
             return errmsg;
           }else{
@@ -177,8 +231,8 @@ define('pgadmin.node.view', [
           }
           if (_.isUndefined(field_def) || _.isNull(field_def) ||
             String(field_def).replace(/^\s+|\s+$/g, '') == '') {
-            err['definition'] = gettext('Please enter view definition.');
-            errmsg = errmsg || err['definition'];
+            err['definition'] = gettext('Please enter view code.');
+            errmsg = err['definition'];
             this.errorModel.set('definition', errmsg);
             return errmsg;
           }else{
