@@ -10,19 +10,17 @@ SELECT
     tup_inserted AS {{ conn|qtIdent(_('Tuples inserted')) }},
     tup_updated AS {{ conn|qtIdent(_('Tuples updated')) }},
     tup_deleted AS {{ conn|qtIdent(_('Tuples deleted')) }},
-    stats_reset AS {{ conn|qtIdent(_('Last statistics reset')) }},
-    slave.confl_tablespace AS {{ conn|qtIdent(_('Tablespace conflicts')) }},
-    slave.confl_lock AS {{ conn|qtIdent(_('Lock conflicts')) }},
-    slave.confl_snapshot AS {{ conn|qtIdent(_('Snapshot conflicts')) }},
-    slave.confl_bufferpin AS {{ conn|qtIdent(_('Bufferpin conflicts')) }},
-    slave.confl_deadlock AS {{ conn|qtIdent(_('Deadlock conflicts')) }},
-    pg_size_pretty(pg_database_size(db.datid)) AS {{ conn|qtIdent(_('Size')) }}
+    pg_database_size(db.datid) AS {{ conn|qtIdent(_('Size')) }}
 FROM
     pg_stat_database db
-    LEFT JOIN pg_stat_database_conflicts slave ON db.datid=slave.datid
 WHERE {% if did %}
 db.datid = {{ did|qtLiteral }}::OID{% else %}
 db.datid > {{ last_system_oid|qtLiteral }}::OID
 {% endif %}
+{% if db_restrictions %}
+
+AND
+db.datname in ({{db_restrictions}})
+{% endif %}
 
 ORDER BY db.datname;
