@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -44,7 +44,7 @@ class DatabaseMultipleDeleteTestCase(BaseTestGenerator):
                 follow_redirects=True,
                 data=json.dumps(data),
                 content_type='html/json')
-            self.assertEquals(response.status_code, 200)
+            self.assertEqual(response.status_code, 200)
         else:
             raise Exception("Could not connect to server to delete the "
                             "database.")
