@@ -2,12 +2,12 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
-from flask_babel import gettext
+from flask_babelex import gettext
 from pgadmin.browser.server_groups.servers.types import ServerType
 
 
@@ -22,8 +22,7 @@ class GPDB(ServerType):
     def icon(self):
         return "gpdb.png"
 
-
-    def instanceOf(self, ver):
+    def instance_of(self, ver):
         return "Greenplum Database" in ver
 
 
