@@ -2,23 +2,25 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
 """Implements pgAgent Job Schedule Node"""
 
-import json
+import simplejson as json
 from functools import wraps
 
 from flask import render_template, request, jsonify
-from flask_babel import gettext
+from flask_babelex import gettext
 from pgadmin.browser.collection import CollectionNodeModule
 from pgadmin.browser.utils import PGChildNodeView
 from pgadmin.utils.ajax import make_json_response, gone, \
     make_response as ajax_response, internal_server_error
 from pgadmin.utils.driver import get_driver
+from pgadmin.browser.server_groups.servers.pgagent.utils \
+    import format_schedule_data
 
 from config import PG_DEFAULT_DRIVER
 
@@ -38,8 +40,8 @@ class JobScheduleModule(CollectionNodeModule):
       - Method is overridden from its base class to make the node as leaf node.
     """
 
-    NODE_TYPE = 'pga_schedule'
-    COLLECTION_LABEL = gettext("Schedules")
+    _NODE_TYPE = 'pga_schedule'
+    _COLLECTION_LABEL = gettext("Schedules")
 
     def get_nodes(self, gid, sid, jid):
         """
@@ -71,6 +73,23 @@ class JobScheduleModule(CollectionNodeModule):
         """
         return 'pga_job'
 
+    @property
+    def csssnippets(self):
+        """
+        Returns a snippet of css to include in the page
+        """
+        snippets = [
+            render_template(
+                "pga_schedule/css/pga_schedule.css",
+                node_type=self.node_type
+            )
+        ]
+
+        for submodule in self.submodules:
+            snippets.extend(submodule.csssnippets)
+
+        return snippets
+
     @property
     def module_use_template_javascript(self):
         """
@@ -140,7 +159,7 @@ class JobScheduleView(PGChildNodeView):
     operations = dict({
         'obj': [
             {'get': 'properties', 'put': 'update', 'delete': 'delete'},
-            {'get': 'list', 'post': 'create'}
+            {'get': 'list', 'post': 'create', 'delete': 'delete'}
         ],
         'nodes': [{'get': 'nodes'}, {'get': 'nodes'}],
         'msql': [{'get': 'msql'}, {'get': 'msql'}],
@@ -195,7 +214,7 @@ class JobScheduleView(PGChildNodeView):
             jid: Job ID
         """
         sql = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             jid=jid
         )
         status, res = self.conn.execute_dict(sql)
@@ -221,7 +240,7 @@ class JobScheduleView(PGChildNodeView):
         """
         res = []
         sql = render_template(
-            "/".join([self.template_path, 'nodes.sql']),
+            "/".join([self.template_path, self._NODES_SQL]),
             jscid=jscid,
             jid=jid
         )
@@ -243,7 +262,8 @@ class JobScheduleView(PGChildNodeView):
                     row['jscid'],
                     row['jscjobid'],
                     row['jscname'],
-                    icon="icon-pga_schedule",
+                    icon="icon-pga_schedule" if row['jscenabled'] else
+                    "icon-pga_schedule-disabled",
                     enabled=row['jscenabled']
                 )
             )
@@ -254,7 +274,8 @@ class JobScheduleView(PGChildNodeView):
                     row['jscid'],
                     row['jscjobid'],
                     row['jscname'],
-                    icon="icon-pga_schedule",
+                    icon="icon-pga_schedule" if row['jscenabled'] else
+                    "icon-pga_schedule-disabled",
                     enabled=row['jscenabled']
                 )
             )
@@ -276,7 +297,7 @@ class JobScheduleView(PGChildNodeView):
             jscid: JobSchedule ID
         """
         sql = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             jscid=jscid, jid=jid
         )
         status, res = self.conn.execute_dict(sql)
@@ -294,20 +315,6 @@ class JobScheduleView(PGChildNodeView):
             status=200
         )
 
-    @staticmethod
-    def format_list_data(value):
-        """
-        Converts to proper array data for sql
-        Args:
-            value: data to be converted
-
-        Returns:
-            Converted data
-        """
-        if not isinstance(value, list):
-            return value.replace("[", "{").replace("]", "}")
-        return value
-
     @check_precondition
     def create(self, gid, sid, jid):
         """
@@ -318,39 +325,15 @@ class JobScheduleView(PGChildNodeView):
             sid: Server ID
             jid: Job ID
         """
-        data = {}
-        if request.args:
-            for k, v in request.args.items():
-                try:
-                    data[k] = json.loads(
-                        v.decode('utf-8') if hasattr(v, 'decode') else v
-                    )
-                except ValueError:
-                    data[k] = v
-        else:
-            data = json.loads(request.data.decode())
-            # convert python list literal to postgres array literal.
-            data['jscminutes'] = JobScheduleView.format_list_data(
-                data['jscminutes']
-            )
-            data['jschours'] = JobScheduleView.format_list_data(
-                data['jschours']
-            )
-            data['jscweekdays'] = JobScheduleView.format_list_data(
-                data['jscweekdays']
-            )
-            data['jscmonthdays'] = JobScheduleView.format_list_data(
-                data['jscmonthdays']
-            )
-            data['jscmonths'] = JobScheduleView.format_list_data(
-                data['jscmonths']
-            )
+        data = json.loads(request.data, encoding='utf-8')
+        # convert python list literal to postgres array literal.
+        format_schedule_data(data)
 
         sql = render_template(
-            "/".join([self.template_path, 'create.sql']),
+            "/".join([self.template_path, self._CREATE_SQL]),
             jid=jid,
             data=data,
-            fetch_id=False
+            fetch_id=True
         )
 
         status, res = self.conn.execute_void('BEGIN')
@@ -358,7 +341,6 @@ class JobScheduleView(PGChildNodeView):
             return internal_server_error(errormsg=res)
 
         status, res = self.conn.execute_scalar(sql)
-
         if not status:
             if self.conn.connected():
                 self.conn.execute_void('END')
@@ -366,7 +348,7 @@ class JobScheduleView(PGChildNodeView):
 
         self.conn.execute_void('END')
         sql = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             jscid=res,
             jid=jid
         )
@@ -386,12 +368,12 @@ class JobScheduleView(PGChildNodeView):
                 row['jscid'],
                 row['jscjobid'],
                 row['jscname'],
-                icon="icon-pga_schedule",
+                icon="icon-pga_schedule" if row['jscenabled'] else
+                "icon-pga_schedule-disabled",
                 enabled=row['jscenabled']
             )
         )
 
-
     @check_precondition
     def update(self, gid, sid, jid, jscid):
         """
@@ -415,29 +397,10 @@ class JobScheduleView(PGChildNodeView):
         else:
             data = json.loads(request.data.decode())
             # convert python list literal to postgres array literal.
-            if 'jscminutes' in data and data['jscminutes'] is not None:
-                data['jscminutes'] = JobScheduleView.format_list_data(
-                    data['jscminutes']
-                )
-            if 'jschours' in data and data['jschours'] is not None:
-                data['jschours'] = JobScheduleView.format_list_data(
-                    data['jschours']
-                )
-            if 'jscweekdays' in data and data['jscweekdays'] is not None:
-                data['jscweekdays'] = JobScheduleView.format_list_data(
-                    data['jscweekdays']
-                )
-            if 'jscmonthdays' in data and data['jscmonthdays'] is not None:
-                data['jscmonthdays'] = JobScheduleView.format_list_data(
-                    data['jscmonthdays']
-                )
-            if 'jscmonths' in data and data['jscmonths'] is not None:
-                data['jscmonths'] = JobScheduleView.format_list_data(
-                    data['jscmonths']
-                )
+            format_schedule_data(data)
 
         sql = render_template(
-            "/".join([self.template_path, 'update.sql']),
+            "/".join([self.template_path, self._UPDATE_SQL]),
             jid=jid,
             jscid=jscid,
             data=data
@@ -449,7 +412,7 @@ class JobScheduleView(PGChildNodeView):
             return internal_server_error(errormsg=res)
 
         sql = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             jscid=jscid,
             jid=jid
         )
@@ -469,23 +432,32 @@ class JobScheduleView(PGChildNodeView):
                 jscid,
                 jid,
                 row['jscname'],
-                icon="icon-pga_schedule",
+                icon="icon-pga_schedule" if row['jscenabled'] else
+                "icon-pga_schedule-disabled",
                 enabled=row['jscenabled']
             )
         )
 
     @check_precondition
-    def delete(self, gid, sid, jid, jscid):
+    def delete(self, gid, sid, jid, jscid=None):
         """Delete the Job Schedule."""
 
-        status, res = self.conn.execute_void(
-            render_template(
-                "/".join([self.template_path, 'delete.sql']),
-                jid=jid, jscid=jscid, conn=self.conn
+        if jscid is None:
+            data = request.form if request.form else json.loads(
+                request.data, encoding='utf-8'
             )
-        )
-        if not status:
-            return internal_server_error(errormsg=res)
+        else:
+            data = {'ids': [jscid]}
+
+        for jscid in data['ids']:
+            status, res = self.conn.execute_void(
+                render_template(
+                    "/".join([self.template_path, self._DELETE_SQL]),
+                    jid=jid, jscid=jscid, conn=self.conn
+                )
+            )
+            if not status:
+                return internal_server_error(errormsg=res)
 
         return make_json_response(success=1)
 
@@ -513,14 +485,14 @@ class JobScheduleView(PGChildNodeView):
 
         if jscid is None:
             sql = render_template(
-                "/".join([self.template_path, 'create.sql']),
+                "/".join([self.template_path, self._CREATE_SQL]),
                 jid=jid,
                 data=data,
                 fetch_id=False
             )
         else:
             sql = render_template(
-                "/".join([self.template_path, 'update.sql']),
+                "/".join([self.template_path, self._UPDATE_SQL]),
                 jid=jid,
                 jscid=jscid,
                 data=data
