@@ -8,4 +8,5 @@ FROM
     pgagent.pga_joblog
 WHERE
     jlgjobid = {{ jid|qtLiteral }}::integer
-ORDER BY jlgid DESC;
+ORDER BY jlgid DESC
+LIMIT {{ rows_threshold }};
