@@ -2,12 +2,12 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
-from flask_babel import gettext
+from flask_babelex import gettext
 from pgadmin.browser.server_groups.servers.types import ServerType
 
 
@@ -18,8 +18,8 @@ class PPAS(ServerType):
         " programs (pg_dump, pg_restore etc)."
     )
 
-    def instanceOf(self, ver):
-        return ver.startswith("EnterpriseDB")
+    def instance_of(self, ver):
+        return "EnterpriseDB" in ver
 
 
 # Default Server Type
