@@ -1,3 +1,12 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define('pgadmin.node.resource_group', [
   'sources/gettext', 'sources/url_for', 'underscore', 'pgadmin.browser',
   'pgadmin.browser.collection',
@@ -21,7 +30,7 @@ define('pgadmin.node.resource_group', [
     pgBrowser.Nodes['resource_group'] = pgBrowser.Node.extend({
       parent_type: 'server',
       type: 'resource_group',
-      dialogHelp: url_for('help.static', {'ilename': 'resource_group_dialog.html'}),
+      dialogHelp: url_for('help.static', {'filename': 'resource_group_dialog.html'}),
       label: gettext('Resource Group'),
       hasSQL:  true,
       canDrop: true,
@@ -69,6 +78,7 @@ define('pgadmin.node.resource_group', [
         defaults: {
           oid: undefined,
           name: undefined,
+          is_sys_obj: undefined,
           cpu_rate_limit: 0.0,
           dirty_rate_limit: 0.0,
         },
@@ -76,12 +86,15 @@ define('pgadmin.node.resource_group', [
         // Defining schema for the resource group node
         schema: [{
           id: 'oid', label: gettext('OID'), type: 'text',
-          editable: false, disabled: true, mode:['properties'],
+          editable: false, mode:['properties'],
         },{
           id: 'name', label: gettext('Name'), cell: 'string',
           type: 'text',
         },{
-          id: 'cpu_rate_limit', label: gettext('CPU rate limit (%%)'), cell: 'string',
+          id: 'is_sys_obj', label: gettext('System resource group?'),
+          cell:'boolean', type: 'switch', mode: ['properties'],
+        },{
+          id: 'cpu_rate_limit', label: gettext('CPU rate limit (percentage)'), cell: 'string',
           type: 'numeric', min:0, max:16777216,
         },{
           id: 'dirty_rate_limit', label: gettext('Dirty rate limit (KB)'), cell: 'string',
