@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -52,7 +52,7 @@ class ResourceGroupsDeleteTestCase(BaseTestGenerator):
             "{0}{1}/{2}/{3}".format(self.url, utils.SERVER_GROUP,
                                     self.server_id, self.resource_group_id),
             follow_redirects=True)
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
 
     def tearDown(self):
         """This function delete the resource group from the database."""
