@@ -1,6 +1,15 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
-  'backgrid', 'alertify', 'pgadmin.browser.node', 'pgadmin.browser.node.ui',
-], function(gettext, _, $, Backbone, Backform, Backgrid, Alertify, pgNode) {
+  'backgrid', 'alertify', 'pgadmin.browser.node', 'sources/utils', 'pgadmin.browser.node.ui',
+], function(gettext, _, $, Backbone, Backform, Backgrid, Alertify, pgNode, commonUtils) {
   /**
    * Each Privilege, supporeted by an database object, will be represented
    * using this Model.
@@ -95,11 +104,11 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
           }
 
           var rerender = function (m) {
-            var self = this;
+            var _self = this;
             if ('grantee' in m.changed && this.model.cid != m.cid) {
               setTimeout(
                 function() {
-                  self.render();
+                  _self.render();
                 }, 50
               );
             }
@@ -124,8 +133,8 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
             // it is not loaded yet
             node_info = (_.has(model.top, 'node_info')
               && !_.isUndefined(model.top.node_info)) ?
-            model.top.node_info :
-            model.handler.top.node_info,
+              model.top.node_info :
+              model.handler.top.node_info,
             curr_user = node_info.server.user.name;
 
           model.collection.each(function(m) {
@@ -160,7 +169,7 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
         );
       },
     },{
-      id: 'grantor', label: gettext('Grantor'), type: 'text', disabled: true,
+      id: 'grantor', label: gettext('Grantor'), type: 'text', readonly: true,
       cell: 'node-list-by-name', node: 'role',
     }],
 
@@ -273,11 +282,11 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
         this.attributes['privileges']) {
         var anyPrivSelected = false;
         this.attributes['privileges'].each(
-            function(p) {
-              if (p.get('privilege')) {
-                anyPrivSelected = true;
-              }
-            });
+          function(p) {
+            if (p.get('privilege')) {
+              anyPrivSelected = true;
+            }
+          });
 
         if (!anyPrivSelected) {
           msg = gettext('At least one privilege should be selected.');
@@ -319,22 +328,27 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
       template: _.template([
         '<tr class="<%= header ? "header" : "" %>">',
         ' <td class="renderable">',
-        '  <label class="privilege_label">',
-        '   <input type="checkbox" name="privilege" privilege="<%- privilege_type %>" target="<%- target %>" <%= privilege ? \'checked\' : "" %>></input>',
-        '   <%- privilege_label %>',
-        '  </label>',
+        '  <div class="custom-control custom-checkbox privilege-checkbox">',
+        '    <input tabindex="0" type="checkbox" class="custom-control-input" id="<%= checkbox_id %>" name="privilege" privilege="<%- privilege_type %>" target="<%- target %>" <%= privilege ? \'checked\' : "" %>/>',
+        '    <label class="custom-control-label" for="<%= checkbox_id %>">',
+        '      <%- privilege_label %>',
+        '    </label>',
+        '  </div>',
         ' </td>',
         ' <td class="renderable">',
-        '  <label class="privilege_label">',
-        '   <input type="checkbox" name="with_grant" privilege="<%- privilege_type %>" target="<%- target %>" <%= with_grant ? \'checked\' : "" %> <%= enable_with_grant ? "" : \'disabled\'%>></input>',
-        '   WITH GRANT OPTION',
-        '  </label>',
+        '  <div class="custom-control custom-checkbox privilege-checkbox">',
+        '    <input tabindex="0" type="checkbox" class="custom-control-input" id="wgo_<%= checkbox_id %>" name="with_grant" privilege="<%- privilege_type %>" target="<%- target %>" <%= with_grant ? \'checked\' : "" %> <%= enable_with_grant ? "" : \'disabled\'%>/>',
+        '    <label class="custom-control-label" for="wgo_<%= checkbox_id %>">',
+        '      WITH GRANT OPTION',
+        '    </label>',
+        '  </div>',
         ' </td>',
         '</tr>'].join(' '), null, {variable: null}),
 
       events: {
         'change': 'privilegeChanged',
         'blur': 'lostFocus',
+        'keydown': 'lostFocus',
       },
 
       render: function () {
@@ -343,12 +357,13 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
         this.$el.attr('target', this.elId);
 
         var collection = this.model.get(this.column.get('name')),
-          tbl = $('<table></table>').appendTo(this.$el),
+          tbl = $('<table aria-label='+this.column.get('label')+'></table>').appendTo(this.$el),
           self = this,
           privilege = true, with_grant = true;
 
         // For each privilege generate html template.
         // List down all the Privilege model.
+        var checkbox_id = _.uniqueId();
         collection.each(function(m) {
           var d = m.toJSON();
 
@@ -362,6 +377,7 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
               'privilege_label': self.Labels[d.privilege_type],
               'with_grant': (self.model.get('grantee') != 'PUBLIC' && d.with_grant),
               'enable_with_grant': (self.model.get('grantee') != 'PUBLIC' && d.privilege),
+              'checkbox_id': d.privilege_type + '' + checkbox_id,
             });
           privilege = (privilege && d.privilege);
           with_grant = (with_grant && privilege && d.with_grant);
@@ -379,6 +395,7 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
               'with_grant': (self.model.get('grantee') != 'PUBLIC' && with_grant),
               'enable_with_grant': (self.model.get('grantee') != 'PUBLIC' && privilege),
               'header': true,
+              'checkbox_id': 'all' + '' + checkbox_id,
             }));
         }
         self.$el.find('input[type=checkbox]').first().trigger('focus');
@@ -462,8 +479,8 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
                  * 3. Deselect and disable the checkbox for ALL with grant privilege.
                  */
                 $allPrivileges.prop('checked', false);
-                $elGrant.prop('checked', false),
-                  $allGrants.prop('checked', false);
+                $elGrant.prop('checked', false);
+                $allGrants.prop('checked', false);
                 $elGrant.prop('disabled', true);
                 $allGrants.prop('disabled', true);
               }
@@ -536,8 +553,8 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
 
                 if (type == 'with_grant') {
                   $allGrants = $tbl.find(
-                      'input[name=with_grant][privilege!=\'ALL\']:checked'
-                    );
+                    'input[name=with_grant][privilege!=\'ALL\']:checked'
+                  );
                   if ($allGrants.length == collection.models.length) {
                     $allGrants.prop('disabled', false);
                     $allGrants.prop('checked', true);
@@ -599,7 +616,33 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
               node = node.parentNode;
             }
             return false;
-          };
+          },
+          model = this.model,
+          column = this.column,
+          command = new Backgrid.Command(ev),
+          coll = this.model.get(this.column.get('name'));
+
+        if (ev.key == 'Tab'){
+          commonUtils.handleKeyNavigation(event);
+        }
+
+        if (command.moveUp() || command.moveDown() || command.save() || command.cancel() ||
+          (command.moveLeft() && ev.target.name === 'privilege' && $(ev.target).attr('privilege') === 'ALL')) {
+          // undo
+          ev.stopPropagation();
+          model.trigger('backgrid:edited', model, column, command);
+          return;
+        } else if (command.moveRight()) {
+          // If we are at the last privilege then we should move to next cell
+          if (coll.last().get('privilege_type') === $(ev.target).attr('privilege')) {
+            if ((ev.target.name === 'privilege' && !ev.target.checked ) ||
+              $(ev.target).attr('name') === 'with_grant') {
+              ev.stopPropagation();
+              model.trigger('backgrid:edited', model, column, command);
+              return;
+            }
+          }
+        }
 
         /*
          * Between leaving the old element focus and entering the new element focus the
@@ -621,7 +664,6 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
             var m = self.model;
             m.trigger('backgrid:edited', m, self.column, new Backgrid.Command(ev));
           }},10);
-        return;
       },
     });
 
@@ -691,6 +733,30 @@ define(['sources/gettext', 'underscore', 'jquery', 'backbone', 'backform',
         }
       });
     },
+
+    events: {
+      'click': 'enterEditMode',
+      'keydown': 'saveOrCancel',
+    },
+
+    saveOrCancel: function (e) {
+      var model = this.model;
+      var column = this.column;
+      var command = new Backgrid.Command(e);
+
+      if (command.moveUp() || command.moveDown() || command.moveLeft() || command.moveRight() ||
+        command.save()) {
+        e.preventDefault();
+        e.stopPropagation();
+        model.trigger('backgrid:edited', model, column, command);
+      }
+      // esc
+      else if (command.cancel()) {
+      // undo
+        e.stopPropagation();
+        model.trigger('backgrid:edited', model, column, command);
+      }
+    },
   });
 
   return PrivilegeRoleModel;
