@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -22,15 +22,11 @@ from pgadmin.utils.ajax import make_json_response, \
 from pgadmin.utils.ajax import precondition_required
 from pgadmin.utils.driver import get_driver
 from config import PG_DEFAULT_DRIVER
-from pgadmin.utils import IS_PY2
-# If we are in Python3
-if not IS_PY2:
-    unicode = str
 
 
 class TablespaceModule(CollectionNodeModule):
-    NODE_TYPE = 'tablespace'
-    COLLECTION_LABEL = gettext("Tablespaces")
+    _NODE_TYPE = 'tablespace'
+    _COLLECTION_LABEL = gettext("Tablespaces")
 
     def __init__(self, import_name, **kwargs):
         super(TablespaceModule, self).__init__(import_name, **kwargs)
@@ -48,7 +44,7 @@ class TablespaceModule(CollectionNodeModule):
         Load the module script for server, when any of the server-group node is
         initialized.
         """
-        return servers.ServerModule.NODE_TYPE
+        return servers.ServerModule.node_type
 
     @property
     def module_use_template_javascript(self):
@@ -89,25 +85,11 @@ class TablespaceView(PGChildNodeView):
         'stats': [{'get': 'statistics'}, {'get': 'statistics'}],
         'dependency': [{'get': 'dependencies'}],
         'dependent': [{'get': 'dependents'}],
-        'module.js': [{}, {}, {'get': 'module_js'}],
         'vopts': [{}, {'get': 'variable_options'}],
         'move_objects': [{'put': 'move_objects'}],
         'move_objects_sql': [{'get': 'move_objects_sql'}],
     })
 
-    def module_js(self):
-        """
-        This property defines (if javascript) exists for this node.
-        Override this property for your own logic.
-        """
-        return make_response(
-            render_template(
-                "tablespaces/js/tablespaces.js",
-                _=gettext
-            ),
-            200, {'Content-Type': 'application/x-javascript'}
-        )
-
     def check_precondition(f):
         """
         This function will behave as a decorator which will checks
@@ -125,6 +107,18 @@ class TablespaceView(PGChildNodeView):
                 kwargs['sid']
             )
             self.conn = self.manager.connection()
+            self.datlastsysoid = \
+                self.manager.db_info[self.manager.did]['datlastsysoid'] \
+                if self.manager.db_info is not None and \
+                self.manager.did in self.manager.db_info else 0
+            self.datistemplate = False
+            if (
+                self.manager.db_info is not None and
+                self.manager.did in self.manager.db_info and
+                'datistemplate' in self.manager.db_info[self.manager.did]
+            ):
+                self.datistemplate = self.manager.db_info[
+                    self.manager.did]['datistemplate']
 
             # If DB not connected then return error to browser
             if not self.conn.connected():
@@ -153,7 +147,7 @@ class TablespaceView(PGChildNodeView):
     @check_precondition
     def list(self, gid, sid):
         SQL = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             conn=self.conn
         )
         status, res = self.conn.execute_dict(SQL)
@@ -168,7 +162,7 @@ class TablespaceView(PGChildNodeView):
     @check_precondition
     def node(self, gid, sid, tsid):
         SQL = render_template(
-            "/".join([self.template_path, 'nodes.sql']),
+            "/".join([self.template_path, self._NODES_SQL]),
             tsid=tsid, conn=self.conn
         )
         status, rset = self.conn.execute_2darray(SQL)
@@ -194,7 +188,7 @@ class TablespaceView(PGChildNodeView):
     def nodes(self, gid, sid, tsid=None):
         res = []
         SQL = render_template(
-            "/".join([self.template_path, 'nodes.sql']),
+            "/".join([self.template_path, self._NODES_SQL]),
             tsid=tsid, conn=self.conn
         )
         status, rset = self.conn.execute_2darray(SQL)
@@ -244,7 +238,7 @@ class TablespaceView(PGChildNodeView):
 
         # We need to parse & convert ACL coming from database to json format
         SQL = render_template(
-            "/".join([self.template_path, 'acl.sql']),
+            "/".join([self.template_path, self._ACL_SQL]),
             tsid=tsid, conn=self.conn
         )
         status, acl = self.conn.execute_dict(SQL)
@@ -267,7 +261,7 @@ class TablespaceView(PGChildNodeView):
     @check_precondition
     def properties(self, gid, sid, tsid):
         SQL = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             tsid=tsid, conn=self.conn
         )
         status, res = self.conn.execute_dict(SQL)
@@ -281,6 +275,8 @@ class TablespaceView(PGChildNodeView):
 
         # Making copy of output for future use
         copy_data = dict(res['rows'][0])
+        copy_data['is_sys_obj'] = (
+            copy_data['oid'] <= self.datlastsysoid or self.datistemplate)
         copy_data = self._formatter(copy_data, tsid)
 
         return ajax_response(
@@ -309,9 +305,8 @@ class TablespaceView(PGChildNodeView):
                     status=410,
                     success=0,
                     errormsg=gettext(
-                        "Could not find the required parameter (%s)." %
-                        required_args[arg]
-                    )
+                        "Could not find the required parameter ({})."
+                    ).format(arg)
                 )
 
         # To format privileges coming from client
@@ -320,7 +315,7 @@ class TablespaceView(PGChildNodeView):
 
         try:
             SQL = render_template(
-                "/".join([self.template_path, 'create.sql']),
+                "/".join([self.template_path, self._CREATE_SQL]),
                 data=data, conn=self.conn
             )
 
@@ -329,7 +324,7 @@ class TablespaceView(PGChildNodeView):
             if not status:
                 return internal_server_error(errormsg=res)
             SQL = render_template(
-                "/".join([self.template_path, 'alter.sql']),
+                "/".join([self.template_path, self._ALTER_SQL]),
                 data=data, conn=self.conn
             )
 
@@ -341,7 +336,7 @@ class TablespaceView(PGChildNodeView):
 
             # To fetch the oid of newly created tablespace
             SQL = render_template(
-                "/".join([self.template_path, 'alter.sql']),
+                "/".join([self.template_path, self._ALTER_SQL]),
                 tablespace=data['name'], conn=self.conn
             )
 
@@ -374,7 +369,7 @@ class TablespaceView(PGChildNodeView):
         try:
             SQL, name = self.get_sql(gid, sid, data, tsid)
             # Most probably this is due to error
-            if not isinstance(SQL, (str, unicode)):
+            if not isinstance(SQL, str):
                 return SQL
 
             SQL = SQL.strip('\n').strip(' ')
@@ -411,7 +406,7 @@ class TablespaceView(PGChildNodeView):
                 # Get name for tablespace from tsid
                 status, rset = self.conn.execute_dict(
                     render_template(
-                        "/".join([self.template_path, 'nodes.sql']),
+                        "/".join([self.template_path, self._NODES_SQL]),
                         tsid=tsid, conn=self.conn
                     )
                 )
@@ -432,7 +427,7 @@ class TablespaceView(PGChildNodeView):
 
                 # drop tablespace
                 SQL = render_template(
-                    "/".join([self.template_path, 'delete.sql']),
+                    "/".join([self.template_path, self._DELETE_SQL]),
                     tsname=(rset['rows'][0])['name'], conn=self.conn
                 )
 
@@ -457,14 +452,19 @@ class TablespaceView(PGChildNodeView):
         data = dict()
         for k, v in request.args.items():
             try:
-                data[k] = json.loads(v, encoding='utf-8')
+                # comments should be taken as is because if user enters a
+                # json comment it is parsed by loads which should not happen
+                if k in ('description',):
+                    data[k] = v
+                else:
+                    data[k] = json.loads(v, encoding='utf-8')
             except ValueError as ve:
                 current_app.logger.exception(ve)
                 data[k] = v
 
         sql, name = self.get_sql(gid, sid, data, tsid)
         # Most probably this is due to error
-        if not isinstance(sql, (str, unicode)):
+        if not isinstance(sql, str):
             return sql
 
         sql = sql.strip('\n').strip(' ')
@@ -475,6 +475,22 @@ class TablespaceView(PGChildNodeView):
             status=200
         )
 
+    def _format_privilege_data(self, data):
+        for key in ['spcacl']:
+            if key in data and data[key] is not None:
+                if 'added' in data[key]:
+                    data[key]['added'] = parse_priv_to_db(
+                        data[key]['added'], self.acl
+                    )
+                if 'changed' in data[key]:
+                    data[key]['changed'] = parse_priv_to_db(
+                        data[key]['changed'], self.acl
+                    )
+                if 'deleted' in data[key]:
+                    data[key]['deleted'] = parse_priv_to_db(
+                        data[key]['deleted'], self.acl
+                    )
+
     def get_sql(self, gid, sid, data, tsid=None):
         """
         This function will genrate sql from model/properties data
@@ -485,7 +501,7 @@ class TablespaceView(PGChildNodeView):
 
         if tsid is not None:
             SQL = render_template(
-                "/".join([self.template_path, 'properties.sql']),
+                "/".join([self.template_path, self._PROPERTIES_SQL]),
                 tsid=tsid, conn=self.conn
             )
             status, res = self.conn.execute_dict(SQL)
@@ -494,7 +510,7 @@ class TablespaceView(PGChildNodeView):
 
             if len(res['rows']) == 0:
                 return gone(
-                    _("Could not find the tablespace on the server.")
+                    gettext("Could not find the tablespace on the server.")
                 )
 
             # Making copy of output for further processing
@@ -502,20 +518,7 @@ class TablespaceView(PGChildNodeView):
             old_data = self._formatter(old_data, tsid)
 
             # To format privileges data coming from client
-            for key in ['spcacl']:
-                if key in data and data[key] is not None:
-                    if 'added' in data[key]:
-                        data[key]['added'] = parse_priv_to_db(
-                            data[key]['added'], self.acl
-                        )
-                    if 'changed' in data[key]:
-                        data[key]['changed'] = parse_priv_to_db(
-                            data[key]['changed'], self.acl
-                        )
-                    if 'deleted' in data[key]:
-                        data[key]['deleted'] = parse_priv_to_db(
-                            data[key]['deleted'], self.acl
-                        )
+            self._format_privilege_data(data)
 
             # If name is not present with in update data then copy it
             # from old data
@@ -524,7 +527,7 @@ class TablespaceView(PGChildNodeView):
                     data[arg] = old_data[arg]
 
             SQL = render_template(
-                "/".join([self.template_path, 'update.sql']),
+                "/".join([self.template_path, self._UPDATE_SQL]),
                 data=data, o_data=old_data
             )
         else:
@@ -533,12 +536,12 @@ class TablespaceView(PGChildNodeView):
                 data['spcacl'] = parse_priv_to_db(data['spcacl'], self.acl)
             # If the request for new object which do not have tsid
             SQL = render_template(
-                "/".join([self.template_path, 'create.sql']),
+                "/".join([self.template_path, self._CREATE_SQL]),
                 data=data
             )
             SQL += "\n"
             SQL += render_template(
-                "/".join([self.template_path, 'alter.sql']),
+                "/".join([self.template_path, self._ALTER_SQL]),
                 data=data, conn=self.conn
             )
         SQL = re.sub('\n{2,}', '\n\n', SQL)
@@ -550,7 +553,7 @@ class TablespaceView(PGChildNodeView):
         This function will generate sql for sql panel
         """
         SQL = render_template(
-            "/".join([self.template_path, 'properties.sql']),
+            "/".join([self.template_path, self._PROPERTIES_SQL]),
             tsid=tsid, conn=self.conn
         )
         status, res = self.conn.execute_dict(SQL)
@@ -559,7 +562,7 @@ class TablespaceView(PGChildNodeView):
 
         if len(res['rows']) == 0:
             return gone(
-                _("Could not find the tablespace on the server.")
+                gettext("Could not find the tablespace on the server.")
             )
         # Making copy of output for future use
         old_data = dict(res['rows'][0])
@@ -574,16 +577,16 @@ class TablespaceView(PGChildNodeView):
         # We are not showing create sql for system tablespace
         if not old_data['name'].startswith('pg_'):
             SQL = render_template(
-                "/".join([self.template_path, 'create.sql']),
+                "/".join([self.template_path, self._CREATE_SQL]),
                 data=old_data
             )
             SQL += "\n"
         SQL += render_template(
-            "/".join([self.template_path, 'alter.sql']),
+            "/".join([self.template_path, self._ALTER_SQL]),
             data=old_data, conn=self.conn
         )
 
-        sql_header = u"""
+        sql_header = """
 -- Tablespace: {0}
 
 -- DROP TABLESPACE {0};
@@ -677,6 +680,61 @@ class TablespaceView(PGChildNodeView):
             status=200
         )
 
+    def _handel_dependents_type(self, types, type_str, row, rel_name):
+        type_name = ''
+        if types[type_str[0]] is None:
+            if type_str[0] == 'i':
+                type_name = 'index'
+                rel_name = row['indname'] + ' ON ' + rel_name
+            elif type_str[0] == 'o':
+                type_name = 'operator'
+                rel_name = row['relname']
+        else:
+            type_name = types[type_str[0]]
+        return type_name, rel_name
+
+    def _check_dependents_type(self, types, dependents, db_row, result):
+        for row in result['rows']:
+            rel_name = row['nspname']
+            if rel_name is not None:
+                rel_name += '.'
+
+            if rel_name is None:
+                rel_name = row['relname']
+            else:
+                rel_name += row['relname']
+
+            type_str = row['relkind']
+            # Fetch the type name from the dictionary
+            # if type is not present in the types dictionary then
+            # we will continue and not going to add it.
+            if type_str[0] in types:
+                # if type is present in the types dictionary, but it's
+                # value is None then it requires special handling.
+                type_name, rel_name = self._handel_dependents_type(types,
+                                                                   type_str,
+                                                                   row,
+                                                                   rel_name)
+            else:
+                continue
+
+            dependents.append(
+                {
+                    'type': type_name,
+                    'name': rel_name,
+                    'field': db_row['datname']
+                }
+            )
+
+    def _create_dependents_data(self, types, result, dependents, db_row,
+                                is_connected, manager):
+
+        self._check_dependents_type(types, dependents, db_row, result)
+
+        # Release only those connections which we have created above.
+        if not is_connected:
+            manager.release(db_row['datname'])
+
     def get_dependents(self, conn, sid, tsid):
         """
         This function is used to fetch the dependents for the selected node.
@@ -754,48 +812,8 @@ class TablespaceView(PGChildNodeView):
                 if not status:
                     current_app.logger.error(result)
 
-                for row in result['rows']:
-                    rel_name = row['nspname']
-                    if rel_name is not None:
-                        rel_name += '.'
-
-                    if rel_name is None:
-                        rel_name = row['relname']
-                    else:
-                        rel_name += row['relname']
-
-                    type_name = ''
-                    type_str = row['relkind']
-                    # Fetch the type name from the dictionary
-                    # if type is not present in the types dictionary then
-                    # we will continue and not going to add it.
-                    if type_str[0] in types:
-
-                        # if type is present in the types dictionary, but it's
-                        # value is None then it requires special handling.
-                        if types[type_str[0]] is None:
-                            if type_str[0] == 'i':
-                                type_name = 'index'
-                                rel_name = row['indname'] + ' ON ' + rel_name
-                            elif type_str[0] == 'o':
-                                type_name = 'operator'
-                                rel_name = row['relname']
-                        else:
-                            type_name = types[type_str[0]]
-                    else:
-                        continue
-
-                    dependents.append(
-                        {
-                            'type': type_name,
-                            'name': rel_name,
-                            'field': db_row['datname']
-                        }
-                    )
-
-                # Release only those connections which we have created above.
-                if not is_connected:
-                    manager.release(db_row['datname'])
+                self._create_dependents_data(types, result, dependents, db_row,
+                                             is_connected, manager)
 
         return dependents
 
