@@ -2,12 +2,11 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
-from __future__ import print_function
 
 import json
 import uuid
@@ -55,9 +54,9 @@ class TableSpaceDeleteTestCase(BaseTestGenerator):
             '/' + str(self.server_id) + '/' + str(self.tablespace_id),
             follow_redirects=True
         )
-        self.assertEquals(response.status_code, 200)
+        self.assertEqual(response.status_code, 200)
         delete_response_data = json.loads(response.data.decode('utf-8'))
-        self.assertEquals(delete_response_data['success'], 1)
+        self.assertEqual(delete_response_data['success'], 1)
 
     def tearDown(self):
         """This function deletes the tablespace"""
