@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -57,7 +57,7 @@ class TableSpaceUpdateTestCase(BaseTestGenerator):
             data=json.dumps(data),
             follow_redirects=True
         )
-        self.assertEquals(put_response.status_code, 200)
+        self.assertEqual(put_response.status_code, 200)
 
     def tearDown(self):
         """This function deletes the tablespace"""
