@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -14,16 +14,16 @@ from regression.python_test_utils import test_utils
 class TestSSLConnection(BaseTestGenerator):
     """This will check if SSL is used in our database connection"""
     scenarios = [
-        ("Test for SSL connection", dict())
+        ('Test for SSL connection', dict())
     ]
 
     def runTest(self):
-        if hasattr(self, "ignore_test"):
+        if hasattr(self, 'ignore_test'):
             return
         supported_modes = ['require', 'verify-ca', 'verify-full']
         if self.server['sslmode'] in supported_modes:
             with test_utils.Database(self.server) as (
-                    connection, database_name
+                connection, database_name
             ):
 
                 cursor = connection.cursor()
@@ -31,9 +31,8 @@ class TestSSLConnection(BaseTestGenerator):
                 connection.commit()
                 cursor.execute("SELECT ssl_is_used()")
                 is_ssl_used = cursor.fetchone()[0]
-                self.assertEquals(True, is_ssl_used)
+                self.assertEqual(True, is_ssl_used)
         else:
-            self.skipTest("Cannot run SSL connection check test "
-                          "with '{0}' sslmode".format(
-                self.server['sslmode']
-            ))
+            self.skipTest(
+                'Cannot run SSL connection check test '
+                'with \'{0}\' sslmode'.format(self.server['sslmode']))
