@@ -2,14 +2,15 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
 import os
 
-from regression.python_test_utils.sql_template_test_base import SQLTemplateTestBase
+from regression.python_test_utils.sql_template_test_base import \
+    SQLTemplateTestBase
 from regression.python_test_utils.template_helper import file_as_template
 
 
@@ -30,9 +31,14 @@ class TestDependenciesSql(SQLTemplateTestBase):
         self.table_id = cursor.fetchone()[0]
 
     def generate_sql(self, version):
-        template_file = self.get_template_file(version, "dependencies.sql")
+        file_path = os.path.join(os.path.dirname(__file__), "..", "templates",
+                                 "depends", self.server['type'])
+        template_file = self.get_template_file(version, file_path,
+                                               "dependencies.sql")
         template = file_as_template(template_file)
-        sql = template.render(where_clause="WHERE dep.objid=%s::oid" % self.table_id)
+        sql = template.render(
+            where_clause="WHERE dep.objid=%s::oid" % self.table_id,
+            object_id=self.table_id)
 
         return sql
 
@@ -45,7 +51,3 @@ class TestDependenciesSql(SQLTemplateTestBase):
 
         self.assertEqual('n', first_row["deptype"])
         self.assertEqual('public', first_row["refname"])
-
-    @staticmethod
-    def get_template_file(version, filename):
-        return os.path.join(os.path.dirname(__file__), "..", "templates", "depends", "sql", version, filename)
