@@ -2,20 +2,17 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
 
-//import * as commonUtils from '../../../static/js/utils';
-//import Mousetrap from 'mousetrap';
-
 define([
-  'sources/gettext', 'jquery', 'underscore', 'underscore.string', 'sources/pgadmin',
+  'sources/gettext', 'jquery', 'underscore', 'sources/pgadmin',
   'backbone', 'alertify', 'backform', 'backgrid', 'sources/browser/generate_url',
   'pgadmin.backform', 'pgadmin.backgrid',
   'pgadmin.browser.node', 'backgrid.select.all',
-], function(gettext, $, _, S, pgAdmin, Backbone, Alertify, Backform, Backgrid, generateUrl) {
+], function(gettext, $, _, pgAdmin, Backbone, Alertify, Backform, Backgrid, generateUrl) {
 
   var pgBrowser = pgAdmin.Browser = pgAdmin.Browser || {};
 
@@ -42,21 +39,30 @@ define([
         if (this.node_initialized)
           return;
         this.node_initialized = true;
+
         pgAdmin.Browser.add_menus([{
           name: 'refresh', node: this.type, module: this,
           applies: ['object', 'context'], callback: 'refresh',
           priority: 1, label: gettext('Refresh...'),
-          icon: 'fa fa-refresh',
+          icon: 'fa fa-sync-alt',
         }]);
 
         // show query tool only in context menu of supported nodes.
-        if (pgAdmin.DataGrid && pgAdmin.unsupported_nodes) {
-          if (_.indexOf(pgAdmin.unsupported_nodes, this.type) == -1) {
+        if (pgAdmin.unsupported_nodes && _.indexOf(pgAdmin.unsupported_nodes, this.type) == -1) {
+          if ((this.type == 'database' && this.allowConn) || this.type != 'database') {
             pgAdmin.Browser.add_menus([{
               name: 'show_query_tool', node: this.type, module: this,
               applies: ['context'], callback: 'show_query_tool',
-              priority: 998, label: gettext('Query Tool...'),
-              icon: 'fa fa-bolt',
+              priority: 998, label: gettext('Query Tool'),
+              icon: 'pg-font-icon icon-query-tool',
+            }]);
+
+            // show search objects same as query tool
+            pgAdmin.Browser.add_menus([{
+              name: 'search_objects', node: this.type, module: this,
+              applies: ['context'], callback: 'show_search_objects',
+              priority: 997, label: gettext('Search Objects...'),
+              icon: 'fa fa-search',
             }]);
           }
         }
@@ -75,35 +81,30 @@ define([
           j = panel.$container.find('.obj_properties').first(),
           view = j.data('obj-view'),
           content = $('<div></div>')
-          .addClass('pg-prop-content col-12 has-pg-prop-btn-group'),
+            .addClass('pg-prop-content col-12 has-pg-prop-btn-group'),
           node = pgBrowser.Nodes[that.node],
+          $msgContainer = '',
           // This will be the URL, used for object manipulation.
           urlBase = this.generate_url(item, 'properties', data),
-          collection = new (node.Collection.extend({
-            url: urlBase,
-            model: node.model,
-          }))(),
           info = this.getTreeNodeHierarchy.apply(this, [item]),
           gridSchema = Backform.generateGridColumnsFromModel(
             info, node.model, 'properties', that.columns
           ),
-          createButtons = function(buttons, location, extraClasses) {
+          createButtons = function(buttonsList, location, extraClasses) {
             // Arguments must be non-zero length array of type
             // object, which contains following attributes:
             // label, type, extraClasses, register
-            if (buttons && _.isArray(buttons) && buttons.length > 0) {
+            if (buttonsList && _.isArray(buttonsList) && buttonsList.length > 0) {
               // All buttons will be created within a single
               // div area.
               var btnGroup =
-                $('<div></div>').addClass(
-                  'pg-prop-btn-group'
-                ),
+                $('<div class="pg-prop-btn-group"></div>'),
                 // Template used for creating a button
                 tmpl = _.template([
-                  '<button type="<%= type %>" ',
+                  '<button tabindex="0" type="<%= type %>" ',
                   'class="btn <%=extraClasses.join(\' \')%>"',
                   '<% if (disabled) { %> disabled="disabled"<% } %> title="<%-tooltip%>">',
-                  '<span class="<%= icon %>"></span><% if (label != "") { %>&nbsp;<%-label%><% } %></button>',
+                  '<span class="<%= icon %>" role="img"></span><% if (label != "") { %>&nbsp;<%-label%><% } %><span class="sr-only"><%-tooltip%></span></button>',
                 ].join(' '));
               if (location == 'header') {
                 btnGroup.appendTo(that.header);
@@ -113,7 +114,7 @@ define([
               if (extraClasses) {
                 btnGroup.addClass(extraClasses);
               }
-              _.each(buttons, function(btn) {
+              _.each(buttonsList, function(btn) {
                 // Create the actual button, and append to
                 // the group div
 
@@ -132,11 +133,15 @@ define([
             return null;
           }.bind(panel);
 
+        that.collection = new (node.Collection.extend({
+          url: urlBase,
+          model: node.model,
+        }))();
         // Add the new column for the multi-select menus
         if((_.isFunction(that.canDrop) ?
-              that.canDrop.apply(that, [data, item]) : that.canDrop) ||
+          that.canDrop.apply(that, [data, item]) : that.canDrop) ||
               (_.isFunction(that.canDropCascade) ?
-              that.canDropCascade.apply(that, [data, item]) : that.canDropCascade)) {
+                that.canDropCascade.apply(that, [data, item]) : that.canDropCascade)) {
           gridSchema.columns.unshift({
             name: 'oid',
             cell: Backgrid.Extension.SelectRowCell.extend({
@@ -147,11 +152,11 @@ define([
                 }
 
                 var column = this.column, model = this.model, $el = this.$el;
-                this.listenTo(column, 'change:renderable', function (column, renderable) {
+                this.listenTo(column, 'change:renderable', function (col, renderable) {
                   $el.toggleClass('renderable', renderable);
                 });
 
-                if (Backgrid.callByNeed(column.renderable(), column, model)) $el.addClass('renderable');
+                if (Backgrid.callByNeed(column.renderable(), column, model)) $el.addClass('renderable width_percent_3');
 
                 this.listenTo(model, 'backgrid:select', this.toggleCheckbox);
               },
@@ -162,13 +167,18 @@ define([
               },
               render: function() {
                 let model = this.model.toJSON();
-
                 // canDrop can be set to false for individual row from the server side to disable the checkbox
-                if ('canDrop' in model && model.canDrop === false)
-                  this.$el.empty().append('<input tabindex="-1" type="checkbox" disabled="disabled"/>');
-                else
-                  this.$el.empty().append('<input tabindex="-1" type="checkbox" />');
+                let disabled = ('canDrop' in model && model.canDrop === false);
+                let id = `row-${_.uniqueId(model.oid || model.name)}`;
 
+                this.$el.empty().append(`
+                  <div class="custom-control custom-checkbox custom-checkbox-no-label">
+                    <input tabindex="-1" type="checkbox" class="custom-control-input" id="${id}" ${disabled?'disabled':''}/>
+                    <label class="custom-control-label" for="${id}">
+                      <span class="sr-only">` + gettext('Select') + `<span>
+                    </label>
+                  </div>
+                `);
                 this.delegateEvents();
                 return this;
               },
@@ -176,11 +186,15 @@ define([
             headerCell: Backgrid.Extension.SelectAllHeaderCell,
           });
         }
-          // Initialize a new Grid instance
+        /* Columns should be always non-editable  */
+        gridSchema.columns.forEach((col)=>{
+          col.disabled = true;
+        });
+        // Initialize a new Grid instance
         that.grid = new Backgrid.Grid({
-          emptyText: 'No data found',
+          emptyText: gettext('No data found'),
           columns: gridSchema.columns,
-          collection: collection,
+          collection: that.collection,
           className: 'backgrid table presentation table-bordered table-noouter-border table-hover',
         });
 
@@ -188,7 +202,7 @@ define([
           'remove': function() {
             if (this.grid) {
               if (this.grid.collection) {
-                this.grid.collection.reset({silent: true});
+                this.grid.collection.reset([], {silent: true});
                 delete (this.grid.collection);
               }
               delete (this.grid);
@@ -199,6 +213,7 @@ define([
         };
 
         if (view) {
+
           // Avoid unnecessary reloads
           if (_.isEqual($(panel).data('node-prop'), urlBase)) {
             return;
@@ -215,19 +230,24 @@ define([
         j.empty();
         j.data('obj-view', gridView);
 
-        that.header = $('<div></div>').addClass(
-            'pg-prop-header'
-        ).appendTo(j);
+        $msgContainer = '<div role="status" class="pg-panel-message pg-panel-properties-message">' +
+         gettext('Retrieving data from the server...') + '</div>';
 
-         // Render the buttons
+        $msgContainer = $($msgContainer).appendTo(j);
+
+        that.header = $('<div></div>').addClass(
+          'pg-prop-header'
+        );
+
+        // Render the buttons
         var buttons = [];
 
         buttons.push({
           label: '',
           type: 'delete',
           tooltip: gettext('Delete/Drop'),
-          extraClasses: ['btn-secondary m-1', 'delete_multiple'],
-          icon: 'fa fa-lg fa-trash-o',
+          extraClasses: ['btn-primary-icon m-1', 'delete_multiple'],
+          icon: 'fa fa-trash-alt',
           disabled:  (_.isFunction(that.canDrop)) ? !(that.canDrop.apply(self, [data, item])) : (!that.canDrop),
           register: function(btn) {
             btn.on('click',() => {
@@ -240,8 +260,8 @@ define([
           label: '',
           type: 'delete',
           tooltip: gettext('Drop Cascade'),
-          extraClasses: ['btn-secondary m-1', 'delete_multiple_cascade'],
-          icon: 'icon-delete_multiple_cascade',
+          extraClasses: ['btn-primary-icon m-1', 'delete_multiple_cascade'],
+          icon: 'pg-font-icon icon-drop-cascade',
           disabled: (_.isFunction(that.canDropCascade)) ? !(that.canDropCascade.apply(self, [data, item])) : (!that.canDropCascade),
           register: function(btn) {
             btn.on('click',() => {
@@ -255,39 +275,96 @@ define([
         // Render subNode grid
         content.append('<div class="pg-prop-coll-container"></div>');
         content.find('.pg-prop-coll-container').append(that.grid.render().$el);
-        j.append(content);
 
-        // Fetch Data
-        collection.fetch({
-          reset: true,
-          error: function(model, error, xhr) {
-            pgBrowser.Events.trigger(
-              'pgadmin:collection:retrieval:error', 'properties', xhr, error,
-              error.message, item, that
-            );
-            if (!Alertify.pgHandleItemError(
-              xhr, error, error.message, {item: item, info: info}
-            )) {
-              Alertify.pgNotifier(error, xhr, S(
-                gettext('Error retrieving properties - %s.')
-              ).sprintf(error.message || that.label).value(), function() {
-                console.warn(arguments);
-              });
+        var timer;
+        var getAjaxHook = function() {
+          $.ajax({
+            url: urlBase,
+            type: 'GET',
+            beforeSend: function(xhr) {
+              xhr.setRequestHeader(pgAdmin.csrf_token_header, pgAdmin.csrf_token);
+              // Generate a timer for the request
+              timer = setTimeout(function() {
+                // notify user if request is taking longer than 1 second
+
+                $msgContainer.text(gettext('Retrieving data from the server...'));
+                $msgContainer.removeClass('d-none');
+                if (self.grid) {
+                  self.grid.remove();
+                }
+              }, 1000);
+            },
+          }).done(function(res) {
+            clearTimeout(timer);
+
+            if (_.isUndefined(that.grid) || _.isNull(that.grid)) return;
+
+            that.data = res;
+
+            if (that.data.length > 0) {
+
+              if (!$msgContainer.hasClass('d-none')) {
+                $msgContainer.addClass('d-none');
+              }
+              that.header.appendTo(j);
+              j.append(content);
+
+              // Listen scroll event to load more rows
+              $('.pg-prop-content').on('scroll', that.__loadMoreRows.bind(that));
+
+              that.collection.reset(that.data.splice(0, 50));
+
+              // Listen to select all checkbox event
+              that.collection.on('backgrid:select-all', that.__loadAllRows.bind(that));
+            } else {
+            // Do not listen the scroll event
+              $('.pg-prop-content').off('scroll', that.__loadMoreRows);
+
+              $msgContainer.text(gettext('No properties are available for the selected object.'));
+
             }
-          },
-        });
+          }).fail(function(xhr, error) {
+            pgBrowser.Events.trigger(
+              'pgadmin:node:retrieval:error', 'properties', xhr, error.message, item, that
+            );
+            if (!Alertify.pgHandleItemError(xhr, error.message, {
+              item: item,
+              info: info,
+            })) {
+              Alertify.pgNotifier(
+                error, xhr, gettext('Error retrieving properties - %s', error.message || that.label),
+                function(msg) {
+                  if(msg === 'CRYPTKEY_SET') {
+                    getAjaxHook();
+                  } else {
+                    console.warn(arguments);
+                  }
+                }
+              );
+            }
+            // show failed message.
+            $msgContainer.text(gettext('Failed to retrieve data from the server.'));
+          });
+        };
+        getAjaxHook();
 
-        var onDrop = function(type) {
+        var onDrop = function(type, confirm=true) {
           let sel_row_models = this.grid.getSelectedModels(),
             sel_rows = [],
-            item = pgBrowser.tree.selected(),
-            d = item ? pgBrowser.tree.itemData(item) : null,
-            node = pgBrowser.Nodes[d._type],
+            sel_item = pgBrowser.tree.selected(),
+            d = sel_item ? pgBrowser.tree.itemData(sel_item) : null,
+            sel_node = d && pgBrowser.Nodes[d._type],
             url = undefined,
             msg = undefined,
             title = undefined;
 
-          _.each(sel_row_models, function(r){ sel_rows.push(r.id); });
+          if (sel_node && sel_node.type && sel_node.type == 'coll-constraints') {
+            // In order to identify the constraint type, the type should be passed to the server
+            sel_rows = sel_row_models.map(row => ({id: row.get('oid'), _type: row.get('_type')}));
+          }
+          else {
+            sel_rows = sel_row_models.map(row => row.id);
+          }
 
           if (sel_rows.length === 0) {
             Alertify.alert(gettext('Drop Multiple'),
@@ -296,69 +373,84 @@ define([
             return;
           }
 
+          if (!sel_node)
+            return;
+
           if (type === 'dropCascade') {
-            url = node.generate_url(item, 'delete'),
-            msg = gettext('Are you sure you want to drop all the selected objects and all the objects that depend on them?'),
+            url = sel_node.generate_url(sel_item, 'delete');
+            msg = gettext('Are you sure you want to drop all the selected objects and all the objects that depend on them?');
             title = gettext('DROP CASCADE multiple objects?');
           } else {
-            url = node.generate_url(item, 'drop');
+            url = sel_node.generate_url(sel_item, 'drop');
             msg = gettext('Are you sure you want to drop all the selected objects?');
             title = gettext('DROP multiple objects?');
           }
 
-
-          Alertify.confirm(title, msg,
-            function() {
-              $.ajax({
-                url: url,
-                type: 'DELETE',
-                data: JSON.stringify({'ids': sel_rows}),
-                contentType: 'application/json; charset=utf-8',
-              })
-              .done(function(res) {
-                if (res.success == 0) {
-                  pgBrowser.report_error(res.errormsg, res.info);
-                } else {
-                  $(pgBrowser.panels['properties'].panel).removeData('node-prop');
-                  pgBrowser.Events.trigger(
-                  'pgadmin:browser:tree:refresh', item || pgBrowser.tree.selected(), {
-                    success: function() {
-                      node.callbacks.selected.apply(node, [item]);
-                    },
-                  });
-                }
-                return true;
-              })
-              .fail(function(jqx) {
-                var msg = jqx.responseText;
-                /* Error from the server */
-                if (jqx.status == 417 || jqx.status == 410 || jqx.status == 500) {
-                  try {
-                    var data = JSON.parse(jqx.responseText);
-                    msg = data.errormsg;
-                  } catch (e) {
-                    console.warn(e.stack || e);
-                  }
-                }
-                pgBrowser.report_error(
-                  S(gettext('Error dropping %s'))
-                  .sprintf(d._label.toLowerCase())
-                  .value(), msg);
-
+          let dropAjaxHook = function() {
+            $.ajax({
+              url: url,
+              type: 'DELETE',
+              data: JSON.stringify({'ids': sel_rows}),
+              contentType: 'application/json; charset=utf-8',
+            }).done(function(res) {
+              if (res.success == 0) {
+                pgBrowser.report_error(res.errormsg, res.info);
+              } else {
                 $(pgBrowser.panels['properties'].panel).removeData('node-prop');
                 pgBrowser.Events.trigger(
-                  'pgadmin:browser:tree:refresh', item || pgBrowser.tree.selected(), {
+                  'pgadmin:browser:tree:refresh', sel_item || pgBrowser.tree.selected(), {
                     success: function() {
-                      node.callbacks.selected.apply(node, [item]);
+                      sel_node.callbacks.selected.apply(sel_node, [sel_item]);
                     },
+                  });
+              }
+              return true;
+            }).fail(function(xhr, error) {
+              Alertify.pgNotifier(
+                error, xhr,
+                gettext('Error dropping %s', d._label.toLowerCase()),
+                function(alertMsg) {
+                  if (alertMsg == 'CRYPTKEY_SET') {
+                    onDrop(type, false);
+                  } else {
+                    $(pgBrowser.panels['properties'].panel).removeData('node-prop');
+                    pgBrowser.Events.trigger(
+                      'pgadmin:browser:tree:refresh', sel_item || pgBrowser.tree.selected(), {
+                        success: function() {
+                          sel_node.callbacks.selected.apply(sel_node, [sel_item]);
+                        },
+                      }
+                    );
                   }
-                );
-              });
-            },
-            null).show();
+                }
+              );
+            });
+          };
+
+          if(confirm) {
+            Alertify.confirm(title, msg, dropAjaxHook, null).show();
+          } else {
+            dropAjaxHook();
+          }
           return;
         }.bind(that);
       },
+      __loadMoreRows: function(e) {
+        let elem = e.currentTarget;
+        if ((elem.scrollHeight - 10) < elem.scrollTop + elem.offsetHeight) {
+          if (this.data.length > 0) {
+            this.collection.add(this.data.splice(0, 50));
+          }
+        }
+      },
+      __loadAllRows: function(tmp, checked) {
+        if (this.data.length > 0) {
+          this.collection.add(this.data);
+          this.collection.each(function (model) {
+            model.trigger('backgrid:select', model, checked);
+          });
+        }
+      },
       generate_url: function(item, type) {
         /*
          * Using list, and collection functions of a node to get the nodes
@@ -380,6 +472,16 @@ define([
           collectionPickFunction
         );
       },
+      show_query_tool: function() {
+        if(pgAdmin.DataGrid) {
+          pgAdmin.DataGrid.show_query_tool('', pgAdmin.Browser.tree.selected());
+        }
+      },
+      show_search_objects: function() {
+        if(pgAdmin.SearchObjects) {
+          pgAdmin.SearchObjects.show_search_objects('', pgAdmin.Browser.tree.selected());
+        }
+      },
     });
 
   return pgBrowser.Collection;
