@@ -2,14 +2,14 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
 
 define([
-  'underscore', 'underscore.string', 'sources/pgadmin', 'jquery', 'backbone',
-], function(_, S, pgAdmin, $, Backbone) {
+  'underscore', 'sources/pgadmin', 'jquery', 'backbone', 'sources/utils',
+], function(_, pgAdmin, $, Backbone, pgadminUtils) {
   var pgBrowser = pgAdmin.Browser = pgAdmin.Browser || {};
 
   pgBrowser.DataModel = Backbone.Model.extend({
@@ -36,7 +36,7 @@ define([
                   obj = new(pgBrowser.Node.Collection)(val, {
                     model: ((_.isString(s.model) &&
                           s.model in pgBrowser.Nodes) ?
-                        pgBrowser.Nodes[s.model].model : s.model),
+                      pgBrowser.Nodes[s.model].model : s.model),
                     top: self.top || self,
                     handler: self,
                     parse: true,
@@ -44,7 +44,7 @@ define([
                     attrName: s.id,
                   });
 
-                    /*
+                  /*
                      * Nested collection models may or may not have idAttribute.
                      * So to decide whether model is new or not set 'on_server'
                      * flag on such models.
@@ -56,7 +56,7 @@ define([
                     on_server: true,
                   });
                 } else {
-                    /*
+                  /*
                      * Nested collection models may or may not have idAttribute.
                      * So to decide whether model is new or not set 'on_server'
                      * flag on such models.
@@ -83,13 +83,13 @@ define([
                   if (_.isString(s.model) &&
                       s.model in pgBrowser.Nodes[s.model]) {
                     obj = new(pgBrowser.Nodes[s.model].Model)(
-                        obj, {
-                          silent: true,
-                          top: self.top || self,
-                          handler: self,
-                          attrName: s.id,
-                        }
-                      );
+                      obj, {
+                        silent: true,
+                        top: self.top || self,
+                        handler: self,
+                        attrName: s.id,
+                      }
+                    );
                   } else {
                     obj = new(s.model)(obj, {
                       silent: true,
@@ -154,7 +154,6 @@ define([
 
       if (_.isUndefined(options) || _.isNull(options)) {
         options = attributes || {};
-        attributes = null;
       }
 
       self.sessAttrs = {};
@@ -162,8 +161,8 @@ define([
       self.origSessAttrs = {};
       self.objects = [];
       self.arrays = [];
-      self.attrName = options.attrName,
-        self.top = (options.top || self.collection && self.collection.top || self.collection || self);
+      self.attrName = options.attrName;
+      self.top = (options.top || self.collection && self.collection.top || self.collection || self);
       self.handler = options.handler ||
         (self.collection && self.collection.handler);
       self.trackChanges = false;
@@ -234,19 +233,19 @@ define([
                 if (_.isString(s.model) &&
                     s.model in pgBrowser.Nodes[s.model]) {
                   obj = new(pgBrowser.Nodes[s.model].Model)(
-                      obj, {
-                        top: self.top || self,
-                        handler: self,
-                        attrName: s.id,
-                      }
-                    );
+                    obj, {
+                      top: self.top || self,
+                      handler: self,
+                      attrName: s.id,
+                    }
+                  );
                 } else {
                   obj = new(s.model)(
-                      obj, {
-                        top: self.top || self,
-                        handler: self,
-                        attrName: s.id,
-                      });
+                    obj, {
+                      top: self.top || self,
+                      handler: self,
+                      attrName: s.id,
+                    });
                 }
               }
 
@@ -283,11 +282,17 @@ define([
     },
     // Create a reset function, which allow us to remove the nested object.
     reset: function(opts) {
-      var obj;
+      var obj,
+        reindex = !!(opts && opts.reindex);
 
       if (opts && opts.stop)
         this.stopSession();
 
+      // Let's not touch the child attributes, if reindex is false.
+      if (!reindex) {
+        return;
+      }
+
       for (var id in this.objects) {
         obj = this.get(id);
 
@@ -297,7 +302,7 @@ define([
           } else if (obj instanceof Backbone.Model) {
             obj.clear(opts);
           } else if (obj instanceof pgBrowser.DataCollection) {
-            obj.reset(opts);
+            obj.reset([], opts);
           } else if (obj instanceof Backbone.Collection) {
             obj.each(function(m) {
               if (m instanceof Backbone.DataModel) {
@@ -305,14 +310,11 @@ define([
                 obj.clear(opts);
               }
             });
-            if (!(opts instanceof Array)) {
-              opts = [opts];
-            }
-            Backbone.Collection.prototype.reset.apply(obj, opts);
+            Backbone.Collection.prototype.reset.call(obj, [], opts);
           }
         }
       }
-      this.clear(opts);
+      Backbone.Collection.prototype.reset.apply(this, arguments);
     },
     sessChanged: function() {
       var self = this;
@@ -364,7 +366,12 @@ define([
             return;
           }
           attrs[k] = v;
-          if (_.isEqual(self.origSessAttrs[k], v)) {
+          const attrsDefined = self.origSessAttrs[k] && v;
+          /* If the orig value was null and new one is empty string, then its a "no change" */
+          /* If the orig value and new value are of different datatype but of same value(numeric) "no change" */
+          if (_.isEqual(self.origSessAttrs[k], v)
+            || (self.origSessAttrs[k] === null && v === '')
+            || (attrsDefined ? _.isEqual(self.origSessAttrs[k].toString(), v.toString()) : false)) {
             delete self.sessAttrs[k];
           } else {
             self.sessAttrs[k] = v;
@@ -379,8 +386,8 @@ define([
         }
 
         self.trigger('pgadmin-session:set', self, attrs);
-        if (!options || !options.silent) {
-          self.trigger('change', self, options);
+        if (!opts || !opts.silent) {
+          self.trigger('change', self, opts);
         }
 
         // Perform default validations.
@@ -532,6 +539,8 @@ define([
 
       // Let me listen to the my child invalid/valid messages
       self.on('pgadmin-session:model:invalid', self.onChildInvalid);
+      self.on('pgadmin-session:collection:changed', self.onChildCollectionChanged);
+      self.on('pgadmin-session:model-msg:changed', self.onModelChangedMsg);
       self.on('pgadmin-session:model:valid', self.onChildValid);
       self.on('pgadmin-session:changed', self.onChildChanged);
       self.on('pgadmin-session:added', self.onChildChanged);
@@ -549,26 +558,26 @@ define([
             typeof(obj.primary_key) === 'function';
           var key = hasPrimaryKey ? obj.primary_key() : obj.cid,
             comparator = hasPrimaryKey ?
-            function(k) {
-              var o = self.get('k');
+              function(k) {
+                var o = self.get('k');
 
-              if (o && o.primary_key() === key) {
-                objName = k;
-                return true;
-              }
+                if (o && o.primary_key() === key) {
+                  objName = k;
+                  return true;
+                }
 
-              return false;
-            } :
-            function(k) {
-              var o = self.get(k);
+                return false;
+              } :
+              function(k) {
+                var o = self.get(k);
 
-              if (o.cid === key) {
-                objName = k;
-                return true;
-              }
+                if (o.cid === key) {
+                  objName = k;
+                  return true;
+                }
 
-              return false;
-            };
+                return false;
+              };
           _.findIndex(self.objects, comparator);
         }
 
@@ -599,26 +608,26 @@ define([
             (typeof(obj.primary_key) === 'function'));
           var key = hasPrimaryKey ? obj.primary_key() : obj.cid,
             comparator = hasPrimaryKey ?
-            function(k) {
-              var o = self.get('k');
+              function(k) {
+                var o = self.get('k');
 
-              if (o && o.primary_key() === key) {
-                objName = k;
-                return true;
-              }
+                if (o && o.primary_key() === key) {
+                  objName = k;
+                  return true;
+                }
 
-              return false;
-            } :
-            function(k) {
-              var o = self.get('k');
+                return false;
+              } :
+              function(k) {
+                var o = self.get('k');
 
-              if (o && o.cid === key) {
-                objName = k;
-                return true;
-              }
+                if (o && o.cid === key) {
+                  objName = k;
+                  return true;
+                }
 
-              return false;
-            };
+                return false;
+              };
 
           _.findIndex(self.objects, comparator);
         }
@@ -667,10 +676,14 @@ define([
           !validate(self, (objName && [objName]))) {
           if (self.handler) {
             (self.handler).trigger('pgadmin-session:model:valid', self, self.handler);
+            (self.handler).trigger('pgadmin-session:collection:changed', self, self.handler);
           } else {
             self.trigger(
               'pgadmin-session:valid', self.sessChanged(), self
             );
+            self.trigger(
+              'pgadmin-session:collection:changed', self.sessChanged(), self
+            );
           }
         } else {
           msg = msg || _.values(self.errorModel.attributes)[0];
@@ -679,19 +692,84 @@ define([
             (self.handler).trigger(
               'pgadmin-session:model:invalid', msg, self, self.handler
             );
+            (self.handler).trigger('pgadmin-session:collection:changed', self, self.handler);
           } else {
             self.trigger('pgadmin-session:invalid', msg, self);
+            self.trigger('pgadmin-session:collection:changed', self);
           }
         }
       }
     },
 
-    onChildChanged: function() {
+    onChildCollectionChanged: function(obj, obj_hand) {
+
+      var self = this;
+
+      setTimeout(() => {
+
+        var msg = null,
+          validate = function(m, attrs) {
+            if ('default_validate' in m && typeof(m.default_validate) == 'function') {
+              msg = m.default_validate();
+              if (_.isString(msg)) {
+                return msg;
+              }
+            }
+
+            if ('validate' in m && typeof(m.validate) == 'function') {
+              msg = m.validate(attrs);
+
+              return msg;
+            }
+            return null;
+          };
+
+        let handler, parentTr;
+
+        let collection = self.collection || obj_hand;
+        if(collection) {
+          var collection_selector = collection.attrName || collection.name;
+          let activeTab = $('.show.active div.'+collection_selector);
+          $(activeTab).find('.error-in-grid').removeClass('error-in-grid');
+
+          model_collection_exit : if (collection instanceof Backbone.Collection) {
+
+            for (var cid in collection.models) {
+              let model = collection.models[cid];
+
+              for(let mod_obj of model.objects) {
+                let mod_attr = model.attributes[mod_obj];
+                if (mod_attr && mod_attr.models.length > 0) {
+                  for(let mod_attr_prop in mod_attr.models) {
+                    if(validate(mod_attr.models[mod_attr_prop])) {
+                      handler = mod_attr.models[mod_attr_prop];
+                      parentTr = model.parentTr;
+                      break model_collection_exit;
+                    }
+                  }
+                }
+              }
+            }
+          }
+        }
+
+        if(msg && handler) {
+          msg = msg || _.values(handler.errorModel.attributes)[0];
+
+          handler.trigger('pgadmin-session:model:invalid', msg, handler);
+          $(parentTr).addClass('error-in-grid');
+        }
+        return this;
+      }, 120);
+    },
+
+    onChildChanged: function(obj) {
       var self = this;
 
       if (self.trackChanges && self.collection) {
         (self.collection).trigger('change', self);
       }
+      self.trigger('pgadmin-session:collection:changed', self, obj);
     },
 
     stopSession: function() {
@@ -703,6 +781,7 @@ define([
         self.off('pgadmin-session:changed', self.onChildChanged);
         self.off('pgadmin-session:added', self.onChildChanged);
         self.off('pgadmin-session:removed', self.onChildChanged);
+        self.off('pgadmin-session:collection:changed', self.onChildCollectionChanged);
       }
 
       self.trackChanges = false;
@@ -735,9 +814,7 @@ define([
         field = this.fieldData[keys[i]];
         msg = null;
 
-        if (!(_.isUndefined(value) || _.isNull(value) ||
-            String(value).replace(/^\s+|\s+$/g, '') == '')) {
-
+        if (!(_.isUndefined(value) || _.isNull(value) || String(value) === '')) {
           if (!field) {
             continue;
           }
@@ -777,23 +854,23 @@ define([
         max_value = field.max;
 
       if (!_.isUndefined(min_value) && value < min_value) {
-        return S(pgAdmin.Browser.messages.MUST_GR_EQ).sprintf(label, min_value).value();
+        return pgadminUtils.sprintf(pgAdmin.Browser.messages.MUST_GR_EQ, label, min_value);
       } else if (!_.isUndefined(max_value) && value > max_value) {
-        return S(pgAdmin.Browser.messages.MUST_LESS_EQ).sprintf(label, max_value).value();
+        return pgadminUtils.sprintf(pgAdmin.Browser.messages.MUST_LESS_EQ, label, max_value);
       }
       return null;
     },
     number_validate: function(value, field) {
       var pattern = new RegExp('^-?[0-9]+(\.?[0-9]*)?$');
       if (!pattern.test(value)) {
-        return S(pgAdmin.Browser.messages.MUST_BE_NUM).sprintf(field.label).value();
+        return pgadminUtils.sprintf(pgAdmin.Browser.messages.MUST_BE_NUM, field.label);
       }
       return this.check_min_max(value, field);
     },
     integer_validate: function(value, field) {
       var pattern = new RegExp('^-?[0-9]*$');
       if (!pattern.test(value)) {
-        return S(pgAdmin.Browser.messages.MUST_BE_INT).sprintf(field.label).value();
+        return pgadminUtils.sprintf(pgAdmin.Browser.messages.MUST_BE_INT, field.label);
       }
       return this.check_min_max(value, field);
     },
@@ -996,7 +1073,7 @@ define([
     },
     // Override the reset function, so that - we can reset the model
     // properly.
-    reset: function(opts) {
+    reset: function(_set, opts) {
       if (opts && opts.stop)
         this.stopSession();
       this.each(function(m) {
@@ -1023,129 +1100,127 @@ define([
       return (_.findIndex(this.sessAttrs[type], comparator));
     },
     onModelAdd: function(obj) {
-      if (!this.trackChanges)
-        return true;
+      if (this.trackChanges) {
+        var self = this,
+          msg,
+          idx = self.objFindInSession(obj, 'deleted');
 
-      var self = this,
-        msg,
-        idx = self.objFindInSession(obj, 'deleted');
+        // Hmm.. - it was originally deleted from this collection, we should
+        // remove it from the 'deleted' list.
+        if (idx >= 0) {
+          var origObj = self.sessAttrs['deleted'][idx];
 
-      // Hmm.. - it was originally deleted from this collection, we should
-      // remove it from the 'deleted' list.
-      if (idx >= 0) {
-        var origObj = self.sessAttrs['deleted'][idx];
+          obj.origSessAttrs = _.clone(origObj.origSessAttrs);
+          obj.attributes = _.extend(obj.attributes, origObj.attributes);
+          obj.sessAttrs = _.clone(origObj.sessAttrs);
 
-        obj.origSessAttrs = _.clone(origObj.origSessAttrs);
-        obj.attributes = _.extend(obj.attributes, origObj.attributes);
-        obj.sessAttrs = _.clone(origObj.sessAttrs);
+          self.sessAttrs['deleted'].splice(idx, 1);
 
-        self.sessAttrs['deleted'].splice(idx, 1);
-
-        // It has been changed originally!
-        if ((!('sessChanged' in obj)) || obj.sessChanged()) {
-          self.sessAttrs['changed'].push(obj);
-        }
-
-        (self.handler || self).trigger('pgadmin-session:added', self, obj);
-
-
-        if ('default_validate' in obj && typeof(obj.default_validate) == 'function') {
-          msg = obj.default_validate();
-        }
-
-        if (_.isString(msg)) {
-          (self.sessAttrs['invalid'])[obj.cid] = msg;
-        } else if ('validate' in obj && typeof(obj.validate) === 'function') {
-          msg = obj.validate();
-
-          if (msg) {
-            (self.sessAttrs['invalid'])[obj.cid] = msg;
+          // It has been changed originally!
+          if ((!('sessChanged' in obj)) || obj.sessChanged()) {
+            self.sessAttrs['changed'].push(obj);
           }
-        }
-      } else {
 
-        if ('default_validate' in obj && typeof(obj.default_validate) == 'function') {
-          msg = obj.default_validate();
-        }
+          (self.handler || self).trigger('pgadmin-session:added', self, obj);
 
-        if (_.isString(msg)) {
-          (self.sessAttrs['invalid'])[obj.cid] = msg;
-        } else if ('validate' in obj && typeof(obj.validate) === 'function') {
-          msg = obj.validate();
 
-          if (msg) {
-            (self.sessAttrs['invalid'])[obj.cid] = msg;
+          if ('default_validate' in obj && typeof(obj.default_validate) == 'function') {
+            msg = obj.default_validate();
           }
-        }
-        self.sessAttrs['added'].push(obj);
 
-        /*
-         * Session has been changed
-         */
-        (self.handler || self).trigger('pgadmin-session:added', self, obj);
+          if (_.isString(msg)) {
+            (self.sessAttrs['invalid'])[obj.cid] = msg;
+          } else if ('validate' in obj && typeof(obj.validate) === 'function') {
+            msg = obj.validate();
+
+            if (msg) {
+              (self.sessAttrs['invalid'])[obj.cid] = msg;
+            }
+          }
+        } else {
+
+          if ('default_validate' in obj && typeof(obj.default_validate) == 'function') {
+            msg = obj.default_validate();
+          }
+
+          if (_.isString(msg)) {
+            (self.sessAttrs['invalid'])[obj.cid] = msg;
+          } else if ('validate' in obj && typeof(obj.validate) === 'function') {
+            msg = obj.validate();
+
+            if (msg) {
+              (self.sessAttrs['invalid'])[obj.cid] = msg;
+            }
+          }
+          self.sessAttrs['added'].push(obj);
+
+          /*
+           * Session has been changed
+           */
+          (self.handler || self).trigger('pgadmin-session:added', self, obj);
+        }
+
+        // Let the parent/listener know about my status (valid/invalid).
+        this.triggerValidationEvent.apply(this);
+        self.trigger('pgadmin-session:collection:changed', self);
       }
 
-      // Let the parent/listener know about my status (valid/invalid).
-      this.triggerValidationEvent.apply(this);
-
       return true;
     },
     onModelRemove: function(obj) {
-      if (!this.trackChanges)
-        return true;
+      if (this.trackChanges) {
+        /* Once model is removed from collection clear its errorModel as it's no longer relevant
+         * for us. Otherwise it creates problem in 'clearInvalidSessionIfModelValid' function.
+         */
+        obj.errorModel.clear();
 
-      /* Once model is removed from collection clear its errorModel as it's no longer relevant
-       * for us. Otherwise it creates problem in 'clearInvalidSessionIfModelValid' function.
-       */
-      obj.errorModel.clear();
+        var self = this,
+          invalidModels = self.sessAttrs['invalid'],
+          copy = _.clone(obj),
+          idx = self.objFindInSession(obj, 'added');
 
-      var self = this,
-        invalidModels = self.sessAttrs['invalid'],
-        copy = _.clone(obj),
-        idx = self.objFindInSession(obj, 'added');
+        // We need to remove it from the invalid object list first.
+        if (obj.cid in invalidModels) {
+          delete invalidModels[obj.cid];
+        }
 
-      // We need to remove it from the invalid object list first.
-      if (obj.cid in invalidModels) {
-        delete invalidModels[obj.cid];
+        // Hmm - it was newly added, we can safely remove it.
+        if (idx >= 0) {
+          self.sessAttrs['added'].splice(idx, 1);
+
+          (self.handler || self).trigger('pgadmin-session:removed', self, copy);
+
+          self.checkDuplicateWithModel(copy);
+
+          // Let the parent/listener know about my status (valid/invalid).
+          this.triggerValidationEvent.apply(this);
+        } else {
+          // Hmm - it was changed in this session, we should remove it from the
+          // changed models.
+          idx = self.objFindInSession(obj, 'changed');
+
+          if (idx >= 0) {
+            self.sessAttrs['changed'].splice(idx, 1);
+            (self.handler || self).trigger('pgadmin-session:removed', self, copy);
+          } else {
+            (self.handler || self).trigger('pgadmin-session:removed', self, copy);
+          }
+          /* When removing the object use the original session attrs and not the changed ones */
+          obj.set(obj.origSessAttrs, {silent: true});
+          self.sessAttrs['deleted'].push(obj);
+
+          self.checkDuplicateWithModel(obj);
+
+          // Let the parent/listener know about my status (valid/invalid).
+          this.triggerValidationEvent.apply(this);
+        }
+
+        /*
+         * This object has been remove, we need to check (if we still have any
+         * other invalid message pending).
+         */
       }
 
-      // Hmm - it was newly added, we can safely remove it.
-      if (idx >= 0) {
-        self.sessAttrs['added'].splice(idx, 1);
-
-        (self.handler || self).trigger('pgadmin-session:removed', self, copy);
-
-        self.checkDuplicateWithModel(copy);
-
-        // Let the parent/listener know about my status (valid/invalid).
-        this.triggerValidationEvent.apply(this);
-
-        return true;
-      }
-
-      // Hmm - it was changed in this session, we should remove it from the
-      // changed models.
-      idx = self.objFindInSession(obj, 'changed');
-
-      if (idx >= 0) {
-        self.sessAttrs['changed'].splice(idx, 1);
-        (self.handler || self).trigger('pgadmin-session:removed', self, copy);
-      } else {
-        (self.handler || self).trigger('pgadmin-session:removed', self, copy);
-      }
-
-      self.sessAttrs['deleted'].push(obj);
-
-      self.checkDuplicateWithModel(obj);
-
-      // Let the parent/listener know about my status (valid/invalid).
-      this.triggerValidationEvent.apply(this);
-
-      /*
-       * This object has been remove, we need to check (if we still have any
-       * other invalid message pending).
-       */
-
       return true;
     },
     triggerValidationEvent: function() {
@@ -1188,52 +1263,39 @@ define([
     onModelChange: function(obj) {
       var self = this;
 
-      if (!this.trackChanges || !(obj instanceof pgBrowser.Node.Model))
-        return true;
-
-      var idx = self.objFindInSession(obj, 'added');
-
-      // It was newly added model, we don't need to add into the changed
-      // list.
-      if (idx >= 0) {
-        (self.handler || self).trigger('pgadmin-session:changed', self, obj);
-
-        return true;
-      }
-
-      idx = self.objFindInSession(obj, 'changed');
-
-      if (!('sessChanged' in obj)) {
-        (self.handler || self).trigger('pgadmin-session:changed', self, obj);
+      if (this.trackChanges && obj instanceof pgBrowser.Node.Model) {
+        var idx = self.objFindInSession(obj, 'added');
 
+        // It was newly added model, we don't need to add into the changed
+        // list.
         if (idx >= 0) {
-          return true;
-        }
-
-        self.sessAttrs['changed'].push(obj);
-
-        return true;
-      }
-
-      if (idx >= 0) {
-
-        if (!obj.sessChanged()) {
-          // This object is no more updated, removing it from the changed
-          // models list.
-          self.sessAttrs['changed'].splice(idx, 1);
-
           (self.handler || self).trigger('pgadmin-session:changed', self, obj);
-          return true;
+        } else {
+          idx = self.objFindInSession(obj, 'changed');
+
+          if (!('sessChanged' in obj)) {
+            (self.handler || self).trigger('pgadmin-session:changed', self, obj);
+
+            if (idx < 0) {
+              self.sessAttrs['changed'].push(obj);
+            }
+          } else {
+            if (idx >= 0) {
+              if (!obj.sessChanged()) {
+                // This object is no more updated, removing it from the changed
+                // models list.
+                self.sessAttrs['changed'].splice(idx, 1);
+
+                (self.handler || self).trigger('pgadmin-session:changed', self, obj);
+              } else {
+                (self.handler || self).trigger('pgadmin-session:changed', self, obj);
+              }
+            } else if (obj.sessChanged()) {
+              self.sessAttrs['changed'].push(obj);
+              (self.handler || self).trigger('pgadmin-session:changed', self, obj);
+            }
+          }
         }
-
-        (self.handler || self).trigger('pgadmin-session:changed', self, obj);
-
-        return true;
-      }
-
-      if (obj.sessChanged()) {
-        self.sessAttrs['changed'].push(obj);
-        (self.handler || self).trigger('pgadmin-session:changed', self, obj);
       }
 
       return true;
@@ -1280,12 +1342,12 @@ define([
       } else {
         var msg = 'Duplicate rows.';
         setTimeout(function() {
-          _.each(new_conflicting_models, function(m) {
+          _.each(new_conflicting_models, function(local_model) {
             self.trigger(
-              'pgadmin-session:model:invalid', msg, m, self.handler
+              'pgadmin-session:model:invalid', msg, local_model, self.handler
             );
-            m.trigger(
-              'pgadmin-session:model:duplicate', m, msg
+            local_model.trigger(
+              'pgadmin-session:model:duplicate', local_model, msg
             );
           });
         }, 10);
