@@ -1,10 +1,20 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 import _ from 'underscore';
 import pgAdmin from '../../../static/js/pgadmin';
 import $ from 'jquery';
 import Mousetrap from 'mousetrap';
 import * as commonUtils from '../../../static/js/utils';
 import dialogTabNavigator from '../../../static/js/dialog_tab_navigator';
-import {default as keyboardFunc} from 'sources/keyboard_shortcuts';
+import * as keyboardFunc from 'sources/keyboard_shortcuts';
+import pgWindow from 'sources/window';
 
 const pgBrowser = pgAdmin.Browser = pgAdmin.Browser || {};
 
@@ -24,6 +34,7 @@ _.extend(pgBrowser.keyboardNavigation, {
         'tabbed_panel_forward': commonUtils.parseShortcutValue(pgBrowser.get_preference('browser', 'tabbed_panel_forward').value),
         'sub_menu_query_tool': commonUtils.parseShortcutValue(pgBrowser.get_preference('browser', 'sub_menu_query_tool').value),
         'sub_menu_view_data': commonUtils.parseShortcutValue(pgBrowser.get_preference('browser', 'sub_menu_view_data').value),
+        'sub_menu_search_objects': commonUtils.parseShortcutValue(pgBrowser.get_preference('browser', 'sub_menu_search_objects').value),
         'sub_menu_properties': commonUtils.parseShortcutValue(pgBrowser.get_preference('browser', 'sub_menu_properties').value),
         'sub_menu_create': commonUtils.parseShortcutValue(pgBrowser.get_preference('browser', 'sub_menu_create').value),
         'sub_menu_delete': commonUtils.parseShortcutValue(pgBrowser.get_preference('browser', 'sub_menu_delete').value),
@@ -32,6 +43,7 @@ _.extend(pgBrowser.keyboardNavigation, {
         'direct_debugging': commonUtils.parseShortcutValue(pgBrowser.get_preference('browser', 'direct_debugging').value),
         'drop_multiple_objects': commonUtils.parseShortcutValue(pgBrowser.get_preference('browser', 'grid_menu_drop_multiple').value),
         'drop_cascade_multiple_objects': commonUtils.parseShortcutValue(pgBrowser.get_preference('browser', 'grid_menu_drop_cascade_multiple').value),
+        'add_grid_row': commonUtils.parseShortcutValue(pgBrowser.get_preference('browser', 'add_grid_row').value),
 
       };
       this.shortcutMethods = {
@@ -41,10 +53,10 @@ _.extend(pgBrowser.keyboardNavigation, {
             this.keyboardShortcut.help_shortcut],
         }, // Main menu
         'bindRightPanel': {'shortcuts': [this.keyboardShortcut.tabbed_panel_backward, this.keyboardShortcut.tabbed_panel_forward]}, // Main window panels
-        'bindSubMenuClose': {'shortcuts': ['esc','enter']},
         'bindLeftTree': {'shortcuts': this.keyboardShortcut.left_tree_shortcut}, // Main menu,
         'bindSubMenuQueryTool': {'shortcuts': this.keyboardShortcut.sub_menu_query_tool}, // Sub menu - Open Query Tool,
         'bindSubMenuViewData': {'shortcuts': this.keyboardShortcut.sub_menu_view_data}, // Sub menu - Open View Data,
+        'bindSubMenuSearchObjects': {'shortcuts': this.keyboardShortcut.sub_menu_search_objects}, // Sub menu - Open search objects,
         'bindSubMenuProperties': {'shortcuts': this.keyboardShortcut.sub_menu_properties}, // Sub menu - Edit Properties,
         'bindSubMenuCreate': {'shortcuts': this.keyboardShortcut.sub_menu_create}, // Sub menu - Create Object,
         'bindSubMenuDelete': {'shortcuts': this.keyboardShortcut.sub_menu_delete}, // Sub menu - Delete object,
@@ -53,6 +65,7 @@ _.extend(pgBrowser.keyboardNavigation, {
         'bindDirectDebugging': {'shortcuts': this.keyboardShortcut.direct_debugging}, // Sub menu - Direct Debugging
         'bindDropMultipleObjects': {'shortcuts': this.keyboardShortcut.drop_multiple_objects}, // Grid Menu Drop Multiple
         'bindDropCascadeMultipleObjects': {'shortcuts': this.keyboardShortcut.drop_cascade_multiple_objects}, // Grid Menu Drop Cascade Multiple
+        'bindAddGridRow': {'shortcuts': this.keyboardShortcut.add_grid_row}, // Subnode Grid Add Row
       };
       this.bindShortcuts();
     }
@@ -123,7 +136,7 @@ _.extend(pgBrowser.keyboardNavigation, {
   bindRightPanel: function(event, combo) {
     let allPanels = pgAdmin.Browser.docker.findPanels();
     let activePanel = 0;
-    let nextPanel = allPanels.length;
+    let nextPanel = allPanels.length - 1;
     let prevPanel = 1;
     let activePanelId = 0;
     let activePanelFlag = false;
@@ -166,6 +179,7 @@ _.extend(pgBrowser.keyboardNavigation, {
       let currMenu = currLi.closest('.dropdown-menu');
       if(currMenu.closest('.dropdown-submenu').length > 0) {
         currMenu.removeClass('show');
+        currMenu.closest('.dropdown-submenu').removeClass('dropdown-submenu-visible');
         currLi = currMenu.closest('.dropdown-submenu');
         currLi.find('.dropdown-item').trigger('focus');
       }
@@ -189,8 +203,9 @@ _.extend(pgBrowser.keyboardNavigation, {
 
       /*open submenu if any*/
       if(currLi.hasClass('dropdown-submenu')){
+        currLi.addClass('dropdown-submenu-visible');
         currLi.find('.dropdown-menu').addClass('show');
-        currLi = currLi.find('.dropdown-menu .dropdown-item').first().trigger('focus');
+        currLi.find('.dropdown-menu .dropdown-item').first().trigger('focus');
       }
     }
   },
@@ -200,8 +215,9 @@ _.extend(pgBrowser.keyboardNavigation, {
       keyboardFunc._stopEventPropagation(event);
       let currLi = $(event.target).closest('li');
       /*close all the submenus on movement*/
-      $(event.target).closest('.dropdown-menu').find('.show').removeClass('show');
-
+      let dropMenu = $(event.target).closest('.dropdown-menu');
+      dropMenu.find('.show').removeClass('show');
+      dropMenu.find('.dropdown-submenu').removeClass('dropdown-submenu-visible');
       if(combo === 'up') {
         currLi = currLi.prev();
       }
@@ -222,12 +238,6 @@ _.extend(pgBrowser.keyboardNavigation, {
       currLi.find('.dropdown-item').trigger('focus');
     }
   },
-  bindSubMenuClose: function() {
-    if($(event.target).hasClass('dropdown-item')
-        && $(event.target).closest('.dropdown-submenu').length > 0) {
-      $(event.target).closest('.dropdown').find('.dropdown-submenu .dropdown-menu').removeClass('show');
-    }
-  },
   bindLeftTree: function() {
     const tree = this.getTreeDetails();
 
@@ -253,6 +263,15 @@ _.extend(pgBrowser.keyboardNavigation, {
     // Call data grid method to render view data
     pgAdmin.DataGrid.show_data_grid({'mnuid': 1}, tree.i);
   },
+  bindSubMenuSearchObjects: function() {
+    const tree = this.getTreeDetails();
+
+    if (!tree.d)
+      return;
+
+    // Call data grid method to render view data
+    pgAdmin.SearchObjects.show_search_objects('', tree.i);
+  },
   bindSubMenuProperties: function() {
     const tree = this.getTreeDetails();
 
@@ -266,13 +285,20 @@ _.extend(pgBrowser.keyboardNavigation, {
   },
   bindSubMenuCreate: function() {
     const tree = this.getTreeDetails();
-
-    if (!tree.d || pgAdmin.Browser.Nodes[tree.t.itemData(tree.i)._type].collection_node === true)
+    let node_obj = pgAdmin.Browser.Nodes[tree.t.itemData(tree.i)._type];
+    if (!tree.d){
       return;
+    } else if(node_obj.collection_node === true) {
+      if(node_obj.node) {
+        node_obj = pgAdmin.Browser.Nodes[node_obj.node];
+      } else {
+        return;
+      }
+    }
 
     // Open properties dialog in edit mode
     pgAdmin.Browser.Node.callbacks.show_obj_properties.call(
-      pgAdmin.Browser.Nodes[tree.t.itemData(tree.i)._type], {action: 'create'}
+      node_obj, {action: 'create', item: tree.i}
     );
   },
   bindSubMenuDelete: function() {
@@ -293,8 +319,9 @@ _.extend(pgBrowser.keyboardNavigation, {
   },
   bindContextMenu: function(event) {
     const tree = this.getTreeDetails();
-    const left = $(event.srcElement).find('.aciTreeEntry').position().left + 70;
-    const top = $(event.srcElement).find('.aciTreeEntry').position().top + 70;
+    let _srcElement = event.srcElement || event.target;
+    const left = $(_srcElement).find('.aciTreeEntry').position().left + 70;
+    const top = $(_srcElement).find('.aciTreeEntry').position().top + 70;
 
     tree.t.blur(tree.i);
     $('#tree').trigger('blur');
@@ -325,6 +352,12 @@ _.extend(pgBrowser.keyboardNavigation, {
       $('button.delete_multiple_cascade').click();
     }
   },
+  bindAddGridRow: function() {
+    let subNode = $(document.activeElement).closest('.object.subnode');
+    if ($(subNode).length) {
+      $(subNode).find('.add').click();
+    }
+  },
   isPropertyPanelVisible: function() {
     let isPanelVisible = false;
     _.each(pgAdmin.Browser.docker.findPanels(), (panel) => {
@@ -344,11 +377,11 @@ _.extend(pgBrowser.keyboardNavigation, {
       d: selectedTreeNodeData,
     };
   },
-  getDialogTabNavigator: function(dialog) {
-    const backward_shortcut = pgBrowser.get_preference('browser', 'dialog_tab_backward').value;
-    const forward_shortcut = pgBrowser.get_preference('browser', 'dialog_tab_forward').value;
+  getDialogTabNavigator: function(dialogContainer) {
+    const backward_shortcut = pgWindow.pgAdmin.Browser.get_preference('browser', 'dialog_tab_backward').value;
+    const forward_shortcut = pgWindow.pgAdmin.Browser.get_preference('browser', 'dialog_tab_forward').value;
 
-    return new dialogTabNavigator.dialogTabNavigator(dialog, backward_shortcut, forward_shortcut);
+    return new dialogTabNavigator.dialogTabNavigator(dialogContainer, backward_shortcut, forward_shortcut);
   },
 });
 
