@@ -1,6 +1,15 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define([
-  'underscore', 'underscore.string', 'sources/pgadmin', 'jquery',
-], function(_, S, pgAdmin, $) {
+  'underscore', 'sources/pgadmin', 'jquery', 'sources/utils', 'sources/gettext',
+], function(_, pgAdmin, $, pgadminUtils, gettext) {
   'use strict';
 
   pgAdmin.Browser = pgAdmin.Browser || {};
@@ -10,6 +19,7 @@ define([
     var menu_opts = [
         'name', 'label', 'priority', 'module', 'callback', 'data', 'enable',
         'category', 'target', 'url' /* Do not show icon in the menus, 'icon' */ , 'node',
+        'checked', 'menu_items',
       ],
       defaults = {
         url: '#',
@@ -46,31 +56,49 @@ define([
      * Create the jquery element for the menu-item.
      */
     create_el: function(node, item) {
-      var url = $('<a></a>', {
-        'id': this.name,
-        'href': this.url,
-        'target': this.target,
-        'data-toggle': 'pg-menu',
-      }).data('pgMenu', {
-        module: this.module || pgAdmin.Browser,
-        cb: this.callback,
-        data: this.data,
-      }).addClass('dropdown-item');
 
-      this.is_disabled = this.disabled(node, item);
-      if (this.icon) {
-        url.append($('<i></i>', {
-          'class': this.icon,
-        }));
+      if(this.menu_items) {
+        _.each(this.menu_items, function(submenu_item){
+          submenu_item.generate(node, item);
+        });
+        var create_submenu = pgAdmin.Browser.MenuGroup({
+          'label': this.label,
+          'id': this.name,
+        }, this.menu_items);
+        this.$el = create_submenu.$el;
+      } else {
+        var url = $('<a></a>', {
+          'id': this.name,
+          'href': this.url,
+          'target': this.target,
+          'data-toggle': 'pg-menu',
+          'role': 'menuitem',
+        }).data('pgMenu', {
+          module: this.module || pgAdmin.Browser,
+          cb: this.callback,
+          data: this.data,
+        }).addClass('dropdown-item');
+
+        this.is_disabled = this.disabled(node, item);
+        if (this.icon) {
+          url.append($('<i></i>', {
+            'class': this.icon,
+          }));
+        } else if(!_.isUndefined(this.checked)) {
+          url.append($('<i></i>', {
+            'class': 'fa fa-check '+ (this.checked?'':'visibility-hidden'),
+          }));
+        }
+
+        url.addClass((this.is_disabled ? ' disabled' : ''));
+
+        var textSpan = $('<span data-test="menu-item-text"></span>').text('  ' + this.label);
+
+        url.append(textSpan);
+
+        this.$el = $('<li/>').append(url);
       }
 
-      url.addClass((this.is_disabled ? ' disabled' : ''));
-
-      var textSpan = $('<span data-test="menu-item-text"></span>').text('  ' + this.label);
-
-      url.append(textSpan);
-
-      this.$el = $('<li/>').append(url);
     },
     /*
      * Updates the enable/disable state of the menu-item based on the current
@@ -104,8 +132,8 @@ define([
         cb;
 
       if (o.module['callbacks'] && (
-          o.callback in o.module['callbacks']
-        )) {
+        o.callback in o.module['callbacks']
+      )) {
         cb = o.module['callbacks'][o.callback];
       } else if (o.callback in o.module) {
         cb = o.module[o.callback];
@@ -114,7 +142,7 @@ define([
         cb.apply(o.module, [o.data, item]);
       } else {
         pgAdmin.Browser.report_error(
-          S('Developer Warning: Callback - "%s" not found!').sprintf(o.cb).value()
+          pgadminUtils.sprintf('Developer Warning: Callback - "%s" not found!', o.cb)
         );
       }
     },
@@ -132,8 +160,8 @@ define([
           return true;
         }
         if (_.isArray(this.node) ? (
-            _.indexOf(this.node, node) == -1
-          ) : (this.node != node._type)) {
+          _.indexOf(this.node, node) == -1
+        ) : (this.node != node._type)) {
           return true;
         }
       }
@@ -145,6 +173,20 @@ define([
 
       return false;
     },
+
+    /*
+     * Change the checked value and update the checked icon on the menu
+     */
+    change_checked(isChecked) {
+      if(!_.isUndefined(this.checked)) {
+        this.checked = isChecked;
+        if(this.checked) {
+          this.$el.find('.fa-check').removeClass('visibility-hidden');
+        } else {
+          this.$el.find('.fa-check').addClass('visibility-hidden');
+        }
+      }
+    },
   });
 
   /*
@@ -166,12 +208,12 @@ define([
   pgAdmin.Browser.MenuGroup = function(opts, items, prev, ctx) {
     var template = _.template([
         '<% if (above) { %><li class="dropdown-divider"></li><% } %>',
-        '<li class="dropdown-submenu">',
+        '<li class="dropdown-submenu" role="menuitem">',
         ' <a href="#" class="dropdown-item">',
         '  <% if (icon) { %><i class="<%= icon %>"></i><% } %>',
-        '  <span class="text-white"><%= label %></span>',
+        '  <span><%= label %></span>',
         ' </a>',
-        ' <ul class="dropdown-menu bg-dark">',
+        ' <ul class="dropdown-menu">',
         ' </ul>',
         '</li>',
         '<% if (below) { %><li class="dropdown-divider"></li><% } %>',
@@ -232,20 +274,38 @@ define([
    * menu-items.
    *
    * Arguments:
-   * 1. jQuery Element on which you may want to created the menus
-   * 2. list of menu-items
-   * 3. categories - metadata information about the categories, based on which
+   * 1. nodes_obj - Nodes object contains each node object.
+   * 2. jQuery Element on which you may want to created the menus
+   * 3. list of menu-items
+   * 4. categories - metadata information about the categories, based on which
    *                 the submenu (menu-group) will be created (if any).
-   * 4. d - Data object for the selected browser tree item.
-   * 5. item - The selected browser tree item
-   * 6. menu_items - A empty object on which the context menu for the given
+   * 5. d - Data object for the selected browser tree item.
+   * 6. item - The selected browser tree item
+   * 7. menu_items - A empty object on which the context menu for the given
    *                 list of menu-items.
    *
    * Returns if any menu generated for the given input.
    */
   pgAdmin.Browser.MenuCreator = function(
-    $mnu, menus, categories, d, item, menu_items
+    nodes_obj, $mnu, menus, categories, d, item, menu_items
   ) {
+    let showMenu = true;
+
+    /* We check showMenu function is defined by the respective node, if it is
+     * defined then call the function which will return true or false.
+     */
+    if (d && nodes_obj[d._type] && !_.isUndefined(nodes_obj[d._type].showMenu))
+      showMenu = nodes_obj[d._type].showMenu(d, item);
+
+    if (!showMenu) {
+      menu_items = menu_items || {};
+      menu_items[_.uniqueId('ctx_')+ '1_1_ms'] = {
+        disabled : true,
+        name: gettext('No menu available for this object.'),
+      };
+      return;
+    }
+
     var groups = {
         'common': [],
       },
@@ -338,7 +398,6 @@ define([
     return (len > 0);
   };
 
-
   // MENU PUBLIC CLASS DEFINITION
   // ==============================
   var Menu = function(element, options) {
@@ -355,6 +414,7 @@ define([
       ev.preventDefault();
       return false;
     }
+
     var d = this.$element.data('pgMenu');
     if (d.cb) {
       var cb = d.module && d.module['callbacks'] && d.module['callbacks'][d.cb] || d.module && d.module[d.cb];
@@ -410,11 +470,27 @@ define([
     .on(
       'focus.pg.menu.data-api blur.pg.menu.data-api',
       '[data-toggle^="pg-menu"]',
-      function(e) {
-        $(e.target).closest('.menu').toggleClass(
-          'focus', /^focus(in)?$/.test(e.type)
+      function(ev) {
+        $(ev.target).closest('.menu').toggleClass(
+          'focus', /^focus(in)?$/.test(ev.type)
         );
-      });
+      })
+    .on('mouseenter', '.dropdown-submenu', function(ev) {
+      $(ev.currentTarget).removeClass('dropdown-submenu-visible')
+        .addClass('dropdown-submenu-visible');
+      $(ev.currentTarget).find('.dropdown-menu').first().addClass('show');
+    })
+    .on('mouseleave', '.dropdown-submenu', function(ev) {
+      $(ev.currentTarget).removeClass('dropdown-submenu-visible');
+      $(ev.currentTarget).find('.dropdown-menu').first().removeClass('show');
+    })
+    .on('hidden.bs.dropdown', function(ev) {
+      $(ev.target)
+        .find('.dropdown-submenu.dropdown-submenu-visible')
+        .removeClass('dropdown-submenu-visible')
+        .find('.dropdown-menu.show')
+        .removeClass('show');
+    });
 
   return pgAdmin.Browser.MenuItem;
 });
