@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -63,6 +63,3 @@ class InternalVersionTestCase(BaseTestGenerator):
                 self.assertTrue(url.endswith(version_string))
             else:
                 self.assertFalse(url.endswith(version_string))
-
-    def tearDown(self):
-        config = self.config_bak
