@@ -1,12 +1,20 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define('pgadmin.dashboard', [
   'sources/url_for', 'sources/gettext', 'require', 'jquery', 'underscore',
-  'sources/pgadmin', 'backbone', 'backgrid', './charting',
-  'pgadmin.alertifyjs', 'pgadmin.backform',
-  'sources/nodes/dashboard', 'backgrid.filter',
-  'pgadmin.browser', 'bootstrap', 'wcdocker',
+  'sources/pgadmin', 'backbone', 'backgrid',
+  'pgadmin.alertifyjs', 'pgadmin.backform', 'sources/nodes/dashboard',
+  'sources/window', './ChartsDOM', 'pgadmin.browser', 'bootstrap', 'wcdocker',
 ], function(
-  url_for, gettext, r, $, _, pgAdmin, Backbone, Backgrid, charting,
-  Alertify, Backform, NodesDashboard
+  url_for, gettext, r, $, _, pgAdmin, Backbone, Backgrid,
+  Alertify, Backform, NodesDashboard, pgWindow, ChartsDOM
 ) {
 
   pgAdmin.Browser = pgAdmin.Browser || {};
@@ -35,15 +43,22 @@ define('pgadmin.dashboard', [
         this.$el.html(
           '<i class=\'fa fa-stop\' data-toggle=\'tooltip\' ' +
           'title=\'' + gettext('Cancel the active query') +
-          '\'></i>'
+          '\' aria-label=\''+ gettext('Cancel the active query') +'\'></i>'
         );
       } else {
         this.$el.html(
-          '<i class=\'fa fa-times-circle\' data-toggle=\'tooltip\' ' +
+          '<i class=\'fa fa-times-circle text-danger\' data-toggle=\'tooltip\' ' +
           'title=\'' + gettext('Terminate the session') +
-          '\'></i>'
+          '\' aria-label=\''+ gettext('Terminate the session') +'\'></i>'
         );
       }
+      this.$el.attr('tabindex', 0);
+      this.$el.on('keydown', function(e) {
+        // terminating session or cancel the active query.
+        if (e.keyCode == 32) {
+          self.$el.click();
+        }
+      });
       this.delegateEvents();
       return this;
     },
@@ -62,12 +77,8 @@ define('pgadmin.dashboard', [
         return;
 
       // This will refresh the grid
-      var refresh_grid = function() {
-        if (is_server_dashboard) {
-          $('#btn_server_activity_refresh').trigger('click');
-        } else if (is_database_dashboard) {
-          $('#btn_database_activity_refresh').trigger('click');
-        }
+      let refresh_grid = () => {
+        $('#btn_refresh').trigger('click');
       };
 
       if (cell_action === 'cancel') {
@@ -91,17 +102,17 @@ define('pgadmin.dashboard', [
             url: action_url,
             type: 'DELETE',
           })
-          .done(function(res) {
-            if (res == gettext('Success')) {
-              Alertify.success(txtSuccess);
-              refresh_grid();
-            } else {
-              Alertify.error(txtError);
-            }
-          })
-          .fail(function(xhr, status, error) {
-            Alertify.pgRespErrorNotify(xhr, error);
-          });
+            .done(function(res) {
+              if (res == gettext('Success')) {
+                Alertify.success(txtSuccess);
+                refresh_grid();
+              } else {
+                Alertify.error(txtError);
+              }
+            })
+            .fail(function(xhr, status, error) {
+              Alertify.pgRespErrorNotify(xhr, error);
+            });
         },
         function() {
           return true;
@@ -136,7 +147,7 @@ define('pgadmin.dashboard', [
       this.$el.html(
         '<i class=\'fa fa-caret-right\' data-toggle=\'tooltip\' ' +
         'title=\'' + gettext('View the active session details') +
-        '\'></i>'
+        '\' aria-label=\''+ gettext('View the active session details') +'\'></i>'
       );
       this.delegateEvents();
       if (this.grabFocus)
@@ -156,7 +167,7 @@ define('pgadmin.dashboard', [
       label: gettext('Backend type'),
       type: 'text',
       editable: true,
-      disabled: true,
+      readonly: true,
       group: gettext('Details'),
       visible: function() {
         return this.version >= 100000;
@@ -166,21 +177,21 @@ define('pgadmin.dashboard', [
       label: gettext('Query started at'),
       type: 'text',
       editable: false,
-      disabled: true,
+      readonly: true,
       group: gettext('Details'),
     }, {
       id: 'state_change',
       label: gettext('Last state changed at'),
       type: 'text',
       editable: true,
-      disabled: true,
+      readonly: true,
       group: gettext('Details'),
     }, {
       id: 'query',
       label: gettext('SQL'),
       type: 'text',
       editable: true,
-      disabled: true,
+      readonly: true,
       control: Backform.SqlFieldControl,
       group: gettext('Details'),
     }],
@@ -192,6 +203,7 @@ define('pgadmin.dashboard', [
         return;
 
       this.initialized = true;
+      this.chartsDomObj = null;
 
       this.sid = this.did = -1;
       this.version = -1;
@@ -210,27 +222,40 @@ define('pgadmin.dashboard', [
       // Load the default welcome dashboard
       var url = url_for('dashboard.index');
 
-      /* Store the chart objects and there interval ids in this store */
-      this.chartStore = {};
-
       var dashboardPanel = pgBrowser.panels['dashboard'].panel;
       if (dashboardPanel) {
         var div = dashboardPanel.layout().scene().find('.pg-panel-content');
 
         if (div) {
-          $.ajax({
-            url: url,
-            type: 'GET',
-            dataType: 'html',
-          })
-          .done(function(data) {
-            $(div).html(data);
-          })
-          .fail(function() {
-            $(div).html(
-              '<div class="alert alert-danger pg-panel-message" role="alert">' + gettext('An error occurred whilst loading the dashboard.') + '</div>'
-            );
-          });
+          var ajaxHook = function() {
+            $.ajax({
+              url: url,
+              type: 'GET',
+              dataType: 'html',
+            })
+              .done(function(data) {
+                $(div).html(data);
+              })
+              .fail(function(xhr, error) {
+                Alertify.pgNotifier(
+                  error, xhr,
+                  gettext('An error occurred whilst loading the dashboard.'),
+                  function(msg) {
+                    if(msg === 'CRYPTKEY_SET') {
+                      ajaxHook();
+                    } else {
+                      $(div).html(
+                        '<div class="pg-panel-message" role="alert">' + gettext('An error occurred whilst loading the dashboard.') + '</div>'
+                      );
+                    }
+                  }
+                );
+              });
+          };
+          $(div).html(
+            '<div class="pg-panel-message" role="alert">' + gettext('Loading dashboard...') + '</div>'
+          );
+          ajaxHook();
 
           // Cache the current IDs for next time
           $(dashboardPanel).data('sid', -1);
@@ -255,18 +280,7 @@ define('pgadmin.dashboard', [
     object_selected: function(item, itemData, node) {
       let self = this;
 
-      if (dashboardVisible === false) {
-        /*
-         * Clear all the interval functions, even when dashboard is not
-         * visible (in case of connection of the object got disconnected).
-         */
-        if (
-          !_.isUndefined(itemData.connected) &&
-            itemData.connected !== true
-        ) {
-          self.clearChartFromStore();
-        }
-      } else if (itemData && itemData._type) {
+      if (itemData && itemData._type) {
         var treeHierarchy = node.getTreeNodeHierarchy(item),
           url = NodesDashboard.url(itemData, item, treeHierarchy);
 
@@ -327,38 +341,45 @@ define('pgadmin.dashboard', [
                   $(dashboardPanel).data('server_status') == false
                 )
               ) {
+                this.chartsDomObj && this.chartsDomObj.unmount();
                 $(div).empty();
 
-                /* Clear all the charts previous dashboards */
-                self.clearChartFromStore();
-
-                $.ajax({
-                  url: url,
-                  type: 'GET',
-                  dataType: 'html',
-                })
-                .done(function(data) {
-                  $(div).html(data);
-                  self.init_dashboard();
-                })
-                .fail(function() {
-                  $(div).html(
-                    '<div class="alert alert-danger pg-panel-message" role="alert">' + gettext('An error occurred whilst loading the dashboard.') + '</div>'
-                  );
-                });
+                let ajaxHook = function() {
+                  $.ajax({
+                    url: url,
+                    type: 'GET',
+                    dataType: 'html',
+                  })
+                    .done(function(data) {
+                      $(div).html(data);
+                      self.init_dashboard();
+                    })
+                    .fail(function(xhr, error) {
+                      Alertify.pgNotifier(
+                        error, xhr,
+                        gettext('An error occurred whilst loading the dashboard.'),
+                        function(msg) {
+                          if(msg === 'CRYPTKEY_SET') {
+                            ajaxHook();
+                          } else {
+                            $(div).html(
+                              '<div class="pg-panel-message" role="alert">' + gettext('An error occurred whilst loading the dashboard.') + '</div>'
+                            );
+                          }
+                        }
+                      );
+                    });
+                };
+                $(div).html(
+                  '<div class="pg-panel-message" role="alert">' + gettext('Loading dashboard...') + '</div>'
+                );
+                ajaxHook();
                 $(dashboardPanel).data('server_status', true);
               }
             } else {
-              $(div).empty();
-              if (
-                !_.isUndefined(itemData.connected) &&
-                  itemData.connected !== true
-              ) {
-                /* Clear all the charts previous dashboards */
-                self.clearChartFromStore();
-              }
+              this.chartsDomObj && this.chartsDomObj.unmount();
               $(div).html(
-                '<div class="alert alert-info pg-panel-message" role="alert">' + gettext('Please connect to the selected server to view the dashboard.') + '</div>'
+                '<div class="pg-panel-message" role="alert">' + gettext('Please connect to the selected server to view the dashboard.') + '</div>'
               );
               $(dashboardPanel).data('server_status', false);
             }
@@ -369,185 +390,12 @@ define('pgadmin.dashboard', [
       }
     },
 
-    renderChartLoop: function(chartObj, sid, did, url, counter, refresh) {
-      var data = [],
-        dataset = [];
-
-      var theIntervalFunc = function() {
-        var path = url + sid;
-        if (did != -1) {
-          path += '/' + did;
-        }
-        $.ajax({
-          url: path,
-          type: 'GET',
-          dataType: 'html',
-        })
-        .done(function(resp) {
-          $(chartObj.getContainer()).removeClass('graph-error');
-          data = JSON.parse(resp);
-
-          var y = 0,
-            x;
-          if (dataset.length == 0) {
-            if (counter == true) {
-              // Have we stashed initial values?
-              if (_.isUndefined(chartObj.getOtherData('counter_previous_vals'))) {
-                chartObj.setOtherData('counter_previous_vals', data[0]);
-              } else {
-                // Create the initial data structure
-                for (x in data[0]) {
-                  dataset.push({
-                    'data': [
-                      [0, data[0][x] - chartObj.getOtherData('counter_previous_vals')[x]],
-                    ],
-                    'label': x,
-                  });
-                }
-              }
-            } else {
-              // Create the initial data structure
-              for (x in data[0]) {
-                dataset.push({
-                  'data': [
-                    [0, data[0][x]],
-                  ],
-                  'label': x,
-                });
-              }
-            }
-          } else {
-            for (x in data[0]) {
-              // Push new values onto the existing data structure
-              // If this is a counter stat, we need to subtract the previous value
-              if (counter == false) {
-                dataset[y]['data'].unshift([0, data[0][x]]);
-              } else {
-                // Store the current value, minus the previous one we stashed.
-                // It's possible the tab has been reloaded, in which case out previous values are gone
-                if (_.isUndefined(chartObj.getOtherData('counter_previous_vals')))
-                  return;
-
-                dataset[y]['data'].unshift([0, data[0][x] - chartObj.getOtherData('counter_previous_vals')[x]]);
-              }
-
-              // Reset the time index to get a proper scrolling display
-              for (var z = 0; z < dataset[y]['data'].length; z++) {
-                dataset[y]['data'][z][0] = z;
-              }
-
-              y++;
-            }
-            chartObj.setOtherData('counter_previous_vals', data[0]);
-          }
-
-          // Remove uneeded elements
-          for (x = 0; x < dataset.length; x++) {
-            // Remove old data points
-            if (dataset[x]['data'].length > 101) {
-              dataset[x]['data'].pop();
-            }
-          }
-
-          if (chartObj.isInPage()) {
-            if (chartObj.isVisible()) {
-              chartObj.draw(dataset);
-            }
-          } else {
-            return;
-          }
-
-        })
-        .fail(function(xhr) {
-          let err = '';
-          let msg = '';
-          let cls = 'info';
-
-          if (xhr.readyState === 0) {
-            msg = gettext('Not connected to the server or the connection to the server has been closed.');
-          } else {
-            err = JSON.parse(xhr.responseText);
-            msg = err.errormsg;
-
-            // If we get a 428, it means the server isn't connected
-            if (xhr.status === 428) {
-              if (_.isUndefined(msg) || _.isNull(msg)) {
-                msg = gettext('Please connect to the selected server to view the graph.');
-              }
-            } else {
-              msg = gettext('An error occurred whilst rendering the graph.');
-              cls = 'danger';
-            }
-          }
-
-          $(chartObj.getContainer()).addClass('graph-error');
-          $(chartObj.getContainer()).html(
-            '<div class="alert alert-' + cls + ' pg-panel-message" role="alert">' + msg + '</div>'
-          );
-        });
-      };
-      /* Execute once for the first time as setInterval will not do */
-      theIntervalFunc();
-      return setInterval(theIntervalFunc, refresh * 1000);
-    },
-
-    // Render a chart
-    render_chart: function(
-      container, url, options, counter, chartName, prefName
-    ) {
-
-      // Data format:
-      // [
-      //     { data: [[0, y0], [1, y1]...], label: 'Label 1', [options] },
-      //     { data: [[0, y0], [1, y1]...], label: 'Label 2', [options] },
-      //     { data: [[0, y0], [1, y1]...], label: 'Label 3', [options] }
-      // ]
-
-      let self = this,
-        tooltipFormatter = function(refresh, currVal) {
-          return(`Seconds ago: ${parseInt(currVal.x * refresh)}</br>
-                  Value: ${currVal.y}`);
-        };
-
-      if(self.chartStore[chartName]
-        && self.old_preferences[prefName] != self.preferences[prefName]) {
-        self.clearChartFromStore(chartName);
-      }
-
-      if(self.chartStore[chartName]) {
-        let chartObj = self.chartStore[chartName].chartObj;
-        chartObj.setOptions(options, false);
-        chartObj.setTooltipFormatter(
-          tooltipFormatter.bind(null, self.preferences[prefName])
-        );
-      }
-
-      if(!self.chartStore[chartName]) {
-
-        let chartObj = new charting.Chart(container, options);
-
-        chartObj.setTooltipFormatter(
-          tooltipFormatter.bind(null, self.preferences[prefName])
-        );
-
-        self.chartStore[chartName] = {
-          'chartObj' : chartObj,
-          'intervalId' : undefined,
-        };
-
-        self.chartStore[chartName]['intervalId'] = self.renderChartLoop(
-          self.chartStore[chartName]['chartObj'], self.sid, self.did, url,
-          counter, self.preferences[prefName]
-        );
-      }
-    },
-
     // Handler function to support the "Add Server" link
     add_new_server: function() {
       if (pgBrowser && pgBrowser.tree) {
         var i = pgBrowser.tree.selected().length != 0 ?
-          pgBrowser.tree.selected() :
-          pgBrowser.tree.first(null, false),
+            pgBrowser.tree.selected() :
+            pgBrowser.tree.first(null, false),
           serverModule = require('pgadmin.node.server'),
           itemData = pgBrowser.tree.itemData(i);
 
@@ -590,9 +438,10 @@ define('pgadmin.dashboard', [
 
       // Set up the grid
       var grid = new Backgrid.Grid({
+        emptyText: gettext('No data found'),
         columns: columns,
         collection: data,
-        className: 'backgrid table-bordered presentation table backgrid-striped',
+        className: 'backgrid presentation table table-bordered table-noouter-border table-hover',
       });
 
       // Render the grid
@@ -605,17 +454,7 @@ define('pgadmin.dashboard', [
         collection: data,
       });
 
-      // Render the filter
-      $('#' + container.id + '_filter').before(filter.render().el);
-
-      // Add some space to the filter and move it to the right
-      $(filter.el).css({
-        float: 'right',
-        margin: '5px',
-        'margin-right': '2px',
-        'margin-top': '3px',
-      });
-
+      filter.setCustomSearchBox($('#txtGridSearch'));
       // Stash objects for future use
       $(container).data('data', data);
       $(container).data('grid', grid);
@@ -663,7 +502,7 @@ define('pgadmin.dashboard', [
               }
             } else {
               msg = gettext('An error occurred whilst rendering the table.');
-              cls = 'danger';
+              cls = 'error';
             }
           }
 
@@ -674,7 +513,7 @@ define('pgadmin.dashboard', [
           }
 
           $(container).html(
-            '<div class="alert alert-' + cls + ' pg-panel-message" role="alert">' + msg + '</div>'
+            '<div class="pg-panel-' + cls + ' pg-panel-message" role="alert">' + msg + '</div>'
           );
 
           // Try again
@@ -685,30 +524,24 @@ define('pgadmin.dashboard', [
       });
     },
 
-    clearChartFromStore: function(chartName) {
-      var self = this;
-      if(!chartName){
-        _.each(self.chartStore, function(chart, key) {
-          clearInterval(chart.intervalId);
-          delete self.chartStore[key];
-        });
-      }
-      else {
-        clearInterval(self.chartStore[chartName].intervalId);
-        delete self.chartStore[chartName];
-      }
-    },
-
     // Rock n' roll on the dashboard
     init_dashboard: function() {
       let self = this;
 
+      this.chartsDomObj = new ChartsDOM.default(
+        document.getElementById('dashboard-graphs'),
+        self.preferences,
+        self.sid,
+        self.did,
+        $('.dashboard-container')[0].clientHeight <=0 ? false : true
+      );
+
       /* Cache may take time to load for the first time
        * Keep trying till available
        */
       let cacheIntervalId = setInterval(function() {
         try {
-          if(window.top.pgAdmin.Browser.preference_version() > 0) {
+          if(pgWindow.default.pgAdmin.Browser.preference_version() > 0) {
             clearInterval(cacheIntervalId);
             self.reflectPreferences();
           }
@@ -723,76 +556,14 @@ define('pgadmin.dashboard', [
       pgBrowser.onPreferencesChange('dashboards', function() {
         self.reflectPreferences();
       });
-
     },
 
     reflectPreferences: function() {
       var self = this;
-
-      /* We will use old preferences for selective graph updates on preference change */
-      if(self.preferences) {
-        self.old_preferences = self.preferences;
-        self.preferences = window.top.pgAdmin.Browser.get_preferences_for_module('dashboards');
-      }
-      else {
-        self.preferences = window.top.pgAdmin.Browser.get_preferences_for_module('dashboards');
-        self.old_preferences = self.preferences;
-      }
+      self.preferences = pgWindow.default.pgAdmin.Browser.get_preferences_for_module('dashboards');
+      this.chartsDomObj.reflectPreferences(self.preferences);
 
       if(is_server_dashboard || is_database_dashboard) {
-        /* Common things can come here */
-        var div_sessions = $('.dashboard-container').find('#graph-sessions')[0];
-        var div_tps = $('.dashboard-container').find('#graph-tps')[0];
-        var div_ti = $('.dashboard-container').find('#graph-ti')[0];
-        var div_to = $('.dashboard-container').find('#graph-to')[0];
-        var div_bio = $('.dashboard-container').find('#graph-bio')[0];
-        var options_line = {
-          parseFloat: false,
-          xaxis: {
-            min: 100,
-            max: 0,
-            autoscale: 0,
-          },
-          yaxis: {
-            autoscale: 1,
-          },
-        };
-
-        if(self.preferences.graph_data_points) {
-          /* Merge data points related options */
-          options_line = {
-            ...options_line,
-            ...{
-              points: {
-                show:true,
-                radius: 1,
-                hitRadius: 3,
-              },
-            },
-          };
-        }
-
-        if(self.preferences.graph_mouse_track) {
-          /* Merge mouse track related options */
-          options_line = {
-            ...options_line,
-            ...{
-              mouse: {
-                track:true,
-                position: 'sw',
-              },
-            },
-          };
-        }
-
-        if(self.preferences.show_graphs && $('#dashboard-graphs').hasClass('dashboard-hidden')) {
-          $('#dashboard-graphs').removeClass('dashboard-hidden');
-        }
-        else if(!self.preferences.show_graphs) {
-          $('#dashboard-graphs').addClass('dashboard-hidden');
-          self.clearChartFromStore();
-        }
-
         if (self.preferences.show_activity && $('#dashboard-activity').hasClass('dashboard-hidden')) {
           $('#dashboard-activity').removeClass('dashboard-hidden');
         }
@@ -800,35 +571,11 @@ define('pgadmin.dashboard', [
           $('#dashboard-activity').addClass('dashboard-hidden');
         }
 
-        if(self.preferences.show_graphs) {
-          // Render the graphs
-          pgAdmin.Dashboard.render_chart(
-            div_sessions, url_for('dashboard.session_stats'), options_line, false,
-            'session_stats', 'session_stats_refresh'
-          );
-          pgAdmin.Dashboard.render_chart(
-            div_tps, url_for('dashboard.tps_stats'), options_line, true,
-            'tps_stats','tps_stats_refresh'
-          );
-          pgAdmin.Dashboard.render_chart(
-            div_ti, url_for('dashboard.ti_stats'), options_line, true,
-            'ti_stats', 'ti_stats_refresh'
-          );
-          pgAdmin.Dashboard.render_chart(
-            div_to, url_for('dashboard.to_stats'), options_line, true,
-            'to_stats','to_stats_refresh'
-          );
-          pgAdmin.Dashboard.render_chart(
-            div_bio, url_for('dashboard.bio_stats'), options_line, true,
-            'bio_stats','bio_stats_refresh'
-          );
+        if (self.preferences.show_activity && $('#dashboard-activity').hasClass('dashboard-hidden')) {
+          $('#dashboard-activity').removeClass('dashboard-hidden');
         }
-
-        if(!self.preferences.show_graphs && !self.preferences.show_activity) {
-          $('#dashboard-none-show').removeClass('dashboard-hidden');
-        }
-        else {
-          $('#dashboard-none-show').addClass('dashboard-hidden');
+        else if(!self.preferences.show_activity) {
+          $('#dashboard-activity').addClass('dashboard-hidden');
         }
 
         /* Dashboard specific preferences can be updated in the
@@ -850,6 +597,13 @@ define('pgadmin.dashboard', [
       var div_server_prepared = $dashboardContainer.find('#server_prepared').get(0);
       var div_server_config = $dashboardContainer.find('#server_config').get(0);
 
+      var tab_grid_map = {
+        'tab_server_activity': div_server_activity,
+        'tab_server_locks': div_server_locks,
+        'tab_server_prepared': div_server_prepared,
+        'tab_server_config': div_server_config,
+      };
+
       // Display server activity
       if (self.preferences.show_activity) {
         var server_activity_columns = [{
@@ -901,7 +655,7 @@ define('pgadmin.dashboard', [
           server_activity_columns = server_activity_columns.concat(
             [{
               name: 'wait_event',
-              label: gettext('Wait Event'),
+              label: gettext('Wait event'),
               editable: false,
               cell: 'string',
             }, {
@@ -1099,45 +853,17 @@ define('pgadmin.dashboard', [
 
         // (Re)render the appropriate tab
         $('a[data-toggle="tab"]').on('shown.bs.tab', function(e) {
-          switch ($(e.target).attr('aria-controls')) {
-          case 'tab_server_activity':
-            pgAdmin.Dashboard.render_grid_data(div_server_activity);
-            break;
+          let prevGrid = tab_grid_map[$(e.relatedTarget).attr('aria-controls')];
+          $(prevGrid).data('filtertext', $('#txtGridSearch').val());
 
-          case 'tab_server_locks':
-            pgAdmin.Dashboard.render_grid_data(div_server_locks);
-            break;
-
-          case 'tab_server_prepared':
-            pgAdmin.Dashboard.render_grid_data(div_server_prepared);
-            break;
-
-          case 'tab_server_config':
-            pgAdmin.Dashboard.render_grid_data(div_server_config);
-            break;
-          }
+          let currGrid = tab_grid_map[$(e.target).attr('aria-controls')];
+          $('#txtGridSearch').val($(currGrid).data('filtertext'));
+          pgAdmin.Dashboard.render_grid_data(currGrid);
         });
 
-        $('button').off('click').on('click', (e) => {
-          let $this = $(e.currentTarget);
-          let targetID = $this.length ? $this[0].id : undefined;
-          switch (targetID) {
-          case 'btn_server_activity_refresh':
-            pgAdmin.Dashboard.render_grid_data(div_server_activity);
-            break;
-
-          case 'btn_server_locks_refresh':
-            pgAdmin.Dashboard.render_grid_data(div_server_locks);
-            break;
-
-          case 'btn_server_prepared_refresh':
-            pgAdmin.Dashboard.render_grid_data(div_server_prepared);
-            break;
-
-          case 'btn_server_config_refresh':
-            pgAdmin.Dashboard.render_grid_data(div_server_config);
-            break;
-          }
+        $('#btn_refresh').off('click').on('click', () => {
+          let currGrid = tab_grid_map[$('#dashboard-activity .nav-tabs .active').attr('aria-controls')];
+          pgAdmin.Dashboard.render_grid_data(currGrid);
         });
       }
     },
@@ -1147,6 +873,12 @@ define('pgadmin.dashboard', [
       var div_database_locks = document.getElementById('database_locks');
       var div_database_prepared = document.getElementById('database_prepared');
 
+      var tab_grid_map = {
+        'tab_database_activity': div_database_activity,
+        'tab_database_locks': div_database_locks,
+        'tab_database_prepared': div_database_prepared,
+      };
+
       // Display server activity
       if (self.preferences.show_activity) {
         var database_activity_columns = [{
@@ -1193,7 +925,7 @@ define('pgadmin.dashboard', [
           database_activity_columns = database_activity_columns.concat(
             [{
               name: 'wait_event',
-              label: gettext('Wait Event'),
+              label: gettext('Wait event'),
               editable: false,
               cell: 'string',
             }, {
@@ -1348,46 +1080,27 @@ define('pgadmin.dashboard', [
         pgAdmin.Dashboard.render_grid_data(div_database_activity);
 
         // (Re)render the appropriate tab
-        $('a[data-toggle="tab"]').off('shown.bs.tab').on('shown.bs.tab', function(e) {
-          switch ($(e.target).attr('aria-controls')) {
-          case 'tab_database_activity':
-            pgAdmin.Dashboard.render_grid_data(div_database_activity);
-            break;
+        $('a[data-toggle="tab"]').on('shown.bs.tab', function(e) {
+          let prevGrid = tab_grid_map[$(e.relatedTarget).attr('aria-controls')];
+          $(prevGrid).data('filtertext', $('#txtGridSearch').val());
 
-          case 'tab_database_locks':
-            pgAdmin.Dashboard.render_grid_data(div_database_locks);
-            break;
-
-          case 'tab_database_prepared':
-            pgAdmin.Dashboard.render_grid_data(div_database_prepared);
-            break;
-          }
+          let currGrid = tab_grid_map[$(e.target).attr('aria-controls')];
+          $('#txtGridSearch').val($(currGrid).data('filtertext'));
+          pgAdmin.Dashboard.render_grid_data(currGrid);
         });
 
-        // Handle button clicks
-        $('button').off('click').on('click', (e) => {
-          let $this = $(e.currentTarget);
-          let targetID = $this.length ? $this[0].id : undefined;
-          switch (targetID) {
-          case 'btn_database_activity_refresh':
-            pgAdmin.Dashboard.render_grid_data(div_database_activity);
-            break;
-
-          case 'btn_database_locks_refresh':
-            pgAdmin.Dashboard.render_grid_data(div_database_locks);
-            break;
-
-          case 'btn_database_prepared_refresh':
-            pgAdmin.Dashboard.render_grid_data(div_database_prepared);
-            break;
-          }
+        $('#btn_refresh').off('click').on('click', () => {
+          let currGrid = tab_grid_map[$('#dashboard-activity .nav-tabs .active').attr('aria-controls')];
+          pgAdmin.Dashboard.render_grid_data(currGrid);
         });
       }
     },
     toggleVisibility: function(visible, closed=false) {
       dashboardVisible = visible;
       if(closed) {
-        this.clearChartFromStore();
+        this.chartsDomObj && this.chartsDomObj.unmount();
+      } else {
+        this.chartsDomObj && this.chartsDomObj.setPageVisible(dashboardVisible);
       }
     },
     can_take_action: function(m) {
@@ -1400,7 +1113,7 @@ define('pgadmin.dashboard', [
         pg_version = this.get('postgres_version') || null,
         cell_action = this.get('cell_action') || null,
         is_cancel_session = cell_action === 'cancel',
-        txtAction = is_cancel_session ? gettext('cancel') : gettext('terminate');
+        txtMessage;
 
       // With PG10, We have background process showing on dashboard
       // We will not allow user to cancel them as they will fail with error
@@ -1408,20 +1121,22 @@ define('pgadmin.dashboard', [
 
       // Background processes do not have database field populated
       if (pg_version && pg_version >= 100000 && !m.get('datname')) {
-        Alertify.info(
-          gettext('You cannot ') +
-          txtAction +
-          gettext(' background worker processes.')
-        );
+        if (is_cancel_session) {
+          txtMessage = gettext('You cannot cancel background worker processes.');
+        } else {
+          txtMessage = gettext('You cannot terminate background worker processes.');
+        }
+        Alertify.info(txtMessage);
         return false;
         // If it is the last active connection on maintenance db then error out
       } else if (maintenance_database == m.get('datname') &&
         m.get('state') == 'active' && active_sessions.length == 1) {
-        Alertify.error(
-          gettext('You are not allowed to ') +
-          txtAction +
-          gettext(' the main active session.')
-        );
+        if (is_cancel_session) {
+          txtMessage = gettext('You are not allowed to cancel the main active session.');
+        } else {
+          txtMessage = gettext('You are not allowed to terminate the main active session.');
+        }
+        Alertify.error(txtMessage);
         return false;
       } else if (is_cancel_session && m.get('state') == 'idle') {
         // If this session is already idle then do nothing
@@ -1437,11 +1152,12 @@ define('pgadmin.dashboard', [
         return true;
       } else {
         // Do not allow to cancel someone else session to non-super user
-        Alertify.error(
-          gettext('Superuser privileges are required to ') +
-          txtAction +
-          gettext(' another users query.')
-        );
+        if (is_cancel_session) {
+          txtMessage = gettext('Superuser privileges are required to cancel another users query.');
+        } else {
+          txtMessage = gettext('Superuser privileges are required to terminate another users query.');
+        }
+        Alertify.error(txtMessage);
         return false;
       }
     },
