@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -15,17 +15,19 @@ from selenium.webdriver import ActionChains
 from regression.python_test_utils import test_utils
 from regression.feature_utils.base_feature_test import BaseFeatureTest
 from selenium.webdriver.common.by import By
+from selenium.webdriver.common.keys import Keys
 from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.support.ui import WebDriverWait
+from regression.feature_utils.locators import QueryToolLocators, \
+    NavMenuLocators
+from regression.feature_utils.tree_area_locators import TreeAreaLocators
 
 CURRENT_PATH = os.path.dirname(os.path.realpath(__file__))
 
-try:
-    with open(CURRENT_PATH + '/test_data.json') as data_file:
-        config_data = json.load(data_file)[
-            'table_insert_update_cases']['add_update']
-except Exception as e:
-    print(str(e))
+config_data = config_data_json = {}
+# try:
+with open(CURRENT_PATH + '/test_data.json') as data_file:
+    config_data_json = json.load(data_file)
 
 
 class CheckForViewDataTest(BaseFeatureTest):
@@ -80,6 +82,15 @@ CREATE TABLE public.defaults_{0}
     CONSTRAINT defaults_pkey_{0} PRIMARY KEY ({1})
 )
 """
+    non_int_pkey_table = """
+CREATE TABLE public.nonintpkey
+(
+    charid text COLLATE pg_catalog."default" NOT NULL,
+    col1 text,
+    col2 numeric(100),
+    CONSTRAINT nonintpkey_pkey PRIMARY KEY (charid)
+)
+    """
 
     def before(self):
         with test_utils.Database(self.server) as (connection, _):
@@ -95,6 +106,12 @@ CREATE TABLE public.defaults_{0}
                 self.test_db,
                 CheckForViewDataTest.defaults_query.format(k, v))
 
+        test_utils.create_table_with_query(
+            self.server,
+            self.test_db,
+            CheckForViewDataTest.non_int_pkey_table
+        )
+
         # Initialize an instance of WebDriverWait with timeout of 3 seconds
         self.wait = WebDriverWait(self.driver, 3)
 
@@ -104,24 +121,31 @@ CREATE TABLE public.defaults_{0}
     def runTest(self):
         self.page.wait_for_spinner_to_disappear()
         self.page.add_server(self.server)
-        self._tables_node_expandable()
+
+        self.page.expand_database_node(
+            self.server['name'],
+            self.server['db_password'], self.test_db)
+        self.page.toggle_open_tables_node(self.server['name'],
+                                          self.server['db_password'],
+                                          self.test_db, 'public')
+
+        self._load_config_data('table_insert_update_cases')
+        data_local = config_data
         # iterate on both tables
         for cnt in (1, 2):
-            self.page.select_tree_item('defaults_{0}'.format(str(cnt)))
-            # Open Object -> View/Edit data
-            self._view_data_grid()
-
-            self.page.wait_for_query_tool_loading_indicator_to_disappear()
-            # Run test to insert a new row in table with default values
-            self._add_row()
-            self._verify_row_data(True)
-
-            # Run test to copy/paste a row
-            self._copy_paste_row()
-            self.page.close_data_grid()
+            self._perform_test_for_table('defaults_{0}'.format(str(cnt)),
+                                         data_local)
+        # test nonint pkey table
+        self._load_config_data('table_insert_update_nonint')
+        data_local = config_data
+        self._perform_test_for_table('nonintpkey', data_local)
 
     def after(self):
         self.page.remove_server(self.server)
+        for cnt in (1, 2):
+            test_utils.delete_table(
+                self.server, self.test_db, 'defaults_{0}'.format(str(cnt)))
+        test_utils.delete_table(self.server, self.test_db, 'nonintpkey')
 
     @staticmethod
     def _get_cell_xpath(cell, row):
@@ -139,6 +163,41 @@ CREATE TABLE public.defaults_{0}
 
         return xpath_cell
 
+    @staticmethod
+    def _load_config_data(config_key):
+        global config_data
+        config_data = config_data_json[config_key]
+
+    def _perform_test_for_table(self, table_name, config_data_local):
+        self.page.click_a_tree_node(
+            table_name,
+            TreeAreaLocators.sub_nodes_of_tables_node)
+        # Open Object -> View/Edit data
+        self._view_data_grid(table_name)
+
+        self.page.wait_for_query_tool_loading_indicator_to_disappear()
+        # Run test to insert a new row in table with default values
+        self._add_row(config_data_local)
+        self._verify_row_data(row_height=0,
+                              config_check_data=config_data_local['add'])
+
+        # Run test to copy/paste a row
+        self._copy_paste_row(config_data_local)
+
+        self._update_row(config_data_local)
+        self.page.click_tab("Messages")
+        self._verify_messsages("")
+        self.page.click_tab("Data Output")
+        updated_row_data = {
+            i: config_data_local['update'][i] if i in config_data_local[
+                'update'] else val
+            for i, val in config_data_local['add'].items()
+        }
+        self._verify_row_data(row_height=0,
+                              config_check_data=updated_row_data)
+
+        self.page.close_data_grid()
+
     def _compare_cell_value(self, xpath, value):
         # Initialize an instance of WebDriverWait with timeout of 5 seconds
         wait = WebDriverWait(self.driver, 5)
@@ -164,12 +223,11 @@ CREATE TABLE public.defaults_{0}
         Returns: None
 
         """
-
         self.wait.until(EC.visibility_of_element_located(
             (By.XPATH, xpath)), CheckForViewDataTest.TIMEOUT_STRING
         )
         cell_el = self.page.find_by_xpath(xpath)
-        self.page.driver.execute_script("arguments[0].scrollIntoView()",
+        self.page.driver.execute_script("arguments[0].scrollIntoView(false)",
                                         cell_el)
         ActionChains(self.driver).move_to_element(cell_el).double_click(
             cell_el
@@ -181,18 +239,18 @@ CREATE TABLE public.defaults_{0}
             if value == 'clear':
                 cell_el.find_element_by_css_selector('input').clear()
             else:
-                ActionChains(self.driver).send_keys(value).perform()
-
+                ActionChains(self.driver).send_keys(value). \
+                    send_keys(Keys.ENTER).perform()
         elif cell_type in ['text', 'json', 'text[]', 'boolean[]']:
-            self.page.find_by_xpath(
-                "//*[contains(@class, 'pg_textarea')]").click()
-            ActionChains(self.driver).send_keys(value).perform()
+            text_area_ele = self.page.find_by_css_selector(
+                QueryToolLocators.row_editor_text_area_css)
+            text_area_ele.clear()
+            text_area_ele.click()
+            text_area_ele.send_keys(value)
 
             # Click on editor's Save button
-            self.page.find_by_xpath(
-                "//*[contains(@class, 'pg_text_editor')]"
-                "//button[contains(@class, 'fa-save')]"
-            ).click()
+            self.page.find_by_css_selector(
+                QueryToolLocators.text_editor_ok_btn_css).click()
         else:
             # Boolean editor test for to True click
             if data[1] == 'true':
@@ -208,25 +266,21 @@ CREATE TABLE public.defaults_{0}
                 # Sets false
                 ActionChains(self.driver).click(checkbox_el).perform()
 
-    def _tables_node_expandable(self):
-        self.page.toggle_open_tree_item(self.server['name'])
-        self.page.toggle_open_tree_item('Databases')
-        self.page.toggle_open_tree_item(self.test_db)
-        self.page.toggle_open_tree_item('Schemas')
-        self.page.toggle_open_tree_item('public')
-        self.page.toggle_open_tree_item('Tables')
-
-    def _view_data_grid(self):
+    def _view_data_grid(self, table_name):
         self.page.driver.find_element_by_link_text("Object").click()
         ActionChains(
             self.page.driver
         ).move_to_element(
-            self.page.driver.find_element_by_link_text("View/Edit Data")
+            self.page.driver.find_element_by_link_text(
+                NavMenuLocators.view_data_link_text)
         ).perform()
         self.page.find_by_partial_link_text("All Rows").click()
-        time.sleep(1)
+
         # wait until datagrid frame is loaded.
-        self.page.click_tab('Edit Data -')
+        self.page.wait_for_query_tool_loading_indicator_to_appear()
+        self.page.wait_for_query_tool_loading_indicator_to_disappear()
+
+        self.page.click_tab(table_name)
 
         self.wait.until(
             EC.visibility_of_element_located(
@@ -237,84 +291,94 @@ CREATE TABLE public.defaults_{0}
             self.page.driver.find_element_by_tag_name('iframe')
         )
 
-    def _copy_paste_row(self):
+    def _copy_paste_row(self, config_data_l):
         row0_cell0_xpath = CheckForViewDataTest._get_cell_xpath("r0", 1)
-        row1_cell1_xpath = CheckForViewDataTest._get_cell_xpath("r1", 2)
-        row1_cell2_xpath = CheckForViewDataTest._get_cell_xpath("r2", 2)
 
         self.page.find_by_xpath(row0_cell0_xpath).click()
-        self.page.find_by_xpath("//*[@id='btn-copy-row']").click()
-        self.page.find_by_xpath("//*[@id='btn-paste-row']").click()
+        self.page.find_by_css_selector(
+            QueryToolLocators.copy_button_css).click()
+        self.page.find_by_css_selector(
+            QueryToolLocators.paste_button_css).click()
+
         # Update primary key of copied cell
-        self._update_cell(row1_cell1_xpath, [2, "", "int"])
-        self.page.find_by_xpath(
-            CheckForViewDataTest._get_cell_xpath("r1", "3")
-        ).click()
-
-        # Check if removing a cell value with default value sets
-        # markup to [default] if cell is cleared
-        self._update_cell(row1_cell2_xpath, ["clear", "", "int"])
-        # click outside
-        self.page.find_by_xpath(
-            CheckForViewDataTest._get_cell_xpath("r1", "3")
-        ).click()
-
-        self._compare_cell_value(row1_cell2_xpath, "[default]")
-        # reset cell value to previous one
-        self._update_cell(row1_cell2_xpath, ["1", "", "int"])
-
-        self.page.find_by_id("btn-save").click()  # Save data
-        # There should be some delay after save button is clicked, as it
-        # takes some time to complete save ajax call otherwise discard unsaved
-        # changes dialog will appear if we try to execute query before previous
-        # save ajax is completed.
-        time.sleep(2)
+        self._add_update_save_row(config_data_l['copy'], row=2)
 
         # Verify row 1 and row 2 data
-        self._verify_row_data(False)
+        updated_row_data = {
+            i: config_data_l['copy'][i] if i in config_data_l['copy'] else val
+            for i, val in config_data_l['add'].items()
+        }
+        self._verify_row_data(row_height=25,
+                              config_check_data=updated_row_data)
 
-    def _add_row(self):
-        for idx in range(1, len(config_data.keys()) + 1):
+    def _add_update_save_row(self, data, row=1):
+        items = list(data.keys())
+        for item in range(0, len(items)):
+            items[item] = int(items[item])
+        items.sort(reverse=False)
+        for idx in items:
             cell_xpath = CheckForViewDataTest._get_cell_xpath(
-                'r' + str(idx), 1
+                'r' + str(idx), row
             )
             time.sleep(0.2)
-            self._update_cell(cell_xpath, config_data[str(idx)])
-        self.page.find_by_id("btn-save").click()  # Save data
+            self._update_cell(cell_xpath, data[str(idx)])
+        self.page.find_by_css_selector(
+            QueryToolLocators.btn_save_data).click()
         # There should be some delay after save button is clicked, as it
         # takes some time to complete save ajax call otherwise discard unsaved
         # changes dialog will appear if we try to execute query before previous
         # save ajax is completed.
         time.sleep(2)
 
-    def _verify_row_data(self, is_new_row):
-        self.page.find_by_id("btn-flash").click()
+    def _add_row(self, config_data_l):
+        self._add_update_save_row(config_data_l['add'], 1)
 
-        # First row if row height = 0, second row if its 25
-        row_height = 0 if is_new_row else 25
+    def _update_row(self, config_data_l):
+        self._add_update_save_row(config_data_l['update'], 1)
+
+    def _verify_messsages(self, text):
+        messages_ele = self.page.find_by_css_selector(
+            QueryToolLocators.query_messages_panel)
+        self.assertEqual(text, messages_ele.text)
+
+    def _verify_row_data(self, row_height, config_check_data):
+        self.page.click_execute_query_button()
 
         xpath = "//*[contains(@class, 'ui-widget-content') and " \
                 "contains(@style, 'top:" + str(row_height) + "px')]"
+        scroll_on_arg_for_js = "arguments[0].scrollIntoView(false)"
 
         self.page.wait_for_query_tool_loading_indicator_to_disappear()
 
-        result_row = self.page.find_by_xpath(xpath)
-
-        # List of row values in an array
-        for idx in range(1, len(config_data.keys()) + 1):
-            # after copy & paste row, the first cell of row 1 and
-            # row 2(being primary keys) won't match
-            # see if cell values matched to actual value
-            element = result_row.find_element_by_class_name("r" + str(idx))
-            self.page.driver.execute_script("arguments[0].scrollIntoView()",
-                                            element)
-
-            if (idx != 1 and not is_new_row) or is_new_row:
-                self.assertEquals(element.text, config_data[str(idx)][1])
+        # Verify the List of actual values with the expected list
+        actual_list = list(config_check_data.keys())
+        for value in range(0, len(actual_list)):
+            actual_list[value] = int(actual_list[value])
+        actual_list.sort(reverse=False)
+        retry = 5
+        for idx in actual_list:
+            while retry > 0:
+                try:
+                    result_row = self.page.find_by_xpath(xpath)
+                    element = \
+                        result_row.find_element_by_class_name("r" + str(idx))
+                    self.page.driver.execute_script(
+                        scroll_on_arg_for_js, element)
+                    break
+                except Exception:
+                    print("stale reference exception at id:", idx)
+                    retry -= 1
+            time.sleep(0.4)
+            self.assertEqual(element.text, config_check_data[str(idx)][1])
 
         # scroll browser back to the left
         # to reset position so other assertions can succeed
-        for idx in range(len(config_data.keys()), 1, -1):
+        list_item = list(config_check_data.keys())
+        for item in range(0, len(list_item)):
+            list_item[item] = int(list_item[item])
+        list_item.sort(reverse=True)
+        for idx in list_item:
+            time.sleep(0.4)
             element = result_row.find_element_by_class_name("r" + str(idx))
-            self.page.driver.execute_script("arguments[0].scrollIntoView()",
-                                            element)
+            self.page.driver.execute_script(
+                scroll_on_arg_for_js, element)
