@@ -2,16 +2,21 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
-from __future__ import print_function
+import sys
+import random
+
 from regression.python_test_utils import test_utils
 from regression.feature_utils.base_feature_test import BaseFeatureTest
 from selenium.webdriver import ActionChains
-import sys
+from selenium.common.exceptions import StaleElementReferenceException, \
+    WebDriverException
+from regression.feature_utils.locators import QueryToolLocators
+from regression.feature_utils.tree_area_locators import TreeAreaLocators
 
 
 class CheckForXssFeatureTest(BaseFeatureTest):
@@ -32,10 +37,13 @@ class CheckForXssFeatureTest(BaseFeatureTest):
     scenarios = [
         ("Test XSS check for panels and query tool", dict())
     ]
-    test_table_name = "<h1>X"
     test_type_name = '"<script>alert(1)</script>"'
+    check_xss_chars = '&lt;h1&gt;X'
+    check_xss_chars_set2 = '&lt;script&gt;alert(1)&lt;/script&gt;'
 
     def before(self):
+        self.test_table_name = "<h1>X" + str(random.randint(1000, 3000))
+
         test_utils.create_type(
             self.server, self.test_db, self.test_type_name,
             ['"<script>alert(1)</script>" "char"',
@@ -58,15 +66,24 @@ class CheckForXssFeatureTest(BaseFeatureTest):
         self.page.add_server(self.server)
         self._tables_node_expandable()
         self._check_xss_in_browser_tree()
-        self._check_xss_in_properties_tab()
         self._check_xss_in_sql_tab()
 
         # sometime the tab for dependent does not show info, so refreshing
         # the page and then again collapsing until the table node
-        self.page.refresh_page()
-        self.page.toggle_open_servers_group()
-        self._tables_node_expandable()
-        self._check_xss_in_dependents_tab()
+        retry = 2
+        while retry > 0:
+            try:
+                self.page.refresh_page()
+                self.page.toggle_open_servers_group()
+                self._tables_node_expandable()
+                self._check_xss_in_dependents_tab()
+                retry = 0
+            except WebDriverException as e:
+                print("Exception in dependent check {0}".format(retry),
+                      file=sys.stderr)
+                if retry == 1:
+                    raise e
+                retry -= 1
 
         # Query tool
         self.page.open_query_tool()
@@ -85,15 +102,19 @@ class CheckForXssFeatureTest(BaseFeatureTest):
 
     def after(self):
         self.page.remove_server(self.server)
+        test_utils.delete_table(
+            self.server, self.test_db, self.test_table_name)
 
     def _tables_node_expandable(self):
-        self.page.toggle_open_server(self.server['name'])
-        self.page.toggle_open_tree_item('Databases')
-        self.page.toggle_open_tree_item(self.test_db)
-        self.page.toggle_open_tree_item('Schemas')
-        self.page.toggle_open_tree_item('public')
-        self.page.toggle_open_tree_item('Tables')
-        self.page.select_tree_item(self.test_table_name)
+        self.page.expand_database_node(
+            self.server['name'],
+            self.server['db_password'], self.test_db)
+        self.page.toggle_open_tables_node(self.server['name'],
+                                          self.server['db_password'],
+                                          self.test_db, 'public')
+        self.page.click_a_tree_node(
+            self.test_table_name,
+            TreeAreaLocators.sub_nodes_of_tables_node)
 
     def _check_xss_in_browser_tree(self):
         print(
@@ -107,25 +128,10 @@ class CheckForXssFeatureTest(BaseFeatureTest):
 
         self._check_escaped_characters(
             source_code,
-            "&lt;h1&gt;X",
+            self.check_xss_chars,
             "Browser tree"
         )
 
-    def _check_xss_in_properties_tab(self):
-        print(
-            "\n\tChecking the Properties tab for XSS vulnerabilities",
-            file=sys.stderr, end=""
-        )
-        self.page.click_tab("Properties")
-        source_code = self.page.find_by_xpath(
-            "//span[contains(@class,'uneditable-input')]"
-        ).get_attribute('innerHTML')
-        self._check_escaped_characters(
-            source_code,
-            "&lt;h1&gt;X",
-            "Properties tab (Backform Control)"
-        )
-
     def _check_xss_in_sql_tab(self):
         print(
             "\n\tChecking the SQL tab for for XSS vulnerabilities",
@@ -140,7 +146,7 @@ class CheckForXssFeatureTest(BaseFeatureTest):
 
         self._check_escaped_characters(
             source_code,
-            "&lt;h1&gt;X",
+            self.check_xss_chars,
             "SQL tab (Code Mirror)"
         )
 
@@ -151,12 +157,24 @@ class CheckForXssFeatureTest(BaseFeatureTest):
             "\n\tChecking the Dependents tab for XSS vulnerabilities",
             file=sys.stderr, end=""
         )
-        self.page.click_tab("Dependents")
 
-        source_code = self.page.find_by_xpath(
-            "//*[@id='5']/table/tbody/tr/td/div/div/div[2]/"
-            "table/tbody/tr/td[2]"
-        ).get_attribute('innerHTML')
+        retry = 2
+        while retry > 0:
+            try:
+                self.page.click_tab("Dependents")
+                source_code = \
+                    self.page.find_by_xpath("//*[@id='5']/table/tbody/tr/td/"
+                                            "div/div/div[2]/table/tbody/tr/"
+                                            "td[2]").get_attribute('innerHTML')
+                retry = 0
+            except WebDriverException as e:
+                print("Exception in dependent tab {0}".format(retry),
+                      file=sys.stderr)
+                self.page.click_tab("Dependencies")
+                if retry == 1:
+                    self.page.click_tab("Dependents")
+                    raise e
+                retry -= 1
 
         self._check_escaped_characters(
             source_code,
@@ -198,7 +216,8 @@ class CheckForXssFeatureTest(BaseFeatureTest):
         self.page.fill_codemirror_area_with(
             "select '<script>alert(1)</script>"
         )
-        self.page.find_by_id("btn-flash").click()
+        self.page.find_by_css_selector(
+            QueryToolLocators.btn_execute_query_css).click()
 
         self.page.click_tab('Query History')
 
@@ -212,32 +231,42 @@ class CheckForXssFeatureTest(BaseFeatureTest):
 
         self._check_escaped_characters(
             source_code,
-            '&lt;script&gt;alert(1)&lt;/script&gt;',
+            self.check_xss_chars_set2,
             "Query tool (History Entry)"
         )
 
-        # Check for history details message
-        history_ele = self.driver\
-            .find_element_by_css_selector(".query-detail .content-value")
-
-        source_code = history_ele.get_attribute('innerHTML')
+        retry = 2
+        while retry > 0:
+            try:
+                history_ele = self.driver \
+                    .find_element_by_css_selector(
+                        ".query-detail .content-value")
+                source_code = history_ele.get_attribute('innerHTML')
+                break
+            except StaleElementReferenceException:
+                retry -= 1
 
         self._check_escaped_characters(
             source_code,
-            '&lt;script&gt;alert(1)&lt;/script&gt;',
+            self.check_xss_chars_set2,
             "Query tool (History Details-Message)"
         )
 
-        # Check for history details error message
-        history_ele = self.page.find_by_css_selector(
-            ".query-detail .history-error-text"
-        )
-
-        source_code = history_ele.get_attribute('innerHTML')
+        retry = 2
+        while retry > 0:
+            try:
+                # Check for history details error message
+                history_ele = self.page.find_by_css_selector(
+                    ".query-detail .history-error-text"
+                )
+                source_code = history_ele.get_attribute('innerHTML')
+                break
+            except StaleElementReferenceException:
+                retry -= 1
 
         self._check_escaped_characters(
             source_code,
-            '&lt;script&gt;alert(1)&lt;/script&gt;',
+            self.check_xss_chars_set2,
             "Query tool (History Details-Error)"
         )
 
@@ -259,7 +288,7 @@ class CheckForXssFeatureTest(BaseFeatureTest):
 
         self._check_escaped_characters(
             source_code,
-            '&lt;script&gt;alert(1)&lt;/script&gt;',
+            self.check_xss_chars_set2,
             "View Data (SlickGrid)"
         )
 
@@ -272,7 +301,8 @@ class CheckForXssFeatureTest(BaseFeatureTest):
             'select * from "{0}"'.format(self.test_table_name)
         )
 
-        self.page.find_by_id("btn-explain").click()
+        self.page.find_by_css_selector(
+            QueryToolLocators.btn_explain).click()
         self.page.wait_for_query_tool_loading_indicator_to_disappear()
         self.page.click_tab('Explain')
 
@@ -283,9 +313,9 @@ class CheckForXssFeatureTest(BaseFeatureTest):
                     self.driver.find_element_by_css_selector(
                         'div.pgadmin-explain-container > svg > g > g > image'
                     )
-                ).perform()
+                ).click().perform()
                 break
-            except Exception as e:
+            except Exception:
                 if idx != 2:
                     continue
                 else:
@@ -296,12 +326,12 @@ class CheckForXssFeatureTest(BaseFeatureTest):
                     )
                     raise
 
-        source_code = self.driver.find_element_by_id(
-            'toolTip').get_attribute('innerHTML')
+        source_code = self.driver.find_element_by_css_selector(
+            '.pgadmin-explain-details:not(.d-none)').get_attribute('innerHTML')
 
         self._check_escaped_characters(
             source_code,
-            "&lt;h1&gt;X",
+            self.check_xss_chars,
             "Explain tab (Graphical explain plan)"
         )
 
