@@ -2,15 +2,18 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
+import random
+
 from selenium.webdriver import ActionChains
 from selenium.common.exceptions import TimeoutException
 from regression.python_test_utils import test_utils
 from regression.feature_utils.base_feature_test import BaseFeatureTest
+from regression.feature_utils.tree_area_locators import TreeAreaLocators
 from selenium.webdriver.support.ui import WebDriverWait
 from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.common.by import By
@@ -22,6 +25,7 @@ class CheckDebuggerForXssFeatureTest(BaseFeatureTest):
     scenarios = [
         ("Tests to check if Debugger is vulnerable to XSS", dict())
     ]
+    function_name = ""
 
     def before(self):
         with test_utils.Database(self.server) as (connection, _):
@@ -32,10 +36,16 @@ class CheckDebuggerForXssFeatureTest(BaseFeatureTest):
                 )
 
         # Some test function is needed for debugger
+        self.function_name = "a_test_function" + \
+                             str(random.randint(10000, 65535))
         test_utils.create_debug_function(
-            self.server, "postgres", "a_test_function"
+            self.server, self.test_db, self.function_name
         )
 
+        if test_utils.does_function_exist(self.server, self.test_db,
+                                          self.function_name) != 'True':
+            raise RuntimeError("The required function is not found")
+
     def runTest(self):
         self.page.wait_for_spinner_to_disappear()
         self.page.add_server(self.server)
@@ -44,17 +54,20 @@ class CheckDebuggerForXssFeatureTest(BaseFeatureTest):
 
     def after(self):
         self.page.remove_server(self.server)
-        test_utils.drop_debug_function(self.server, "postgres",
-                                       "a_test_function")
+        test_utils.drop_debug_function(self.server, self.test_db,
+                                       self.function_name)
 
     def _function_node_expandable(self):
-        self.page.toggle_open_server(self.server['name'])
-        self.page.toggle_open_tree_item('Databases')
-        self.page.toggle_open_tree_item('postgres')
-        self.page.toggle_open_tree_item('Schemas')
-        self.page.toggle_open_tree_item('public')
-        self.page.toggle_open_tree_item('Functions')
-        self.page.select_tree_item("a_test_function()")
+        self.page.expand_database_node(
+            self.server['name'],
+            self.server['db_password'], self.test_db)
+        self.page.toggle_open_schema_node(self.server['name'],
+                                          self.server['db_password'],
+                                          self.test_db, 'public')
+        self.page.toggle_open_function_node()
+        self.page.click_a_tree_node(
+            self.function_name + "()",
+            TreeAreaLocators.sub_nodes_of_functions_node)
 
     def _debug_function(self):
         self.page.driver.find_element_by_link_text("Object").click()
@@ -74,12 +87,23 @@ class CheckDebuggerForXssFeatureTest(BaseFeatureTest):
                            "contains(@class,'ajs-header')]")
             ))
 
-        except TimeoutException as e:
+        except TimeoutException:
             is_error = None
 
         # If debugger plugin is not found
         if is_error and is_error.text == "Debugger Error":
-            self.page.click_modal('OK')
+            click = True
+            while click:
+                try:
+                    self.page.click_modal('OK')
+                    wait.until(EC.invisibility_of_element(
+                        (By.XPATH, "//div[contains(@class, 'alertify') and "
+                                   "not(contains(@class, 'ajs-hidden'))]//div["
+                                   "contains(@class,'ajs-header')]")
+                    ))
+                    click = False
+                except TimeoutException:
+                    pass
             self.skipTest(
                 "Please make sure that debugger plugin is properly configured"
             )
