@@ -3,7 +3,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL License
 #
 ##########################################################################
@@ -18,10 +18,12 @@ import psutil
 from abc import ABCMeta, abstractproperty, abstractmethod
 from datetime import datetime
 from pickle import dumps, loads
-from subprocess import Popen
+from subprocess import Popen, PIPE
+import logging
 
-from pgadmin.utils import IS_PY2, u, file_quote, fs_encoding, \
-    get_complete_file_path
+from pgadmin.utils import u_encode, file_quote, fs_encoding, \
+    get_complete_file_path, get_storage_directory, IS_WIN
+from pgadmin.browser.server_groups.servers.utils import does_server_exists
 
 import pytz
 from dateutil import parser
@@ -31,15 +33,13 @@ from flask_security import current_user
 
 import config
 from pgadmin.model import Process, db
-if IS_PY2:
-    from StringIO import StringIO
-else:
-    from io import StringIO
+from io import StringIO
 
 PROCESS_NOT_STARTED = 0
 PROCESS_STARTED = 1
 PROCESS_FINISHED = 2
 PROCESS_TERMINATED = 3
+PROCESS_NOT_FOUND = _("Could not find a process with the specified ID.")
 
 
 def get_current_time(format='%Y-%m-%d %H:%M:%S.%f %z'):
@@ -51,9 +51,7 @@ def get_current_time(format='%Y-%m-%d %H:%M:%S.%f %z'):
     ).strftime(format)
 
 
-class IProcessDesc(object):
-    __metaclass__ = ABCMeta
-
+class IProcessDesc(object, metaclass=ABCMeta):
     @abstractproperty
     def message(self):
         pass
@@ -62,6 +60,51 @@ class IProcessDesc(object):
     def details(self, cmd, args):
         pass
 
+    @property
+    def current_storage_dir(self):
+
+        if config.SERVER_MODE:
+
+            file = self.bfile
+            try:
+                # check if file name is encoded with UTF-8
+                file = self.bfile.decode('utf-8')
+            except Exception:
+                # do nothing if bfile is not encoded.
+                pass
+
+            path = get_complete_file_path(file)
+            path = file if path is None else path
+
+            if IS_WIN:
+                path = os.path.realpath(path)
+
+            storage_directory = os.path.basename(get_storage_directory())
+
+            if storage_directory in path:
+                start = path.index(storage_directory)
+                end = start + (len(storage_directory))
+                last_dir = os.path.dirname(path[end:])
+            else:
+                last_dir = file
+
+            last_dir = replace_path_for_win(last_dir)
+
+            return None if hasattr(self, 'is_import') and self.is_import \
+                else last_dir
+
+        return None
+
+
+def replace_path_for_win(last_dir=None):
+    if IS_WIN:
+        if '\\' in last_dir and len(last_dir) == 1:
+            last_dir = last_dir.replace('\\', '\\\\')
+        else:
+            last_dir = last_dir.replace('\\', '/')
+
+    return last_dir
+
 
 class BatchProcess(object):
     def __init__(self, **kwargs):
@@ -82,19 +125,9 @@ class BatchProcess(object):
         p = Process.query.filter_by(pid=_id, user_id=current_user.id).first()
 
         if p is None:
-            raise LookupError(
-                _("Could not find a process with the specified ID.")
-            )
+            raise LookupError(PROCESS_NOT_FOUND)
 
-        try:
-            tmp_desc = loads(p.desc.encode('latin-1')) if \
-                IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
-        except UnicodeDecodeError:
-            tmp_desc = loads(p.desc.encode('utf-8')) if \
-                IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
-        except Exception as e:
-            tmp_desc = loads(p.desc.encode('utf-8', 'ignore')) if \
-                IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
+        tmp_desc = loads(p.desc)
 
         # ID
         self.id = _id
@@ -139,11 +172,11 @@ class BatchProcess(object):
 
         created = False
         size = 0
-        id = ctime
+        uid = ctime
         while not created:
             try:
-                id += random_number(size)
-                log_dir = os.path.join(log_dir, id)
+                uid += random_number(size)
+                log_dir = os.path.join(log_dir, uid)
                 size += 1
                 if not os.path.exists(log_dir):
                     os.makedirs(log_dir, int('700', 8))
@@ -182,33 +215,15 @@ class BatchProcess(object):
         csv_writer = csv.writer(
             args_csv_io, delimiter=str(','), quoting=csv.QUOTE_MINIMAL
         )
-        if sys.version_info[0] == 2:
-            csv_writer.writerow(
-                [
-                    a.encode('utf-8')
-                    if isinstance(a, unicode) else a for a in _args
-                ]
-            )
-        else:
-            csv_writer.writerow(_args)
+        csv_writer.writerow(_args)
 
         args_val = args_csv_io.getvalue().strip(str('\r\n'))
         tmp_desc = dumps(self.desc)
-        try:
-            tmp_desc = tmp_desc.decode('utf-8') if \
-                IS_PY2 and hasattr(tmp_desc, 'decode') else tmp_desc
-        except UnicodeDecodeError:
-            tmp_desc = tmp_desc.decode('latin-1') if \
-                IS_PY2 and hasattr(tmp_desc, 'decode') else tmp_desc
-        except Exception:
-            tmp_desc = tmp_desc.decode('utf-8', 'ignore') if \
-                IS_PY2 and hasattr(tmp_desc, 'decode') else tmp_desc
 
         j = Process(
-            pid=int(id),
+            pid=int(uid),
             command=_cmd,
-            arguments=args_val.decode('utf-8', 'replace')
-            if IS_PY2 and hasattr(args_val, 'decode') else args_val,
+            arguments=args_val,
             logdir=log_dir,
             desc=tmp_desc,
             user_id=current_user.id
@@ -216,99 +231,32 @@ class BatchProcess(object):
         db.session.add(j)
         db.session.commit()
 
-    def start(self, cb=None):
-
-        def which(program, paths):
-            def is_exe(fpath):
-                return os.path.exists(fpath) and os.access(fpath, os.X_OK)
-
-            for path in paths:
-                if not os.path.isdir(path):
-                    continue
-                exe_file = os.path.join(u(path, fs_encoding), program)
-                if is_exe(exe_file):
-                    return file_quote(exe_file)
-            return None
-
-        def convert_environment_variables(env):
-            """
-            This function is use to convert environment variable to string
-            because environment variable must be string in popen
-            :param env: Dict of environment variable
-            :return: Encoded environment variable as string
-            """
-            encoding = sys.getdefaultencoding()
-            if encoding is None or encoding == 'ascii':
-                encoding = 'utf-8'
-            temp_env = dict()
-            for key, value in env.items():
-                if not isinstance(key, str):
-                    key = key.encode(encoding)
-                if not isinstance(value, str):
-                    value = value.encode(encoding)
-                temp_env[key] = value
-            return temp_env
-
+    def check_start_end_time(self):
+        """
+        Check start and end time to check process is still executing or not.
+        :return:
+        """
         if self.stime is not None:
             if self.etime is None:
-                raise Exception(_('The process has already been started.'))
-            raise Exception(
+                raise RuntimeError(_('The process has already been started.'))
+            raise RuntimeError(
                 _('The process has already finished and cannot be restarted.')
             )
 
+    def start(self, cb=None):
+        self.check_start_end_time()
+
         executor = file_quote(os.path.join(
-            os.path.dirname(u(__file__)), u'process_executor.py'
+            os.path.dirname(u_encode(__file__)), 'process_executor.py'
         ))
         paths = os.environ['PATH'].split(os.pathsep)
-        interpreter = None
 
-        if os.name == 'nt':
-            paths.insert(0, os.path.join(u(sys.prefix), u'Scripts'))
-            paths.insert(0, u(sys.prefix))
+        current_app.logger.info(
+            "Process Executor: Operating System Path %s",
+            str(paths)
+        )
 
-            interpreter = which(u'pythonw.exe', paths)
-            if interpreter is None:
-                interpreter = which(u'python.exe', paths)
-
-            if interpreter is None and current_app.PGADMIN_RUNTIME:
-                # We've faced an issue with Windows 2008 R2 (x86) regarding,
-                # not honouring the environment variables set under the Qt
-                # (e.g. runtime), and also setting PYTHONHOME same as
-                # sys.executable (i.e. pgAdmin4.exe).
-                #
-                # As we know, we're running it under the runtime, we can assume
-                # that 'venv' directory will be available outside of 'bin'
-                # directory.
-                #
-                # We would try out luck to find python executable based on that
-                # assumptions.
-                bin_path = os.path.dirname(sys.executable)
-
-                venv = os.path.realpath(
-                    os.path.join(bin_path, u'..\\venv')
-                )
-
-                interpreter = which(u'pythonw.exe', [venv])
-                if interpreter is None:
-                    interpreter = which(u'pythonw.exe', [venv])
-
-                if interpreter is not None:
-                    # Our assumptions are proven right.
-                    # Let's append the 'bin' directory to the PATH environment
-                    # variable. And, also set PYTHONHOME environment variable
-                    # to 'venv' directory.
-                    os.environ['PATH'] = bin_path + ';' + os.environ['PATH']
-                    os.environ['PYTHONHOME'] = venv
-        else:
-            # Let's not use sys.prefix in runtime.
-            # 'sys.prefix' is not identified on *nix systems for some unknown
-            # reason, while running under the runtime.
-            # We're already adding '<installation path>/pgAdmin 4/venv/bin'
-            # directory in the PATH environment variable. Hence - it will
-            # anyway be the redundant value in paths.
-            if not current_app.PGADMIN_RUNTIME:
-                paths.insert(0, os.path.join(u(sys.prefix), u'bin'))
-            interpreter = which(u'python', paths)
+        interpreter = self.get_interpreter(paths)
 
         p = None
         cmd = [
@@ -317,24 +265,10 @@ class BatchProcess(object):
         ]
         cmd.extend(self.args)
 
-        if os.name == 'nt' and IS_PY2:
-            command = []
-            for c in cmd:
-                command.append(
-                    c.encode('utf-8') if isinstance(c, unicode) else str(c)
-                )
-
-            current_app.logger.info(
-                u"Executing the process executor with the arguments: %s",
-                ''.join(command)
-            )
-
-            cmd = command
-        else:
-            current_app.logger.info(
-                u"Executing the process executor with the arguments: %s",
-                str(cmd)
-            )
+        current_app.logger.info(
+            "Executing the process executor with the arguments: %s",
+            str(cmd)
+        )
 
         # Make a copy of environment, and add new variables to support
         env = os.environ.copy()
@@ -348,10 +282,6 @@ class BatchProcess(object):
         if cb is not None:
             cb(env)
 
-        if IS_PY2:
-            # We need environment variables & values in string
-            env = convert_environment_variables(env)
-
         if os.name == 'nt':
             DETACHED_PROCESS = 0x00000008
             from subprocess import CREATE_NEW_PROCESS_GROUP
@@ -375,17 +305,15 @@ class BatchProcess(object):
                 creationflags=(CREATE_NEW_PROCESS_GROUP | DETACHED_PROCESS)
             )
         else:
-            def preexec_function():
-                import signal
-                # Detaching from the parent process group
-                os.setpgrp()
-                # Explicitly ignoring signals in the child process
-                signal.signal(signal.SIGINT, signal.SIG_IGN)
-
-            p = Popen(
-                cmd, close_fds=True, stdout=None, stderr=None, stdin=None,
-                preexec_fn=preexec_function, env=env
-            )
+            # if in debug mode, wait for process to complete and
+            # get the stdout and stderr of popen.
+            if config.CONSOLE_LOG_LEVEL <= logging.DEBUG:
+                output, error = self.get_process_output(cmd, env)
+            else:
+                p = Popen(
+                    cmd, close_fds=True, stdout=None, stderr=None, stdin=None,
+                    preexec_fn=self.preexec_function, env=env
+                )
 
         self.ecode = p.poll()
 
@@ -412,60 +340,167 @@ class BatchProcess(object):
             p.process_state = PROCESS_STARTED
             db.session.commit()
 
-    def status(self, out=0, err=0):
-        import re
+    def get_process_output(self, cmd, env):
+        """
+        :param cmd:
+        :param env:
+        :return:
+        """
+        p = Popen(
+            cmd, close_fds=True, stdout=PIPE, stderr=PIPE, stdin=None,
+            preexec_fn=self.preexec_function, env=env
+        )
 
-        ctime = get_current_time(format='%Y%m%d%H%M%S%f')
+        output, errors = p.communicate()
+        output = output.decode() \
+            if hasattr(output, 'decode') else output
+        errors = errors.decode() \
+            if hasattr(errors, 'decode') else errors
+        current_app.logger.debug(
+            'Process Watcher Out:{0}'.format(output))
+        current_app.logger.debug(
+            'Process Watcher Err:{0}'.format(errors))
+
+        return output, errors
+
+    def preexec_function(self):
+        import signal
+        # Detaching from the parent process group
+        os.setpgrp()
+        # Explicitly ignoring signals in the child process
+        signal.signal(signal.SIGINT, signal.SIG_IGN)
+
+    def get_interpreter(self, paths):
+        """
+        Get interpreter.
+        :param paths:
+        :return:
+        """
+        if os.name == 'nt':
+            paths.insert(0, os.path.join(u_encode(sys.prefix), 'Scripts'))
+            paths.insert(0, u_encode(sys.prefix))
+
+            interpreter = self.which('pythonw.exe', paths)
+            if interpreter is None:
+                interpreter = self.which('python.exe', paths)
+
+            current_app.logger.info(
+                "Process Executor: Interpreter value in path: %s",
+                str(interpreter)
+            )
+            if interpreter is None and current_app.PGADMIN_RUNTIME:
+                # We've faced an issue with Windows 2008 R2 (x86) regarding,
+                # not honouring the environment variables set under the Qt
+                # (e.g. runtime), and also setting PYTHONHOME same as
+                # sys.executable (i.e. pgAdmin4.exe).
+                #
+                # As we know, we're running it under the runtime, we can assume
+                # that 'venv' directory will be available outside of 'bin'
+                # directory.
+                #
+                # We would try out luck to find python executable based on that
+                # assumptions.
+                bin_path = os.path.dirname(sys.executable)
+
+                venv = os.path.realpath(
+                    os.path.join(bin_path, '..\\venv')
+                )
+
+                interpreter = self.which('pythonw.exe', [venv])
+                if interpreter is None:
+                    interpreter = self.which('python.exe', [venv])
+
+                current_app.logger.info(
+                    "Process Executor: Interpreter value in virtual "
+                    "environment: %s", str(interpreter)
+                )
+
+                if interpreter is not None:
+                    # Our assumptions are proven right.
+                    # Let's append the 'bin' directory to the PATH environment
+                    # variable. And, also set PYTHONHOME environment variable
+                    # to 'venv' directory.
+                    os.environ['PATH'] = bin_path + ';' + os.environ['PATH']
+                    os.environ['PYTHONHOME'] = venv
+        else:
+            # Let's not use sys.prefix in runtime.
+            # 'sys.prefix' is not identified on *nix systems for some unknown
+            # reason, while running under the runtime.
+            # We're already adding '<installation path>/pgAdmin 4/venv/bin'
+            # directory in the PATH environment variable. Hence - it will
+            # anyway be the redundant value in paths.
+            if not current_app.PGADMIN_RUNTIME:
+                paths.insert(0, os.path.join(u_encode(sys.prefix), 'bin'))
+            python_binary_name = 'python{0}'.format(sys.version_info[0])
+            interpreter = self.which(u_encode(python_binary_name), paths)
+
+        return interpreter
+
+    def which(self, program, paths):
+        def is_exe(fpath):
+            return os.path.exists(fpath) and os.access(fpath, os.X_OK)
+
+        for path in paths:
+            if not os.path.isdir(path):
+                continue
+            exe_file = os.path.join(u_encode(path, fs_encoding), program)
+            if is_exe(exe_file):
+                return file_quote(exe_file)
+        return None
+
+    def read_log(self, logfile, log, pos, ctime, ecode=None, enc='utf-8'):
+        import re
+        completed = True
+        idx = 0
+        c = re.compile(r"(\d+),(.*$)")
+
+        if not os.path.isfile(logfile):
+            return 0, False
+
+        with open(logfile, 'rb') as f:
+            eofs = os.fstat(f.fileno()).st_size
+            f.seek(pos, 0)
+            if pos == eofs and ecode is None:
+                completed = False
+
+            while pos < eofs:
+                idx += 1
+                line = f.readline()
+                line = line.decode(enc, 'replace')
+                r = c.split(line)
+                if len(r) < 3:
+                    # ignore this line
+                    pos = f.tell()
+                    continue
+                if r[1] > ctime:
+                    completed = False
+                    break
+                log.append([r[1], r[2]])
+                pos = f.tell()
+                if idx >= 1024:
+                    completed = False
+                    break
+                if pos == eofs:
+                    if ecode is None:
+                        completed = False
+                    break
+
+        return pos, completed
+
+    def status(self, out=0, err=0):
+        ctime = get_current_time(format='%y%m%d%H%M%S%f')
 
         stdout = []
         stderr = []
         out_completed = err_completed = False
         process_output = (out != -1 and err != -1)
-        enc = sys.getdefaultencoding()
-        if enc is None or enc == 'ascii':
-            enc = 'utf-8'
-
-        def read_log(logfile, log, pos, ctime, ecode=None):
-            completed = True
-            idx = 0
-            c = re.compile(r"(\d+),(.*$)")
-
-            if not os.path.isfile(logfile):
-                return 0, False
-
-            with open(logfile, 'rb') as f:
-                eofs = os.fstat(f.fileno()).st_size
-                f.seek(pos, 0)
-                if pos == eofs and ecode is None:
-                    completed = False
-
-                while pos < eofs:
-                    idx += 1
-                    line = f.readline()
-                    line = line.decode(enc, 'replace')
-                    r = c.split(line)
-                    if len(r) < 3:
-                        # ignore this line
-                        pos = f.tell()
-                        continue
-                    if r[1] > ctime:
-                        completed = False
-                        break
-                    log.append([r[1], r[2]])
-                    pos = f.tell()
-                    if idx >= 1024:
-                        completed = False
-                        break
-                    if pos == eofs:
-                        if ecode is None:
-                            completed = False
-                        break
-
-            return pos, completed
 
         j = Process.query.filter_by(
             pid=self.id, user_id=current_user.id
         ).first()
+        enc = sys.getdefaultencoding()
+        if enc == 'ascii':
+            enc = 'utf-8'
 
         execution_time = None
 
@@ -484,11 +519,11 @@ class BatchProcess(object):
                 execution_time = BatchProcess.total_seconds(etime - stime)
 
             if process_output:
-                out, out_completed = read_log(
-                    self.stdout, stdout, out, ctime, self.ecode
+                out, out_completed = self.read_log(
+                    self.stdout, stdout, out, ctime, self.ecode, enc
                 )
-                err, err_completed = read_log(
-                    self.stderr, stderr, err, ctime, self.ecode
+                err, err_completed = self.read_log(
+                    self.stderr, stderr, err, ctime, self.ecode, enc
                 )
         else:
             out_completed = err_completed = False
@@ -518,6 +553,26 @@ class BatchProcess(object):
             'process_state': self.process_state
         }
 
+    @staticmethod
+    def _check_start_time(p, data):
+        """
+        Check start time and its related other timing checks.
+        :param p: Process.
+        :param data: Data
+        :return:
+        """
+        if 'start_time' in data and data['start_time']:
+            p.start_time = data['start_time']
+
+            # We can't have 'exit_code' without the 'start_time'
+            if 'exit_code' in data and \
+                    data['exit_code'] is not None:
+                p.exit_code = data['exit_code']
+
+                # We can't have 'end_time' without the 'exit_code'.
+                if 'end_time' in data and data['end_time']:
+                    p.end_time = data['end_time']
+
     @staticmethod
     def update_process_info(p):
         if p.start_time is None or p.end_time is None:
@@ -531,18 +586,7 @@ class BatchProcess(object):
                     data = json.load(fp)
 
                     #  First - check for the existance of 'start_time'.
-                    if 'start_time' in data and data['start_time']:
-                        p.start_time = data['start_time']
-
-                        # We can't have 'exit_code' without the 'start_time'
-                        if 'exit_code' in data and \
-                                data['exit_code'] is not None:
-                            p.exit_code = data['exit_code']
-
-                            # We can't have 'end_time' without the 'exit_code'.
-                            if 'end_time' in data and data['end_time']:
-                                p.end_time = data['end_time']
-
+                    BatchProcess._check_start_time(p, data)
                     # get the pid of the utility.
                     if 'pid' in data:
                         p.utility_pid = data['pid']
@@ -558,6 +602,39 @@ class BatchProcess(object):
                     return False, False
         return True, False
 
+    @staticmethod
+    def _check_process_desc(p):
+        """
+        Check process desc instance and return data according to process.
+        :param p: process
+        :return: return value for details, type_desc and desc related
+        to process
+        """
+        desc = loads(p.desc)
+        details = desc
+        type_desc = ''
+        current_storage_dir = None
+
+        if isinstance(desc, IProcessDesc):
+
+            from pgadmin.tools.backup import BackupMessage
+            from pgadmin.tools.import_export import IEMessage
+            args = []
+            args_csv = StringIO(
+                p.arguments.encode('utf-8')
+                if hasattr(p.arguments, 'decode') else p.arguments
+            )
+            args_reader = csv.reader(args_csv, delimiter=str(','))
+            for arg in args_reader:
+                args = args + arg
+            details = desc.details(p.command, args)
+            type_desc = desc.type_desc
+            if isinstance(desc, (BackupMessage, IEMessage)):
+                current_storage_dir = desc.current_storage_dir
+            desc = desc.message
+
+        return desc, details, type_desc, current_storage_dir
+
     @staticmethod
     def list():
         processes = Process.query.filter_by(user_id=current_user.id)
@@ -568,8 +645,7 @@ class BatchProcess(object):
             status, updated = BatchProcess.update_process_info(p)
             if not status:
                 continue
-
-            if not changed:
+            elif not changed:
                 changed = updated
 
             if p.start_time is None or (
@@ -577,37 +653,18 @@ class BatchProcess(object):
             ):
                 continue
 
+            if BatchProcess._operate_orphan_process(p):
+                continue
+
             execution_time = None
 
             stime = parser.parse(p.start_time)
             etime = parser.parse(p.end_time or get_current_time())
 
             execution_time = BatchProcess.total_seconds(etime - stime)
-            desc = ""
-            try:
-                desc = loads(p.desc.encode('latin-1')) if \
-                    IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
-            except UnicodeDecodeError:
-                desc = loads(p.desc.encode('utf-8')) if \
-                    IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
-            except Exception:
-                desc = loads(p.desc.encode('utf-8', 'ignore')) if \
-                    IS_PY2 and hasattr(p.desc, 'encode') else loads(p.desc)
 
-            details = desc
-
-            if isinstance(desc, IProcessDesc):
-                args = []
-                args_csv = StringIO(
-                    p.arguments.encode('utf-8')
-                    if hasattr(p.arguments, 'decode') else p.arguments
-                )
-                args_reader = csv.reader(args_csv, delimiter=str(','))
-                for arg in args_reader:
-                    args = args + arg
-                details = desc.details(p.command, args)
-                type_desc = desc.type_desc
-                desc = desc.message
+            desc, details, type_desc, current_storage_dir = BatchProcess.\
+                _check_process_desc(p)
 
             res.append({
                 'id': p.pid,
@@ -619,7 +676,8 @@ class BatchProcess(object):
                 'exit_code': p.exit_code,
                 'acknowledge': p.acknowledge,
                 'execution_time': execution_time,
-                'process_state': p.process_state
+                'process_state': p.process_state,
+                'current_storage_dir': current_storage_dir,
             })
 
         if changed:
@@ -627,6 +685,29 @@ class BatchProcess(object):
 
         return res
 
+    @staticmethod
+    def _operate_orphan_process(p):
+
+        if p and p.desc:
+            desc = loads(p.desc)
+            if does_server_exists(desc.sid, current_user.id) is False:
+                current_app.logger.warning(
+                    _("Server with id '{0}' is either removed or does "
+                      "not exists for the background process "
+                      "'{1}'").format(desc.sid, p.pid)
+                )
+                try:
+                    BatchProcess.acknowledge(p.pid)
+                except LookupError as lerr:
+                    current_app.logger.warning(
+                        _("Status for the background process '{0}' could "
+                          "not be loaded.").format(p.pid)
+                    )
+                    current_app.logger.exception(lerr)
+                return True
+
+        return False
+
     @staticmethod
     def total_seconds(dt):
         return round(dt.total_seconds(), 2)
@@ -645,9 +726,7 @@ class BatchProcess(object):
         ).first()
 
         if p is None:
-            raise LookupError(
-                _("Could not find a process with the specified ID.")
-            )
+            raise LookupError(PROCESS_NOT_FOUND)
 
         if p.end_time is not None:
             logdir = p.logdir
@@ -688,9 +767,7 @@ class BatchProcess(object):
         ).first()
 
         if p is None:
-            raise LookupError(
-                _("Could not find a process with the specified ID.")
-            )
+            raise LookupError(PROCESS_NOT_FOUND)
 
         try:
             process = psutil.Process(p.utility_pid)
