@@ -1,8 +1,17 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 define('misc.bgprocess', [
   'sources/pgadmin', 'sources/gettext', 'sources/url_for', 'underscore',
-  'underscore.string', 'jquery', 'pgadmin.browser', 'alertify',
+  'jquery', 'pgadmin.browser', 'alertify',
 ], function(
-  pgAdmin, gettext, url_for, _, S, $, pgBrowser, Alertify
+  pgAdmin, gettext, url_for, _, $, pgBrowser, Alertify
 ) {
 
   pgBrowser.BackgroundProcessObsorver = pgBrowser.BackgroundProcessObsorver || {};
@@ -11,6 +20,8 @@ define('misc.bgprocess', [
     return pgBrowser.BackgroundProcessObsorver;
   }
 
+  var isServerMode = (function() { return pgAdmin.server_mode == 'True'; })();
+
   var wcDocker = window.wcDocker;
 
   var BGProcess = function(info, notify) {
@@ -24,6 +35,27 @@ define('misc.bgprocess', [
 
   _.extend(
     BGProcess.prototype, {
+      success_status_tpl: _.template(`
+      <div class="d-flex px-2 py-1 bg-success-light border border-success rounded">
+        <div class="pr-2">
+          <i class="fa fa-check text-success pg-bg-status-icon" aria-hidden="true" role="img"></i>
+        </div>
+        <div class="mx-auto pg-bg-status-text alert-text-body"><%-status_text%></div>
+      </div>`),
+      failed_status_tpl: _.template(`
+      <div class="d-flex px-2 py-1 bg-danger-lighter border border-danger rounded">
+        <div class="pr-2">
+          <i class="fa fa-times fa-lg text-danger pg-bg-status-icon" aria-hidden="true" role="img"></i>
+        </div>
+        <div class="mx-auto pg-bg-status-text alert-text-body"><%-status_text%></div>
+      </div>`),
+      other_status_tpl: _.template(`
+      <div class="d-flex px-2 py-1 bg-primary-light border border-primary rounded">
+        <div class="pr-2">
+          <i class="fa fa-info fa-lg text-primary pg-bg-status-icon" aria-hidden="true" role="img"></i>
+        </div>
+        <div class="mx-auto pg-bg-status-text alert-text-body"><%-status_text%></div>
+      </div>`),
       initialize: function(info, notify) {
         _.extend(this, {
           details: false,
@@ -31,8 +63,10 @@ define('misc.bgprocess', [
           curr_status: null,
           state: 0, // 0: NOT Started, 1: Started, 2: Finished, 3: Terminated
           completed: false,
+          current_storage_dir: null,
 
           id: info['id'],
+          type_desc: null,
           desc: null,
           detailed_desc: null,
           stime: null,
@@ -81,12 +115,12 @@ define('misc.bgprocess', [
         case 'status':
           if (this.details && this.out != -1 && this.err != -1) {
             return url_for(
-                'bgprocess.detailed_status', {
-                  'pid': this.id,
-                  'out': this.out,
-                  'err': this.err,
-                }
-              );
+              'bgprocess.detailed_status', {
+                'pid': this.id,
+                'out': this.out,
+                'err': this.err,
+              }
+            );
           }
           return url_for('bgprocess.status', {
             'pid': this.id,
@@ -115,6 +149,9 @@ define('misc.bgprocess', [
         if ('execution_time' in data)
           self.execution_time = parseFloat(data.execution_time);
 
+        if ('type_desc' in data)
+          self.type_desc = data.type_desc;
+
         if ('desc' in data)
           self.desc = data.desc;
 
@@ -127,6 +164,9 @@ define('misc.bgprocess', [
         if ('process_state' in data)
           self.state = data.process_state;
 
+        if ('current_storage_dir' in data)
+          self.current_storage_dir = data.current_storage_dir;
+
         if ('out' in data) {
           self.out = data.out && data.out.pos;
 
@@ -159,7 +199,13 @@ define('misc.bgprocess', [
           if (pgAdmin.natural_sort(out[io][0], err[ie][0]) <= 0) {
             res.push('<li class="pg-bg-res-out">' + escapeHTML(out[io++][1]) + '</li>');
           } else {
-            res.push('<li class="pg-bg-res-err">' + escapeHTML(err[ie++][1]) + '</li>');
+            let log_msg = escapeHTML(err[ie++][1]);
+            let regex_obj = new RegExp(': (' + gettext('error') + '|' + gettext('fatal') + '):', 'i');
+            if (regex_obj.test(log_msg)) {
+              res.push('<li class="pg-bg-res-err">' + log_msg + '</li>');
+            } else {
+              res.push('<li class="pg-bg-res-out">' + log_msg + '</li>');
+            }
           }
         }
 
@@ -168,7 +214,13 @@ define('misc.bgprocess', [
         }
 
         while (ie < err.length) {
-          res.push('<li class="pg-bg-res-err">' + escapeHTML(err[ie++][1]) + '</li>');
+          let log_msg = escapeHTML(err[ie++][1]);
+          let regex_obj = new RegExp(': (' + gettext('error') + '|' + gettext('fatal') + '):', 'i');
+          if (regex_obj.test(log_msg)) {
+            res.push('<li class="pg-bg-res-err">' + log_msg + '</li>');
+          } else {
+            res.push('<li class="pg-bg-res-out">' + log_msg + '</li>');
+          }
         }
 
         if (res.length) {
@@ -178,23 +230,30 @@ define('misc.bgprocess', [
           });
         }
 
+        if(self.logs_loading) {
+          self.logs_loading.remove();
+          self.logs_loading = null;
+        }
+
         if (self.stime) {
-          self.curr_status = gettext('Started');
+          self.curr_status = self.other_status_tpl({status_text:gettext('Started')});
 
           if (self.execution_time >= 2) {
-            self.curr_status = gettext['Running...'];
+            self.curr_status = self.other_status_tpl({status_text:gettext('Running...')});
           }
 
           if (!_.isNull(self.exit_code)) {
             if (self.state === 3) {
-              self.curr_status = gettext('Terminated by user.');
+              self.curr_status = self.failed_status_tpl({status_text:gettext('Terminated by user.')});
             } else if (self.exit_code == 0) {
-              self.curr_status = gettext('Successfully completed.');
+              self.curr_status = self.success_status_tpl({status_text:gettext('Successfully completed.')});
             } else {
-              self.curr_status = S(
-                gettext('Failed (exit code: %s).')
-              ).sprintf(String(self.exit_code)).value();
+              self.curr_status = self.failed_status_tpl(
+                {status_text:gettext('Failed (exit code: %s).', String(self.exit_code))}
+              );
             }
+          } else if (_.isNull(self.exit_code) && self.state === 3) {
+            self.curr_status = self.other_status_tpl({status_text:gettext('Terminating the process...')});
           }
 
           if (self.state == 0 && self.stime) {
@@ -236,18 +295,18 @@ define('misc.bgprocess', [
           async: true,
           contentType: 'application/json',
         })
-        .done(function(res) {
-          setTimeout(function() {
-            self.update(res);
-          }, 10);
-        })
-        .fail(function(res) {
-          // Try after some time only if job id present
-          if (res.status != 410)
+          .done(function(res) {
             setTimeout(function() {
               self.update(res);
-            }, 10000);
-        });
+            }, 10);
+          })
+          .fail(function(res) {
+          // Try after some time only if job id present
+            if (res.status != 410)
+              setTimeout(function() {
+                self.update(res);
+              }, 10000);
+          });
       },
 
       show: function() {
@@ -255,55 +314,35 @@ define('misc.bgprocess', [
 
         if (self.notify && !self.details) {
           if (!self.notifier) {
-            var header = $('<div></div>', {
-                class: 'pg-bg-notify-header',
-              }).append($('<span></span>').text(_.unescape(self.desc))),
-              content = $('<div class="pg-bg-bgprocess"></div>').append(
-                header
-              ).append(
-                $('<div></div>', {
-                  class: 'pg-bg-notify-body',
-                }).append(
-                  $('<div></div>', {
-                    class: 'pg-bg-start',
-                  }).append(
-                    $('<div></div>', {
-                      class: 'row align-items-center',
-                    }).append(
-                      $('<div></div>', {
-                        class: 'col-9',
-                      }).text(self.stime.toString())
-                    ).append(
-                      $('<div></div>', {
-                        class: 'col-3',
-                      }).append(
-                        $('<button></button>', {
-                          type: 'button',
-                          class: 'btn btn-danger btn-sm float-right bg-process-stop',
-                        }).text('Stop Process')
-                      )
-                    )
-                  ).append(
-                    $('<div class="pg-bg-etime"></div>')
-                  )
-                )
-              ),
-              for_details = $('<div></div>', {
-                class: 'text-center pg-bg-click',
-              }).append(
-                $('<span></span>').text(gettext('Click here for details.'))
-              ).appendTo(content),
-              close_me = $(
-                '<div class="bg-close"><i class="fa fa-close"></i></div>'
-              ).appendTo(header);
+            let content = $(`
+            <div class="card">
+              <div class="card-header bg-primary d-flex">
+                <div>${self.type_desc}</div>
+                <div class="ml-auto">
+                  <button class="btn btn-sm-sq btn-primary pg-bg-close" aria-label='close'><i class="fa fa-lg fa-times" role="img"></i></button>
+                </div>
+              </div>
+              <div class="card-body px-2">
+                <div class="py-1">${self.desc}</div>
+                <div class="py-1">${self.stime.toString()}</div>
+                <div class="d-flex py-1">
+                  <div class="my-auto mr-2">
+                    <span class="fa fa-clock fa-lg" role="img"></span>
+                  </div>
+                  <div class="pg-bg-etime my-auto mr-2"></div>
+                  <div class="ml-auto">
+                    <button class="btn btn-secondary pg-bg-more-details" title="More Details"><span class="fa fa-info-circle" role="img"></span>&nbsp;` + gettext('More details...') + `</button>
+                    <button class="btn btn-danger bg-process-stop" disabled><span class="fa fa-times-circle" role="img" title="Stop the operation"></span>&nbsp;` + gettext('Stop Process') + `</button>
+                  </div>
+                </div>
+                <div class="pg-bg-status py-1">
+                </div>
+              </div>
+            </div>
+            `);
 
-            $('<div></div>', {
-              class: 'pg-bg-status ' + ((self.exit_code === 0) ?
-                'bg-success' : (self.exit_code == 1) ?
-                'bg-failed' : ''),
-            }).appendTo(content);
-
-            $('<div></div>').appendTo(content);
+            let for_details = content.find('.pg-bg-more-details');
+            let close_me = content.find('.pg-bg-close');
 
             self.container = content;
             self.notifier = Alertify.notify(
@@ -330,7 +369,7 @@ define('misc.bgprocess', [
 
             // Do not close the notifier, when clicked on the container, which
             // is a default behaviour.
-            content.on('click', function(ev) {
+            self.container.on('click', function(ev) {
               ev = ev || window.event;
               ev.cancelBubble = true;
               ev.stopPropagation();
@@ -341,6 +380,7 @@ define('misc.bgprocess', [
             // On Click event to stop the process.
             content.find('.bg-process-stop').off('click').on('click', self.stop_process.bind(this));
           }
+
           // TODO:: Formatted execution time
           self.container.find('.pg-bg-etime').empty().append(
             $('<span></span>').text(
@@ -349,24 +389,16 @@ define('misc.bgprocess', [
           ).append(
             $('<span></span>').text(' ' + gettext('seconds'))
           );
-          var $status_bar = $(self.container.find('.pg-bg-status'));
-          $status_bar.empty().append(
-            self.curr_status
-          );
 
-          // Enable/Disable stop process button
+          var $status_bar = $(self.container.find('.pg-bg-status'));
+          $status_bar.html(self.curr_status);
           var $btn_stop_process = $(self.container.find('.bg-process-stop'));
-          if (isNaN(parseInt(self.exit_code))) {
+
+          // Enable Stop Process button only when process is running
+          if (parseInt(self.state) === 1) {
             $btn_stop_process.attr('disabled', false);
           } else {
             $btn_stop_process.attr('disabled', true);
-            if (self.exit_code === 0 && self.state != 3) {
-              $status_bar.addClass('bg-success');
-              $status_bar.removeClass('bg-failed');
-            } else {
-              $status_bar.addClass('bg-failed');
-              $status_bar.removeClass('bg-success');
-            }
           }
         } else {
           self.show_detailed_view.apply(self);
@@ -382,28 +414,41 @@ define('misc.bgprocess', [
           is_new = true;
           panel = this.panel =
             pgBrowser.BackgroundProcessObsorver.create_panel();
-
-          panel.title('Process Watcher - ' + _.escape(self.desc));
+          panel.title(gettext('Process Watcher - %s', self.type_desc));
           panel.focus();
         }
 
         var container = panel.$container,
-          status_class = '',
           $logs = container.find('.bg-process-watcher'),
           $header = container.find('.bg-process-details'),
           $footer = container.find('.bg-process-footer'),
-          $btn_stop_process = container.find('.bg-process-stop');
+          $btn_stop_process = container.find('.bg-process-stop'),
+          $btn_storage_manager = container.find('.bg-process-storage-manager');
 
-        // Enable/Disable stop process button
-        if (isNaN(parseInt(self.exit_code))) {
+        if(self.current_storage_dir && isServerMode) { //for backup & exports with server mode, operate over storage manager
+
+          if($btn_storage_manager.length == 0) {
+            var str_storage_manager_btn = '<button id="bg-process-storage-manager" class="btn btn-secondary bg-process-storage-manager" title="Click to open file location" aria-label="Storage Manager" tabindex="0" disabled><span class="pg-font-icon icon-storage-manager" role="img"></span></button>&nbsp;';
+            container.find('.bg-process-details .bg-btn-section').prepend(str_storage_manager_btn);
+            $btn_storage_manager = container.find('.bg-process-storage-manager');
+          }
+
+          // Disable storage manager button only when process is running
+          if (parseInt(self.state) === 1) {
+            $btn_storage_manager.attr('disabled', true);
+          }
+          else {
+            $btn_storage_manager.attr('disabled', false);
+          }
+          // On Click event for storage manager button.
+          $btn_storage_manager.off('click').on('click', self.storage_manager.bind(this));
+        }
+
+        // Enable Stop Process button only when process is running
+        if (parseInt(self.state) === 1) {
           $btn_stop_process.attr('disabled', false);
         } else {
           $btn_stop_process.attr('disabled', true);
-          if (self.exit_code === 0 && self.state != 3) {
-            status_class = 'bg-bgprocess-success';
-          } else {
-            status_class = 'bg-bgprocess-failed';
-          }
         }
 
         // On Click event to stop the process.
@@ -415,8 +460,10 @@ define('misc.bgprocess', [
           setTimeout(function() {
             self.logs[0].scrollTop = self.logs[0].scrollHeight;
           });
+          self.logs_loading = $('<li class="pg-bg-res-out loading-logs">' + gettext('Loading process logs...') + '</li>');
+          self.logs.append(self.logs_loading);
           // set bgprocess detailed description
-          $header.find('.bg-detailed-desc').html(_.unescape(self.detailed_desc));
+          $header.find('.bg-detailed-desc').html(self.detailed_desc);
         }
 
         // set bgprocess start time
@@ -425,9 +472,7 @@ define('misc.bgprocess', [
         );
 
         // set status
-        $footer.find('.bg-process-status p').removeClass().addClass(
-          status_class
-        ).html(self.curr_status);
+        $footer.find('.bg-process-status').html(self.curr_status);
 
         // set bgprocess execution time
         $footer.find('.bg-process-exec-time p').empty().append(
@@ -448,9 +493,9 @@ define('misc.bgprocess', [
             }, 1000
           );
 
-          var resize_log_container = function($logs, $header, $footer) {
-            var h = $header.outerHeight() + $footer.outerHeight();
-            $logs.css('padding-bottom', h);
+          var resize_log_container = function(logs, header, footer) {
+            var h = header.outerHeight() + footer.outerHeight();
+            logs.css('padding-bottom', h);
           }.bind(panel, $logs, $header, $footer);
 
           panel.on(wcDocker.EVENT.RESIZED, resize_log_container);
@@ -480,12 +525,12 @@ define('misc.bgprocess', [
           async: true,
           contentType: 'application/json',
         })
-        .done(function() {
-          return;
-        })
-        .fail(function() {
-          console.warn(arguments);
-        });
+          .done(function() {
+            return;
+          })
+          .fail(function() {
+            console.warn(arguments);
+          });
       },
 
       stop_process: function() {
@@ -500,14 +545,23 @@ define('misc.bgprocess', [
           async: true,
           contentType: 'application/json',
         })
-        .done(function() {
-          return;
-        })
-        .fail(function() {
-          console.warn(arguments);
-        });
+          .done(function() {
+            return;
+          })
+          .fail(function() {
+            console.warn(arguments);
+          });
       },
 
+      storage_manager: function() {
+
+        var self = this;
+        if(self.current_storage_dir) {
+          pgBrowser.Events.trigger(
+            'pgadmin:tools:storage_manager', self.current_storage_dir
+          );
+        }
+      },
     });
 
   _.extend(
@@ -550,31 +604,31 @@ define('misc.bgprocess', [
           async: true,
           contentType: 'application/json',
         })
-        .done(function(res) {
-          if (!res || !_.isArray(res)) {
-            return;
-          }
-          for (var idx in res) {
-            var process = res[idx];
-            if ('id' in process) {
-              if (!(process.id in observer.bgprocesses)) {
-                observer.bgprocesses[process.id] = new BGProcess(process);
+          .done(function(res) {
+            if (!res || !_.isArray(res)) {
+              return;
+            }
+            for (var idx in res) {
+              var process = res[idx];
+              if ('id' in process) {
+                if (!(process.id in observer.bgprocesses)) {
+                  observer.bgprocesses[process.id] = new BGProcess(process);
+                }
               }
             }
-          }
-          if (recheck && res.length == 0) {
+            if (recheck && res.length == 0) {
             // Recheck after some more time
-            setTimeout(
-              function() {
-                observer.update_process_list(false);
-              }, 3000
-            );
-          }
-        })
-        .fail(function() {
+              setTimeout(
+                function() {
+                  observer.update_process_list(false);
+                }, 3000
+              );
+            }
+          })
+          .fail(function() {
           // FIXME:: What to do now?
-          console.warn(arguments);
-        });
+            console.warn(arguments);
+          });
       },
 
       create_panel: function() {
@@ -605,35 +659,35 @@ define('misc.bgprocess', [
           showTitle: true,
           isCloseable: true,
           isPrivate: true,
+          isLayoutMember: false,
           content: '<div class="bg-process-details">' +
-            '<p class="bg-detailed-desc"></p>' +
-            '<div class="bg-process-stats" style="padding-bottom: 5px">' +
-              '<div class="row align-items-center">' +
-                '<div class="col-9">' +
-                  '<span><b>' + gettext('Start time') + ': </b>' +
-                    '<span class="bgprocess-start-time"></span>' +
-                  '</span>' +
+              '<div class="bg-detailed-desc"></div>' +
+              '<div class="bg-process-stats d-flex py-1">' +
+                '<div class="my-auto mr-2">' +
+                  '<span class="fa fa-clock fa-lg" role="img"></span>' +
                 '</div>' +
-                '<div class="col-3">' +
-                  '<button type="button" class="btn btn-danger btn-sm float-right bg-process-stop">' +
-                    gettext('Stop Process') + '</button>' +
+                '<div class="pg-bg-etime my-auto mr-2">'+
+                  '<span>' + gettext('Start time') + ': <span class="bgprocess-start-time"></span>' +
+                  '</span>'+
+                '</div>' +
+                '<div class="ml-auto bg-btn-section">' +
+                  '<button type="button" class="btn btn-danger bg-process-stop" disabled><span class="fa fa-times-circle" role="img"></span>&nbsp;' + gettext('Stop Process') + '</button>' +
                 '</div>' +
               '</div>' +
             '</div>' +
             '<div class="bg-process-watcher">' +
             '</div>' +
-            '<div class="bg-process-footer row">' +
-              '<div class="bg-process-status col-6">' +
-              '<span><b>' + gettext('Status') + ':</b></span><p></p>' +
+            '<div class="bg-process-footer p-2 d-flex">' +
+              '<div class="bg-process-status flex-grow-1">' +
               '</div>' +
-              '<div class="bg-process-exec-time col-6">' +
-              '<div class="exec-div pull-right">' +
-              '<span><b>' + gettext('Execution time') + ':</b></span><p></p>' +
+              '<div class="bg-process-exec-time ml-4 my-auto">' +
+                '<div class="exec-div">' +
+                  '<span>' + gettext('Execution time') + ':</span><p></p>' +
+                '</div>' +
               '</div>' +
-            '</div>' +
             '</div>',
           onCreate: function(myPanel, $container) {
-            $container.addClass('pg-no-overflow');
+            $container.addClass('pg-no-overflow p-2');
           },
         });
         p.load(pgBrowser.docker);
