@@ -2,7 +2,7 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
@@ -28,11 +28,7 @@ module.exports =  Alertify.dialog('createModeDlg', function() {
           text: gettext('Create'),
           key: 13,
           className: 'btn btn-primary fa fa-file file_manager_create file_manager_ok pg-alertify-button disabled',
-        },
-        ],
-        focus: {
-          element: 0,
-        },
+        }],
         options: {
           closableByDimmer: false,
           maximizable: false,
@@ -98,14 +94,14 @@ module.exports =  Alertify.dialog('createModeDlg', function() {
         contentType: 'application/x-download; charset=utf-8',
         async: false,
       })
-      .done(function(resp) {
-        var data = resp.data.result;
-        if (data['Code'] === 1) {
-          is_exist = true;
-        } else {
-          is_exist = false;
-        }
-      });
+        .done(function(resp) {
+          var data = resp.data.result;
+          if (data['Code'] === 1) {
+            is_exist = true;
+          } else {
+            is_exist = false;
+          }
+        });
       return is_exist;
     },
     check_permission: function(path) {
@@ -125,46 +121,67 @@ module.exports =  Alertify.dialog('createModeDlg', function() {
         contentType: 'application/json; charset=utf-8',
         async: false,
       })
-      .done(function(resp) {
-        var data = resp.data.result;
-        if (data.Code === 1) {
-          permission = true;
-        } else {
+        .done(function(resp) {
+          var data = resp.data.result;
+          if (data.Code === 1) {
+            permission = true;
+          } else {
+            $('.file_manager_ok').addClass('disabled');
+            Alertify.error(data.Error);
+          }
+        })
+        .fail(function() {
           $('.file_manager_ok').addClass('disabled');
-          Alertify.error(data.Error);
-        }
-      })
-      .fail(function() {
-        $('.file_manager_ok').addClass('disabled');
-        Alertify.error(gettext('Error occurred while checking access permission.'));
-      });
+          Alertify.error(gettext('Error occurred while checking access permission.'));
+        });
       return permission;
     },
     callback: function(closeEvent) {
+      closeEvent.cancel = false;
       if (closeEvent.button.text == gettext('Create')) {
-        var newFile = $('.storage_dialog #uploader .input-path').val(),
-          file_data = {
-            'path': $('.currentpath').val(),
-          },
-          innerbody;
+        var act_variable = document.activeElement.id;
+        if(act_variable != 'refresh_list') {
+          var newFile = $('.storage_dialog #uploader .input-path').val(),
+            file_data = {
+              'path': $('.currentpath').val(),
+            },
+            innerbody,
+            ext = $('.allowed_file_types select').val();
 
-        if (!this.check_permission(newFile)) {
-          closeEvent.cancel = true;
-          return;
-        }
+          /*
+             Add the file extension if necessary, and if the file type selector
+             isn't set to "All Files". If there's no . at all in the path, or
+             there is a . already but it's not following the last /, AND the
+             extension isn't *, then we add the extension.
+           */
+          if ((!newFile.includes('.') ||
+              newFile.split('.').pop() != ext) &&
+              ext != '*') {
+            newFile = newFile + '.' + ext;
+            $('.storage_dialog #uploader .input-path').val(newFile);
+          }
 
-        if (!_.isUndefined(newFile) && newFile !== '' && this.is_file_exist()) {
-          this.replace_file();
-          closeEvent.cancel = true;
+          if (!this.check_permission(newFile)) {
+            closeEvent.cancel = true;
+            return;
+          }
+
+          if (!_.isUndefined(newFile) && newFile !== '' && this.is_file_exist()) {
+            this.replace_file();
+            this.$container.find('.replace_file').find('.btn_yes').trigger('focus');
+            closeEvent.cancel = true;
+          } else {
+            pgAdmin.Browser.Events.trigger('pgadmin-storage:finish_btn:create_file', newFile);
+            innerbody = $(this.elements.body).find('.storage_content');
+            $(innerbody).find('*').off();
+            innerbody.remove();
+            removeTransId(this.trans_id);
+          }
+
+          set_last_traversed_dir(file_data, this.trans_id);
         } else {
-          pgAdmin.Browser.Events.trigger('pgadmin-storage:finish_btn:create_file', newFile);
-          innerbody = $(this.elements.body).find('.storage_content');
-          $(innerbody).find('*').off();
-          innerbody.remove();
-          removeTransId(this.trans_id);
+          closeEvent.cancel = true;
         }
-
-        set_last_traversed_dir(file_data, this.trans_id);
       } else if (closeEvent.button.text == gettext('Cancel')) {
         innerbody = $(this.elements.body).find('.storage_content');
         $(innerbody).find('*').off();
