@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -22,6 +22,7 @@ from pgadmin.utils.ajax import make_json_response, bad_request,\
 from pgadmin.utils.menu import MenuItem
 
 from pgadmin.model import db, Setting
+from pgadmin.utils.constants import MIMETYPE_APP_JS
 
 MODULE_NAME = 'settings'
 
@@ -39,7 +40,7 @@ class SettingsModule(PgAdminModule):
             'file_items': [
                 MenuItem(
                     name='mnu_resetlayout',
-                    priority=999,
+                    priority=998,
                     module="pgAdmin.Settings",
                     callback='show',
                     icon='fa fa-retweet',
@@ -55,7 +56,8 @@ class SettingsModule(PgAdminModule):
         """
         return [
             'settings.store', 'settings.store_bulk', 'settings.reset_layout',
-            'settings.save_tree_state', 'settings.get_tree_state'
+            'settings.save_tree_state', 'settings.get_tree_state',
+            'settings.reset_tree_state'
         ]
 
 
@@ -70,7 +72,7 @@ def store_setting(setting, value):
     db.session.commit()
 
 
-def get_setting(setting, default=None):
+def get_setting(setting, default=''):
     """Retrieve a configuration setting for the current user, or return the
     default value specified by the caller."""
     data = Setting.query.filter_by(
@@ -85,7 +87,7 @@ def get_setting(setting, default=None):
 @blueprint.route("/")
 @login_required
 def index():
-    return bad_request(errormsg=_("This URL cannot be called directly."))
+    return bad_request(errormsg=gettext("This URL cannot be called directly."))
 
 
 @blueprint.route("/settings.js")
@@ -94,7 +96,7 @@ def script():
     """Render the required Javascript"""
     return Response(response=render_template("settings/settings.js"),
                     status=200,
-                    mimetype="application/javascript")
+                    mimetype=MIMETYPE_APP_JS)
 
 
 @blueprint.route("/store", methods=['POST'], endpoint='store_bulk')
@@ -136,8 +138,31 @@ def store(setting=None, value=None):
 def reset_layout():
     """Reset configuration setting"""
 
-    # There can be only one record at most
-    data = Setting.query.filter_by(user_id=current_user.id).first()
+    try:
+        db.session.query(Setting) \
+            .filter(Setting.user_id == current_user.id)\
+            .filter((Setting.setting == 'Browser/Layout') |
+                    (Setting.setting == 'SQLEditor/Layout') |
+                    (Setting.setting == 'Debugger/Layout'))\
+            .delete()
+
+        db.session.commit()
+    except Exception as e:
+        return make_json_response(
+            status=410, success=0, errormsg=str(e)
+        )
+
+    return make_json_response(result=request.form)
+
+
+@blueprint.route("/reset_tree_state", methods=['DELETE'],
+                 endpoint='reset_tree_state')
+@login_required
+def reset_tree_state():
+    """Reset the saved tree state."""
+
+    data = Setting.query.filter_by(user_id=current_user.id,
+                                   setting='browser_tree_state').first()
     try:
         if data is not None:
             db.session.delete(data)
@@ -147,7 +172,7 @@ def reset_layout():
             status=410, success=0, errormsg=str(e)
         )
 
-    return make_json_response(result=request.form)
+    return success_return()
 
 
 @blueprint.route("/save_tree_state/", endpoint="save_tree_state",
