@@ -2,14 +2,12 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
 from pgadmin.model import Version
-import config
-import sys
 
 
 def get_version():
@@ -18,11 +16,9 @@ def get_version():
     except Exception:
         return -1
 
-    if int(version.value) > int(config.SETTINGS_SCHEMA_VERSION):
-        print(u"""
-    The database schema version is {0}, whilst the version required by the \
-    software is {1}.
-    Exiting...""".format(version.value, config.SETTINGS_SCHEMA_VERSION))
-        sys.exit(1)
-
     return version.value
+
+
+def set_version(new_version):
+    version = Version.query.filter_by(name='ConfigDB').first()
+    version.value = new_version
