@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -14,21 +14,19 @@ from flask import Response, url_for
 from flask import render_template, request, current_app
 from flask_babelex import gettext
 from flask_security import login_required
+from urllib.parse import unquote
+
 from pgadmin.browser.server_groups.servers.utils import parse_priv_to_db
 from pgadmin.utils import PgAdminModule
 from pgadmin.utils.ajax import make_response as ajax_response, \
-    make_json_response, internal_server_error
+    make_json_response, internal_server_error, bad_request
 from pgadmin.utils.driver import get_driver
 
 from config import PG_DEFAULT_DRIVER
-
-try:
-    from urllib import unquote
-except ImportError:
-    from urllib.parse import unquote
 from pgadmin.utils.ajax import precondition_required
 from functools import wraps
 from pgadmin.utils.preferences import Preferences
+from pgadmin.utils.constants import MIMETYPE_APP_JS
 
 # set template path for sql scripts
 MODULE_NAME = 'grant_wizard'
@@ -147,7 +145,9 @@ def check_precondition(f):
 @blueprint.route("/")
 @login_required
 def index():
-    pass
+    return bad_request(
+        errormsg=gettext("This URL cannot be called directly.")
+    )
 
 
 @blueprint.route("/grant_wizard.js")
@@ -157,7 +157,7 @@ def script():
     return Response(response=render_template(
         "grant_wizard/js/grant_wizard.js", _=gettext),
         status=200,
-        mimetype="application/javascript")
+        mimetype=MIMETYPE_APP_JS)
 
 
 @blueprint.route(
@@ -174,6 +174,86 @@ def acl_list(sid, did):
         mimetype="application/json")
 
 
+def _get_rows_for_type(conn, ntype, server_prop, node_id):
+    """
+    Used internally by properties to get rows for an object type
+    :param conn: connection object
+    :param ntype: object type
+    :param server_prop: server properties
+    :param node_id: oid
+    :return: status, execute response
+    """
+    function_sql_url = '/sql/function.sql'
+    status, res = True, []
+
+    if ntype in ['function']:
+        sql = render_template("/".join(
+            [server_prop['template_path'], function_sql_url]),
+            node_id=node_id, type='function')
+
+        status, res = conn.execute_dict(sql)
+    # Fetch procedures only if server type is EPAS or PG >= 11
+    elif len(server_prop) > 0 and (
+        server_prop['server_type'] == 'ppas' or (
+            server_prop['server_type'] == 'pg' and
+            server_prop['version'] >= 11000
+        )
+    ) and ntype in ['procedure']:
+        sql = render_template("/".join(
+            [server_prop['template_path'], function_sql_url]),
+            node_id=node_id, type='procedure')
+
+        status, res = conn.execute_dict(sql)
+
+    # Fetch trigger functions
+    elif ntype in ['trigger_function']:
+        sql = render_template("/".join(
+            [server_prop['template_path'], function_sql_url]),
+            node_id=node_id, type='trigger_function')
+        status, res = conn.execute_dict(sql)
+
+    # Fetch Sequences against schema
+    elif ntype in ['sequence']:
+        sql = render_template("/".join(
+            [server_prop['template_path'], '/sql/sequence.sql']),
+            node_id=node_id)
+
+        status, res = conn.execute_dict(sql)
+
+    # Fetch Tables against schema
+    elif ntype in ['table']:
+        sql = render_template("/".join(
+            [server_prop['template_path'], '/sql/table.sql']),
+            node_id=node_id)
+
+        status, res = conn.execute_dict(sql)
+
+    # Fetch Views against schema
+    elif ntype in ['view']:
+        sql = render_template("/".join(
+            [server_prop['template_path'], '/sql/view.sql']),
+            node_id=node_id, node_type='v')
+
+        status, res = conn.execute_dict(sql)
+
+    # Fetch Materialzed Views against schema
+    elif ntype in ['mview']:
+        sql = render_template("/".join(
+            [server_prop['template_path'], '/sql/view.sql']),
+            node_id=node_id, node_type='m')
+
+        status, res = conn.execute_dict(sql)
+    # Fetch Foreign tables.
+    elif ntype in ['foreign_table']:
+        sql = render_template("/".join(
+            [server_prop['template_path'], '/sql/foreign_table.sql']),
+            node_id=node_id, node_type='m')
+
+        status, res = conn.execute_dict(sql)
+
+    return status, res
+
+
 @blueprint.route(
     '/<int:sid>/<int:did>/<int:node_id>/<node_type>/',
     methods=['GET'], endpoint='objects'
@@ -185,6 +265,8 @@ def properties(sid, did, node_id, node_type):
        and render into selection page of wizard
     """
 
+    get_schema_sql_url = '/sql/get_schemas.sql'
+
     # unquote encoded url parameter
     node_type = unquote(node_type)
 
@@ -195,131 +277,70 @@ def properties(sid, did, node_id, node_type):
     manager = get_driver(PG_DEFAULT_DRIVER).connection_manager(sid)
     conn = manager.connection(did=did)
 
-    node_types = []
     show_sysobj = blueprint.show_system_objects().get()
     if node_type == 'database':
-
-        # Fetch list of schemas
-        # Get sys_obj_values and get list of schemas
-        ntype = 'schema'
-        SQL = render_template("/".join(
-            [server_prop['template_path'], '/sql/get_schemas.sql']),
+        sql = render_template("/".join(
+            [server_prop['template_path'], get_schema_sql_url]),
             show_sysobj=show_sysobj)
-        status, res = conn.execute_dict(SQL)
-
-        if not status:
-            return internal_server_error(errormsg=res)
-        node_types = res['rows']
+        ntype = 'schema'
     else:
-        SQL = render_template("/".join(
-            [server_prop['template_path'], '/sql/get_schemas.sql']),
-            nspid=node_id, show_sysobj=False)
-        status, res = conn.execute_dict(SQL)
-
-        if not status:
-            return internal_server_error(errormsg=res)
-        node_types = res['rows']
+        sql = render_template("/".join(
+            [server_prop['template_path'], get_schema_sql_url]),
+            show_sysobj=show_sysobj, nspid=node_id)
         ntype = node_type
 
+    status, res = conn.execute_dict(sql)
+
+    if not status:
+        return internal_server_error(errormsg=res)
+    node_types = res['rows']
+
+    def _append_rows(status, res, disp_type):
+        if not status:
+            current_app.logger.error(res)
+            failed_objects.append(disp_type)
+        else:
+            res_data.extend(res['rows'])
+
     for row in node_types:
         if 'oid' in row:
             node_id = row['oid']
 
-        # Fetch functions against schema
-        if ntype in ['schema', 'function']:
-            SQL = render_template("/".join(
-                [server_prop['template_path'], '/sql/function.sql']),
-                node_id=node_id, type='function')
+        if ntype == 'schema':
+            status, res = _get_rows_for_type(
+                conn, 'function', server_prop, node_id)
+            _append_rows(status, res, 'function')
 
-            status, res = conn.execute_dict(SQL)
-            if not status:
-                current_app.logger.error(res)
-                failed_objects.append('function')
-            else:
-                res_data.extend(res['rows'])
+            status, res = _get_rows_for_type(
+                conn, 'procedure', server_prop, node_id)
+            _append_rows(status, res, 'procedure')
 
-        # Fetch procedures only if server type is EPAS or PG >= 11
-        if (len(server_prop) > 0 and
-            (server_prop['server_type'] == 'ppas' or
-             (server_prop['server_type'] == 'pg' and
-              server_prop['version'] >= 11000)) and
-                ntype in ['schema', 'procedure']):
-            SQL = render_template("/".join(
-                [server_prop['template_path'], '/sql/function.sql']),
-                node_id=node_id, type='procedure')
+            status, res = _get_rows_for_type(
+                conn, 'trigger_function', server_prop, node_id)
+            _append_rows(status, res, 'trigger function')
 
-            status, res = conn.execute_dict(SQL)
+            status, res = _get_rows_for_type(
+                conn, 'sequence', server_prop, node_id)
+            _append_rows(status, res, 'sequence')
 
-            if not status:
-                current_app.logger.error(res)
-                failed_objects.append('procedure')
-            else:
-                res_data.extend(res['rows'])
+            status, res = _get_rows_for_type(
+                conn, 'table', server_prop, node_id)
+            _append_rows(status, res, 'table')
 
-        # Fetch trigger functions
-        if ntype in ['schema', 'trigger_function']:
-            SQL = render_template("/".join(
-                [server_prop['template_path'], '/sql/function.sql']),
-                node_id=node_id, type='trigger_function')
-            status, res = conn.execute_dict(SQL)
+            status, res = _get_rows_for_type(
+                conn, 'view', server_prop, node_id)
+            _append_rows(status, res, 'view')
 
-            if not status:
-                current_app.logger.error(res)
-                failed_objects.append('trigger function')
-            else:
-                res_data.extend(res['rows'])
+            status, res = _get_rows_for_type(
+                conn, 'mview', server_prop, node_id)
+            _append_rows(status, res, 'materialized view')
 
-        # Fetch Sequences against schema
-        if ntype in ['schema', 'sequence']:
-            SQL = render_template("/".join(
-                [server_prop['template_path'], '/sql/sequence.sql']),
-                node_id=node_id)
-
-            status, res = conn.execute_dict(SQL)
-            if not status:
-                current_app.logger.error(res)
-                failed_objects.append('sequence')
-            else:
-                res_data.extend(res['rows'])
-
-        # Fetch Tables against schema
-        if ntype in ['schema', 'table']:
-            SQL = render_template("/".join(
-                [server_prop['template_path'], '/sql/table.sql']),
-                node_id=node_id)
-
-            status, res = conn.execute_dict(SQL)
-            if not status:
-                current_app.logger.error(res)
-                failed_objects.append('table')
-            else:
-                res_data.extend(res['rows'])
-
-        # Fetch Views against schema
-        if ntype in ['schema', 'view']:
-            SQL = render_template("/".join(
-                [server_prop['template_path'], '/sql/view.sql']),
-                node_id=node_id, node_type='v')
-
-            status, res = conn.execute_dict(SQL)
-            if not status:
-                current_app.logger.error(res)
-                failed_objects.append('view')
-            else:
-                res_data.extend(res['rows'])
-
-        # Fetch Materialzed Views against schema
-        if ntype in ['schema', 'mview']:
-            SQL = render_template("/".join(
-                [server_prop['template_path'], '/sql/view.sql']),
-                node_id=node_id, node_type='m')
-
-            status, res = conn.execute_dict(SQL)
-            if not status:
-                current_app.logger.error(res)
-                failed_objects.append('materialized view')
-            else:
-                res_data.extend(res['rows'])
+            status, res = _get_rows_for_type(
+                conn, 'foreign_table', server_prop, node_id)
+            _append_rows(status, res, 'foreign table')
+        else:
+            status, res = _get_rows_for_type(conn, ntype, server_prop, node_id)
+            _append_rows(status, res, 'function')
 
     msg = None
     if len(failed_objects) > 0:
@@ -336,7 +357,7 @@ def properties(sid, did, node_id, node_type):
 
 @blueprint.route(
     '/sql/<int:sid>/<int:did>/',
-    methods=['GET'], endpoint='modified_sql'
+    methods=['POST'], endpoint='modified_sql'
 )
 @login_required
 @check_precondition
@@ -344,15 +365,8 @@ def msql(sid, did):
     """
     This function will return modified SQL
     """
-
     server_prop = server_info
-    data = {}
-    for k, v in request.args.items():
-        try:
-            data[k] = json.loads(v)
-        except ValueError:
-            data[k] = v
-
+    data = request.form if request.form else json.loads(request.data.decode())
     # Form db connection
     manager = get_driver(PG_DEFAULT_DRIVER).connection_manager(sid)
     conn = manager.connection(did=did)
@@ -367,7 +381,6 @@ def msql(sid, did):
         current_app.logger.exception(e)
 
     try:
-
         # Parse privileges
         data['priv'] = {}
         if 'acl' in data:
@@ -384,35 +397,48 @@ def msql(sid, did):
                 data['acl'],
                 acls['table']['acl'])
 
+            data['priv']['foreign_table'] = parse_priv_to_db(
+                data['acl'],
+                acls['foreign_table']['acl'])
+
         # Pass database objects and get SQL for privileges
-        SQL_data = ''
+        sql_data = ''
         data_func = {'objects': data['objects'],
                      'priv': data['priv']['function']}
-        SQL = render_template(
+        sql = render_template(
             "/".join([server_prop['template_path'],
                       '/sql/grant_function.sql']),
             data=data_func, conn=conn)
-        if SQL and SQL.strip('\n') != '':
-            SQL_data += SQL
+        if sql and sql.strip('\n') != '':
+            sql_data += sql
 
         data_seq = {'objects': data['objects'],
                     'priv': data['priv']['sequence']}
-        SQL = render_template(
+        sql = render_template(
             "/".join([server_prop['template_path'],
                       '/sql/grant_sequence.sql']),
             data=data_seq, conn=conn)
-        if SQL and SQL.strip('\n') != '':
-            SQL_data += SQL
+        if sql and sql.strip('\n') != '':
+            sql_data += sql
 
         data_table = {'objects': data['objects'],
                       'priv': data['priv']['table']}
-        SQL = render_template(
+        sql = render_template(
             "/".join([server_prop['template_path'], '/sql/grant_table.sql']),
             data=data_table, conn=conn)
-        if SQL and SQL.strip('\n') != '':
-            SQL_data += SQL
+        if sql and sql.strip('\n') != '':
+            sql_data += sql
 
-        res = {'data': SQL_data}
+        data_table = {'objects': data['objects'],
+                      'priv': data['priv']['foreign_table']}
+        sql = render_template(
+            "/".join([server_prop['template_path'],
+                      '/sql/grant_foreign_table.sql']),
+            data=data_table, conn=conn)
+        if sql and sql.strip('\n') != '':
+            sql_data += sql
+
+        res = {'data': sql_data}
 
         return ajax_response(
             response=res,
@@ -471,36 +497,49 @@ def save(sid, did):
                 data['acl'],
                 acls['table']['acl'])
 
+            data['priv']['foreign_table'] = parse_priv_to_db(
+                data['acl'],
+                acls['foreign_table']['acl'])
+
         # Pass database objects and get SQL for privileges
         # Pass database objects and get SQL for privileges
-        SQL_data = ''
+        sql_data = ''
         data_func = {'objects': data['objects'],
                      'priv': data['priv']['function']}
-        SQL = render_template(
+        sql = render_template(
             "/".join([server_prop['template_path'],
                       '/sql/grant_function.sql']),
             data=data_func, conn=conn)
-        if SQL and SQL.strip('\n') != '':
-            SQL_data += SQL
+        if sql and sql.strip('\n') != '':
+            sql_data += sql
 
         data_seq = {'objects': data['objects'],
                     'priv': data['priv']['sequence']}
-        SQL = render_template(
+        sql = render_template(
             "/".join([server_prop['template_path'],
                       '/sql/grant_sequence.sql']),
             data=data_seq, conn=conn)
-        if SQL and SQL.strip('\n') != '':
-            SQL_data += SQL
+        if sql and sql.strip('\n') != '':
+            sql_data += sql
 
         data_table = {'objects': data['objects'],
                       'priv': data['priv']['table']}
-        SQL = render_template(
+        sql = render_template(
             "/".join([server_prop['template_path'], '/sql/grant_table.sql']),
             data=data_table, conn=conn)
-        if SQL and SQL.strip('\n') != '':
-            SQL_data += SQL
+        if sql and sql.strip('\n') != '':
+            sql_data += sql
 
-        status, res = conn.execute_dict(SQL_data)
+        data_table = {'objects': data['objects'],
+                      'priv': data['priv']['foreign_table']}
+        sql = render_template(
+            "/".join([server_prop['template_path'],
+                      '/sql/grant_foreign_table.sql']),
+            data=data_table, conn=conn)
+        if sql and sql.strip('\n') != '':
+            sql_data += sql
+
+        status, res = conn.execute_dict(sql_data)
         if not status:
             return internal_server_error(errormsg=res)
 
