@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -15,12 +15,13 @@ from flask import url_for, Response, render_template, request, current_app
 from flask_babelex import gettext as _
 from flask_security import login_required, current_user
 from pgadmin.misc.bgprocess.processes import BatchProcess, IProcessDesc
-from pgadmin.utils import PgAdminModule, html, is_utility_exists
+from pgadmin.utils import PgAdminModule, html, does_utility_exist
 from pgadmin.utils.ajax import bad_request, make_json_response
 from pgadmin.utils.driver import get_driver
 
 from config import PG_DEFAULT_DRIVER
 from pgadmin.model import Server
+from pgadmin.utils.constants import MIMETYPE_APP_JS
 
 MODULE_NAME = 'maintenance'
 
@@ -93,10 +94,16 @@ class Message(IProcessDesc):
         if self.data['op'] == "CLUSTER":
             return res.format(_('Cluster'))
 
-    def details(self, cmd, args):
+    @property
+    def type_desc(self):
+        return _("Maintenance")
 
+    def _check_for_vacuum(self):
+        """
+        Check for VACUUM in data and return format response.
+        :return: response.
+        """
         res = None
-
         if self.data['op'] == "VACUUM":
             res = _('VACUUM ({0})')
 
@@ -109,6 +116,10 @@ class Message(IProcessDesc):
                 opts.append(_('VERBOSE'))
 
             res = res.format(', '.join(str(x) for x in opts))
+        return res
+
+    def details(self, cmd, args):
+        res = self._check_for_vacuum()
 
         if self.data['op'] == "ANALYZE":
             res = _('ANALYZE')
@@ -130,13 +141,11 @@ class Message(IProcessDesc):
 
         res = '<div>' + html.safe_str(res)
 
-        res += '</div><div>'
-        res += html.safe_str(
-            _("Running Query:")
-        )
-        res += '</b><br><i>'
+        res += '</div><div class="py-1">'
+        res += _("Running Query:")
+        res += '<div class="pg-bg-cmd enable-selection p-1">'
         res += html.safe_str(self.query)
-        res += '</i></div>'
+        res += '</div></div>'
 
         return res
 
@@ -156,10 +165,27 @@ def script():
     return Response(
         response=render_template("maintenance/js/maintenance.js", _=_),
         status=200,
-        mimetype="application/javascript"
+        mimetype=MIMETYPE_APP_JS
     )
 
 
+def get_index_name(data):
+    """
+    Check and get index name from constraints.
+    :param data: Data.
+    :return: index_name.
+    """
+    index_name = None
+    if 'primary_key' in data and data['primary_key']:
+        index_name = data['primary_key']
+    elif 'unique_constraint' in data and data['unique_constraint']:
+        index_name = data['unique_constraint']
+    elif 'index' in data and data['index']:
+        index_name = data['index']
+
+    return index_name
+
+
 @blueprint.route(
     '/job/<int:sid>/<int:did>', methods=['POST'], endpoint='create_job'
 )
@@ -176,20 +202,11 @@ def create_maintenance_job(sid, did):
         None
     """
     if request.form:
-        # Convert ImmutableDict to dict
-        data = dict(request.form)
-        data = json.loads(data['data'][0], encoding='utf-8')
+        data = json.loads(request.form['data'], encoding='utf-8')
     else:
         data = json.loads(request.data, encoding='utf-8')
 
-    index_name = None
-
-    if 'primary_key' in data and data['primary_key']:
-        index_name = data['primary_key']
-    elif 'unique_constraint' in data and data['unique_constraint']:
-        index_name = data['unique_constraint']
-    elif 'index' in data and data['index']:
-        index_name = data['index']
+    index_name = get_index_name(data)
 
     # Fetch the server details like hostname, port, roles etc
     server = Server.query.filter_by(
@@ -214,7 +231,7 @@ def create_maintenance_job(sid, did):
         )
 
     utility = manager.utility('sql')
-    ret_val = is_utility_exists(utility)
+    ret_val = does_utility_exist(utility)
     if ret_val:
         return make_json_response(
             success=0,
@@ -266,7 +283,7 @@ def create_maintenance_job(sid, did):
     # Return response
     return make_json_response(
         data={'job_id': jid, 'status': True,
-              'info': 'Maintenance job created.'}
+              'info': _('Maintenance job created.')}
     )
 
 
@@ -298,7 +315,7 @@ def check_utility_exists(sid):
     manager = driver.connection_manager(server.id)
 
     utility = manager.utility('sql')
-    ret_val = is_utility_exists(utility)
+    ret_val = does_utility_exist(utility)
     if ret_val:
         return make_json_response(
             success=0,
