@@ -2,21 +2,20 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
-
 define([
   'sources/gettext', 'sources/url_for', 'jquery', 'underscore',
-  'underscore.string', 'pgadmin.alertifyjs', 'sources/pgadmin', 'pgadmin.browser', 'backbone',
+  'pgadmin.alertifyjs', 'sources/pgadmin', 'pgadmin.browser', 'backbone',
   'backgrid', 'backform', 'sources/utils',
   'tools/maintenance/static/js/menu_utils',
   'sources/nodes/supported_database_node',
   'pgadmin.backform', 'pgadmin.backgrid',
   'pgadmin.browser.node.ui',
 ], function(
-  gettext, url_for, $, _, S, Alertify, pgAdmin, pgBrowser, Backbone, Backgrid,
+  gettext, url_for, $, _, Alertify, pgAdmin, pgBrowser, Backbone, Backgrid,
   Backform, commonUtils,
   menuUtils, supportedNodes
 ) {
@@ -29,10 +28,6 @@ define([
   if (pgAdmin.Tools.maintenance)
     return pgAdmin.Tools.maintenance;
 
-  Backform.CustomSwitchControl = Backform.SwitchControl.extend({
-    className: 'pgadmin-control-group form-group pg-el-md-6 pg-el-12 row',
-  });
-
   // Main model for Maintenance functionality
   var MaintenanceModel = Backbone.Model.extend({
     defaults: {
@@ -55,8 +50,11 @@ define([
       id: 'op',
       label: gettext('Maintenance operation'),
       cell: 'string',
-      type: 'text',
+      type: 'radioModern',
+      controlsClassName: 'pgadmin-controls col-12 col-sm-8',
+      controlLabelClassName: 'control-label col-sm-4 col-12',
       group: gettext('Options'),
+      value: 'VACUUM',
       options: [{
         'label': 'VACUUM',
         'value': 'VACUUM',
@@ -74,24 +72,6 @@ define([
         'value': 'CLUSTER',
       },
       ],
-      control: Backform.RadioControl.extend({
-        template: _.template([
-          '<label class="control-label col-sm-4 col-12"><%=label%></label>',
-          '<div class="pgadmin-controls col-12 col-sm-8 btn-group pg-maintenance-op pgadmin-controls-radio-none" data-toggle="buttons">',
-          ' <% for (var i=0; i < options.length; i++) { %>',
-          ' <% var option = options[i]; %>',
-          ' <label class="btn btn-primary<% if (i == 0) { %> active<%}%>">',
-          '  <input type="radio" name="op" id="op" autocomplete="off" value=<%-formatter.fromRaw(option.value)%><% if (i == 0) { %> selected<%}%> > <%-option.label%>',
-          ' </label>',
-          ' <% } %>',
-          '</div>',
-        ].join('\n')),
-      }),
-      select2: {
-        allowClear: false,
-        width: '100%',
-        placeholder: gettext('Select from list...'),
-      },
     },
     {
       type: 'nested',
@@ -103,21 +83,30 @@ define([
         id: 'vacuum_full',
         group: gettext('Vacuum'),
         disabled: 'isDisabled',
-        control: Backform.CustomSwitchControl,
+        type: 'switch',
+        extraToggleClasses: 'pg-el-sm-4',
+        controlLabelClassName: 'control-label pg-el-sm-5 pg-el-12',
+        controlsClassName: 'pgadmin-controls pg-el-sm-7 pg-el-12',
         label: gettext('FULL'),
         deps: ['op'],
       }, {
         id: 'vacuum_freeze',
         deps: ['op'],
         disabled: 'isDisabled',
-        control: Backform.CustomSwitchControl,
+        type: 'switch',
+        extraToggleClasses: 'pg-el-sm-4',
+        controlLabelClassName: 'control-label pg-el-sm-5 pg-el-12',
+        controlsClassName: 'pgadmin-controls pg-el-sm-7 pg-el-12',
         label: gettext('FREEZE'),
         group: gettext('Vacuum'),
       }, {
         id: 'vacuum_analyze',
         deps: ['op'],
         disabled: 'isDisabled',
-        control: Backform.CustomSwitchControl,
+        type: 'switch',
+        extraToggleClasses: 'pg-el-sm-4',
+        controlLabelClassName: 'control-label pg-el-sm-5 pg-el-12',
+        controlsClassName: 'pgadmin-controls pg-el-sm-7 pg-el-12',
         label: gettext('ANALYZE'),
         group: gettext('Vacuum'),
       }],
@@ -126,7 +115,7 @@ define([
       id: 'verbose',
       group: gettext('Options'),
       deps: ['op'],
-      control: Backform.CustomSwitchControl,
+      type: 'switch',
       label: gettext('Verbose Messages'),
       disabled: 'isDisabled',
     },
@@ -244,7 +233,7 @@ define([
           return;
         }
       } else {
-        Alertify.alert(S(gettext('Failed to load preference %s of module %s')).sprintf(preference_name, module).value());
+        Alertify.alert(gettext('Failed to load preference %s of module %s', preference_name, module));
         return;
       }
 
@@ -258,7 +247,7 @@ define([
       if (!d)
         return;
 
-      var node = d && pgBrowser.Nodes[d._type],
+      var node = pgBrowser.Nodes[d._type],
         treeInfo = node.getTreeNodeHierarchy.apply(node, [i]);
 
       if (treeInfo.database._label.indexOf('=') >= 0) {
@@ -281,21 +270,23 @@ define([
               return {
                 buttons: [{
                   text: '',
-                  className: 'btn btn-secondary pull-left fa fa-info pg-alertify-icon-button',
+                  className: 'btn btn-primary-icon pull-left fa fa-info pg-alertify-icon-button',
                   attrs: {
                     name: 'object_help',
                     type: 'button',
                     url: 'maintenance.html',
                     label: gettext('Maintenance'),
+                    'aria-label': gettext('Object Help'),
                   },
                 }, {
                   text: '',
                   key: 112,
-                  className: 'btn btn-secondary pull-left fa fa-question pg-alertify-icon-button',
+                  className: 'btn btn-primary-icon pull-left fa fa-question pg-alertify-icon-button',
                   attrs: {
                     name: 'dialog_help',
                     type: 'button',
                     label: gettext('Maintenance'),
+                    'aria-label': gettext('Help'),
                     url: url_for(
                       'help.static', {
                         'filename': 'maintenance_dialog.html',
@@ -324,14 +315,14 @@ define([
             },
             // Callback functions when click on the buttons of the Alertify dialogs
             callback: function(e) {
-              var i = pgBrowser.tree.selected(),
-                d = i && i.length == 1 ? pgBrowser.tree.itemData(i) : undefined,
-                node = d && pgBrowser.Nodes[d._type];
+              var sel_item = pgBrowser.tree.selected(),
+                itemData = sel_item && sel_item.length == 1 ? pgBrowser.tree.itemData(sel_item) : undefined,
+                sel_node = itemData && pgBrowser.Nodes[itemData._type];
 
               if (e.button.element.name == 'dialog_help' || e.button.element.name == 'object_help') {
                 e.cancel = true;
                 pgBrowser.showHelp(e.button.element.name, e.button.element.getAttribute('url'),
-                  node, i, e.button.element.getAttribute('label'));
+                  sel_node, sel_item);
                 return;
               }
 
@@ -343,31 +334,33 @@ define([
                   unique_constraint = undefined,
                   index = undefined;
 
-                if (!d)
+                if (!itemData)
                   return;
 
-                var treeInfo = node.getTreeNodeHierarchy.apply(node, [i]);
+                var node_hierarchy = sel_node.getTreeNodeHierarchy.apply(sel_node, [sel_item]);
 
-                if (treeInfo.schema != undefined) {
-                  schema = treeInfo.schema._label;
+                if (node_hierarchy.schema != undefined) {
+                  schema = node_hierarchy.schema._label;
                 }
 
-                if (treeInfo.partition != undefined) {
-                  table = treeInfo.partition._label;
-                } else if (treeInfo.table != undefined) {
-                  table = treeInfo.table._label;
+                if (node_hierarchy.partition != undefined) {
+                  table = node_hierarchy.partition._label;
+                } else if (node_hierarchy.table != undefined) {
+                  table = node_hierarchy.table._label;
+                } else if (node_hierarchy.mview != undefined) {
+                  table = node_hierarchy.mview._label;
                 }
 
-                if (treeInfo.primary_key != undefined) {
-                  primary_key = treeInfo.primary_key._label;
-                } else if (treeInfo.unique_constraint != undefined) {
-                  unique_constraint = treeInfo.unique_constraint._label;
-                } else if (treeInfo.index != undefined) {
-                  index = treeInfo.index._label;
+                if (node_hierarchy.primary_key != undefined) {
+                  primary_key = node_hierarchy.primary_key._label;
+                } else if (node_hierarchy.unique_constraint != undefined) {
+                  unique_constraint = node_hierarchy.unique_constraint._label;
+                } else if (node_hierarchy.index != undefined) {
+                  index = node_hierarchy.index._label;
                 }
 
                 this.view.model.set({
-                  'database': treeInfo.database._label,
+                  'database': node_hierarchy.database._label,
                   'schema': schema,
                   'table': table,
                   'primary_key': primary_key,
@@ -378,31 +371,31 @@ define([
                 $.ajax({
                   url: url_for(
                     'maintenance.create_job', {
-                      'sid': treeInfo.server._id,
-                      'did': treeInfo.database._id,
+                      'sid': node_hierarchy.server._id,
+                      'did': node_hierarchy.database._id,
                     }),
                   method: 'POST',
                   data: {
                     'data': JSON.stringify(this.view.model.toJSON()),
                   },
                 })
-                .done(function(res) {
-                  if (res.data && res.data.status) {
+                  .done(function(res) {
+                    if (res.data && res.data.status) {
                     //Do nothing as we are creating the job and exiting from the main dialog
-                    Alertify.success(res.data.info);
-                    pgBrowser.Events.trigger('pgadmin-bgprocess:created', self);
-                  } else {
+                      Alertify.success(res.data.info);
+                      pgBrowser.Events.trigger('pgadmin-bgprocess:created', self);
+                    } else {
+                      Alertify.alert(
+                        gettext('Maintenance job creation failed.'),
+                        res.errormsg
+                      );
+                    }
+                  })
+                  .fail(function() {
                     Alertify.alert(
-                      gettext('Maintenance job creation failed.'),
-                      res.errormsg
+                      gettext('Maintenance job creation failed.')
                     );
-                  }
-                })
-                .fail(function() {
-                  Alertify.alert(
-                    gettext('Maintenance job creation failed.')
-                  );
-                });
+                  });
               }
             },
             build: function() {
@@ -418,26 +411,29 @@ define([
                   });
                 }
               },
+              onshow: function() {
+                var container = $(this.elements.body).find('.tab-content:first > .tab-pane.active:first');
+                commonUtils.findAndSetFocus(container);
+              },
             },
             prepare: function() {
               // Main maintenance tool dialog container
               var $container = $('<div class=\'maintenance_dlg\'></div>');
+              var tree = pgBrowser.tree,
+                sel_item = tree.selected(),
+                itemInfo = sel_item && sel_item.length == 1 ? tree.itemData(sel_item) : undefined,
+                nodeData = itemInfo && pgBrowser.Nodes[itemInfo._type];
 
-              var t = pgBrowser.tree,
-                i = t.selected(),
-                d = i && i.length == 1 ? t.itemData(i) : undefined,
-                node = d && pgBrowser.Nodes[d._type];
-
-              if (!d)
+              if (!itemInfo)
                 return;
 
-              var treeInfo = node.getTreeNodeHierarchy.apply(node, [i]);
+              var treeData = nodeData.getTreeNodeHierarchy.apply(nodeData, [sel_item]);
 
               var newModel = new MaintenanceModel({}, {
-                  node_info: treeInfo,
+                  node_info: treeData,
                 }),
                 fields = Backform.generateViewSchema(
-                  treeInfo, newModel, 'create', node, treeInfo.server, true
+                  treeData, newModel, 'create', nodeData, treeData.server, true
                 );
 
               var view = this.view = new Backform.Dialog({
@@ -450,20 +446,17 @@ define([
               view.render();
 
               // If node is Index, Unique or Primary key then disable vacuum & analyze button
-              if (d._type == 'primary_key' || d._type == 'unique_constraint' ||
-                d._type == 'index') {
+              if (itemInfo._type == 'primary_key' || itemInfo._type == 'unique_constraint' ||
+                itemInfo._type == 'index') {
                 var vacuum_analyze_btns = $container.find(
-                  '.pgadmin-controls label:lt(2)'
-                ).removeClass('active').addClass('disabled');
+                  '.btn-group label.btn:lt(2)'
+                ).addClass('disabled');
                 // Find reindex button element & add active class to it
                 var reindex_btn = vacuum_analyze_btns[1].nextElementSibling;
-                $(reindex_btn).addClass('active');
+                $(reindex_btn).trigger('click');
               }
 
               view.$el.attr('tabindex', -1);
-              var container = view.$el.find('.tab-content:first > .tab-pane.active:first');
-              commonUtils.findAndSetFocus(container);
-
               this.elements.content.appendChild($container.get(0));
             },
           };
@@ -479,25 +472,25 @@ define([
         url: baseUrl,
         type:'GET',
       })
-      .done(function(res) {
-        if (!res.success) {
+        .done(function(res) {
+          if (!res.success) {
+            Alertify.alert(
+              gettext('Utility not found'),
+              res.errormsg
+            );
+            return;
+          }
+          // Open the Alertify dialog
+          Alertify.MaintenanceDialog(gettext('Maintenance...')).set('resizable', true)
+            .resizeTo(pgAdmin.Browser.stdW.md,pgAdmin.Browser.stdH.md);
+        })
+        .fail(function() {
           Alertify.alert(
             gettext('Utility not found'),
-            res.errormsg
+            gettext('Failed to fetch Utility information')
           );
           return;
-        }
-        // Open the Alertify dialog
-        Alertify.MaintenanceDialog(gettext('Maintenance...')).set('resizable', true)
-          .resizeTo(pgAdmin.Browser.stdW.md,pgAdmin.Browser.stdH.md);
-      })
-      .fail(function() {
-        Alertify.alert(
-          gettext('Utility not found'),
-          gettext('Failed to fetch Utility information')
-        );
-        return;
-      });
+        });
     },
   };
 
