@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -14,7 +14,7 @@ import os
 
 from pgadmin.utils.route import BaseTestGenerator
 from regression import parent_node_dict
-from pgadmin.utils import is_utility_exists
+from pgadmin.utils import does_utility_exist
 
 
 class MaintenanceJobTest(BaseTestGenerator):
@@ -52,7 +52,11 @@ class MaintenanceJobTest(BaseTestGenerator):
 
         binary_path = os.path.join(
             self.server['default_binary_paths'][self.server['type']], 'psql')
-        retVal = is_utility_exists(binary_path)
+
+        if os.name == 'nt':
+            binary_path = binary_path + '.exe'
+
+        retVal = does_utility_exist(binary_path)
         if retVal is not None:
             self.skipTest(retVal)
 
@@ -70,42 +74,51 @@ class MaintenanceJobTest(BaseTestGenerator):
         job_id = response_data['data']['job_id']
 
         cnt = 0
-        while 1:
+        the_process = None
+        while True:
             if cnt >= 10:
                 break
             # Check the process list
-            response1 = self.tester.get('/misc/bgprocess/?_='.format(
+            response1 = self.tester.get('/misc/bgprocess/?_={0}'.format(
                 random.randint(1, 9999999)))
             self.assertEqual(response1.status_code, 200)
             process_list = json.loads(response1.data.decode('utf-8'))
 
-            if len(process_list) > 0 and 'execution_time' in process_list[0]:
+            try:
+                the_process = next(
+                    p for p in process_list if p['id'] == job_id)
+            except Exception:
+                the_process = None
+
+            if the_process and 'execution_time' in the_process:
                 break
             time.sleep(0.5)
             cnt += 1
 
-        self.assertTrue('execution_time' in process_list[0])
-        self.assertTrue('stime' in process_list[0])
-        self.assertTrue('exit_code' in process_list[0])
-        self.assertTrue(process_list[0]['exit_code'] in
+        self.assertTrue('execution_time' in the_process)
+        self.assertTrue('stime' in the_process)
+        self.assertTrue('exit_code' in the_process)
+        self.assertTrue(the_process['exit_code'] in
                         self.expected_exit_code)
 
-        self.assertIn(self.expected_cmd, process_list[0]['details'])
+        self.assertIn(self.expected_cmd, the_process['details'])
 
         # Check the process details
-        p_details = self.tester.get('/misc/bgprocess/{0}?_='.format(
+        p_details = self.tester.get('/misc/bgprocess/{0}?_={1}'.format(
             job_id, random.randint(1, 9999999))
         )
         self.assertEqual(p_details.status_code, 200)
 
-        p_details = self.tester.get('/misc/bgprocess/{0}/{1}/{2}/?_='.format(
-            job_id, 0, 0, random.randint(1, 9999999))
+        p_details = self.tester.get(
+            '/misc/bgprocess/{0}/{1}/{2}/?_={3}'.format(
+                job_id, 0, 0, random.randint(1, 9999999)
+            )
         )
         self.assertEqual(p_details.status_code, 200)
         p_details_data = json.loads(p_details.data.decode('utf-8'))
 
         # Retrieve the backup job process logs
-        while 1:
+        while True:
             out, err, status = MaintenanceJobTest.get_params(p_details_data)
             if status:
                 break
