@@ -2,25 +2,20 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
 
-import sys
 import simplejson as json
 import os
 
 from pgadmin.utils.route import BaseTestGenerator
 from regression import parent_node_dict
-from pgadmin.utils import server_utils as server_utils, is_utility_exists
+from pgadmin.utils import server_utils as server_utils, does_utility_exist
 from pgadmin.browser.server_groups.servers.databases.tests import utils as \
     database_utils
-
-if sys.version_info < (3, 3):
-    from mock import patch, MagicMock
-else:
-    from unittest.mock import patch, MagicMock
+from unittest.mock import patch, MagicMock
 
 
 class RestoreCreateJobTest(BaseTestGenerator):
@@ -233,7 +228,7 @@ class RestoreCreateJobTest(BaseTestGenerator):
              not_expected_cmd_opts=[],
              expected_exit_code=[0, None]
          )),
-        ('When restore object with option - Disbale',
+        ('When restore object with option - Disable Triggers',
          dict(
              class_params=dict(
                  sid=1,
@@ -305,7 +300,11 @@ class RestoreCreateJobTest(BaseTestGenerator):
         binary_path = os.path.join(
             self.server['default_binary_paths'][self.server['type']],
             'pg_restore')
-        retVal = is_utility_exists(binary_path)
+
+        if os.name == 'nt':
+            binary_path = binary_path + '.exe'
+
+        retVal = does_utility_exist(binary_path)
         if retVal is not None:
             self.skipTest(retVal)
 
