@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -10,16 +10,20 @@
 """ Implemented classes for the different object type used by data grid """
 
 from abc import ABCMeta, abstractmethod
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
+from collections import OrderedDict
 import six
 from flask import render_template
 from flask_babelex import gettext
+from werkzeug.exceptions import InternalServerError
 from pgadmin.utils.ajax import forbidden
 from pgadmin.utils.driver import get_driver
-
+from pgadmin.tools.sqleditor.utils.is_query_resultset_updatable \
+    import is_query_resultset_updatable
+from pgadmin.tools.sqleditor.utils.save_changed_data import save_changed_data
+from pgadmin.tools.sqleditor.utils.get_column_types import get_columns_types
+from pgadmin.utils.preferences import Preferences
+from pgadmin.utils.exception import ObjectGone, ExecuteError
+from pgadmin.utils.constants import SERVER_CONNECTION_CLOSED
 from config import PG_DEFAULT_DRIVER
 
 VIEW_FIRST_100_ROWS = 1
@@ -42,15 +46,15 @@ class ObjectRegistry(ABCMeta):
 
     registry = dict()
 
-    def __init__(cls, name, bases, d):
+    def __init__(self, name, bases, d):
         """
         This method is used to register the objects based on object type.
         """
 
         if d and 'object_type' in d:
-            ObjectRegistry.registry[d['object_type']] = cls
+            ObjectRegistry.registry[d['object_type']] = self
 
-        ABCMeta.__init__(cls, name, bases, d)
+        ABCMeta.__init__(self, name, bases, d)
 
     @classmethod
     def get_object(cls, name, **kwargs):
@@ -164,9 +168,10 @@ class SQLFilter(object):
         self.sid = kwargs['sid']
         self.did = kwargs['did']
         self.obj_id = kwargs['obj_id']
-        self.__row_filter = kwargs.get('sql_filter', None)
-        self.__dara_sorting = kwargs.get('data_sorting', None)
-        self.__set_sorting_from_filter_dialog = False
+        sql_filter = kwargs.get('sql_filter', None)
+        self._row_filter = sql_filter if isinstance(sql_filter, str) else None
+        self._data_sorting = kwargs.get('data_sorting', None)
+        self._set_sorting_from_filter_dialog = False
 
         manager = get_driver(PG_DEFAULT_DRIVER).connection_manager(self.sid)
         conn = manager.connection(did=self.did)
@@ -183,21 +188,21 @@ class SQLFilter(object):
 
             status, result = conn.execute_dict(query)
             if not status:
-                raise Exception(result)
+                raise ExecuteError(result)
+            if len(result['rows']) == 0:
+                raise ObjectGone(
+                    gettext("The specified object could not be found."))
 
             self.nsp_name = result['rows'][0]['nspname']
             self.object_name = result['rows'][0]['relname']
         else:
-            raise Exception(gettext(
-                'Not connected to server or connection with the server '
-                'has been closed.')
-            )
+            raise InternalServerError(SERVER_CONNECTION_CLOSED)
 
     def get_filter(self):
         """
         This function returns the filter.
         """
-        return self.__row_filter
+        return self._row_filter
 
     def set_filter(self, row_filter):
         """
@@ -207,11 +212,13 @@ class SQLFilter(object):
         Args:
             row_filter: sql query
         """
+        if not isinstance(row_filter, str):
+            row_filter = None
 
         status, msg = self.validate_filter(row_filter)
 
         if status:
-            self.__row_filter = row_filter
+            self._row_filter = row_filter
 
         return status, msg
 
@@ -219,8 +226,8 @@ class SQLFilter(object):
         """
         This function returns the filter.
         """
-        if self.__dara_sorting and len(self.__dara_sorting) > 0:
-            return self.__dara_sorting
+        if self._data_sorting and len(self._data_sorting) > 0:
+            return self._data_sorting
         return None
 
     def set_data_sorting(self, data_filter, set_from_filter_dialog=False):
@@ -228,24 +235,24 @@ class SQLFilter(object):
         This function validates the filter and set the
         given filter to member variable.
         """
-        self.__dara_sorting = data_filter['data_sorting']
-        self.__set_sorting_from_filter_dialog = set_from_filter_dialog
+        self._data_sorting = data_filter['data_sorting']
+        self._set_sorting_from_filter_dialog = set_from_filter_dialog
 
     def is_sorting_set_from_filter_dialog(self):
         """This function return whether sorting is set from filter dialog"""
-        return self.__set_sorting_from_filter_dialog
+        return self._set_sorting_from_filter_dialog
 
     def is_filter_applied(self):
         """
         This function returns True if filter is applied else False.
         """
         is_filter_applied = True
-        if self.__row_filter is None or self.__row_filter == '':
+        if self._row_filter is None or self._row_filter == '':
             is_filter_applied = False
 
-        if not is_filter_applied:
-            if self.__dara_sorting and len(self.__dara_sorting) > 0:
-                is_filter_applied = True
+        if not is_filter_applied and \
+                self._data_sorting and len(self._data_sorting) > 0:
+            is_filter_applied = True
 
         return is_filter_applied
 
@@ -253,8 +260,8 @@ class SQLFilter(object):
         """
         This function remove the filter by setting value to None.
         """
-        self.__row_filter = None
-        self.__dara_sorting = None
+        self._row_filter = None
+        self._data_sorting = None
 
     def append_filter(self, row_filter):
         """
@@ -268,9 +275,9 @@ class SQLFilter(object):
         existing_filter = self.get_filter()
 
         if existing_filter is None or existing_filter == '':
-            self.__row_filter = row_filter
+            self._row_filter = row_filter
         else:
-            self.__row_filter = existing_filter + ' \n    AND ' + row_filter
+            self._row_filter = existing_filter + ' \n    AND ' + row_filter
 
     def validate_filter(self, row_filter):
         """
@@ -354,6 +361,7 @@ class GridCommand(BaseCommand, SQLFilter, FetchedRowTracker):
         self.conn_id = kwargs['conn_id'] if 'conn_id' in kwargs else None
         self.cmd_type = kwargs['cmd_type'] if 'cmd_type' in kwargs else None
         self.limit = -1
+        self._OBJECT_QUERY_SQL = 'objectquery.sql'
 
         if self.cmd_type in (VIEW_FIRST_100_ROWS, VIEW_LAST_100_ROWS):
             self.limit = 100
@@ -392,15 +400,13 @@ class GridCommand(BaseCommand, SQLFilter, FetchedRowTracker):
             )
             status, result = conn.execute_dict(query)
             if not status:
-                raise Exception(result)
+                raise ExecuteError(result)
 
             for row in result['rows']:
                 all_columns.append(row['attname'])
         else:
-            raise Exception(
-                gettext('Not connected to server or connection with the '
-                        'server has been closed.')
-            )
+            raise InternalServerError(SERVER_CONNECTION_CLOSED)
+
         # If user has custom data sorting then pass as it as it is
         if data_sorting and len(data_sorting) > 0:
             all_sorted_columns = data_sorting
@@ -456,6 +462,11 @@ class TableCommand(GridCommand):
         # call base class init to fetch the table name
         super(TableCommand, self).__init__(**kwargs)
 
+        # Set the default sorting on table data by primary key if user
+        # preference value is set
+        self.data_sorting_by_pk = Preferences.module('sqleditor').preference(
+            'table_view_data_by_pk').get()
+
     def get_sql(self, default_conn=None):
         """
         This method is used to create a proper SQL query
@@ -479,7 +490,7 @@ class TableCommand(GridCommand):
         if data_sorting is None and \
             not self.is_sorting_set_from_filter_dialog() \
             and (self.cmd_type in (VIEW_FIRST_100_ROWS, VIEW_LAST_100_ROWS) or
-                 (self.cmd_type == VIEW_ALL_ROWS and self.limit > 0)):
+                 (self.cmd_type == VIEW_ALL_ROWS and self.data_sorting_by_pk)):
             sorting = {'data_sorting': []}
             for pk in primary_keys:
                 sorting['data_sorting'].append(
@@ -489,14 +500,14 @@ class TableCommand(GridCommand):
 
         if sql_filter is None:
             sql = render_template(
-                "/".join([self.sql_path, 'objectquery.sql']),
+                "/".join([self.sql_path, self._OBJECT_QUERY_SQL]),
                 object_name=self.object_name,
                 nsp_name=self.nsp_name, limit=self.limit, has_oids=has_oids,
                 data_sorting=data_sorting
             )
         else:
             sql = render_template(
-                "/".join([self.sql_path, 'objectquery.sql']),
+                "/".join([self.sql_path, self._OBJECT_QUERY_SQL]),
                 object_name=self.object_name,
                 nsp_name=self.nsp_name, limit=self.limit, has_oids=has_oids,
                 sql_filter=sql_filter, data_sorting=data_sorting
@@ -528,7 +539,7 @@ class TableCommand(GridCommand):
 
             status, result = conn.execute_dict(query)
             if not status:
-                raise Exception(result)
+                raise ExecuteError(result)
 
             for row in result['rows']:
                 pk_names += driver.qtIdent(conn, row['attname']) + ','
@@ -538,10 +549,7 @@ class TableCommand(GridCommand):
                 # Remove last character from the string
                 pk_names = pk_names[:-1]
         else:
-            raise Exception(
-                gettext('Not connected to server or connection with the '
-                        'server has been closed.')
-            )
+            raise InternalServerError(SERVER_CONNECTION_CLOSED)
 
         return pk_names, primary_keys
 
@@ -576,7 +584,7 @@ class TableCommand(GridCommand):
         status, result = conn.execute_dict(query)
 
         if not status:
-            raise Exception(result)
+            raise ExecuteError(result)
 
         for row in result['rows']:
             all_columns.append(row['attname'])
@@ -588,7 +596,7 @@ class TableCommand(GridCommand):
         )
         status, result = conn.execute_dict(query)
         if not status:
-            raise Exception(result)
+            raise ExecuteError(result)
 
         for row in result['rows']:
             # Only append if not already present in the list
@@ -612,8 +620,14 @@ class TableCommand(GridCommand):
         This function checks whether the table has oids or not.
         """
         driver = get_driver(PG_DEFAULT_DRIVER)
+        manager = driver.connection_manager(self.sid)
+
+        # Remove the special behavior of OID columns from
+        # PostgreSQL 12 onwards, so returning False.
+        if manager.sversion >= 120000:
+            return False
+
         if default_conn is None:
-            manager = driver.connection_manager(self.sid)
             conn = manager.connection(did=self.did, conn_id=self.conn_id)
         else:
             conn = default_conn
@@ -626,13 +640,10 @@ class TableCommand(GridCommand):
 
             status, has_oids = conn.execute_scalar(query)
             if not status:
-                raise Exception(has_oids)
+                raise ExecuteError(has_oids)
 
         else:
-            raise Exception(
-                gettext('Not connected to server or connection with the '
-                        'server has been closed.')
-            )
+            raise InternalServerError(SERVER_CONNECTION_CLOSED)
 
         return has_oids
 
@@ -659,244 +670,21 @@ class TableCommand(GridCommand):
         else:
             conn = default_conn
 
-        status = False
-        res = None
-        query_res = dict()
-        count = 0
-        list_of_rowid = []
-        operations = ('added', 'updated', 'deleted')
-        list_of_sql = {}
-        _rowid = None
+        return save_changed_data(changed_data=changed_data,
+                                 columns_info=columns_info,
+                                 command_obj=self,
+                                 client_primary_key=client_primary_key,
+                                 conn=conn)
 
-        if conn.connected():
-
-            # Start the transaction
-            conn.execute_void('BEGIN;')
-
-            # Iterate total number of records to be updated/inserted
-            for of_type in changed_data:
-                # No need to go further if its not add/update/delete operation
-                if of_type not in operations:
-                    continue
-                # if no data to be save then continue
-                if len(changed_data[of_type]) < 1:
-                    continue
-
-                column_type = {}
-                column_data = {}
-                for each_col in columns_info:
-                    if (
-                        columns_info[each_col]['not_null'] and
-                        not columns_info[each_col]['has_default_val']
-                    ):
-                        column_data[each_col] = None
-                        column_type[each_col] =\
-                            columns_info[each_col]['type_name']
-                    else:
-                        column_type[each_col] = \
-                            columns_info[each_col]['type_name']
-
-                # For newly added rows
-                if of_type == 'added':
-                    # Python dict does not honour the inserted item order
-                    # So to insert data in the order, we need to make ordered
-                    # list of added index We don't need this mechanism in
-                    # updated/deleted rows as it does not matter in
-                    # those operations
-                    added_index = OrderedDict(
-                        sorted(
-                            changed_data['added_index'].items(),
-                            key=lambda x: int(x[0])
-                        )
-                    )
-                    list_of_sql[of_type] = []
-
-                    # When new rows are added, only changed columns data is
-                    # sent from client side. But if column is not_null and has
-                    # no_default_value, set column to blank, instead
-                    # of not null which is set by default.
-                    column_data = {}
-                    pk_names, primary_keys = self.get_primary_keys()
-                    has_oids = 'oid' in column_type
-
-                    for each_row in added_index:
-                        # Get the row index to match with the added rows
-                        # dict key
-                        tmp_row_index = added_index[each_row]
-                        data = changed_data[of_type][tmp_row_index]['data']
-                        # Remove our unique tracking key
-                        data.pop(client_primary_key, None)
-                        data.pop('is_row_copied', None)
-                        list_of_rowid.append(data.get(client_primary_key))
-
-                        # Update columns value with columns having
-                        # not_null=False and has no default value
-                        column_data.update(data)
-
-                        sql = render_template(
-                            "/".join([self.sql_path, 'insert.sql']),
-                            data_to_be_saved=column_data,
-                            primary_keys=None,
-                            object_name=self.object_name,
-                            nsp_name=self.nsp_name,
-                            data_type=column_type,
-                            pk_names=pk_names,
-                            has_oids=has_oids
-                        )
-
-                        select_sql = render_template(
-                            "/".join([self.sql_path, 'select.sql']),
-                            object_name=self.object_name,
-                            nsp_name=self.nsp_name,
-                            primary_keys=primary_keys,
-                            has_oids=has_oids
-                        )
-
-                        list_of_sql[of_type].append({
-                            'sql': sql, 'data': data,
-                            'client_row': tmp_row_index,
-                            'select_sql': select_sql
-                        })
-                        # Reset column data
-                        column_data = {}
-
-                # For updated rows
-                elif of_type == 'updated':
-                    list_of_sql[of_type] = []
-                    for each_row in changed_data[of_type]:
-                        data = changed_data[of_type][each_row]['data']
-                        pk = changed_data[of_type][each_row]['primary_keys']
-                        sql = render_template(
-                            "/".join([self.sql_path, 'update.sql']),
-                            data_to_be_saved=data,
-                            primary_keys=pk,
-                            object_name=self.object_name,
-                            nsp_name=self.nsp_name,
-                            data_type=column_type
-                        )
-                        list_of_sql[of_type].append({'sql': sql, 'data': data})
-                        list_of_rowid.append(data.get(client_primary_key))
-
-                # For deleted rows
-                elif of_type == 'deleted':
-                    list_of_sql[of_type] = []
-                    is_first = True
-                    rows_to_delete = []
-                    keys = None
-                    no_of_keys = None
-                    for each_row in changed_data[of_type]:
-                        rows_to_delete.append(changed_data[of_type][each_row])
-                        # Fetch the keys for SQL generation
-                        if is_first:
-                            # We need to covert dict_keys to normal list in
-                            # Python3
-                            # In Python2, it's already a list & We will also
-                            # fetch column names using index
-                            keys = list(
-                                changed_data[of_type][each_row].keys()
-                            )
-                            no_of_keys = len(keys)
-                            is_first = False
-                    # Map index with column name for each row
-                    for row in rows_to_delete:
-                        for k, v in row.items():
-                            # Set primary key with label & delete index based
-                            # mapped key
-                            try:
-                                row[changed_data['columns']
-                                    [int(k)]['name']] = v
-                            except ValueError:
-                                continue
-                            del row[k]
-
-                    sql = render_template(
-                        "/".join([self.sql_path, 'delete.sql']),
-                        data=rows_to_delete,
-                        primary_key_labels=keys,
-                        no_of_keys=no_of_keys,
-                        object_name=self.object_name,
-                        nsp_name=self.nsp_name
-                    )
-                    list_of_sql[of_type].append({'sql': sql, 'data': {}})
-
-            for opr, sqls in list_of_sql.items():
-                for item in sqls:
-                    if item['sql']:
-                        row_added = None
-
-                        # Fetch oids/primary keys
-                        if 'select_sql' in item and item['select_sql']:
-                            status, res = conn.execute_dict(
-                                item['sql'], item['data'])
-                        else:
-                            status, res = conn.execute_void(
-                                item['sql'], item['data'])
-
-                        if not status:
-                            conn.execute_void('ROLLBACK;')
-                            # If we roll backed every thing then update the
-                            # message for each sql query.
-                            for val in query_res:
-                                if query_res[val]['status']:
-                                    query_res[val]['result'] = \
-                                        'Transaction ROLLBACK'
-
-                            # If list is empty set rowid to 1
-                            try:
-                                if list_of_rowid:
-                                    _rowid = list_of_rowid[count]
-                                else:
-                                    _rowid = 1
-                            except Exception:
-                                _rowid = 0
-
-                            return status, res, query_res, _rowid
-
-                        # Select added row from the table
-                        if 'select_sql' in item:
-                            status, sel_res = conn.execute_dict(
-                                item['select_sql'], res['rows'][0])
-
-                            if not status:
-                                conn.execute_void('ROLLBACK;')
-                                # If we roll backed every thing then update
-                                # the message for each sql query.
-                                for val in query_res:
-                                    if query_res[val]['status']:
-                                        query_res[val]['result'] = \
-                                            'Transaction ROLLBACK'
-
-                                # If list is empty set rowid to 1
-                                try:
-                                    if list_of_rowid:
-                                        _rowid = list_of_rowid[count]
-                                    else:
-                                        _rowid = 1
-                                except Exception:
-                                    _rowid = 0
-
-                                return status, sel_res, query_res, _rowid
-
-                            if 'rows' in sel_res and len(sel_res['rows']) > 0:
-                                row_added = {
-                                    item['client_row']: sel_res['rows'][0]}
-
-                        rows_affected = conn.rows_affected()
-
-                        # store the result of each query in dictionary
-                        query_res[count] = {
-                            'status': status,
-                            'result': None if row_added else res,
-                            'sql': sql, 'rows_affected': rows_affected,
-                            'row_added': row_added
-                        }
-
-                        count += 1
-
-            # Commit the transaction if there is no error found
-            conn.execute_void('COMMIT;')
-
-        return status, res, query_res, _rowid
+    def get_columns_types(self, conn):
+        columns_info = conn.get_column_info()
+        has_oids = self.has_oids()
+        table_oid = self.obj_id
+        return get_columns_types(conn=conn,
+                                 columns_info=columns_info,
+                                 has_oids=has_oids,
+                                 table_oid=table_oid,
+                                 is_query_tool=False)
 
 
 class ViewCommand(GridCommand):
@@ -929,13 +717,13 @@ class ViewCommand(GridCommand):
 
         if sql_filter is None:
             sql = render_template(
-                "/".join([self.sql_path, 'objectquery.sql']),
+                "/".join([self.sql_path, self._OBJECT_QUERY_SQL]),
                 object_name=self.object_name, nsp_name=self.nsp_name,
                 limit=self.limit, data_sorting=data_sorting
             )
         else:
             sql = render_template(
-                "/".join([self.sql_path, 'objectquery.sql']),
+                "/".join([self.sql_path, self._OBJECT_QUERY_SQL]),
                 object_name=self.object_name, nsp_name=self.nsp_name,
                 sql_filter=sql_filter, limit=self.limit,
                 data_sorting=data_sorting
@@ -990,13 +778,13 @@ class ForeignTableCommand(GridCommand):
 
         if sql_filter is None:
             sql = render_template(
-                "/".join([self.sql_path, 'objectquery.sql']),
+                "/".join([self.sql_path, self._OBJECT_QUERY_SQL]),
                 object_name=self.object_name, nsp_name=self.nsp_name,
                 limit=self.limit, data_sorting=data_sorting
             )
         else:
             sql = render_template(
-                "/".join([self.sql_path, 'objectquery.sql']),
+                "/".join([self.sql_path, self._OBJECT_QUERY_SQL]),
                 object_name=self.object_name, nsp_name=self.nsp_name,
                 sql_filter=sql_filter, limit=self.limit,
                 data_sorting=data_sorting
@@ -1041,13 +829,13 @@ class CatalogCommand(GridCommand):
 
         if sql_filter is None:
             sql = render_template(
-                "/".join([self.sql_path, 'objectquery.sql']),
+                "/".join([self.sql_path, self._OBJECT_QUERY_SQL]),
                 object_name=self.object_name, nsp_name=self.nsp_name,
                 limit=self.limit, data_sorting=data_sorting
             )
         else:
             sql = render_template(
-                "/".join([self.sql_path, 'objectquery.sql']),
+                "/".join([self.sql_path, self._OBJECT_QUERY_SQL]),
                 object_name=self.object_name, nsp_name=self.nsp_name,
                 sql_filter=sql_filter, limit=self.limit,
                 data_sorting=data_sorting
@@ -1080,18 +868,101 @@ class QueryToolCommand(BaseCommand, FetchedRowTracker):
         self.auto_rollback = False
         self.auto_commit = True
 
+        # Attributes needed to be able to edit updatable resultsets
+        self.is_updatable_resultset = False
+        self.primary_keys = None
+        self.pk_names = None
+        self.table_has_oids = False
+        self.columns_types = None
+
     def get_sql(self, default_conn=None):
         return None
 
     def get_all_columns_with_order(self, default_conn=None):
         return None
 
+    def get_primary_keys(self):
+        return self.pk_names, self.primary_keys
+
+    def get_columns_types(self, conn=None):
+        return self.columns_types
+
+    def has_oids(self):
+        return self.table_has_oids
+
     def can_edit(self):
-        return False
+        return self.is_updatable_resultset
 
     def can_filter(self):
         return False
 
+    def check_updatable_results_pkeys_oids(self):
+        """
+            This function is used to check whether the last successful query
+            produced updatable results and sets the necessary flags and
+            attributes accordingly.
+            Should be called after polling for the results is successful
+            (results are ready)
+        """
+        # Fetch the connection object
+        driver = get_driver(PG_DEFAULT_DRIVER)
+        manager = driver.connection_manager(self.sid)
+        conn = manager.connection(did=self.did, conn_id=self.conn_id)
+
+        # Get the driver version as a float
+        driver_version = float('.'.join(driver.version().split('.')[:2]))
+
+        # Checking for updatable resultsets uses features in psycopg 2.8
+        if driver_version < 2.8:
+            return False
+
+        # Get the path to the sql templates
+        sql_path = 'sqleditor/sql/#{0}#'.format(manager.version)
+
+        self.is_updatable_resultset, self.table_has_oids,\
+            self.primary_keys, pk_names, table_oid,\
+            self.columns_types = is_query_resultset_updatable(conn, sql_path)
+
+        # Create pk_names attribute in the required format
+        if pk_names is not None:
+            self.pk_names = ''
+
+            for pk_name in pk_names:
+                self.pk_names += driver.qtIdent(conn, pk_name) + ','
+
+            if self.pk_names != '':
+                # Remove last character from the string
+                self.pk_names = self.pk_names[:-1]
+
+        # Add attributes required to be able to update table data
+        if self.is_updatable_resultset:
+            self.__set_updatable_results_attrs(sql_path=sql_path,
+                                               table_oid=table_oid,
+                                               conn=conn)
+        return self.is_updatable_resultset
+
+    def save(self,
+             changed_data,
+             columns_info,
+             client_primary_key='__temp_PK',
+             default_conn=None):
+        if not self.is_updatable_resultset:
+            return False, gettext('Resultset is not updatable.'), None, None
+        else:
+            driver = get_driver(PG_DEFAULT_DRIVER)
+            if default_conn is None:
+                manager = driver.connection_manager(self.sid)
+                conn = manager.connection(did=self.did, conn_id=self.conn_id)
+            else:
+                conn = default_conn
+
+            return save_changed_data(changed_data=changed_data,
+                                     columns_info=columns_info,
+                                     conn=conn,
+                                     command_obj=self,
+                                     client_primary_key=client_primary_key,
+                                     auto_commit=self.auto_commit)
+
     def set_connection_id(self, conn_id):
         self.conn_id = conn_id
 
@@ -1100,3 +971,25 @@ class QueryToolCommand(BaseCommand, FetchedRowTracker):
 
     def set_auto_commit(self, auto_commit):
         self.auto_commit = auto_commit
+
+    def __set_updatable_results_attrs(self, sql_path,
+                                      table_oid, conn):
+        # Set template path for sql scripts and the table object id
+        self.sql_path = sql_path
+        self.obj_id = table_oid
+
+        if conn.connected():
+            # Fetch the Namespace Name and object Name
+            query = render_template(
+                "/".join([self.sql_path, 'objectname.sql']),
+                obj_id=self.obj_id
+            )
+
+            status, result = conn.execute_dict(query)
+            if not status:
+                raise ExecuteError(result)
+
+            self.nsp_name = result['rows'][0]['nspname']
+            self.object_name = result['rows'][0]['relname']
+        else:
+            raise InternalServerError(SERVER_CONNECTION_CLOSED)
