@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -14,6 +14,8 @@ from operator import attrgetter
 
 from flask import Blueprint, current_app
 from flask_babelex import gettext
+from flask_security import current_user, login_required
+from threading import Lock
 
 from .paths import get_storage_directory
 from .preferences import Preferences
@@ -50,6 +52,7 @@ class PgAdminModule(Blueprint):
         self.before_app_first_request(create_module_preference)
 
     def register_preferences(self):
+        # To be implemented by child classes
         pass
 
     def register(self, app, options, first_registration=False):
@@ -66,6 +69,7 @@ class PgAdminModule(Blueprint):
             if first_registration:
                 module.parentmodules.append(self)
             app.register_blueprint(module)
+            app.register_logout_hook(module)
 
     def get_own_stylesheets(self):
         """
@@ -155,7 +159,6 @@ class PgAdminModule(Blueprint):
         return res
 
 
-IS_PY2 = (sys.version_info[0] == 2)
 IS_WIN = (os.name == 'nt')
 
 sys_encoding = sys.getdefaultencoding()
@@ -171,17 +174,11 @@ if not fs_encoding or fs_encoding == 'ascii':
     fs_encoding = 'utf-8'
 
 
-def u(_s, _encoding=sys_encoding):
-    if IS_PY2:
-        if isinstance(_s, str):
-            return unicode(_s, _encoding)
+def u_encode(_s, _encoding=sys_encoding):
     return _s
 
 
 def file_quote(_p):
-    if IS_PY2:
-        if isinstance(_p, unicode):
-            return _p.encode(fs_encoding)
     return _p
 
 
@@ -189,25 +186,10 @@ if IS_WIN:
     import ctypes
     from ctypes import wintypes
 
-    if IS_PY2:
-        def env(name):
-            if IS_PY2:
-                # Make sure string argument is unicode
-                name = unicode(name)
-            n = ctypes.windll.kernel32.GetEnvironmentVariableW(name, None, 0)
-
-            if n == 0:
-                return None
-
-            buf = ctypes.create_unicode_buffer(u'\0' * n)
-            ctypes.windll.kernel32.GetEnvironmentVariableW(name, buf, n)
-
-            return buf.value
-    else:
-        def env(name):
-            if name in os.environ:
-                return os.environ[name]
-            return None
+    def env(name):
+        if name in os.environ:
+            return os.environ[name]
+        return None
 
     _GetShortPathNameW = ctypes.windll.kernel32.GetShortPathNameW
     _GetShortPathNameW.argtypes = [
@@ -253,7 +235,7 @@ else:
         return _path
 
     def document_dir():
-        return os.path.realpath(os.path.expanduser(u'~/'))
+        return os.path.realpath(os.path.expanduser('~/'))
 
 
 def get_complete_file_path(file):
@@ -275,7 +257,7 @@ def get_complete_file_path(file):
     if storage_dir:
         file = os.path.join(
             storage_dir,
-            file.lstrip(u'/').lstrip(u'\\')
+            file.lstrip('/').lstrip('\\')
         )
         if IS_WIN:
             file = file.replace('\\', '/')
@@ -284,15 +266,15 @@ def get_complete_file_path(file):
     return file if os.path.isfile(file) else None
 
 
-def is_utility_exists(file):
+def does_utility_exist(file):
     """
     This function will check the utility file exists on given path.
     :return:
     """
     error_msg = None
     if not os.path.exists(file):
-        error_msg = gettext(u"'%s' file not found. Please correct the Binary"
-                            u" Path in the Preferences dialog" % file)
+        error_msg = gettext("'%s' file not found. Please correct the Binary"
+                            " Path in the Preferences dialog" % file)
     return error_msg
 
 
@@ -329,3 +311,46 @@ SHORTCUT_FIELDS = [
         'label': gettext('Alt/Option')
     }
 ]
+
+
+class KeyManager:
+    def __init__(self):
+        self.users = dict()
+        self.lock = Lock()
+
+    @login_required
+    def get(self):
+        user = self.users.get(current_user.id, None)
+        if user is not None:
+            return user.get('key', None)
+
+    @login_required
+    def set(self, _key, _new_login=True):
+        with self.lock:
+            user = self.users.get(current_user.id, None)
+            if user is None:
+                self.users[current_user.id] = dict(
+                    session_count=1, key=_key)
+            else:
+                if _new_login:
+                    user['session_count'] += 1
+                user['key'] = _key
+
+    @login_required
+    def reset(self):
+        with self.lock:
+            user = self.users.get(current_user.id, None)
+
+            if user is not None:
+                # This will not decrement if session expired
+                user['session_count'] -= 1
+                if user['session_count'] == 0:
+                    del self.users[current_user.id]
+
+    @login_required
+    def hard_reset(self):
+        with self.lock:
+            user = self.users.get(current_user.id, None)
+
+            if user is not None:
+                del self.users[current_user.id]
