@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -14,18 +14,23 @@ object.
 
 """
 import datetime
+import re
 from flask import session
-from flask_babelex import gettext
+from flask_login import current_user
+from werkzeug.exceptions import InternalServerError
 import psycopg2
 from psycopg2.extensions import adapt
+from threading import Lock
 
 import config
-from pgadmin.model import Server, User
-from .keywords import ScanKeyword
+from pgadmin.model import Server
+from .keywords import scan_keyword
 from ..abstract import BaseDriver
 from .connection import Connection
 from .server_manager import ServerManager
 
+connection_restore_lock = Lock()
+
 
 class Driver(BaseDriver):
     """
@@ -60,6 +65,28 @@ class Driver(BaseDriver):
 
         super(Driver, self).__init__()
 
+    def _restore_connections_from_session(self):
+        """
+        Used internally by connection_manager to restore connections
+        from sessions.
+        """
+        if session.sid not in self.managers:
+            self.managers[session.sid] = managers = dict()
+            if '__pgsql_server_managers' in session:
+                session_managers = \
+                    session['__pgsql_server_managers'].copy()
+                for server in \
+                    Server.query.filter_by(
+                        user_id=current_user.id):
+                    manager = managers[str(server.id)] = \
+                        ServerManager(server)
+                    if server.id in session_managers:
+                        manager._restore(session_managers[server.id])
+                        manager.update_session()
+            return managers
+
+        return {}
+
     def connection_manager(self, sid=None):
         """
         connection_manager(...)
@@ -74,23 +101,22 @@ class Driver(BaseDriver):
         assert (sid is not None and isinstance(sid, int))
         managers = None
 
+        server_data = Server.query.filter_by(id=sid).first()
+        if server_data is None:
+            return None
+
         if session.sid not in self.managers:
-            self.managers[session.sid] = managers = dict()
-            if '__pgsql_server_managers' in session:
-                session_managers = session['__pgsql_server_managers'].copy()
-                session['__pgsql_server_managers'] = dict()
-
-                for server_id in session_managers:
-                    s = Server.query.filter_by(id=server_id).first()
-
-                    if not s:
-                        continue
-
-                    manager = managers[str(server_id)] = ServerManager(s)
-                    manager._restore(session_managers[server_id])
-                    manager.update_session()
+            with connection_restore_lock:
+                # The wait is over but the object might have been loaded
+                # by some other thread check again
+                managers = self._restore_connections_from_session()
         else:
             managers = self.managers[session.sid]
+            if str(sid) in managers:
+                manager = managers[str(sid)]
+                with connection_restore_lock:
+                    manager._restore_connections()
+                    manager.update_session()
 
         managers['pinged'] = datetime.datetime.now()
         if str(sid) not in managers:
@@ -105,22 +131,22 @@ class Driver(BaseDriver):
 
         return managers[str(sid)]
 
-    def Version(cls):
+    def version(self):
         """
-        Version(...)
+        version(...)
 
         Returns the current version of psycopg2 driver
         """
-        version = getattr(psycopg2, '__version__', None)
+        _version = getattr(psycopg2, '__version__', None)
 
-        if version:
-            return version
+        if _version:
+            return _version
 
-        raise Exception(
+        raise InternalServerError(
             "Driver Version information for psycopg2 is not available!"
         )
 
-    def libpq_version(cls):
+    def libpq_version(self):
         """
         Returns the loaded libpq version
         """
@@ -128,7 +154,7 @@ class Driver(BaseDriver):
         if version:
             return version
 
-        raise Exception(
+        raise InternalServerError(
             "libpq version information is not available!"
         )
 
@@ -182,7 +208,7 @@ class Driver(BaseDriver):
                 str(sid) in self.managers[session.sid]:
             del self.managers[session.sid][str(sid)]
 
-    def gc(self):
+    def gc_timeout(self):
         """
         Release the connections for the sessions, which have not pinged the
         server for more than config.MAX_SESSION_IDLE_TIME.
@@ -224,7 +250,7 @@ class Driver(BaseDriver):
                 mgr.release()
 
     @staticmethod
-    def qtLiteral(value):
+    def qtLiteral(value, force_quote=False):
         adapted = adapt(value)
 
         # Not all adapted objects have encoding
@@ -238,7 +264,14 @@ class Driver(BaseDriver):
         res = adapted.getquoted()
 
         if isinstance(res, bytes):
-            return res.decode('utf-8')
+            res = res.decode('utf-8')
+
+        if force_quote is True:
+            # Convert the input to the string to use the startsWith(...)
+            res = str(res)
+            if not res.startswith("'"):
+                return "'" + res + "'"
+
         return res
 
     @staticmethod
@@ -247,7 +280,7 @@ class Driver(BaseDriver):
         # COL_NAME_KEYWORD        1
         # TYPE_FUNC_NAME_KEYWORD  2
         # RESERVED_KEYWORD        3
-        extraKeywords = {
+        extra_keywords = {
             'connect': 3,
             'convert': 3,
             'distributed': 0,
@@ -271,48 +304,45 @@ class Driver(BaseDriver):
             'varchar2': 3
         }
 
-        return extraKeywords.get(key, None) or ScanKeyword(key)
+        return extra_keywords.get(key, None) or scan_keyword(key)
 
     @staticmethod
-    def needsQuoting(key, forTypes):
+    def needsQuoting(key, for_types):
         value = key
-        valNoArray = value
+        val_noarray = value
 
         # check if the string is number or not
         if isinstance(value, int):
             return True
         # certain types should not be quoted even though it contains a space.
         # Evilness.
-        elif forTypes and value[-2:] == u"[]":
-            valNoArray = value[:-2]
+        elif for_types and value[-2:] == "[]":
+            val_noarray = value[:-2]
 
-        if forTypes and valNoArray.lower() in [
-            u'bit varying',
-            u'"char"',
-            u'character varying',
-            u'double precision',
-            u'timestamp without time zone',
-            u'timestamp with time zone',
-            u'time without time zone',
-            u'time with time zone',
-            u'"trigger"',
-            u'"unknown"'
+        if for_types and val_noarray.lower() in [
+            'bit varying',
+            '"char"',
+            'character varying',
+            'double precision',
+            'timestamp without time zone',
+            'timestamp with time zone',
+            'time without time zone',
+            'time with time zone',
+            '"trigger"',
+            '"unknown"'
         ]:
             return False
 
         # If already quoted?, If yes then do not quote again
-        if forTypes and valNoArray:
-            if valNoArray.startswith('"') \
-                    or valNoArray.endswith('"'):
-                return False
+        if for_types and val_noarray and \
+                (val_noarray.startswith('"') or val_noarray.endswith('"')):
+            return False
 
-        if u'0' <= valNoArray[0] <= u'9':
+        if '0' <= val_noarray[0] <= '9':
             return True
 
-        for c in valNoArray:
-            if (not (u'a' <= c <= u'z') and c != u'_' and
-                    not (u'0' <= c <= u'9')):
-                return True
+        if re.search('[^a-z_0-9]+', val_noarray):
+            return True
 
         # check string is keywaord or not
         category = Driver.ScanKeywordExtraLookup(value)
@@ -325,7 +355,7 @@ class Driver(BaseDriver):
             return False
 
         # COL_NAME_KEYWORD
-        if forTypes and category == 1:
+        if for_types and category == 1:
             return False
 
         return True
@@ -339,18 +369,15 @@ class Driver(BaseDriver):
         value = None
 
         for val in args:
+            # DataType doesn't have len function then convert it to string
+            if not hasattr(val, '__len__'):
+                val = str(val)
+
             if len(val) == 0:
                 continue
-            if hasattr(str, 'decode') and not isinstance(val, unicode):
-                # Handling for python2
-                try:
-                    val = str(val).encode('utf-8')
-                except UnicodeDecodeError:
-                    # If already unicode, most likely coming from db
-                    val = str(val).decode('utf-8')
             value = val
 
-            if (Driver.needsQuoting(val, True)):
+            if Driver.needsQuoting(val, True):
                 value = value.replace("\"", "\"\"")
                 value = "\"" + value + "\""
 
@@ -366,22 +393,19 @@ class Driver(BaseDriver):
         value = None
 
         for val in args:
-            if type(val) == list:
+            if isinstance(val, list):
                 return map(lambda w: Driver.qtIdent(conn, w), val)
-            if hasattr(str, 'decode') and not isinstance(val, unicode):
-                # Handling for python2
-                try:
-                    val = str(val).encode('utf-8')
-                except UnicodeDecodeError:
-                    # If already unicode, most likely coming from db
-                    val = str(val).decode('utf-8')
+
+            # DataType doesn't have len function then convert it to string
+            if not hasattr(val, '__len__'):
+                val = str(val)
 
             if len(val) == 0:
                 continue
 
             value = val
 
-            if (Driver.needsQuoting(val, False)):
+            if Driver.needsQuoting(val, False):
                 value = value.replace("\"", "\"\"")
                 value = "\"" + value + "\""
 
