@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2019, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -13,15 +13,13 @@ fetching results from it, and also takes care of the duplicate column name in
 result.
 """
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
 
+from collections import OrderedDict
+import psycopg2
 from psycopg2.extensions import cursor as _cursor, encodings
-from .encoding import configureDriverEncodings
+from .encoding import configure_driver_encodings
 
-configureDriverEncodings(encodings)
+configure_driver_encodings(encodings)
 
 
 class _WrapperColumn(object):
@@ -91,7 +89,25 @@ class _WrapperColumn(object):
         Generates an OrderedDict from the fields of the original objects
         with avoiding the duplicate name.
         """
-        ores = OrderedDict(self.orig_col._asdict())
+
+        # In psycopg2 2.8, the description of one result column,
+        # exposed as items of the cursor.description sequence.
+        # Before psycopg2 2.8 the description attribute was a sequence
+        # of simple tuples or namedtuples.
+        if psycopg2.__version__.find('2.8') != -1:
+            ores = OrderedDict()
+            ores['name'] = self.orig_col.name
+            ores['type_code'] = self.orig_col.type_code
+            ores['display_size'] = self.orig_col.display_size
+            ores['internal_size'] = self.orig_col.internal_size
+            ores['precision'] = self.orig_col.precision
+            ores['scale'] = self.orig_col.scale
+            ores['null_ok'] = self.orig_col.null_ok
+            ores['table_oid'] = self.orig_col.table_oid
+            ores['table_column'] = self.orig_col.table_column
+        else:
+            ores = OrderedDict(self.orig_col._asdict())
+
         name = ores['name']
         if self.dummy_name:
             ores['name'] = self.dummy_name
@@ -176,6 +192,9 @@ class DictCursor(_cursor):
         Execute function
         """
         self._odt_desc = None
+        if params is not None and len(params) == 0:
+            params = None
+
         return _cursor.execute(self, query, params)
 
     def executemany(self, query, params=None):
@@ -213,7 +232,7 @@ class DictCursor(_cursor):
         it = _cursor.__iter__(self)
         try:
             yield self._dict_tuple(next(it))
-            while 1:
+            while True:
                 yield self._dict_tuple(next(it))
         except StopIteration:
             pass
