@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -16,12 +16,17 @@ import config
 from flask import current_app, session
 from flask_security import current_user
 from flask_babelex import gettext
+from werkzeug.exceptions import InternalServerError
 
 from pgadmin.utils import get_complete_file_path
 from pgadmin.utils.crypto import decrypt
+from pgadmin.utils.master_password import process_masterpass_disabled
 from .connection import Connection
 from pgadmin.model import Server, User
-from pgadmin.utils.exception import ConnectionLost, SSHTunnelConnectionLost
+from pgadmin.utils.exception import ConnectionLost, SSHTunnelConnectionLost,\
+    CryptKeyMissing
+from pgadmin.utils.master_password import get_crypt_key
+from pgadmin.utils.exception import ObjectGone
 
 if config.SUPPORT_SSH_TUNNEL:
     from sshtunnel import SSHTunnelForwarder, BaseSSHTunnelForwarderError
@@ -34,6 +39,7 @@ class ServerManager(object):
     This class contains the information about the given server.
     And, acts as connection manager for that particular session.
     """
+    _INFORMATION_MSG = gettext("Information is not available.")
 
     def __init__(self, server):
         self.connections = dict()
@@ -60,6 +66,7 @@ class ServerManager(object):
         self.hostaddr = server.hostaddr
         self.port = server.port
         self.db = server.maintenance_db
+        self.shared = server.shared
         self.did = None
         self.user = server.username
         self.password = server.password
@@ -81,9 +88,12 @@ class ServerManager(object):
         if config.SUPPORT_SSH_TUNNEL:
             self.use_ssh_tunnel = server.use_ssh_tunnel
             self.tunnel_host = server.tunnel_host
-            self.tunnel_port = server.tunnel_port
+            self.tunnel_port = \
+                22 if server.tunnel_port is None else server.tunnel_port
             self.tunnel_username = server.tunnel_username
-            self.tunnel_authentication = server.tunnel_authentication
+            self.tunnel_authentication = 0 \
+                if server.tunnel_authentication is None \
+                else server.tunnel_authentication
             self.tunnel_identity_file = server.tunnel_identity_file
             self.tunnel_password = server.tunnel_password
         else:
@@ -102,6 +112,20 @@ class ServerManager(object):
 
         self.connections = dict()
 
+    def _set_password(self, res):
+        """
+        Set password for server manager object.
+        :param res: response dict.
+        :return:
+        """
+        if hasattr(self, 'password') and self.password:
+            if hasattr(self.password, 'decode'):
+                res['password'] = self.password.decode('utf-8')
+            else:
+                res['password'] = str(self.password)
+        else:
+            res['password'] = self.password
+
     def as_dict(self):
         """
         Returns a dictionary object representing the server manager.
@@ -113,18 +137,11 @@ class ServerManager(object):
         res['sid'] = self.sid
         res['ver'] = self.ver
         res['sversion'] = self.sversion
-        if hasattr(self, 'password') and self.password:
-            # If running under PY2
-            if hasattr(self.password, 'decode'):
-                res['password'] = self.password.decode('utf-8')
-            else:
-                res['password'] = str(self.password)
-        else:
-            res['password'] = self.password
+
+        self._set_password(res)
 
         if self.use_ssh_tunnel:
             if hasattr(self, 'tunnel_password') and self.tunnel_password:
-                # If running under PY2
                 if hasattr(self.tunnel_password, 'decode'):
                     res['tunnel_password'] = \
                         self.tunnel_password.decode('utf-8')
@@ -143,54 +160,55 @@ class ServerManager(object):
 
         return res
 
-    def ServerVersion(self):
+    def server_version(self):
         return self.ver
 
     @property
     def version(self):
         return self.sversion
 
-    def MajorVersion(self):
+    def major_version(self):
         if self.sversion is not None:
             return int(self.sversion / 10000)
-        raise Exception("Information is not available.")
+        raise InternalServerError(self._INFORMATION_MSG)
 
-    def MinorVersion(self):
+    def minor_version(self):
         if self.sversion:
             return int(int(self.sversion / 100) % 100)
-        raise Exception("Information is not available.")
+        raise InternalServerError(self._INFORMATION_MSG)
 
-    def PatchVersion(self):
+    def patch_version(self):
         if self.sversion:
             return int(int(self.sversion / 100) / 100)
-        raise Exception("Information is not available.")
+        raise InternalServerError(self._INFORMATION_MSG)
 
     def connection(
             self, database=None, conn_id=None, auto_reconnect=True, did=None,
             async_=None, use_binary_placeholder=False, array_to_string=False
     ):
         if database is not None:
-            if hasattr(str, 'decode') and \
-                    not isinstance(database, unicode):
-                database = database.decode('utf-8')
-            if did is not None:
-                if did in self.db_info:
-                    self.db_info[did]['datname'] = database
+            if did is not None and did in self.db_info:
+                self.db_info[did]['datname'] = database
         else:
             if did is None:
                 database = self.db
             elif did in self.db_info:
                 database = self.db_info[did]['datname']
             else:
-                maintenance_db_id = u'DB:{0}'.format(self.db)
+                maintenance_db_id = 'DB:{0}'.format(self.db)
                 if maintenance_db_id in self.connections:
                     conn = self.connections[maintenance_db_id]
+                    # try to connect maintenance db if not connected
+                    if not conn.connected():
+                        conn.connect()
+
                     if conn.connected():
-                        status, res = conn.execute_dict(u"""
+                        status, res = conn.execute_dict("""
 SELECT
     db.oid as did, db.datname, db.datallowconn,
     pg_encoding_to_char(db.encoding) AS serverencoding,
-    has_database_privilege(db.oid, 'CREATE') as cancreate, datlastsysoid
+    has_database_privilege(db.oid, 'CREATE') as cancreate, datlastsysoid,
+    datistemplate
 FROM
     pg_database db
 WHERE db.oid = {0}""".format(did))
@@ -201,10 +219,14 @@ WHERE db.oid = {0}""".format(did))
                                 database = self.db_info[did]['datname']
 
                         if did not in self.db_info:
-                            raise Exception(gettext(
+                            raise ObjectGone(gettext(
                                 "Could not find the specified database."
                             ))
 
+        if not get_crypt_key()[0]:
+            # the reason its not connected might be missing key
+            raise CryptKeyMissing()
+
         if database is None:
             # Check SSH Tunnel is alive or not.
             if self.use_ssh_tunnel == 1:
@@ -212,8 +234,8 @@ WHERE db.oid = {0}""".format(did))
             else:
                 raise ConnectionLost(self.sid, None, None)
 
-        my_id = (u'CONN:{0}'.format(conn_id)) if conn_id is not None else \
-            (u'DB:{0}'.format(database))
+        my_id = ('CONN:{0}'.format(conn_id)) if conn_id is not None else \
+            ('DB:{0}'.format(database))
 
         self.pinged = datetime.datetime.now()
 
@@ -232,6 +254,76 @@ WHERE db.oid = {0}""".format(did))
 
             return self.connections[my_id]
 
+    @staticmethod
+    def _get_password_to_conn(data, masterpass_processed):
+        """
+        Get password for connect to server with simple and ssh connection.
+        :param data: Data.
+        :param masterpass_processed:
+        :return:
+        """
+        # The data variable is a copy so is not automatically synced
+        # update here
+        if masterpass_processed and 'password' in data:
+            data['password'] = None
+        if masterpass_processed and 'tunnel_password' in data:
+            data['tunnel_password'] = None
+
+    def _get_server_type(self):
+        """
+        Get server type and server cls.
+        :return:
+        """
+        from pgadmin.browser.server_groups.servers.types import ServerType
+
+        if self.ver and not self.server_type:
+            for st in ServerType.types():
+                if st.instance_of(self.ver):
+                    self.server_type = st.stype
+                    self.server_cls = st
+                    break
+
+    def _check_and_reconnect_server(self, conn, conn_info, data):
+        """
+        Check and try to reconnect the server if server previously connected
+        and auto_reconnect is true.
+        :param conn:
+        :type conn:
+        :param conn_info:
+        :type conn_info:
+        :param data:
+        :type data:
+        :return:
+        :rtype:
+        """
+        from pgadmin.browser.server_groups.servers.types import ServerType
+        if conn_info['wasConnected'] and conn_info['auto_reconnect']:
+            try:
+                # Check SSH Tunnel needs to be created
+                if self.use_ssh_tunnel == 1 and \
+                        not self.tunnel_created:
+                    status, error = self.create_ssh_tunnel(
+                        data['tunnel_password'])
+
+                    # Check SSH Tunnel is alive or not.
+                    self.check_ssh_tunnel_alive()
+
+                conn.connect(
+                    password=data['password'],
+                    server_types=ServerType.types()
+                )
+                # This will also update wasConnected flag in
+                # connection so no need to update the flag manually.
+            except CryptKeyMissing:
+                # maintain the status as this will help to restore once
+                # the key is available
+                conn.wasConnected = conn_info['wasConnected']
+                conn.auto_reconnect = conn_info['auto_reconnect']
+            except Exception as e:
+                current_app.logger.exception(e)
+                self.connections.pop(conn_info['conn_id'])
+                raise
+
     def _restore(self, data):
         """
         Helps restoring to reconnect the auto-connect connections smoothly on
@@ -239,28 +331,18 @@ WHERE db.oid = {0}""".format(did))
         """
         # restore server version from flask session if flask server was
         # restarted. As we need server version to resolve sql template paths.
-        from pgadmin.browser.server_groups.servers.types import ServerType
+        masterpass_processed = process_masterpass_disabled()
 
-        self.ver = data.get('ver', None)
-        self.sversion = data.get('sversion', None)
-
-        if self.ver and not self.server_type:
-            for st in ServerType.types():
-                if st.instanceOf(self.ver):
-                    self.server_type = st.stype
-                    self.server_cls = st
-                    break
-
-        # Hmm.. we will not honour this request, when I already have
-        # connections
-        if len(self.connections) != 0:
-            return
+        ServerManager._get_password_to_conn(data, masterpass_processed)
+        # Get server type.
+        self._get_server_type()
 
         # We need to know about the existing server variant supports during
         # first connection for identifications.
         self.pinged = datetime.datetime.now()
         try:
-            if 'password' in data and data['password']:
+            if 'password' in data and data['password'] and \
+                    hasattr(data['password'], 'encode'):
                 data['password'] = data['password'].encode('utf-8')
             if 'tunnel_password' in data and data['tunnel_password']:
                 data['tunnel_password'] = \
@@ -269,56 +351,99 @@ WHERE db.oid = {0}""".format(did))
             current_app.logger.exception(e)
 
         connections = data['connections']
+
         for conn_id in connections:
             conn_info = connections[conn_id]
-            conn = self.connections[conn_info['conn_id']] = Connection(
-                self, conn_info['conn_id'], conn_info['database'],
-                conn_info['auto_reconnect'], conn_info['async_'],
-                use_binary_placeholder=conn_info['use_binary_placeholder'],
-                array_to_string=conn_info['array_to_string']
-            )
+            if conn_info['conn_id'] in self.connections:
+                conn = self.connections[conn_info['conn_id']]
+            else:
+                conn = self.connections[conn_info['conn_id']] = Connection(
+                    self, conn_info['conn_id'], conn_info['database'],
+                    conn_info['auto_reconnect'], conn_info['async_'],
+                    use_binary_placeholder=conn_info[
+                        'use_binary_placeholder'],
+                    array_to_string=conn_info['array_to_string']
+                )
 
-            # only try to reconnect if connection was connected previously and
-            # auto_reconnect is true.
-            if conn_info['wasConnected'] and conn_info['auto_reconnect']:
+            # only try to reconnect
+            self._check_and_reconnect_server(conn, conn_info, data)
+
+    def _restore_connections(self):
+        for conn_id in self.connections:
+            conn = self.connections[conn_id]
+            # only try to reconnect if connection was connected previously
+            # and auto_reconnect is true.
+            wasConnected = conn.wasConnected
+            auto_reconnect = conn.auto_reconnect
+            if conn.wasConnected and conn.auto_reconnect:
                 try:
                     # Check SSH Tunnel needs to be created
-                    if self.use_ssh_tunnel == 1 and not self.tunnel_created:
+                    if self.use_ssh_tunnel == 1 and \
+                       not self.tunnel_created:
                         status, error = self.create_ssh_tunnel(
-                            data['tunnel_password'])
+                            self.tunnel_password
+                        )
 
                         # Check SSH Tunnel is alive or not.
                         self.check_ssh_tunnel_alive()
 
-                    conn.connect(
-                        password=data['password'],
-                        server_types=ServerType.types()
-                    )
-                    # This will also update wasConnected flag in connection so
-                    # no need to update the flag manually.
+                    conn.connect()
+                    # This will also update wasConnected flag in
+                    # connection so no need to update the flag manually.
+                except CryptKeyMissing:
+                    # maintain the status as this will help to restore once
+                    # the key is available
+                    conn.wasConnected = wasConnected
+                    conn.auto_reconnect = auto_reconnect
                 except Exception as e:
+                    self.connections.pop(conn_id)
                     current_app.logger.exception(e)
-                    self.connections.pop(conn_info['conn_id'])
+                    raise
+
+    def _stop_ssh_tunnel(self, did, database, conn_id):
+        """
+        Stop ssh tunnel connection if function call without any parameter.
+        :param did: Database Id.
+        :param database: Database.
+        :param conn_id: COnnection Id.
+        :return:
+        """
+        if database is None and conn_id is None and did is None:
+            self.stop_ssh_tunnel()
+
+    def _check_db_info(self, did, conn_id, database):
+        """
+        Check did is not none and it is resent in db_info.
+        :param did: Database Id.
+        :param conn_id: Connection Id.
+        :return:
+        """
+        if database is None and conn_id is None and did is None:
+            self.stop_ssh_tunnel()
+
+        my_id = None
+        if did is not None:
+            if did in self.db_info and 'datname' in self.db_info[did]:
+                database = self.db_info[did]['datname']
+                if database is None:
+                    return True, False, my_id
+            else:
+                return True, False, my_id
+
+        if conn_id is not None:
+            my_id = 'CONN:{0}'.format(conn_id)
+        elif database is not None:
+            my_id = 'DB:{0}'.format(database)
+
+        return False, True, my_id
 
     def release(self, database=None, conn_id=None, did=None):
         # Stop the SSH tunnel if release() function calls without
         # any parameter.
-        if database is None and conn_id is None and did is None:
-            self.stop_ssh_tunnel()
-
-        if did is not None:
-            if did in self.db_info and 'datname' in self.db_info[did]:
-                database = self.db_info[did]['datname']
-                if hasattr(str, 'decode') and \
-                        not isinstance(database, unicode):
-                    database = database.decode('utf-8')
-                if database is None:
-                    return False
-            else:
-                return False
-
-        my_id = (u'CONN:{0}'.format(conn_id)) if conn_id is not None else \
-            (u'DB:{0}'.format(database)) if database is not None else None
+        is_return, return_value, my_id = self._check_db_info(did, conn_id,
+                                                             database)
+        if is_return:
+            return return_value
 
         if my_id is not None:
             if my_id in self.connections:
@@ -340,8 +465,14 @@ WHERE db.oid = {0}""".format(did))
             else:
                 return False
 
-        for con in self.connections:
-            self.connections[con]._release()
+        for con_key in list(self.connections.keys()):
+            conn = self.connections[con_key]
+            # Cancel the ongoing transaction before closing the connection
+            # as it may hang forever
+            if conn.connected() and conn.conn_id is not None and \
+               conn.conn_id.startswith('CONN:'):
+                conn.cancel_transaction(conn.conn_id[5:])
+            conn._release()
 
         self.connections = dict()
         self.ver = None
@@ -387,9 +518,11 @@ WHERE db.oid = {0}""".format(did))
 
     def export_password_env(self, env):
         if self.password:
-            password = decrypt(
-                self.password, current_user.password
-            ).decode()
+            crypt_key_present, crypt_key = get_crypt_key()
+            if not crypt_key_present:
+                return False, crypt_key
+
+            password = decrypt(self.password, crypt_key).decode()
             os.environ[str(env)] = password
 
     def create_ssh_tunnel(self, tunnel_password):
@@ -405,20 +538,20 @@ WHERE db.oid = {0}""".format(did))
             return False, gettext("Unauthorized request.")
 
         if tunnel_password is not None and tunnel_password != '':
+            crypt_key_present, crypt_key = get_crypt_key()
+            if not crypt_key_present:
+                raise CryptKeyMissing()
+
             try:
-                tunnel_password = decrypt(tunnel_password, user.password)
-                # Handling of non ascii password (Python2)
-                if hasattr(str, 'decode'):
-                    tunnel_password = \
-                        tunnel_password.decode('utf-8').encode('utf-8')
+                tunnel_password = decrypt(tunnel_password, crypt_key)
                 # password is in bytes, for python3 we need it in string
-                elif isinstance(tunnel_password, bytes):
+                if isinstance(tunnel_password, bytes):
                     tunnel_password = tunnel_password.decode()
 
             except Exception as e:
                 current_app.logger.exception(e)
-                return False, "Failed to decrypt the SSH tunnel " \
-                              "password.\nError: {0}".format(str(e))
+                return False, gettext("Failed to decrypt the SSH tunnel "
+                                      "password.\nError: {0}").format(str(e))
 
         try:
             # If authentication method is 1 then it uses identity file
@@ -438,13 +571,14 @@ WHERE db.oid = {0}""".format(did))
                     ssh_password=tunnel_password,
                     remote_bind_address=(self.host, self.port)
                 )
-
+            # flag tunnel threads in daemon mode to fix hang issue.
+            self.tunnel_object.daemon_forward_servers = True
             self.tunnel_object.start()
             self.tunnel_created = True
         except BaseSSHTunnelForwarderError as e:
             current_app.logger.exception(e)
-            return False, "Failed to create the SSH tunnel." \
-                          "\nError: {0}".format(str(e))
+            return False, gettext("Failed to create the SSH tunnel.\n"
+                                  "Error: {0}").format(str(e))
 
         # Update the port to communicate locally
         self.local_bind_port = self.tunnel_object.local_bind_port
