@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2017, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -16,21 +16,24 @@ class MenuItem(object):
 
 
 class Panel(object):
-    def __init__(self, name, title, content='', width=500, height=600, isIframe=True,
-                 showTitle=True, isCloseable=True, isPrivate=False, priority=None,
-                 icon=None, data=None, events=None):
+    def __init__(
+        self, name, title, content='', width=500, height=600, **kwargs
+    ):
         self.name = name
         self.title = title
         self.content = content
         self.width = width
         self.height = height
-        self.isIframe = isIframe
-        self.showTitle = showTitle
-        self.isCloseable = isCloseable
-        self.isPrivate = isPrivate
-        self.icon = icon
-        self.data = data
-        self.events = events
+        self.isIframe = kwargs.get('is_iframe', True)
+        self.showTitle = kwargs.get('show_title', True)
+        self.isCloseable = kwargs.get('is_closeable', True)
+        self.isPrivate = kwargs.get('is_private', None)
+        self.icon = kwargs.get('icon', None)
+        self.data = kwargs.get('data', None)
+        self.events = kwargs.get('events', None)
+        self.limit = kwargs.get('limit', False)
+        self.canHide = kwargs.get('can_hide', False)
+        priority = kwargs.get('priority', None)
         if priority is None:
             global PRIORITY
             PRIORITY += 100
