@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -28,21 +28,15 @@ from uuid import uuid4
 from threading import Lock
 from flask import current_app, request, flash, redirect
 from flask_login import login_url
-from pgadmin.utils.ajax import make_json_response
 
-try:
-    from cPickle import dump, load
-except ImportError:
-    from pickle import dump, load
-
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
+from pickle import dump, load
+from collections import OrderedDict
 
 from flask.sessions import SessionInterface, SessionMixin
 from werkzeug.datastructures import CallbackDict
 
+from pgadmin.utils.ajax import make_json_response
+
 
 def _calc_hmac(body, secret):
     return base64.b64encode(
@@ -74,12 +68,7 @@ class ManagedSession(CallbackDict, SessionMixin):
 
     def sign(self, secret):
         if not self.hmac_digest:
-            if hasattr(string, 'lowercase'):
-                population = string.lowercase
-            # If script is running under python3
-            elif hasattr(string, 'ascii_lowercase'):
-                population = string.ascii_lowercase
-            population += string.digits
+            population = string.ascii_lowercase + string.digits
 
             self.randval = ''.join(random.sample(population, 20))
             self.hmac_digest = _calc_hmac(
@@ -109,11 +98,11 @@ class SessionManager(object):
 
 
 class CachingSessionManager(SessionManager):
-    def __init__(self, parent, num_to_store, skip_paths=[]):
+    def __init__(self, parent, num_to_store, skip_paths=None):
         self.parent = parent
         self.num_to_store = num_to_store
         self._cache = OrderedDict()
-        self.skip_paths = skip_paths
+        self.skip_paths = [] if skip_paths is None else skip_paths
 
     def _normalize(self):
         if len(self._cache) > self.num_to_store:
@@ -153,7 +142,7 @@ class CachingSessionManager(SessionManager):
         with sess_lock:
             if sid in self._cache:
                 session = self._cache[sid]
-                if session.hmac_digest != digest:
+                if session and session.hmac_digest != digest:
                     session = None
 
                 # reset order in Dict
@@ -193,13 +182,13 @@ class CachingSessionManager(SessionManager):
 
 class FileBackedSessionManager(SessionManager):
 
-    def __init__(self, path, secret, disk_write_delay, skip_paths=[]):
+    def __init__(self, path, secret, disk_write_delay, skip_paths=None):
         self.path = path
         self.secret = secret
         self.disk_write_delay = disk_write_delay
         if not os.path.exists(self.path):
             os.makedirs(self.path)
-        self.skip_paths = skip_paths
+        self.skip_paths = [] if skip_paths is None else skip_paths
 
     def exists(self, sid):
         fname = os.path.join(self.path, sid)
@@ -225,7 +214,7 @@ class FileBackedSessionManager(SessionManager):
 
         # touch the file
         with open(fname, 'wb'):
-            pass
+            return ManagedSession(sid=sid)
 
         return ManagedSession(sid=sid)
 
@@ -263,11 +252,10 @@ class FileBackedSessionManager(SessionManager):
         current_time = time.time()
         if not session.hmac_digest:
             session.sign(self.secret)
-        elif not session.force_write:
-            if session.last_write is not None and \
-                (current_time - float(session.last_write)) < \
-                    self.disk_write_delay:
-                return
+        elif not session.force_write and session.last_write is not None and \
+            (current_time - float(session.last_write)) < \
+                self.disk_write_delay:
+            return
 
         session.last_write = current_time
         session.force_write = False
@@ -323,7 +311,11 @@ class ManagedSessionInterface(SessionInterface):
         response.set_cookie(
             app.session_cookie_name,
             '%s!%s' % (session.sid, session.hmac_digest),
-            expires=cookie_exp, httponly=True, domain=domain
+            expires=cookie_exp,
+            secure=config.SESSION_COOKIE_SECURE,
+            httponly=config.SESSION_COOKIE_HTTPONLY,
+            samesite=config.SESSION_COOKIE_SAMESITE,
+            domain=domain
         )
 
 
@@ -352,7 +344,7 @@ def pga_unauthorised():
         else:
             login_message = lm.login_message
 
-    if not lm.login_view or request.is_xhr:
+    if not lm.login_view:
         # Only 401 is not enough to distinguish pgAdmin login is required.
         # There are other cases when we return 401. For eg. wrong password
         # supplied while connecting to server.
@@ -364,7 +356,10 @@ def pga_unauthorised():
             info='PGADMIN_LOGIN_REQUIRED'
         )
 
-    if login_message:
+    # flash messages are only required if the request was from a
+    # security page, otherwise it will be redirected to login page
+    # anyway
+    if login_message and 'security' in request.endpoint:
         flash(login_message, category=lm.login_message_category)
 
     return redirect(login_url(lm.login_view, request.url))
@@ -379,14 +374,11 @@ def cleanup_session_files():
     iterate_session_files = False
 
     global LAST_CHECK_SESSION_FILES
-    if LAST_CHECK_SESSION_FILES is None:
+    if LAST_CHECK_SESSION_FILES is None or \
+        datetime.datetime.now() >= LAST_CHECK_SESSION_FILES + \
+            datetime.timedelta(hours=config.CHECK_SESSION_FILES_INTERVAL):
         iterate_session_files = True
         LAST_CHECK_SESSION_FILES = datetime.datetime.now()
-    else:
-        if datetime.datetime.now() >= LAST_CHECK_SESSION_FILES + \
-                datetime.timedelta(hours=config.CHECK_SESSION_FILES_INTERVAL):
-            iterate_session_files = True
-            LAST_CHECK_SESSION_FILES = datetime.datetime.now()
 
     if iterate_session_files:
         for root, dirs, files in os.walk(
@@ -405,6 +397,6 @@ def cleanup_session_files():
                     current_app.permanent_session_lifetime + \
                     datetime.timedelta(days=1)
 
-                if file_expiration_time <= datetime.datetime.now():
-                    if os.path.exists(absolute_file_name):
-                        os.unlink(absolute_file_name)
+                if file_expiration_time <= datetime.datetime.now() and \
+                        os.path.exists(absolute_file_name):
+                    os.unlink(absolute_file_name)
