@@ -2,7 +2,7 @@
 #
 # pgAdmin 4 - PostgreSQL Tools
 #
-# Copyright (C) 2013 - 2018, The pgAdmin Development Team
+# Copyright (C) 2013 - 2021, The pgAdmin Development Team
 # This software is released under the PostgreSQL Licence
 #
 ##########################################################################
@@ -36,4 +36,4 @@ class TestCompileTemplateName(BaseTestGenerator):
         result = compile_template_name(
             'some/prefix', 'some_file.sql', self.server_type, self.version
         )
-        self.assertEquals(result, self.expected_return_value)
+        self.assertEqual(result, self.expected_return_value)
