@@ -2,7 +2,7 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////
@@ -94,14 +94,16 @@ describe('BackupDialogWrapper', () => {
         return backupJQueryContainerSpy;
       } else if (selector === backupNode.elements.body.childNodes[0]) {
         return backupNodeChildNodeSpy;
+      } else {
+        return jasmine.createSpyObj('obj', ['appendTo']);
       }
     });
+    alertifySpy = jasmine.createSpyObj('alertify', ['alert', 'dialog']);
 
   });
 
   describe('#prepare', () => {
     beforeEach(() => {
-      alertifySpy = jasmine.createSpyObj('alertify', ['alert', 'dialog']);
       backupDialogWrapper = new BackupDialogWrapper(
         '<div class=\'backup_dialog\'></div>',
         'backupDialogTitle',
@@ -245,8 +247,6 @@ describe('BackupDialogWrapper', () => {
               getAttribute: (attributeName) => {
                 if (attributeName === 'url') {
                   return 'http://someurl';
-                } else if (attributeName === 'label') {
-                  return 'some label';
                 }
               },
             },
@@ -260,8 +260,7 @@ describe('BackupDialogWrapper', () => {
           'dialog_help',
           'http://someurl',
           pgBrowser.Nodes['server'],
-          serverTreeNode.getHtmlIdentifier(),
-          'some label'
+          serverTreeNode.getHtmlIdentifier()
         );
       });
 
@@ -288,8 +287,6 @@ describe('BackupDialogWrapper', () => {
               getAttribute: (attributeName) => {
                 if (attributeName === 'url') {
                   return 'http://someurl';
-                } else if (attributeName === 'label') {
-                  return 'some label';
                 }
               },
             },
@@ -303,8 +300,7 @@ describe('BackupDialogWrapper', () => {
           'object_help',
           'http://someurl',
           pgBrowser.Nodes['server'],
-          serverTreeNode.getHtmlIdentifier(),
-          'some label'
+          serverTreeNode.getHtmlIdentifier()
         );
       });
 
@@ -391,11 +387,11 @@ describe('BackupDialogWrapper', () => {
             beforeEach(() => {
               pgBrowser.Events = jasmine.createSpyObj('Events', ['trigger']);
               alertifySpy.success = jasmine.createSpy('success');
-
               networkMock.onPost('/backup/job/10').reply((request) => {
                 dataSentToServer = request.data;
                 return [200, {'success': 1}];
               });
+
             });
 
             it('creates a success alert box', (done) => {
@@ -442,7 +438,7 @@ describe('BackupDialogWrapper', () => {
               });
 
               backupDialogWrapper.callback(event);
-              setTimeout(() => {
+              setTimeout( () => {
                 expect(alertifySpy.alert).toHaveBeenCalledWith(
                   'Backup job failed.',
                   'some-error-message'
@@ -556,7 +552,7 @@ describe('BackupDialogWrapper', () => {
     });
   });
 
-  describe('#setExtraParameters', () => {
+  xdescribe('#setExtraParameters', () => {
     let selectedTreeNode;
     let treeInfo;
     let model;
