@@ -1,3 +1,12 @@
+/////////////////////////////////////////////////////////////
+//
+// pgAdmin 4 - PostgreSQL Tools
+//
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
+// This software is released under the PostgreSQL Licence
+//
+//////////////////////////////////////////////////////////////
+
 import StatisticsModel from '../../../../pgadmin/misc/static/explain/js/explain_statistics';
 import $ from 'jquery';
 
@@ -9,10 +18,14 @@ describe('ExplainStatistics', () => {
   beforeEach(function() {
     statsModel = new StatisticsModel();
     statsDiv = '<div class="pg-explain-stats-area btn-group d-none"></div>';
-    tooltipContainer = $('<div></div>', {
-      id: 'toolTip',
-      class: 'pgadmin-explain-tooltip',
-    });
+    tooltipContainer = $(
+      `<div class="pgadmin-explain-details d-none">
+       <div class="details-header">
+         <div class="details-title"></div>
+       </div>
+       <div class="details-body"></div>
+     </div>`
+    );
   });
 
   describe('No Statistics', () => {
@@ -21,9 +34,10 @@ describe('ExplainStatistics', () => {
 
       statsModel.set('JIT', []);
       statsModel.set('Triggers', []);
+      statsModel.set('Summary', {});
       statsModel.set_statistics(tooltipContainer);
 
-      expect($('.pg-explain-stats-area').hasClass('d-none')).toBe(true);
+      expect($('.pg-explain-stats-area').hasClass('d-none')).toEqual(true);
     });
   });
 
@@ -36,23 +50,15 @@ describe('ExplainStatistics', () => {
     });
 
     it('Statistics button should be visible', () => {
-      expect($('.pg-explain-stats-area').hasClass('d-none')).toBe(false);
+      expect($('.pg-explain-stats-area').hasClass('d-none')).toEqual(false);
     });
 
-    it('Mouse over event should be trigger', () => {
+    it('Mouse click event should be trigger', () => {
       // Trigger mouse over event
-      var hoverEvent = new $.Event('mouseover');
-      $('.pg-explain-stats-area').trigger(hoverEvent);
+      var clickEvent = new $.Event('click');
+      $('.pg-explain-stats-area').trigger(clickEvent);
 
-      expect(tooltipContainer.css('opacity')).toBe('0.8');
-    });
-
-    it('Mouse out event should be trigger', () => {
-      // Trigger mouse out event
-      var hoverEvent = new $.Event('mouseout');
-      $('.pg-explain-stats-area').trigger(hoverEvent);
-
-      expect(tooltipContainer.css('opacity')).toBe('0');
+      expect(tooltipContainer.hasClass('d-none')).toBe(false);
     });
   });
 
@@ -65,23 +71,40 @@ describe('ExplainStatistics', () => {
     });
 
     it('Statistics button should be visible', () => {
-      expect($('.pg-explain-stats-area').hasClass('d-none')).toBe(false);
+      expect($('.pg-explain-stats-area').hasClass('d-none')).toEqual(false);
     });
 
-    it('Mouse over event should be trigger', () => {
+    it('Mouse click event should be trigger', () => {
       // Trigger mouse over event
-      var hoverEvent = new $.Event('mouseover');
-      $('.pg-explain-stats-area').trigger(hoverEvent);
+      var clickEvent = new $.Event('click');
+      $('.pg-explain-stats-area').trigger(clickEvent);
 
-      expect(tooltipContainer.css('opacity')).toBe('0.8');
+      expect(tooltipContainer.hasClass('d-none')).toBe(false);
+    });
+  });
+
+  describe('Summary', () => {
+    beforeEach(function() {
+      $('body').append(statsDiv);
+      statsModel.set('JIT', []);
+      statsModel.set('Triggers', []);
+      statsModel.set('Summary', {
+        'Planning Time': 0.12,
+        'Execution Time': 2.34,
+      });
+      statsModel.set_statistics(tooltipContainer);
     });
 
-    it('Mouse out event should be trigger', () => {
-      // Trigger mouse out event
-      var hoverEvent = new $.Event('mouseout');
-      $('.pg-explain-stats-area').trigger(hoverEvent);
+    it('Statistics button should be visible', () => {
+      expect($('.pg-explain-stats-area').hasClass('d-none')).toEqual(false);
+    });
 
-      expect(tooltipContainer.css('opacity')).toBe('0');
+    it('Mouse click event should be trigger', () => {
+      // Trigger mouse over event
+      var clickEvent = new $.Event('click');
+      $('.pg-explain-stats-area').trigger(clickEvent);
+
+      expect(tooltipContainer.hasClass('d-none')).toBe(false);
     });
   });
 });
