@@ -2,22 +2,24 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////////////////
 
-import keyboardShortcuts from 'sources/keyboard_shortcuts';
+import * as keyboardShortcuts from 'sources/keyboard_shortcuts';
 import {queryToolActions} from 'sources/sqleditor/query_tool_actions';
 import gettext from 'sources/gettext';
 
 describe('the keyboard shortcuts', () => {
   const F1_KEY = 112,
     F5_KEY = 116,
+    F6_KEY = 117,
     F7_KEY = 118,
     F8_KEY = 119,
     PERIOD_KEY = 190,
-    FWD_SLASH_KEY = 191;
+    FWD_SLASH_KEY = 191,
+    C1_KEY = 49;
 
   let sqlEditorControllerSpy, event, queryToolActionsSpy;
   beforeEach(() => {
@@ -69,7 +71,7 @@ describe('the keyboard shortcuts', () => {
           key_code: F7_KEY,
         },
       },
-      download_csv: {
+      download_results: {
         alt: false,
         shift: false,
         control: false,
@@ -93,8 +95,45 @@ describe('the keyboard shortcuts', () => {
           key_code: null,
         },
       },
+      commit_transaction: {
+        alt: false,
+        shift: true,
+        control: true,
+        key: {
+          key_code: 'm',
+        },
+      },
+      rollback_transaction: {
+        alt: false,
+        shift: true,
+        control: true,
+        key: {
+          key_code: 'r',
+        },
+      },
+      save_data: {
+        alt : false,
+        shift: false,
+        control: false,
+        key: {
+          key_code: F6_KEY,
+        },
+      },
     };
 
+    sqlEditorControllerSpy.macros = [
+      {
+        alt: false,
+        control: true,
+        id: 1,
+        key: '1',
+        key_code: C1_KEY,
+        key_label: 'Ctrl + 1',
+        name: 'C1',
+        sql: 'Select 1;',
+      },
+    ];
+
     queryToolActionsSpy = jasmine.createSpyObj(queryToolActions, [
       'explainAnalyze',
       'explain',
@@ -103,6 +142,10 @@ describe('the keyboard shortcuts', () => {
       'commentLineCode',
       'uncommentLineCode',
       'executeQuery',
+      'executeCommit',
+      'executeRollback',
+      'saveDataChanges',
+      'executeMacro',
     ]);
   });
 
@@ -158,6 +201,42 @@ describe('the keyboard shortcuts', () => {
     });
   });
 
+  describe('F6', () => {
+    describe('when there is not a query already running', () => {
+      beforeEach(() => {
+        event.which = F6_KEY;
+        event.altKey = false;
+        event.shiftKey = false;
+        event.ctrlKey = false;
+        keyboardShortcuts.processEventQueryTool(
+          sqlEditorControllerSpy, queryToolActionsSpy, event
+        );
+      });
+
+      it('should save the changed data', () => {
+        expect(queryToolActionsSpy.saveDataChanges).toHaveBeenCalledWith(sqlEditorControllerSpy);
+      });
+
+      expectEventPropagationToStop();
+    });
+
+    describe('when the query is already running', () => {
+      it('does nothing', () => {
+        event.keyCode = F6_KEY;
+        event.altKey = false;
+        event.shiftKey = false;
+        event.ctrlKey = false;
+        sqlEditorControllerSpy.isQueryRunning.and.returnValue(true);
+
+        keyboardShortcuts.processEventQueryTool(
+          sqlEditorControllerSpy, queryToolActionsSpy, event
+        );
+
+        expect(queryToolActionsSpy.saveDataChanges).not.toHaveBeenCalled();
+      });
+    });
+  });
+
   describe('F7', () => {
     describe('when there is not a query already running', () => {
       beforeEach(() => {
@@ -494,30 +573,153 @@ describe('the keyboard shortcuts', () => {
     };
 
     it('shortcut_key',()=>{
-      expect(keyboardShortcuts.shortcut_key(shortcut)).toBe('A');
+      expect(keyboardShortcuts.shortcut_key(shortcut)).toEqual('A');
     });
 
     it('shortcut_accesskey_title',()=>{
       expect(keyboardShortcuts.shortcut_accesskey_title(
-        'Title', shortcut)).toBe(gettext('Title (accesskey + A)'));
+        'Title', shortcut)).toEqual(gettext('Title (accesskey + A)'));
     });
 
     it('shortcut_title',()=>{
-      shortcut.alt = true, shortcut.shift = false, shortcut.control = false;
+      shortcut.alt = true;
+      shortcut.shift = false;
+      shortcut.control = false;
       expect(keyboardShortcuts.shortcut_title(
-        'Title', shortcut)).toBe(gettext('Title (Alt+A)'));
+        'Title', shortcut)).toEqual(gettext('Title (Alt+A)'));
 
-      shortcut.alt = false, shortcut.shift = true, shortcut.control = false;
+      shortcut.alt = false;
+      shortcut.shift = true;
+      shortcut.control = false;
       expect(keyboardShortcuts.shortcut_title(
-        'Title', shortcut)).toBe(gettext('Title (Shift+A)'));
+        'Title', shortcut)).toEqual(gettext('Title (Shift+A)'));
 
-      shortcut.alt = false, shortcut.shift = false, shortcut.control = true;
+      shortcut.alt = false;
+      shortcut.shift = false;
+      shortcut.control = true;
       expect(keyboardShortcuts.shortcut_title(
-        'Title', shortcut)).toBe(gettext('Title (Ctrl+A)'));
+        'Title', shortcut)).toEqual(gettext('Title (Ctrl+A)'));
 
-      shortcut.alt = true, shortcut.shift = true, shortcut.control = true;
+      shortcut.alt = true;
+      shortcut.shift = true;
+      shortcut.control = true;
       expect(keyboardShortcuts.shortcut_title(
-        'Title', shortcut)).toBe(gettext('Title (Alt+Shift+Ctrl+A)'));
+        'Title', shortcut)).toEqual(gettext('Title (Alt+Shift+Ctrl+A)'));
+    });
+  });
+
+  describe('Shift+Ctrl+C', () => {
+    describe('when there is not a query already running', () => {
+      beforeEach(() => {
+        event.shiftKey = true;
+        event.which = 'm';
+        event.altKey = false;
+        event.ctrlKey = true;
+        keyboardShortcuts.processEventQueryTool(
+          sqlEditorControllerSpy, queryToolActionsSpy, event
+        );
+      });
+
+      it('should commit the transaction', () => {
+        expect(queryToolActionsSpy.executeCommit).toHaveBeenCalledWith(sqlEditorControllerSpy);
+      });
+
+      expectEventPropagationToStop();
+    });
+
+    describe('when the query is already running', () => {
+      it('does nothing', () => {
+        event.shiftKey = true;
+        event.which = 'm';
+        event.altKey = false;
+        event.ctrlKey = true;
+
+        sqlEditorControllerSpy.isQueryRunning.and.returnValue(true);
+
+        keyboardShortcuts.processEventQueryTool(
+          sqlEditorControllerSpy, queryToolActionsSpy, event
+        );
+
+        expect(queryToolActionsSpy.executeCommit).not.toHaveBeenCalled();
+      });
+    });
+  });
+
+  describe('Shift+Ctrl+R', () => {
+    describe('when there is not a query already running', () => {
+      beforeEach(() => {
+        event.shiftKey = true;
+        event.which = 'r';
+        event.altKey = false;
+        event.ctrlKey = true;
+        keyboardShortcuts.processEventQueryTool(
+          sqlEditorControllerSpy, queryToolActionsSpy, event
+        );
+      });
+
+      it('should rollback the transaction', () => {
+        expect(queryToolActionsSpy.executeRollback).toHaveBeenCalledWith(sqlEditorControllerSpy);
+      });
+
+      expectEventPropagationToStop();
+    });
+
+    describe('when the query is already running', () => {
+      it('does nothing', () => {
+        event.shiftKey = true;
+        event.which = 'r';
+        event.altKey = false;
+        event.ctrlKey = true;
+
+        sqlEditorControllerSpy.isQueryRunning.and.returnValue(true);
+
+        keyboardShortcuts.processEventQueryTool(
+          sqlEditorControllerSpy, queryToolActionsSpy, event
+        );
+
+        expect(queryToolActionsSpy.executeRollback).not.toHaveBeenCalled();
+      });
+    });
+  });
+
+  describe('Macro Ctrl + 1', () => {
+    describe('when there is not a query already running', () => {
+      beforeEach(() => {
+        event.which = C1_KEY;
+        event.altKey = false;
+        event.shiftKey = false;
+        event.ctrlKey = true;
+
+        keyboardShortcuts.processEventQueryTool(
+          sqlEditorControllerSpy, queryToolActionsSpy, event
+        );
+      });
+
+      it('should execute the macro', () => {
+        expect(queryToolActionsSpy.executeMacro).toHaveBeenCalledWith(sqlEditorControllerSpy,
+          sqlEditorControllerSpy.macros[0].id);
+      });
+
+      it('should stop event propagation', () => {
+        expect(event.preventDefault).toHaveBeenCalled();
+      });
+    });
+
+    describe('when the query is already running', () => {
+      it('does nothing', () => {
+        event.keyCode = C1_KEY;
+        event.altKey = false;
+        event.shiftKey = false;
+        event.ctrlKey = true;
+
+        sqlEditorControllerSpy.isQueryRunning.and.returnValue(true);
+
+        keyboardShortcuts.processEventQueryTool(
+          sqlEditorControllerSpy, queryToolActionsSpy, event
+        );
+
+        expect(queryToolActionsSpy.executeMacro).not.toHaveBeenCalled();
+      });
     });
   });
 
@@ -525,7 +727,7 @@ describe('the keyboard shortcuts', () => {
     describe('stops all event propogation', () => {
 
       it('should cancel the bubble', () => {
-        expect(event.cancelBubble).toBe(true);
+        expect(event.cancelBubble).toEqual(true);
       });
 
       it('should prevent the default behavior', () => {
