@@ -2,7 +2,7 @@
 //
 // pgAdmin 4 - PostgreSQL Tools
 //
-// Copyright (C) 2013 - 2019, The pgAdmin Development Team
+// Copyright (C) 2013 - 2021, The pgAdmin Development Team
 // This software is released under the PostgreSQL Licence
 //
 //////////////////////////////////////////////////////////////////////////
@@ -30,7 +30,7 @@ describe('queryToolActions', () => {
       it('calls the execute function on the sqlEditorController', () => {
         queryToolActions.executeQuery(sqlEditorController);
 
-        expect(sqlEditorController.execute).toHaveBeenCalled();
+        expect(sqlEditorController.check_data_changes_to_execute_query).toHaveBeenCalled();
       });
     });
     describe('when the command is being run from the view data view', () => {
@@ -39,10 +39,10 @@ describe('queryToolActions', () => {
         sqlEditorController.is_query_tool = false;
       });
 
-      it('it calls the execute_data_query function on the sqlEditorController', () => {
+      it('it calls the check_data_changes_to_execute_query function on the sqlEditorController', () => {
         queryToolActions.executeQuery(sqlEditorController);
 
-        expect(sqlEditorController.execute_data_query).toHaveBeenCalled();
+        expect(sqlEditorController.check_data_changes_to_execute_query).toHaveBeenCalled();
       });
     });
   });
@@ -55,6 +55,8 @@ describe('queryToolActions', () => {
         spyOn(queryToolActions, '_costsEnabled').and.returnValue(false);
         spyOn(queryToolActions, '_buffers').and.returnValue(false);
         spyOn(queryToolActions, '_timing').and.returnValue(false);
+        spyOn(queryToolActions, '_summary').and.returnValue(false);
+        spyOn(queryToolActions, '_settings').and.returnValue(false);
       });
 
       it('calls the execute function', () => {
@@ -69,19 +71,22 @@ describe('queryToolActions', () => {
           buffers: false,
           timing: false,
           summary: false,
+          settings: false,
         };
 
-        expect(sqlEditorController.execute).toHaveBeenCalledWith(explainObject);
+        expect(sqlEditorController.check_data_changes_to_execute_query).toHaveBeenCalledWith(explainObject);
       });
     });
 
-    describe('when verbose and costs and buffers and timing are all selected', () => {
+    describe('when all options are selected', () => {
       beforeEach(() => {
         setUpSpies('', '');
         spyOn(queryToolActions, '_verbose').and.returnValue(true);
         spyOn(queryToolActions, '_costsEnabled').and.returnValue(true);
         spyOn(queryToolActions, '_buffers').and.returnValue(true);
         spyOn(queryToolActions, '_timing').and.returnValue(true);
+        spyOn(queryToolActions, '_summary').and.returnValue(true);
+        spyOn(queryToolActions, '_settings').and.returnValue(true);
       });
       it('calls the execute function', () => {
         queryToolActions.explainAnalyze(sqlEditorController);
@@ -92,9 +97,10 @@ describe('queryToolActions', () => {
           costs: true,
           buffers: true,
           timing: true,
-          summary: false,
+          summary: true,
+          settings: true,
         };
-        expect(sqlEditorController.execute).toHaveBeenCalledWith(explainObject);
+        expect(sqlEditorController.check_data_changes_to_execute_query).toHaveBeenCalledWith(explainObject);
       });
     });
 
@@ -105,6 +111,8 @@ describe('queryToolActions', () => {
         spyOn(queryToolActions, '_costsEnabled').and.returnValue(false);
         spyOn(queryToolActions, '_buffers').and.returnValue(true);
         spyOn(queryToolActions, '_timing').and.returnValue(false);
+        spyOn(queryToolActions, '_summary').and.returnValue(false);
+        spyOn(queryToolActions, '_settings').and.returnValue(false);
       });
       it('calls the execute function', () => {
         queryToolActions.explainAnalyze(sqlEditorController);
@@ -117,9 +125,10 @@ describe('queryToolActions', () => {
           buffers: true,
           timing: false,
           summary: false,
+          settings: false,
         };
 
-        expect(sqlEditorController.execute).toHaveBeenCalledWith(explainObject);
+        expect(sqlEditorController.check_data_changes_to_execute_query).toHaveBeenCalledWith(explainObject);
       });
     });
 
@@ -130,6 +139,8 @@ describe('queryToolActions', () => {
         spyOn(queryToolActions, '_costsEnabled').and.returnValue(true);
         spyOn(queryToolActions, '_buffers').and.returnValue(false);
         spyOn(queryToolActions, '_timing').and.returnValue(true);
+        spyOn(queryToolActions, '_summary').and.returnValue(false);
+        spyOn(queryToolActions, '_settings').and.returnValue(false);
       });
       it('calls the execute function', () => {
         queryToolActions.explainAnalyze(sqlEditorController);
@@ -142,9 +153,38 @@ describe('queryToolActions', () => {
           buffers: false,
           timing: true,
           summary: false,
+          settings: false,
         };
 
-        expect(sqlEditorController.execute).toHaveBeenCalledWith(explainObject);
+        expect(sqlEditorController.check_data_changes_to_execute_query).toHaveBeenCalledWith(explainObject);
+      });
+    });
+
+    describe('when all are not selected except summary and settings', () => {
+      beforeEach(() => {
+        setUpSpies('', '');
+        spyOn(queryToolActions, '_verbose').and.returnValue(false);
+        spyOn(queryToolActions, '_costsEnabled').and.returnValue(false);
+        spyOn(queryToolActions, '_buffers').and.returnValue(false);
+        spyOn(queryToolActions, '_timing').and.returnValue(false);
+        spyOn(queryToolActions, '_summary').and.returnValue(true);
+        spyOn(queryToolActions, '_settings').and.returnValue(true);
+      });
+      it('calls the execute function', () => {
+        queryToolActions.explainAnalyze(sqlEditorController);
+
+        const explainObject = {
+          format: 'json',
+          analyze: true,
+          verbose: false,
+          costs: false,
+          buffers: false,
+          timing: false,
+          summary: true,
+          settings: true,
+        };
+
+        expect(sqlEditorController.check_data_changes_to_execute_query).toHaveBeenCalledWith(explainObject);
       });
     });
   });
@@ -155,6 +195,10 @@ describe('queryToolActions', () => {
         setUpSpies('', '');
         spyOn(queryToolActions, '_verbose').and.returnValue(true);
         spyOn(queryToolActions, '_costsEnabled').and.returnValue(true);
+        spyOn(queryToolActions, '_summary').and.returnValue(false);
+        spyOn(queryToolActions, '_settings').and.returnValue(false);
+        spyOn(queryToolActions, '_summary').and.returnValue(false);
+        spyOn(queryToolActions, '_settings').and.returnValue(false);
       });
 
       it('calls the execute function', () => {
@@ -167,8 +211,9 @@ describe('queryToolActions', () => {
           buffers: false,
           timing: false,
           summary: false,
+          settings: false,
         };
-        expect(sqlEditorController.execute).toHaveBeenCalledWith(explainObject);
+        expect(sqlEditorController.check_data_changes_to_execute_query).toHaveBeenCalledWith(explainObject);
       });
     });
 
@@ -177,6 +222,8 @@ describe('queryToolActions', () => {
         setUpSpies('', '');
         spyOn(queryToolActions, '_verbose').and.returnValue(false);
         spyOn(queryToolActions, '_costsEnabled').and.returnValue(false);
+        spyOn(queryToolActions, '_summary').and.returnValue(false);
+        spyOn(queryToolActions, '_settings').and.returnValue(false);
       });
 
       it('calls the execute function', () => {
@@ -189,9 +236,10 @@ describe('queryToolActions', () => {
           buffers: false,
           timing: false,
           summary: false,
+          settings: false,
         };
 
-        expect(sqlEditorController.execute).toHaveBeenCalledWith(explainObject);
+        expect(sqlEditorController.check_data_changes_to_execute_query).toHaveBeenCalledWith(explainObject);
       });
     });
 
@@ -200,6 +248,8 @@ describe('queryToolActions', () => {
         setUpSpies('', '');
         spyOn(queryToolActions, '_verbose').and.returnValue(true);
         spyOn(queryToolActions, '_costsEnabled').and.returnValue(false);
+        spyOn(queryToolActions, '_summary').and.returnValue(false);
+        spyOn(queryToolActions, '_settings').and.returnValue(false);
       });
 
       it('calls the execute function', () => {
@@ -212,8 +262,9 @@ describe('queryToolActions', () => {
           buffers: false,
           timing: false,
           summary: false,
+          settings: false,
         };
-        expect(sqlEditorController.execute).toHaveBeenCalledWith(explainObject);
+        expect(sqlEditorController.check_data_changes_to_execute_query).toHaveBeenCalledWith(explainObject);
       });
     });
   });
@@ -226,7 +277,7 @@ describe('queryToolActions', () => {
       it('does nothing', () => {
         queryToolActions.download(sqlEditorController);
 
-        expect(sqlEditorController.trigger_csv_download).not.toHaveBeenCalled();
+        expect(sqlEditorController.trigger_csv_download).toHaveBeenCalled();
       });
     });
 
@@ -247,12 +298,21 @@ describe('queryToolActions', () => {
           }));
         });
 
-        it('calls trigger_csv_download with the query and the filename', () => {
+        it('calls trigger_csv_download with filename having .csv extension', () => {
           let filename = 'data-' + time + '.csv';
 
           queryToolActions.download(sqlEditorController);
 
-          expect(sqlEditorController.trigger_csv_download).toHaveBeenCalledWith(selectedQueryString, filename);
+          expect(sqlEditorController.trigger_csv_download).toHaveBeenCalledWith(filename);
+        });
+
+        it('calls trigger_csv_download filename having .txt extension', () => {
+          sqlEditorController.preferences.csv_field_separator = ';';
+          let filename = 'data-' + time + '.txt';
+
+          queryToolActions.download(sqlEditorController);
+
+          expect(sqlEditorController.trigger_csv_download).toHaveBeenCalledWith(filename);
         });
       });
 
@@ -273,12 +333,12 @@ describe('queryToolActions', () => {
           }));
         });
 
-        it('calls trigger_csv_download with the query and the filename', () => {
+        it('calls trigger_csv_download with filename', () => {
           let filename = 'data-' + time + '.csv';
 
           queryToolActions.download(sqlEditorController);
 
-          expect(sqlEditorController.trigger_csv_download).toHaveBeenCalledWith(entireQueryString, filename);
+          expect(sqlEditorController.trigger_csv_download).toHaveBeenCalledWith(filename);
         });
       });
     });
@@ -291,7 +351,7 @@ describe('queryToolActions', () => {
 
         queryToolActions.download(sqlEditorController);
 
-        expect(sqlEditorController.trigger_csv_download).toHaveBeenCalledWith(query, 'iAmATable' + '.csv');
+        expect(sqlEditorController.trigger_csv_download).toHaveBeenCalledWith('iAmATable' + '.csv');
       });
     });
 
@@ -315,8 +375,8 @@ describe('queryToolActions', () => {
       beforeEach(() => {
         setUpSpies('', 'a string\nddd\nsss');
 
-        sqlEditorController.gridView.query_tool_obj.getCursor = (isFrom) => {
-          return isFrom ? 3 : 3;
+        sqlEditorController.gridView.query_tool_obj.getCursor = () => {
+          return 3;
         };
       });
 
@@ -362,8 +422,8 @@ describe('queryToolActions', () => {
       beforeEach(() => {
         setUpSpies('', 'a string\nddd\nsss');
 
-        sqlEditorController.gridView.query_tool_obj.getCursor = (isFrom) => {
-          return isFrom ? 3 : 3;
+        sqlEditorController.gridView.query_tool_obj.getCursor = () => {
+          return 3;
         };
       });
 
@@ -409,8 +469,8 @@ describe('queryToolActions', () => {
       beforeEach(() => {
         setUpSpies('', 'a string\nddd\nsss');
 
-        sqlEditorController.gridView.query_tool_obj.getCursor = (isFrom) => {
-          return isFrom ? 3 : 3;
+        sqlEditorController.gridView.query_tool_obj.getCursor = () => {
+          return 3;
         };
       });
 
@@ -454,8 +514,8 @@ describe('queryToolActions', () => {
       beforeEach(() => {
         setUpSpies('', 'a string\nddd\nsss');
 
-        sqlEditorController.gridView.query_tool_obj.getCursor = (isFrom) => {
-          return isFrom ? 3 : 3;
+        sqlEditorController.gridView.query_tool_obj.getCursor = () => {
+          return 3;
         };
       });
 
@@ -474,13 +534,13 @@ describe('queryToolActions', () => {
       it('toggle the selection and string should be in upper case', () => {
         queryToolActions.toggleCaseOfSelectedText(sqlEditorController);
         expect(replaceSelectionSpy
-          ).toHaveBeenCalledWith('STRING');
+        ).toHaveBeenCalledWith('STRING');
       });
 
       it('(negative scenario toggle the selection and string should be in upper case', () => {
         queryToolActions.toggleCaseOfSelectedText(sqlEditorController);
         expect(replaceSelectionSpy
-          ).not.toHaveBeenCalledWith('string');
+        ).not.toHaveBeenCalledWith('string');
       });
     });
 
@@ -492,13 +552,13 @@ describe('queryToolActions', () => {
       it('toggle the selection and string should be in lower case', () => {
         queryToolActions.toggleCaseOfSelectedText(sqlEditorController);
         expect(replaceSelectionSpy
-          ).toHaveBeenCalledWith('string');
+        ).toHaveBeenCalledWith('string');
       });
 
       it('(negative scenario toggle the selection and string should be in lower case', () => {
         queryToolActions.toggleCaseOfSelectedText(sqlEditorController);
         expect(replaceSelectionSpy
-          ).not.toHaveBeenCalledWith('STRING');
+        ).not.toHaveBeenCalledWith('STRING');
       });
     });
 
@@ -510,20 +570,46 @@ describe('queryToolActions', () => {
       it('toggle the selection and string should be in upper case', () => {
         queryToolActions.toggleCaseOfSelectedText(sqlEditorController);
         expect(replaceSelectionSpy
-          ).toHaveBeenCalledWith('STRING');
+        ).toHaveBeenCalledWith('STRING');
       });
 
       it('(negative scenario toggle the selection and string should be in upper case', () => {
         queryToolActions.toggleCaseOfSelectedText(sqlEditorController);
         expect(replaceSelectionSpy
-          ).not.toHaveBeenCalledWith('sTRIng');
+        ).not.toHaveBeenCalledWith('sTRIng');
       });
     });
   });
 
-  function setUpSpies(selectedQueryString, entireQueryString) {
-    getValueSpy = jasmine.createSpy('getValueSpy').and.returnValue(entireQueryString);
-    getSelectionSpy = jasmine.createSpy('getSelectionSpy').and.returnValue(selectedQueryString);
+  describe('executeCommit', () => {
+    describe('when commit action is being run from the query tool', () => {
+      beforeEach(() => {
+        setUpSpies('', '');
+      });
+
+      it('calls the execute commit function', () => {
+        queryToolActions.executeCommit(sqlEditorController);
+        expect(sqlEditorController.special_sql).toEqual('COMMIT;');
+      });
+    });
+  });
+
+  describe('executeRollback', () => {
+    describe('when rollback action is being run from the query tool', () => {
+      beforeEach(() => {
+        setUpSpies('', '');
+      });
+
+      it('calls the execute rollback function', () => {
+        queryToolActions.executeRollback(sqlEditorController);
+        expect(sqlEditorController.special_sql).toEqual('ROLLBACK;');
+      });
+    });
+  });
+
+  function setUpSpies(selectedQueryStringArg, entireQueryStringArg) {
+    getValueSpy = jasmine.createSpy('getValueSpy').and.returnValue(entireQueryStringArg);
+    getSelectionSpy = jasmine.createSpy('getSelectionSpy').and.returnValue(selectedQueryStringArg);
     replaceSelectionSpy = jasmine.createSpy('replaceSelectionSpy');
 
     sqlEditorController = {
@@ -536,7 +622,7 @@ describe('queryToolActions', () => {
           uncomment: jasmine.createSpy('uncommentSpy'),
           replaceSelection: replaceSelectionSpy,
           getCursor: (isFrom) => {
-            return entireQueryString.indexOf(selectedQueryString) + (isFrom ? 0 : selectedQueryString.length);
+            return entireQueryStringArg.indexOf(selectedQueryStringArg) + (isFrom ? 0 : selectedQueryStringArg.length);
           },
         },
       },
@@ -544,8 +630,10 @@ describe('queryToolActions', () => {
       trigger: jasmine.createSpy('trigger'),
       table_name: 'iAmATable',
       is_query_tool: true,
-      execute: jasmine.createSpy('execute'),
-      execute_data_query: jasmine.createSpy('execute_data_query'),
+      check_data_changes_to_execute_query: jasmine.createSpy('check_data_changes_to_execute_query'),
+      preferences: {
+        csv_field_separator: ',',
+      },
     };
   }
 });
